(defvar #:sys-package:colon '#:shirka:moteur:filtrage)

(defvar #:exp:trace nil)

;;; On filtre avec "nom-filtre" et plus particulierement avec une de ses 
;;; instances. Celle-ci a ete partiellement instanciee (pour ses attributs
;;; fixes). Sa propriete "ordre-fixe" est a jour, et donne l'ordre 
;;; provisoire de parcours des attributs, en tenant compte des dependances.
;;; On filtre a partir de la position du marqueur, materialise par la
;;; propriete "MARQ-INST" de l'instance "Nom-inst-f" et qui contient la liste
;;; d'instances restant a filtrer.
;;; Renvoie NIL ou la valeur resultat du filtrage.

(de :FILTRAGE (nom-inst-f nom-filtre)
    (tag ECHEC-FILTRAGE
         ;;; -->maj d'ordre-parcours a partir d'ordre-fixe ,en tenant compte
         ;;; -->des attributs semi-fixes pour un filtre emboite.
         ;;; -->On obtient alors l'ordre dans lequel seront parcourus tous les 
         ;;; -->attributs sous la forme d'une liste ((fixes et semi-fixes) (autres))
         (let ((filt-h-niv (:FILTRE-PLUS-HAUT-NIVEAU? nom-filtre)))
              (if filt-h-niv
                  (putprop nom-inst-f (getprop nom-inst-f 'ordre-att) 
                          'ordre-parcours)
                  (:INSTANCIER-SEMI-FIXES nom-inst-f nom-filtre))
              (tag SUCCES-FILTRAGE
                   ;-->filtrage "d'ordre 0"
                   (let (( filtre-0 (:FILTRAGE-BOITEUX nom-inst-f nom-filtre)))
                        (if filtre-0 (exit SUCCES-FILTRAGE filtre-0)))
                   ; -->On teste s'il existe des instances a filtrer
                   (let ((list-inst (getprop nom-inst-f 'marq-inst)))
                        (ifn list-inst (:ACTION-ECHEC-FILTRAGE nom-inst-f nom-filtre))
                        ; --> filtrage instance par instance a partir du marqueur
                        ; --> un echec sur une instance n'est definitif que s'il s'agit
                        ; --> de la derniere instance a traiter
                        (mapc '(lambda (inst-cour)
                                       (:MAJ-INST-TRAV nom-filtre inst-cour)
                                       (if (:FILTRER-INST nom-inst-f nom-filtre)
                                           (:ACTION-SUCCES-FILTRAGE nom-inst-f
                                                   nom-filtre
                                                   filt-h-niv
                                                   list-inst))
                                       ; -> on met list-inst a jour pour le marquage 
                                       (setq list-inst (cdr list-inst)))
                              list-inst)
                        ;--> si on n'est pas sorti par un succes c'est qu'il y a echec
                        (:ACTION-ECHEC-FILTRAGE nom-inst-f nom-filtre)) ))))

;;; Filtrage tres particulier: si l'instance de filtre
;;; est totalement instanciee, on renvoie la valeur de
;;; l'attribut-resultat
(de :FILTRAGE-BOITEUX (nom-inst-f nom-filtre)
    (let ((l-att-var (cadr  (getprop nom-inst-f 'ordre-parcours)))
          (att-dem   (caar  (getprop nom-filtre 'dem))) )
         ;--> si tous les attributs sont instancies et qu'il existe une
         ;--> facette "$valeur" sur l'attribut demande on renvoie la valeur
         (slet ((val-fac (#:shirka:comp:VAL-FAC-RESEAU nom-filtre att-dem %const))
                (val (if (eq 0 (car val-fac)) (cdr val-fac))))
               (if (and val (equal l-att-var (list att-dem)))
                   val))))

;;; Si aucune instance ne satisfait le filtre:
(de :ACTION-ECHEC-FILTRAGE (nom-inst-f nom-filtre)
    (ifn (:FILTRE-PLUS-HAUT-NIVEAU? nom-filtre)
         (:RAZ-MARQUEUR nom-filtre nom-inst-f) )
    (exit ECHEC-FILTRAGE))

;;; Si l'instance courante satisfait le filtre:
;;; on renvoie la valeur se trouvant dans
;;; l'instance de travail puis on met le marqueur a jour
(de :ACTION-SUCCES-FILTRAGE (nom-inst-f nom-filtre filt-h-niv list-inst)
    (exit SUCCES-FILTRAGE
          (prog1 
                 (:RENVOYER-VAL nom-filtre nom-inst-f)
                 (if filt-h-niv 
                     (putprop nom-inst-f (cdr list-inst) 'marq-inst)
                     (:RAZ-MARQUEUR nom-filtre nom-inst-f) ) ) ) )

;;; Filtrage d'une instance (l'instance de travail)
;;; avec l'instance "nom-inst-f" du filtre "nom-filtre"
;;; Renvoie NIL si echec et T sinon
;;; Place la valeur resultat dans l'instance de travail
(de :FILTRER-INST (nom-inst-f nom-filtre)
    ;-->initialisation du parcours des attributs.
    (if #:exp:trace 
        (#:exp:cons-tr  '(but m-e-cor)))
    (slet ((nom-inst-trav (getprop nom-filtre 'lie))
           (att (getprop nom-inst-f 'ordre-parcours))
           (att-fixes (car att))
           (att-var   (cadr att)))
          (if #:exp:trace 
              (#:exp:cons-tr  
                      (list 'tr-me-cor nom-filtre 
                            (vref (#:shirka:noyau:schema nom-inst-trav) 2))))
          ;-->On commence par parcourir les attributs "fixes"
          ;-->1er parcours: ceux qui sont valorises dans l'instance de travail
          ;-->2eme parcours :les fixes restants
          ;-->On parcours alors les attributs "variables".
          (tag ECHEC-FILT-ATT
               (let (( mec1 (if att-fixes (:MEC-ATT-FIXES-1 nom-filtre
                                                  att-fixes 
                                                  nom-inst-f
                                                  nom-inst-trav))))
                    ;-->si il reste des attributs fixes a traiter :
                    (if mec1 (:MEC-ATT-FIXES-2 mec1
                                     nom-inst-f
                                     nom-filtre
                                     nom-inst-trav)) )   
               ;-->s'il y a des attributs "variables":
               (if att-var (:MEC-AUTRES att-var
                                   nom-inst-f
                                   nom-filtre
                                   nom-inst-trav))
               ;-->On teste un "a-verifier" eventuel sur " lui-meme "
               (:A-VERIF-LUI-MEME nom-filtre)
               (if #:exp:trace 
                   (#:exp:cons-tr  '(succes)))
               T) ) )

(de :A-VERIF-LUI-MEME (nom-filtre)
    (let ((l-a-verif (#:shirka:comp:VAL-FAC-RESEAU 
                             nom-filtre 'lui-meme %a-verif)))
         (if l-a-verif
             (ifn (#:shirka:moteur:A-VERIF l-a-verif) 
                  (exit ECHEC-FILT-ATT)))))

;;; Effectue la mise en correspondance des attributs fixes
;;; de l'instance de filtre "nom-inst-f" avec les attributs
;;; valorises (instancies) dans l'instance de travail.
;;; Renvoie une liste d'attributs que devra parcourir
;;; MEC-ATT-FIXES-2 si tout se passe bien et NIL en cas
;;; d'echec sur un attribut (d'ou EXIT ECHEC-FILT-ATT).
;;; L'admissibilite par rapport au filtre est supposee deja
;;; faite pour les valeurs d'attributs de l'instance de filtre.
(de :MEC-ATT-FIXES-1 (nom-filtre l-att-fixes nom-inst-f nom-inst-trav)
    (mapc '(lambda (nom-att)
                   (let ((val-inst-t (#:shirka:moteur:filtrage:VALEUR-ATT? 
                                             nom-inst-trav nom-att ))
                         (val-inst-f (#:shirka:moteur:filtrage:VALEUR-ATT-F? 
                                             nom-inst-f    nom-att )))
                        (when val-inst-t
                              (when #:exp:trace 
                                    (#:exp:cons-tr  '(but m-e-att))
                                    (#:exp:cons-tr  (list 'tr-me-att1
                                                          nom-att val-inst-t))
                                    )
                              (:MEC-ATT nom-filtre val-inst-t val-inst-f nom-att)
                              (when #:exp:trace 
                                    (#:exp:cons-tr  '(succes))
                                    )
                              (setq l-att-fixes (remq nom-att l-att-fixes)) ) ) )
          l-att-fixes)
    l-att-fixes)

;;; Idem que la precedente mais effectue une mise en 
;;; correspondance avec des attributs de l'instance de travail
;;; dont il faut calculer les valeurs. Celles-ci sont alors placees 
;;; dans l'instance de travail.
;;; Il est inutile de tester leur admissibilite car cela a du etre
;;; fait pour les valeurs d'attributs fixes et semi-fixes de
;;; l'instance de filtre. La mec en decidera donc.
;;; Renvoie toujours NIL.
(de :MEC-ATT-FIXES-2 (l-att-fixes nom-inst-f nom-filtre nom-inst-trav)
    (mapc '(lambda (nom-att)
                   (when #:exp:trace 
                         (#:exp:cons-tr  '(but m-e-att))
                         (#:exp:cons-tr  (list 'tr-me-att2
                                               nom-att))
                         )
                   (let ((val-att-t (#:shirka:moteur:VALEUR? 
                                            nom-att nom-inst-trav))
                         (val-att-f (#:shirka:moteur:filtrage:VALEUR-ATT-F? 
                                            nom-inst-f nom-att )) )
                        (unless val-att-t
                                (when #:exp:trace 
                                      (#:exp:cons-tr  '(echec))
                                      (#:exp:cons-tr  '(echec))
                                      )
                                (exit ECHEC-FILT-ATT)
                                ) 
                        ;-->on place la valeur dans l'instance de travail
                        (ifn  (eq nom-att 'lui-meme)
                             (#:shirka:moteur:AJ-VALEUR 
                                     (#:shirka:noyau:SCHEMA 
                                             nom-inst-trav)
                                     nom-att
                                     val-att-t
                                     T) ) 
                        (:MEC-ATT nom-filtre val-att-t val-att-f nom-att) 
                        ) ) 
          l-att-fixes) )

;;; Mise en correspondance des valeurs donnees en parametre
;;; pour l'attribut "nom-att".
;;; Renvoie toujours NIL.
(de :MEC-ATT ( nom-filtre val-att-t val-att-f nom-att )
    ;-->Test egalite simple. Si ca ne marche pas on teste
    ;-->une eventuelle appartenance (c'est le cas ou nom-att
    ;-->est type $un et reference un attribut type $liste-de
    ;-->dans le filtre)
    (ifn (equal val-att-t val-att-f)
         (unless (and (consp val-att-f)
                      (member val-att-t val-att-f)
                      (APPARTIENT? nom-att nom-filtre) )
                 (when #:exp:trace 
                       (#:exp:cons-tr  '(echec))
                       (#:exp:cons-tr  '(echec))
                       )
                 (exit ECHEC-FILT-ATT) ) ) )

;;; Mise en correspondance des attributs variables ,a savoir:
;;; $var<-- avec une reference interne au filtre
;;; $sib-filtre non encore fixes
;;; $sib-exec ou $sib-choix ou $defaut s'ils ne font pas partie
;;; des attributs deja fixes et qu'ils figurent avec un $sib-filtre
;;; $a-verifier seuls ou combines avec un des cas precedents
(de :MEC-AUTRES (l-att-var nom-inst-f nom-filtre nom-inst-trav)
    ;-->Servira a calculer le type des attributs pour tester le domaine
    (#:shirka:noyau:L-NOM-ATT-INST-HE 
            (#:shirka:noyau:SCHEMA 
                    (#:shirka:noyau:VAL-INST 
                            (#:shirka:noyau:SCHEMA nom-inst-trav) 
                            %EST-UN )))
    (mapc '(lambda (nom-att)
                   (when #:exp:trace 
                         (#:exp:cons-tr  '(but m-e-att))
                         )
                   (if #:exp:trace 
                       (#:exp:cons-tr  (list 'tr-me-att2 nom-att))
                       )
                   (let ((val-att-t (#:shirka:moteur:VALEUR? 
                                            nom-att nom-inst-trav )))
                        (ifn val-att-t (exit ECHEC-FILT-ATT 
                                             (when #:exp:trace 
                                                   (#:exp:cons-tr  '(echec))
                                                   (#:exp:cons-tr  '(echec))
                                                   )
                                             ))
                        ;-->Si on obtient une valeur ,il faut tester son 
                        ;-->admissibilite par rapport au filtre.
                        (ifn (eq nom-att 'lui-meme) 
                             (ifn (:ADMIS-CONST-A-VERIF? val-att-t nom-filtre nom-att)
                                  (exit ECHEC-FILT-ATT
                                        (if #:exp:trace 
                                            (#:exp:cons-tr  '(echec))
                                            )
                                        )
                                  (#:shirka:moteur:AJ-VALEUR 
                                          (#:shirka:noyau:SCHEMA nom-inst-trav)
                                          nom-att
                                          val-att-t 
                                          T ) ) )
                        ;-->On cherche alors a inferer la valeur de nom-att dans
                        ;-->l'instance de filtre pour faire la mise en correspondance
                        (lets ((val-att-reseau 
                                       (#:shirka:comp:VAL-ATT-RESEAU 
                                               nom-filtre nom-att))
                               (inf? (:INFERER? nom-filtre nom-att val-att-reseau)))
                              (when inf?   
                                    (let ((val-att-f (:INFERENCE-FILT val-att-reseau
                                                             nom-filtre
                                                             nom-inst-f
                                                             nom-att)))
                                         (ifn val-att-f (exit ECHEC-FILT-ATT
                                                              (if #:exp:trace 
                                                                  (#:exp:cons-tr  '(echec))
                                                                  )
                                                              )
                                              (:MEC-ATT nom-filtre
                                                      val-att-t      
                                                      val-att-f
                                                      nom-att)) ) )
                              (let ((tr-adm (#:shirka:comp:val-fac-att-reseau 
                                                    val-att-reseau %const)))
                                   (ifn (or inf? (not tr-adm))
                                        (if #:exp:trace 
                                            (#:exp:cons-tr  (list 'tr-admis tr-adm))
                                            )))
                              ) ) )
          l-att-var)
    (if #:exp:trace 
        (#:exp:cons-tr  '(succes))
        )
    )

;;; On teste s"il est necessaire d'inferer la valeur
(dmd :INFERER? (nom-filtre nom-att val-att-reseau)
     `(or (:EXISTE-FILTRE? ,val-att-reseau)
          (:REF-INTERNE? ,nom-filtre ,nom-att ,val-att-reseau)) ) 

(dmd :EXISTE-FILTRE? (val-att-reseau)
     `(#:shirka:comp:VAL-FAC-ATT-RESEAU ,val-att-reseau %sib-filt))

(dmd :REF-INTERNE? (nom-filtre nom-att val-att-reseau)
     `(:REF-VAR-TYPE %var-interne 
              (#:shirka:comp:VAL-FAC-ATT-RESEAU
                      ,val-att-reseau %var)) )

;;; Teste si le filtre est de plus haut niveau. Renvoie NIL si non.
(dmd :FILTRE-PLUS-HAUT-NIVEAU? (nom-filtre)
     `(= %var-result (cadar (getprop ,nom-filtre 'dem))) )

;;; Recopie l'instance courante dans l'instance de travail. Cette
;;; derniere est necessaire pour placer les valeurs calculees au 
;;; cours du filtrage.
(de :MAJ-INST-TRAV  (nom-filtre inst-cour)
    (let (( nom-inst-trav (getprop nom-filtre 'lie)))
         ;-->on place toutes les proprietes de "inst-cour" 
         ;-->dans l'instance de travail.
         (plist nom-inst-trav (plist inst-cour))                            
         (setq nom-inst-trav (copy inst-cour))  ) )

;;; Replace la liste complete d'instances filtrables dans le
;;; marqueur d'instances.
;;; Renvoie cette liste.
(dmd :RAZ-MARQUEUR (nom-filtre nom-inst-f)
     `(putprop ,nom-inst-f 
              (#:shirka:noyau:L-INST 
                      (#:shirka:noyau:SCHEMA ,nom-filtre))
              'marq-inst) )

;;; En cas de succes du filtrage, on renvoie la valeur demandee.
(de :RENVOYER-VAL (nom-filtre nom-inst-f) 
    (let ((att-dem (caar (getprop nom-filtre 'dem))))
         (or (:VALEUR-ATT? 
                     (getprop nom-filtre 'lie);-->inst de travail
                     att-dem)
             (#:shirka:noyau:val-fac-he
                     (#:shirka:noyau:schema nom-filtre) 
                     att-dem
                     '$valeur))))

;;; Teste s'il faut effectuer une appartenance (T) ou une egalite
;;; (NIL) lors de la m.e.c. Il faut tester l'appartenance quand
;;; un attribut "$un" et "$var" reference un attribut "$liste-de".
(de :APPARTIENT? (nom-att nom-filtre)
    (let (( val-fac-var (#:shirka:comp:VAL-FAC-RESEAU 
                                nom-filtre nom-att %var)))
         (if val-fac-var
             (or (#:shirka:moteur:REF-VAR-TYPE 
                         %var-fixe-app val-fac-var)
                 (#:shirka:moteur:REF-TYPE-VAR 
                         %var-semifixe-app val-fac-var) ) ) ) )
;--> si pas de var : egalite.

;;; Teste l'admissibilite pour les facettes constantes ($valeur,
;;; $domaine et $intervalle) du filtre.
;;; renvoie T si la valeur a tester est admissible NIL sinon.
(de :ADMIS? ( val-a-tester nom-filtre nom-att)
    ( SLET (( val-att-reseau 
                    (#:shirka:comp:val-att-reseau nom-filtre nom-att))
            ( val-fac-const 
                    (#:shirka:comp:VAL-FAC-ATT-RESEAU val-att-reseau %const) ))
          (if (null val-fac-const)
              t ;-->aucune facette constante: admissible
              (let (( t-const (car val-fac-const))
                    ( val-fac (cdr val-fac-const)) )
                   (selectq t-const
                           ( 0 (equal val-fac val-a-tester));-->$valeur
                           ( 1 (#:shirka:noyau:APP-INTER? val-a-tester
                                       (#:shirka:comp:VAL-FAC-ATT-RESEAU
                                               val-att-reseau %type-att)
                                       val-fac )) ;-->$intervalle
                           ( 2 (#:shirka:noyau:APP-DOM? val-a-tester  
                                       (#:shirka:comp:VAL-FAC-ATT-RESEAU 
                                               val-att-reseau  %type-att)
                                       val-fac)) ) )) ) )
