(defvar #:sys-package:colon '#:shirka:moteur:filtrage)

;;; Teste l'admissibilite par rapport aux constantes et lance "a-verifier"
;;; Renvoie NIL si non admissible.
(de :ADMIS-CONST-A-VERIF? (val-a-tester nom-filtre nom-att )
    (if (:ADMIS? val-a-tester nom-filtre nom-att )
        (let ((val-fac-a-verif 
                      (#:shirka:comp:VAL-FAC-RESEAU 
                              nom-filtre nom-att %a-verif)))
             (if val-fac-a-verif
                 (#:shirka:moteur:A-VERIF val-fac-a-verif)
                 T ) ) ) )

;;; Recherche d'une valeur par inference pour la placer dans l'instance de
;;; filtre.
;;; Renvoie NIL ou cette valeur si on peut l'obtenir.
(de :INFERENCE-FILT (val-att-reseau nom-filtre nom-inst-f nom-att)
    (if #:exp:trace 
        (#:exp:cons-tr  (list 'but nom-att nom-inst-f))
        )
    (tag SUCCES-INF
         ;-->S'il existe une reference interne, elle doit etre resolue .Sinon erreur
         (let ((val-fac-ref-int 
                       (:REF-INTERNE? 
                               nom-filtre nom-att val-att-reseau)))
              (when val-fac-ref-int
                    (when #:exp:trace 
                          (#:exp:cons-tr  (list 'tr-vr nom-att val-fac-ref-int))
                          (#:exp:cons-tr  '(succes)))
                    (exit SUCCES-INF 
                          (:OBT-VAL-REF-F nom-inst-f val-fac-ref-int))))
         ;-->sinon c'est qu'il y a un sib-filtre (filtres emboites)
         (mapc '(lambda (n-filtre)
                        ;-->Recherche de l'instance de filtre correspondant a "n-filtre"
                        ;-->puis filtrage
                        (when #:exp:trace 
                              (#:exp:cons-tr  (list 'tr-vf nom-att nom-inst-f nom-filtre))
                              )
                        (let (( result-filt 
                                      (:FILTRAGE 
                                              (:RECH-INST-FILT nom-inst-f n-filtre)
                                              n-filtre)))
                             ; s'il y a un resultat admissible on le renvoie
                             (when (and result-filt 
                                        (:ADMIS-CONST-A-VERIF?  result-filt
                                                nom-filtre
                                                nom-att ) )     
                                   (if #:exp:trace 
                                       (#:exp:cons-tr  '(succes))
                                       )
                                   (exit SUCCES-INF result-filt) )  ) )
               (#:shirka:comp:VAL-FAC-ATT-RESEAU val-att-reseau %sib-filt) ) ;--> liste des filtres
         ;-->Si on n'a pas reussi a filtrer une valeur, il faut essayer d'autres 
         ;-->methodes moins prioritaires.
         (let ((result-exec 
                       (:ESSAI-SIB-EX val-att-reseau nom-filtre nom-att )))
              (when result-exec
                    (when #:exp:trace 
                          (#:exp:cons-tr  '(succes))
                          )
                    (exit SUCCES-INF result-exec) ) )
         (let ((result-choix 
                       (:ESSAI-SIB-CHOIX val-att-reseau nom-filtre nom-att)))
              (when result-choix
                    (when #:exp:trace 
                          (#:exp:cons-tr  '(succes))
                          )
                    (exit SUCCES-INF result-choix) ) )
         (let ((defaut (:ESSAI-DEFAUT  val-att-reseau nom-filtre nom-att)))
              (when defaut 
                    (when #:exp:trace 
                          (#:exp:cons-tr  '(succes))
                          )
                    (exit SUCCES-INF defaut) ) )
         (when #:exp:trace 
               (#:exp:cons-tr  '(echec)) ) ) )
        ;-->si rien ne marche , on sort avec NIL.

;;; Recherche de l'instance du filtre "nom-filtre" emboite dans le filtre 
;;; dont l'instance courante est "nom-inst".
(dmd :RECH-INST-FILT (nom-inst nom-filtre)
     `(cassq ,nom-filtre (getprop ,nom-inst 'f-imb)) )

(de :ERREUR () (print 'ERRRRRRRRREUR....))

;;; Definition des types de facettes $var que l'on peut trouver dans le reseau
;;; pour un attribut d'un filtre
(defvar %var-fixe-simple 0)
(defvar %var-fixe-app 1)
(defvar %var-semifixe-simple 2)
(defvar %var-semifixe-app 3) 
(defvar %var-result 4)
(defvar %var-result-imb 5)
(defvar %var-interne 6)
  
;;; Instanciation des attributs semi-fixes (pour un filtre imbrique).A savoir :
;;; ceux n'ayant pas deja ete instancies (fixes)
;;; ceux correspondant a des facettes  $var particulieres
;;; ceux correspondant a $sib-exec ou $sib-choix ou $defaut a condition
;;; qu'il n'existe aucun $sib-filtre.
;;; Mise a jour de la propriete "ordre-parcours" a partir de "ordre-att".
;;; au depart,"ordre-att" et "ordre-parcours" sont de la forme :((fixes) (autres)) 
;;; on parcourt "autres" a la recherche de semi-fixes et on rajoute ceux-ci aux 
;;; "fixes" de "ordre-parcours" .
(de :INSTANCIER-SEMI-FIXES (nom-inst-filtre nom-filtre)
    (slet (( l-att (copy (getprop nom-inst-filtre 'ordre-att))) ;-->"fixes" et "autres"
           (l-att-fixes (car l-att)) ;-->"fixes"
           (l-att-var (cadr l-att))) ;--> "autres"
          (mapc '(lambda (nom-att)
                         (slet ((val-att-reseau 
                                        (#:shirka:comp:VAL-ATT-RESEAU 
                                                nom-filtre nom-att))
                                (val-att 
                                        (:OBT-VAL-ATT-SEMI-FIXES 
                                                val-att-reseau 
                                                nom-inst-filtre nom-filtre nom-att)))
                               (when val-att ;-->l'attribut courant est semi-fixe  
                                     ;-->maj des listes "fixes" et "autres"   
                                     (nconc1 l-att-fixes nom-att)
                                     (setq l-att-var (deLQ nom-att l-att-var))
                                     ;-->placement de la valeur dans l'instance de filtre
                                     (:AJ-VALEUR-INST-FILTRE val-att nom-att nom-inst-filtre))))
                l-att-var)
          ;-->maj de la propriete "ordre-parcours"
          (putprop nom-inst-filtre (list l-att-fixes l-att-var) 'ordre-parcours)))

;;; On cherche si l'attribut de nom "nom-att" est semi-fixe.
;;; Renvoie la valeur de cet attribut s'il est semi-fixe
;;; et NIL sinon.
(de :OBT-VAL-ATT-SEMI-FIXES (val-att-reseau nom-inst-filtre nom-filtre nom-att)
    (tag SORTIE
         (let ((val-var (:ESSAI-VAR val-att-reseau nom-filtre nom-att)))
              (if val-var (exit SORTIE val-var)) )
         (if  (:EXISTE-FILTRE? val-att-reseau) (exit SORTIE))
         (let ((val-sib-ex (:ESSAI-SIB-EX val-att-reseau nom-filtre nom-att)))
              (if val-sib-ex (exit SORTIE val-sib-ex)) )
         (let ((val-sib-ch (:ESSAI-SIB-CHOIX val-att-reseau nom-filtre nom-att)))
              (if val-sib-ch (exit SORTIE val-sib-ch)) )
         (let ((val-def (:ESSAI-DEFAUT val-att-reseau nom-filtre nom-att)))
              (if val-def (exit SORTIE val-def)) ) ) )

;;; Tente de trouver une valeur semi-fixe sur les facettes $var.
;;; Renvoie cette valeur apres en avoir teste l'admissibilite
;;; pour le filtre, ou NIL si aucune valeur n'est obtenue.
(de :ESSAI-VAR (val-att-reseau nom-filtre nom-att)
    (slet ((val-fac-var (#:shirka:comp:VAL-FAC-ATT-RESEAU val-att-reseau %var))
           (ref-var (or (:REF-VAR-TYPE %var-result-imb val-fac-var)
                        (:REF-VAR-TYPE %var-semifixe-simple val-fac-var)
                        (:REF-VAR-TYPE %var-semifixe-app val-fac-var) ) ) )
          (if ref-var
              (let ((val-var (:OBT-VAL-REF-F nom-inst-filtre ref-var)))
                   (if val-var 
                       (if (:ADMIS-CONST-A-VERIF? val-var nom-filtre nom-att)
                           val-var) )))))

;;; Tentative d'inference d'une valeur semi-fixe sur la facette
;;; $sib-filtre  de "nom-att".
;;; Test d'admissibilite sur cette valeur.
;;; Renvoie la valeur ou NIL.
(de :ESSAI-SIB-EX (val-att-reseau nom-filtre nom-att)
    (let ((l-sibex (#:shirka:comp:VAL-FAC-ATT-RESEAU
                           val-att-reseau %sib-exec)))
         (if l-sibex
             (let (( result-exec (:SIBEX1 l-sibex nom-filtre)))
                  (if (and result-exec
                           (:ADMIS-CONST-A-VERIF? result-exec nom-filtre nom-att))
                      result-exec) ) ) ) )
      
;;; REMARQUE : optimisation possible en testant chaque sib-exec independamment.

(de :SIBEX1 (l-sib-exec nom-filtre)
    (#:shirka:moteur:lier nom-filtre (getprop nom-filtre 'lie))
    (tag SORTIE
         (:SIB-EXEC l-sib-exec (#:shirka:noyau:SCHEMA nom-filtre)) ) )

;;; Meme fonction que ESSAI-SIB-EX sur $sib-choix.
(de :ESSAI-SIB-CHOIX (val-att-reseau nom-filtre nom-att)
    (let ((l-sib-choix 
                  (#:shirka:comp:VAL-FAC-ATT-RESEAU 
                          val-att-reseau %sib-choix)))
         (if l-sib-choix
             (let (( result-choix (:SIBCHOIX l-sib-choix)))
                  (if (and result-choix
                           (:ADMIS-CONST-A-VERIF? result-choix nom-filtre nom-att))
                      result-choix) ) ) ) )

(de :ESSAI-DEFAUT (val-att-reseau nom-filtre nom-att)
    (let ((defaut 
                  (#:shirka:comp:VAL-FAC-ATT-RESEAU 
                          val-att-reseau %defaut)))
         (when (and defaut
                    (:ADMIS-CONST-A-VERIF? defaut nom-filtre nom-att))
               (when #:exp:trace 
                     (#:exp:cons-tr  (list 'tr-vd defaut 
                                           nom-att (:inst-cour nom-filt) nom-filt))
                     )
               defaut) ) )

;;; Prefiltrage constant pour tous les filtres du schema "nom-schema"
;;; dont une instance "nom-instance"vient d'etre creee, ainsi que
;;; pour tous les filtres de tous les schemas ancetres de "nom-schema".
;;; Il s'agit de rattacher aux filtres l'instance "nom-instance" si
;;; elle satisfait la partie constante du filtre ,totalement ou
;;; partiellement (cas de l'instance incomplete). 
(de :FILT-CONST-ANC (nom-instance nom-schema)
    ;-->on commence par les filtres du schema courant:
    (:FILTRAGE-CONSTANT nom-instance nom-schema)
    ;-->on poursuit avec ceux des ancetres:
    (mapc '(lambda (nom-sch)
                   (:FILTRAGE-CONSTANT nom-instance nom-sch))
          (#:shirka:noyau:ANCETRES 
                  (#:shirka:noyau:SCHEMA nom-schema)) ) )

;;; Prefiltrage constant pour les filtres du schema "nom-schema"
(de :FILTRAGE-CONSTANT ( nom-instance nom-schema )
    (mapc '(lambda (nom-filtre)
                   (:FILT-CONST-INST nom-instance nom-filtre ) )
          (:CHERCHER-FILTRES nom-schema) ) )

;;; Ramene la liste de tous les filtres specialisant "nom-schema".
(dmd :CHERCHER-FILTRES ( nom-schema )
     `(getprop ,nom-schema 'filtres) )

;;; Teste l'admissibilite pour la partie constante du filtre "nom-filtre"
;;; de l'instance "nom-instance".
;;; Puis teste l'admissibilite par rapport a l'eventuelle 
;;; specialisation locale associee au filtre a condition que celle-ci
;;; soit de meme nature que le filtre (lui-meme renvoye par le filtre).
;;; Si OK: ajoute cette instance aux instances 
;;; du filtre et la renvoie. Sinon: renvoie nil et ne fait rien d'autre.
(de :FILT-CONST-INST ( nom-instance nom-filtre )
    (slet (( filtre (#:shirka:noyau:SCHEMA nom-filtre))
           (incomplete NIL)
           (l-att-filt 
                   (delq 'lui-meme 
                         (#:shirka:noyau:L-NOM-ATT filtre)))
           (att-dem (#:shirka:moteur:OBT-ATT-DEM nom-filtre)) 
           ;--> attribut resultat du filtre
           (sch-cible (:OBT-SCH-CIBLE nom-filtre)) 
           ;--> schema ou est att-dem
           (type (:OBT-TYPE-ATT-CIBLE nom-filtre sch-cible ))
           (spec-loc? (and (eq att-dem 'lui-meme) 
                           ;--> vrai si il existe
                           (not (#:shirka:noyau:TYPE-SIMPLE? type)) 
                           ;--> une specialisation locale associee
                           (getprop type 'spec-loc)))  
           (sch-spec (if spec-loc? (#:shirka:noyau:SCHEMA type))) )
          (tag ECHEC-INST
               (mapc '(lambda (nom-att-filt )
                              (let ((val-att-filt 
                                            (:VALEUR-ATT? 
                                                    nom-instance nom-att-filt )) )
                                   (ifn val-att-filt 
                                        (setq incomplete T)
                                        ;--> si l'attribut a une valeur:
                                        ;--> on teste son admisibilite sur le filtre.
                                        (cond ((:admis? val-att-filt nom-filtre nom-att-filt))
                                              (t (exit echec-inst))))))
                     l-att-filt)
               ;--> admissibilite sur l'eventuelle specialisation locale.
               (if spec-loc?
                   (:ADMIS-SPEC-LOC?  sch-spec nom-instance))
               ;-->on place "nom-instance" dans la liste d'instances de "nom-filtre"
               ;--> en fin de liste si l'instance est incomplete, en tete sinon.
               (if incomplete
                   (#:shirka:moteur:AJ-VALEUR filtre  'inst nom-instance T)
                   (#:shirka:moteur:AJ-VALEUR filtre  'inst nom-instance  NIL ) ) ) ) )

;;; Teste l'admissibilite  de l'instance "nom-instance" pour la 
;;; specialisation locale "sch-spec" dont les attributs forment
;;; la liste l-att-spec.
;;; Renvoie NIL si non admissible et T si elle est admissible ou
;;; incomplete et partiellement admissible. 
(de :ADMIS-SPEC-LOC? ( sch-spec nom-instance)
    (let ((l-att-spec (#:shirka:noyau:L-NOM-ATT sch-spec)))
         (#:shirka:noyau:L-NOM-ATT-INST-HE sch-spec)
         ;--> pour calculer les "natures"
         ;--> admissibilite sur la specialisation locale:
         (mapc '(lambda (nom-att-spec)
                        (let ((val-att-spec 
                                      (#:shirka:moteur:filtrage:VALEUR-ATT? 
                                              nom-instance nom-att-spec )) )
                             (if (and val-att-spec spec-loc?)
                                 (unless (:ADMIS-CONST-SPEC? val-att-spec sch-spec nom-att-spec )
                                         (exit ECHEC-INST) )) ) )
               l-att-spec) ) T )

;;; Ramene le nom du schema dans lequel se trouve l'attribut
;;; filtrable par "nom-filtre".
(dmd :OBT-SCH-CIBLE (nom-filtre)
     `(caddar (getprop ,nom-filtre 'dem)))

;;; Ramene le type de l'attribut auquel est renvoye le resultat
;;; du filtre "nom-filtre".Cet attribut est dans "nom-schema".
(dmd :OBT-TYPE-ATT-CIBLE (nom-filtre nom-schema)
     `(cadr (#:shirka:noyau:NATURE 
                    (#:shirka:noyau:SCHEMA ,nom-schema)
                    ;--> attribut cible
                    (cdddar (getprop ,nom-filtre 'dem)))))

;;; Teste l'admissibilite de "val-a-tester" ,valeur de l'attribut
;;; "nom-att" figurant dans la specialisation locale  "sch-loc".
(de :ADMIS-CONST-SPEC? (val-a-tester sch-loc nom-att )
    (slet ((att (#:shirka:noyau:ATTRIB sch-loc nom-att))
           (val-fac-dom (#:shirka:noyau:VAL-FAC att '$domaine))
           (val-fac-int (#:shirka:noyau:VAL-FAC att '$intervalle))
           (val-fac-val (#:shirka:noyau:VAL-FAC att '$valeur))
           (type-att (if (or val-fac-dom val-fac-int)
                         (cadr(#:shirka:noyau:NATURE sch-loc nom-att)) )) )
          (cond
                (val-fac-dom 
                        (#:shirka:noyau:APP-DOM? 
                                val-a-tester type-att val-fac-dom))
                (val-fac-int 
                        (#:shirka:noyau:APP-INTER? 
                                val-a-tester type-att val-fac-int))
                (val-fac-val (EQUAL val-a-tester val-fac-val))
                (T T) ) ) )
 
;;; Suppression de "nom -instance" dans la liste d'instances
;;; des filtres du schema de nom "nom-schema" et  de tous ses
;;; ancetres .
(de :SUP-INST-FILT-ANC (nom-instance nom-schema)
    (:SUP-INST-FILT nom-instance nom-schema)
    (mapc '(lambda (nom-sch)
                   (:SUP-INST-FILT nom-instance nom-sch)) 
          (#:shirka:noyau:ANCETRES 
                  (#:shirka:noyau:SCHEMA nom-schema)) ) )

;;; idem que la precedente mais sans les ancetres.
(de :SUP-INST-FILT (nom-instance nom-schema )
    (mapc '(lambda (nom-filtre)
                   (#:shirka:moteur:SUP-VALEUR 
                           (#:shirka:noyau:SCHEMA nom-filtre) 
                           'inst nom-instance) )
          (:CHERCHER-FILTRES nom-schema) ) )
