(defvar #:sys-package:colon '#:shirka:moteur)

;;; teste l'admissibilite de "valeur" pour l'attribut "nom-att" de "schema"
;;; "valeur" peut etre une liste ou une valeur simple
(de :admissible? (valeur nom-att schema instance )
    (if (atom valeur)
        ;valeur simple
        (if (:test-admissible valeur nom-att schema instance)
            valeur)
        ;liste de valeurs
        (if (every  
                   '(lambda (val)
                            (:test-admissible val nom-att schema instance))
                   valeur)
            valeur)))

;;; teste l'admissibilite de "valeur" simple pour 
;;; l'attribut "nom-att" de "schema"
(de :test-admissible (valeur nom-att schema instance)
    (setq %%EXPL-NEG-TYPE nil)
    (setq %%EXPL-NEG-PRED nil)
    (tag verif
         (let (((nature type) (#:shirka:noyau:nature schema nom-att))
               (nom-inst (#:shirka:noyau:nom-sch instance))
               (nom-sch  (#:shirka:noyau:nom-sch schema))
               (attrib (#:shirka:noyau:attrib-he schema nom-att)))
              ; compatibilite du type
              (if (selectq type
                          ; types elementaires predefinis
                          (objet-simple (or (symbolp valeur)
                                            (numberp valeur)
                                            (stringp valeur)))               
                          (symbole (symbolp valeur))
                          (nombre  (numberp valeur))
                          (entier  (fixp valeur))
                          (reel    (floatp valeur))
                          (chaine  (stringp valeur))
                          (booleen (member valeur '(vrai faux oui non)))
                          (objet t)
                          ; type complexe
                          (t 
                             (if (or (symbolp valeur) 
                                     (vectorp valeur))                         
                                 (let ((instance 
                                               (if (vectorp valeur)
                                                   valeur
                                                   (let ((sch 
                                                              (tag erreur 
                                                                   (#:shirka:noyau:schema 
                                                                           valeur))))
                                                        (if sch sch
                                                            ; le schema donne en valeur n'est pas encore defini
                                                            ; adjonction a la liste de reprises, incrementation
                                                            ; du nombre d'attributs non verifies de l'instance
                                                            ; et impression d'un message d'avertissement
                                                            (let ((occurrences (assq valeur %liste-reprise)))
                                                                 (ifn occurrences
                                                                      (setq %liste-reprise 
                                                                            (acons valeur (list (list nom-sch nom-inst nom-att))
                                                                                   %liste-reprise))
                                                                      ; le schema apparait deja dans la liste de reprise
                                                                      (rplacd occurrences (cons 
                                                                                                (list nom-sch nom-inst nom-att)
                                                                                                (cdr occurrences)))))
                                                            (putprop nom-inst (add1 (getprop nom-inst 'nb-indet))
                                                                    'nb-indet)
                                                            (prin "Attention: l'admissibilite de la valeur " valeur)
                                                            (print " ne peut etre verifiee pour l'attribut ")
                                                            (prin nom-att " de l'instance " nom-inst)
                                                            (print " de la classe " nom-sch) nil)))))
                                      (if instance
                                          (:spec-inst? instance type)
                                          (exit verif t))))))                           
                  ; verification des saufs et des domaines
                  (unless (#:shirka:noyau:app-sauf? valeur type 
                                  (vref attrib %sauf))
                          (when (#:shirka:noyau:app-dom? valeur type 
                                        (vref attrib %dom))
                                ; verification des intervalles 
                                ; pour les types simples
                                (if (#:shirka:noyau:type-simple? type)
                                    (if (#:shirka:noyau:app-inter? 
                                                valeur type 
                                                (vref attrib %int))
                                        ; verification des $a-verifier
                                        (let ((pred (:a-verifier? 
                                                            valeur nom-att schema instance)))
                                             (if pred pred
                                                 (setq %%EXPL-NEG-PRED %%NOM-PRED) nil))) 
                                    (let ((pred (:a-verifier? 
                                                        valeur nom-att schema instance)))
                                         (if pred pred
                                             (setq %%EXPL-NEG-PRED %%NOM-PRED) nil)))))
                  ; si le type n'est pas admissible
                  (setq %%EXPL-NEG-TYPE type)
                  nil
                  ))))

;;; teste les facettes $a-verifier pour "valeur"
;;; de l'attribut "nom-att" de "instance"
;;; classe est la classe ou doit etre rattachee l'instance
(de :a-verifier? (valeur nom-att schema instance)
    (let ((nom-sch (#:shirka:noyau:nom-sch schema)))
         ; parcours des ancetres pour la facette $a-verifier
         (prog1 
                (:a-verifier-anc valeur nom-att schema 
                        (#:shirka:noyau:nom-sch instance))
                ; demarquage des schemas de classe ancetres
                (#:shirka:noyau:demarquer schema))))

(de :a-verifier-anc (valeur nom-att schema nom-inst)
    (let ((l-a-verif (#:shirka:noyau:val-facette 
                             schema nom-att '$a-verifier))
          (nom-sch (#:shirka:noyau:nom-sch schema)))
         (prog1 
                (tag echec
                     (when l-a-verif  ; la facette $a-verifier est presente
                           (:lier nom-sch nom-inst)
                           (let ((pos 
                                      (#:shirka:noyau:pos-nom-att schema nom-att)))
                                (if pos
                                    (vset (getprop nom-sch 'w-inst)
                                          pos
                                          valeur))))
                     (ifn (:a-verif l-a-verif) ;renvoie t si l-a-verif est nil
                          (exit echec)
                          (mapc 
                                '(lambda (nom-sch)
                                         (ifn (:a-verifier-anc valeur nom-att 
                                                      (#:shirka:noyau:schema 
                                                              nom-sch) nom-inst)
                                              (exit echec)))
                                (#:shirka:noyau:peres schema))
                          t))  ; car aucun predicat n'a echoue en amont
                (when l-a-verif ;la liaison n'a ete faite que si la facette 
                      (:delier nom-sch)) ; $a-verifier etait presente
                )))

;;; verifie les predicats attaches a l'attribut lui-meme
;;; cette fonction est appelee en fin d'instanciation, interactive
;;; ou non et apres une modification et ajout de valeur
(de :verif-lui-meme (schema instance)
    (tag echec
         (let ((nom-sch (#:shirka:noyau:nom-sch schema))
               (nom-inst (#:shirka:noyau:nom-sch instance)))
              (#:shirka:moteur:lier nom-sch nom-inst)
              (prog1 
                     (ifn (tag erreur 
                               (#:shirka:moteur:a-verif 
                                       (#:shirka:noyau:val-facette 
                                               schema 'lui-meme '$a-verifier)))
                          (exit echec)
                          (mapc 
                                '(lambda (nom-sch)
                                         (ifn (:verif-lui-meme (#:shirka:noyau:schema nom-sch) 
                                                      instance)
                                              (exit echec)))
                                (#:shirka:noyau:peres schema))
                          t)  ; car aucun predicat n'a echoue en amont
                     (#:shirka:moteur:delier nom-sch)))))

;;; declenchement des fonctions $avant-xx ou $apres-xx
(de :reflexe  (schema nom-att nom-inst nom-reflexe)
    (let ((meth (#:shirka:noyau:val-fac-he 
                        schema nom-att nom-reflexe)))
         (when meth
               (#:shirka:moteur:lier  nom-sch nom-inst)
               (#:shirka:moteur:attach-proc meth) 
               (#:shirka:moteur:delier nom-sch))))

;;; ajoute "valeur" dans l'attribut "nom-att" de "instance"
;;; en fin de liste si "bfin", en tete sinon.
;;; renvoie nil si l'operation n'a pu etre effectuee (non admissibilite, ...)
;;; sinon renvoie "valeur" 
(de :aj-valeur (instance nom-att valeur bfin)
    (if (and valeur instance)
        (let ((schema (#:shirka:noyau:schema 
                              (#:shirka:noyau:val-inst instance %est-un))))
             (let ((nom-inst (#:shirka:noyau:nom-sch instance))
                   (nom-sch  (#:shirka:noyau:nom-sch schema)))
                  (if (#:shirka:noyau:att-inst? schema nom-att)  
                      ; l'attribut doit etre instanciable
                      (slet ((pos-att (#:shirka:noyau:pos-nom-att schema nom-att))
                             ((nature type)  (#:shirka:noyau:nature schema nom-att))
                             (val-exist (#:shirka:noyau:val-inst instance pos-att)))
                            (tag erreur  ; erreur: non admissibilite,
                                 ; saturation de liste, valeur existante
                                 (if (eq nature '$liste-de)       ; attribut multi-value
                                     ; ajout apres verification de $cardmax, de l'admissibilite 
                                     ; de "valeur" et eventuellement de la liste entiere
                                     (let ((cardmax (#:shirka:noyau:val-fac-he 
                                                            schema nom-att '$cardmax)))
                                          (if (memq valeur val-exist)
                                              ; la valeur apparait deja dans la liste
                                              (exit erreur))
                                          (if (and cardmax
                                                   (eqn (length val-exist) cardmax))
                                              (exit erreur))            ; liste saturee
                                          (ifn (:admissible? valeur nom-att schema instance)
                                               (exit erreur)            ; valeur non admissible
                                               ; declenchement des actions $avant-ajout
                                               (:reflexe schema nom-att nom-inst 
                                                       '$avant-ajout)
                                               (if bfin
                                                   ; ajout en fin de liste
                                                   (vset instance pos-att 
                                                         (nconc1 val-exist valeur))
                                                   ; ajout en tete de liste
                                                   (vset instance pos-att 
                                                         (cons valeur val-exist)))))    
                                     ; attribut mono-value: ajout apres test d'admissibilite
                                     (if val-exist
                                         (exit erreur)       ; une valeur est deja presente
                                         (ifn (:admissible? valeur nom-att schema instance)
                                              (exit erreur)  ; valeur non admissible
                                              ; declenchement des actions $avant-ajout
                                              (:reflexe schema nom-att nom-inst 
                                                      '$avant-ajout)
                                              (vset instance pos-att valeur))))
                                 ; le pre-filtrage de l'instance est a refaire
                                 (#:shirka:moteur:filtrage:sup-inst-filt-anc 
                                         nom-inst nom-sch)
                                 (#:shirka:moteur:filtrage:filt-const-anc nom-inst nom-sch)
                                 ;declenchement des actions $apres-ajout
                                 (:reflexe schema nom-att nom-inst '$apres-ajout)
                                 ; mise a jour du lien "util-par" de "valeur" si celle-ci
                                 ; est une instance ou une reference a une instance
                                 ; (ifn (or (type-simple? type)
                                 ; (and (eq type 'objet);cas d'une valeur de type Objet
                                 ; (not (symbolp valeur))) ;mais en fait de type simple
                                 ; (not (schema valeur)))
                                 ; (slet ((sch-valeur (#:shirka:noyau:schema valeur))
                                 ; (l-util-par (vref sch-valeur %util-par)))
                                 ; (vset sch-valeur %util-par 
                                 ;      (nconc1 l-util-par
                                 ;          (#:shirka:noyau:nom-sch instance)))))               
                                 valeur)))))))

;;; cree le lien est-un de "instance" vers le schema "nom-sch"
;;; ajoute le lien inverse "inst" du schema "nom-sch" vers "instance"
(de :creer-est-un (instance nom-sch)
    (vset instance %est-un nom-sch)
    ; mise a jour de l'attribut "inst" du schema de classe
    (:aj-valeur (#:shirka:noyau:schema nom-sch) 'inst 
            (#:shirka:noyau:val-inst instance %nom-sch) t))

;;; ajoute le lien "util-par" de "instance" vers "nom-objet"
(dmd :aj-util-par (instance nom-objet)
     `(:aj-valeur ,instance 'util-par ,nom-objet t))

;;; cree le lien "sorte-de" de "schema" vers ses "peres"
;;; ajoute les liens inverses "spec", de chacun des peres vers "schema"
(de :creer-sorte-de (schema peres)
    (let ((nom-sch (#:shirka:noyau:nom-sch schema)))
         (mapc 
               '(lambda (pere)
                        (:aj-valeur schema 'sorte-de pere t)
                        (:aj-valeur (#:shirka:noyau:schema pere) 
                                'spec nom-sch t))
               peres)))

;;; supprime "valeur" dans l'attribut "nom-att" de "instance
;;; renvoie nil si l'operation n'a pu etre effectuee
;;; renvoie "valeur" dans le cas contraire
(de :sup-valeur (instance nom-att valeur)
    (let ((schema (#:shirka:noyau:schema 
                          (#:shirka:noyau:val-inst instance %est-un))))
         (slet ((nom-sch (#:shirka:noyau:nom-sch schema))
                (nom-inst (#:shirka:noyau:nom-sch instance)))
               (tag erreur  
                    ; erreurs: attribut inconnu, 
                    ; pas de valeur ou liste trop courte
                    (let ((pos-att (#:shirka:noyau:pos-nom-att schema nom-att)))
                         (ifn pos-att ;attribut inconnu ou non instanciable
                              (exit erreur))  
                         (let (((nature type) 
                                (#:shirka:noyau:nature schema nom-att))
                               (val-exist 
                                       (#:shirka:noyau:val-inst instance pos-att)))
                              (ifn val-exist 
                                   (exit erreur))  ;aucune valeur n'est presente
                              (if (eq nature '$liste-de)  
                                  ; attribut multi-value
                                  ; suppression de "valeur" apres verification 
                                  ; de son existence dans "val-inst" et de $cardmin
                                  (let ((cardmin (#:shirka:noyau:val-fac-he 
                                                         schema nom-att '$cardmin)))
                                       ; la valeur a supprimer doit etre 
                                       ; obligatoirement precisee, admissible 
                                       ; et presente das la liste
                                       (unless (and valeur
                                                    (member valeur val-exist) 
                                                    (:admissible?
                                                            valeur nom-att schema instance))
                                               (exit erreur))
                                       (if (and cardmin
                                                (eqn (length val-exist) cardmin))
                                           (exit erreur)
                                           ; declenchement des actions $avant-sup
                                           (:reflexe schema nom-att nom-inst 
                                                   '$avant-sup)
                                           ; suppression
                                           (vset instance pos-att 
                                                 (delete valeur val-exist))))
                                  ; attribut mono-value
                                  (ifn (or (not valeur)
                                           ; la valeur a supprimer n'est pas  precisee
                                           (:eq-val val-exist valeur type))
                                       (exit erreur)
                                       (:reflexe schema nom-att nom-inst '$avant-sup)
                                       ; suppression
                                       (vset instance pos-att nil)))
                              ; le pre-filtrage de l'instance est a refaire
                              (#:shirka:moteur:filtrage:sup-inst-filt-anc 
                                      nom-inst nom-sch)
                              (#:shirka:moteur:filtrage:filt-const-anc 
                                      nom-inst nom-sch)
                              ; declenchement des actions $apres-sup
                              (:reflexe schema nom-att nom-inst '$apres-sup)
                              ; mise a jour du lien util-par de "valeur" si celle-ci 
                              ; est une instance ou une reference a une instance
                              (if valeur valeur val-exist)))))))

(dmd :eq-val (val1 val2 type)
     `(if (eq ,type 'reel)
          (<= (abs (- ,val1 ,val2)) 1.0e-8)
          (eq ,val1 ,val2)))

;;; remplacement de "valeur1" par "valeur2" dans l'attribut "nom-att"
;;; de "instance". Renvoie nil si l'operation n'a pu etre effectuee.
;;; renvoie "valeur2" dans le cas contraire.
(de :mod-valeur (instance nom-att valeur2 valeur1)
    (let ((schema (#:shirka:noyau:schema 
                          (#:shirka:noyau:val-inst instance %est-un))))
         (slet ((nom-inst (#:shirka:noyau:nom-sch instance))
                (nom-sch  (#:shirka:noyau:nom-sch schema))
                (pos-att (#:shirka:noyau:pos-nom-att schema nom-att)))
               (tag erreur
                    (ifn pos-att (exit erreur))
                    (let (((nature type) 
                           (#:shirka:noyau:nature schema  nom-att))
                          (val-exist 
                                  (#:shirka:noyau:val-inst instance pos-att)))
                         (ifn val-exist (exit erreur));aucune valeur n'est presente
                         (if (eq nature '$liste-de)  ;attribut multi-value
                             ; remplacement apres verification de l'existence de "valeur1"
                             ; dans "val-inst" et de l'admissibilite de "valeur2"
                             (ifn (and (member valeur1 val-exist)
                                       (not (member valeur2 val-exist))
                                       (:admissible? valeur2 nom-att schema instance))
                                  (exit erreur)
                                  ; declenchement des actions $avant-mod
                                  (:reflexe schema nom-att nom-inst '$avant-mod)
                                  ; remplacement proprement dit 
                                  (nsubst valeur2 valeur1 val-exist))
                             ; attribut mono-value
                             (ifn (and (or (not valeur1)
                                           (:eq-val val-exist valeur1 type))
                                       (:admissible? valeur2 nom-att schema instance))
                                  (exit erreur)
                                  (vset instance pos-att valeur2)
                                  ; declenchement des actions $avant-mod
                                  (:reflexe schema nom-att nom-inst '$avant-mod))) 
                         ; le pre-filtrage de l'instance est a refaire
                         (#:shirka:moteur:filtrage:sup-inst-filt-anc
                                 nom-inst nom-sch)
                         (#:shirka:moteur:filtrage:filt-const-anc nom-inst nom-sch)
                         ; declenchement des actions $apres-mod
                         (:reflexe schema nom-att nom-inst '$apres-mod)
                         ; mise a jour du lien util-par de "valeur" si celle-ci
                         ; est une instance ou une reference a une instance
                         valeur2)))))

;;; suppression d'une instance
;;; si bsup-spec est vrai et dans le cas d'une classe,
;;; suppression des sous-classes et des instances
(de :sup-inst (instance bsup-spec)
    (let ((schema (#:shirka:noyau:schema 
                          (#:shirka:noyau:val-inst instance %est-un)))
          (nom-inst (#:shirka:noyau:nom-sch instance)))
         ; suppression du nom de l'instance dans le schema de classe
         (:sup-valeur schema 'inst nom-inst)
         ; suppression dans les filtres (pre-filtrage constant)
         (#:shirka:moteur:filtrage:sup-inst-filt-anc 
                 nom-inst (#:shirka:noyau:nom-sch schema))
         ; si schema de classe : 
         (when (and (#:shirka:noyau:sch-classe? instance)
                    bsup-spec)
               ;  -suppression des instances
               (mapc '(lambda (nom-inst)
                              (:sup-inst 
                                      (#:shirka:noyau:schema nom-inst) nil ))
                     (#:shirka:noyau:l-inst-t instance))
               ;  -suppression du nom dans les super-classes
               (mapc '(lambda (nom-super-cl)
                              (let ((super-cl  
                                            (#:shirka:noyau:schema nom-super-cl)))
                                   (when super-cl
                                         (:sup-valeur super-cl 'spec nom-inst))))
                     (#:shirka:noyau:peres instance))
               ;  -suppression des specialisations
               (mapc '(lambda (nom-spec)
                              (:sup-inst 
                                      (#:shirka:noyau:schema nom-spec) t))
                     (#:shirka:noyau:l-spec instance)))                   
         ; propagation de la suppression grace au lien "util-par"
         ; suppression des diverses proprietes attachees
         (remprop nom-inst 'filtres)
         (remprop nom-inst 'ordre-att)
         (remprop nom-inst 'nb-indet)
         (remprop nom-inst 'w-inst)
         (remprop nom-inst 'lie)
         (remprop nom-inst 'l-att)
         (remprop nom-inst 'dem)
         (remprop nom-inst 'ass)
         (remprop nom-inst 'f-imb)
         (remprop nom-inst 'def)
         (remprop nom-inst 'schema) 
         (remprop nom-inst 'traite)
         nom-inst))

;;; creation d'une instance par fonction
(de :creer-instance (nom-classe liste-valeurs nom-inst)
    (tag erreur-admiss
         (slet ((classe (#:shirka:noyau:schema nom-classe))
                (l-att  (#:shirka:noyau:l-nom-att-inst-he classe))
                (instance (#:shirka:noyau:make-inst nom-classe nom-inst)))
               (mapc 
                     '(lambda ((nom-att . valeur))
                              (let ((valeur (eval valeur)))
                                   (and 
                                        (member nom-att l-att)
                                        (:admissible? valeur nom-att 
                                                classe instance)
                                        (vset instance
                                              (#:shirka:noyau:pos-nom-att classe nom-att)
                                              valeur))))
                     liste-valeurs)
               ; verification des predicats globaux (lui-meme)
               (unless (#:shirka:moteur:verif-lui-meme classe instance)
                       (prin "Attention: schema " nom-inst " incorrect; ")
                       (print "predicats globaux non verifies")
                       (exit erreur-admiss))
               ; pre-filtrage constant de l'instance
               (#:shirka:moteur:filtrage:filt-const-anc nom-inst nom-classe)
               ; reprise des admissibilites non resolues si l'instance est
               ; elle-meme complete
               (if (= (getprop (#:shirka:noyau:nom-sch instance) 'nb-indet) 0)
                   (#:shirka:comp:traiter-liste-reprise  nom-inst))
               instance)))

;;; verifie si "instance" peut appartenir a la classe de nom "nom-sch-dest" 
(de :spec-inst? (instance nom-sch-dest)
    (let ((resultat (#:shirka:classif:spec-inst instance nom-sch-dest 
                            %pour-types %sauf-types %binf %bdial)))
         (memq resultat
               '(sur possible))))
  
;;; macros destinees a faciliter l'ecriture 
;;; des attachements proceduraux
(dmd val?   (nom-att)
     `(#:shirka:moteur:valeur? ,nom-att 
              (#:shirka:noyau:nom-sch inst)))

(dmd affect (nom-att valeur)
     `(#:shirka:moteur:aj-valeur inst ,nom-att ,valeur t))
