;;; ******************************
;;; ATTENTION : modification effectuee dans la fonction :app-inter? pour ne
;;; pas utiliser les fonctions arithmetiques sur des symboles.
;;; C.Capponi.
;;; *******************************

;;; primitives d'acces aux schemas
(defvar #:sys-package:colon '#:shirka:noyau)

;;; vrai si "nom-att" est un meta-attribut
(dmd :meta? (nom-att)
     `(memq ,nom-att '(sorte-de est-un util-par spec inst)))

;;; recherche dans une liste de vecteurs "l-vec" 
;;; du vecteur dont l'element de rang  "rang" est egal a "nom"
(de :rec (l-vec nom rang)
    (tag trouve
         (mapc '(lambda (vec)
                        (ifn (vectorp vec)
                             (setq vec (:schema vec)))
                        (if (equal nom (vref vec rang))
                            (exit trouve vec)))
               l-vec)))

;;; recherche ordonnee dans une liste de vecteurs "l-fac"
;;; du vecteur dont le premier element est egal a "nom-fac"
(de :rec-fac (l-fac nom-fac)
    (tag trouve
         (mapc '(lambda (vec)
                        (slet ((vec (if (vectorp vec) vec (:schema vec)))
                               (prem-el (vref vec 0)))
                              (ifn (alphalessp prem-el nom-fac)
                                   (exit trouve)
                                   (when (eq prem-el nom-fac) 
                                         (exit trouve vec)))))
               l-fac)))

;;; schema donne par son nom externe "nom-sch"
;;; si "nom-sch" est en fait le schema lui-meme, renvoie "nom-sch"
(dmd :schema (nom-sch)
     `(if (vectorp ,nom-sch)
          ,nom-sch
          (let ((schem (getprop ,nom-sch 'schema)))
               (if schem
                   schem
                   (print "Schema " ,nom-sch " indefini")
                   (exit erreur)))))
                   
;;; obtention du schema de classe d'une instance connue par son nom
(de :classe-de (nom-inst)
    (:schema (:val-inst (:schema nom-inst) 
                     %est-un)))

;;; nom d'un schema
(dmd :nom-sch (schema)
     `(vref ,schema %nom-sch))

;;; liste des attributs propres d'un schema        
(dmd :l-att (schema)
     `(vref ,schema %list-att))

;;; liste des noms des attributs propres d'un schema
(de :l-nom-att (schema)
    (mapcar '(lambda (att)
                     (ifn (vectorp att)
                          (setq att (:schema att)))
                     (vref att %nom-att))
            (:l-att schema)))

;;; liste des noms des attributs propres d'un schema
;;; dont la classe apparait dans classes-att 
;;; ou n'apparait pas dans sauf-classes-att
(de :l-nom-att-class (schema classes-att sauf-classes-att)
    (let ((l-nom-att nil))
         (mapc '(lambda (att)
                        (ifn (vectorp att)
                             (setq att (:schema att)))
                        (slet ((nom-att (vref att %nom-att))
                               (classe-att (:classe-att-he schema nom-att)))
                              (when (:considerer-classe-att 
                                            classe-att classes-att sauf-classes-att)
                                    (setq l-nom-att (nconc1 l-nom-att nom-att)))))
               
               (:l-att schema))
         l-nom-att))

;;; determine si la classe d'attribut type-att est a considerer, connaissant
;;; la liste des classes a considerer <pour-types> et
;;; la liste des classes a ne pas considerer <sauf-types>
(de :considerer-classe-att (type-att pour-types sauf-types)
    (or (and (consp pour-types) (:classe-att? type-att pour-types))
        (and (consp sauf-types) (not (:classe-att? type-att sauf-types)))
        (and (not pour-types) (not sauf-types))))

;;; determine si type-att est l'une des classes d'attribut de liste-classes-att
;;; ou une specialisation de l'une de ces classes
(de :classe-att? (type-att liste-classes-att)
    (tag apparait
         (mapc 
               '(lambda (nom-classe-att)
                        (if (:sorte-de? type-att nom-classe-att)
                            (exit apparait type-att)))
               liste-classes-att)))
    
;;; attribut propre donne par son nom
(dmd :attrib (schema nom-att)
     `(:rec (vref ,schema %list-att) ,nom-att %nom-att))

;;; nom d'un attribut
(dmd :nom-att (attribut)
     `(vref ,attribut %nom-att))

;;; liste des facettes propres d'un attribut
(dmd :l-fac (attribut)
     `(vref ,attribut %list-fac))

;;; facette propre nommee d'un attribut connu
(dmd :fac (attribut nom-fac)
     `(:rec-fac (vref ,attribut %list-fac) ,nom-fac))

;;; facette propre nommee d'un attribut nomme
(de :facette (schema nom-att nom-fac)
    (let ((attrib (:attrib schema nom-att)))
         (if attrib
             (:rec-fac (vref attrib %list-fac) nom-fac))))

;;; valeur d'une facette propre nommee d'un attribut connu
(de :val-fac (attribut nom-fac)
    (if attribut
        (let ((facette (:rec-fac (vref attribut %list-fac) nom-fac )))
             (if facette
                 (vref facette %list-att)))))

;;; valeur d'une facette propre nommee d'un attribut nomme de schema
(de :val-facette (schema nom-att nom-fac)
    (let ((attribut (:attrib schema nom-att)))
         (if  attribut
             (:val-fac attribut nom-fac))))

;;; liste des peres d'un schema
(dmd :peres (schema)
     `(vref ,schema %sorte-de))

;;; liste des ancetres d'un schema
(dmd :ancetres (schema)
     `(prog1 (:ramener-anc ,schema)
             (:demarquer ,schema)))

(de :ramener-anc (schema)
    (let ((peres (:peres schema)))
         (if peres
             (append peres
                     (mapcan '(lambda (pere)
                                      (unless (getprop pere 'traite)
                                              (prog1 
                                                     (:ramener-anc (:schema pere))
                                                     (putprop pere t 'traite))))
                             peres)))))

;;; demarquage apres heritage
(de :demarquer (schema)
    (mapc '(lambda (pere)
                   (when (getprop pere 'traite)
                         (:demarquer (:schema pere))
                         (putprop pere nil 'traite)))
          (:peres schema)))

;;; attribut herite donne par son nom
(de :attrib-he (schema nom-att)
    (tag trouve
         (protect 
                 (:obtenir-att schema nom-att)
                 (:demarquer schema))))

(de :obtenir-att (schema nom-att)
    (let ((attrib (:attrib schema nom-att)))
         (if attrib
             (exit trouve attrib)
             (mapc '(lambda (pere)
                            (unless (getprop pere 'traite)
                                    (putprop pere t 'traite )
                                    (let ((pere (:schema pere)))
                                         (:obtenir-att pere nom-att))))
                   (:peres schema)))))

;;; valeur de facette nommee d'un attribut nomme de schema
;;; arret des la premiere rencontre d'une facette de meme nom
(de :val-fac-he (schema nom-att nom-fac)
    (tag trouve
         (protect   ; pour effectuer le demarquage dans tous les cas
                 (:ramener-fac schema nom-att nom-fac)
                 (:demarquer schema))))

(de :ramener-fac (schema nom-att nom-fac)
    (let ((valeur (:val-facette schema nom-att nom-fac)))
         (if valeur
             (exit trouve valeur)
             ; essai d'heritage
             (mapc '(lambda (pere)
                            (unless (getprop pere 'traite) 
                                    (putprop pere t 'traite)
                                    (let ((pere (:schema pere)))
                                         (:ramener-fac pere nom-att nom-fac))))
                   (:peres schema))))))

(de :val-fac-he-sch (schema nom-att nom-fac)
    (tag trouve-fac
         (protect       ; pour effectuer le demarquage dans tous les cas
                 (:ramener-fac-sch schema nom-att nom-fac)
                 (:demarquer schema))))

(de :ramener-fac-sch (schema nom-att nom-fac)
    (let ((valeur (:val-facette schema nom-att nom-fac)))
         (if valeur
             (exit trouve-fac (cons valeur (:nom-sch schema))))
         ;essai d'heritage
         (mapc '(lambda (pere)
                        (unless (getprop pere 'traite) 
                                (putprop pere t 'traite)
                                (let ((pere (:schema pere)))
                                     (:ramener-fac-sch pere nom-att nom-fac))))
               (:peres schema))))))

;;; obtention par heritage de la nature d'un attribut de schema
;;; c'est-a-dire $un ou $liste-de et son type: (nature type)
(de :obtenir-nature (schema nom-att)
    (tag trouve
         (protect    ; pour effectuer le demarquage dans tous les cas
                 (:ramener-nat schema nom-att)
                 (:demarquer schema))))

(de :ramener-nat (schema nom-att)
    (slet ((att (:attrib schema nom-att))
           (nat (if att
                    (or (:fac att '$un)
                        (:fac att '$liste-de)))))
          (if nat
              (exit trouve (list (vref nat %est-un)
                                 (vref nat %list-val)))
              ; essai d'heritage
              (mapc '(lambda (pere)
                             (unless (getprop pere 'traite)
                                     (putprop pere t 'traite)
                                     (let ((pere (:schema pere)))
                                          (:ramener-nat pere nom-att))))
                    (:peres schema)))))

;;; obtention de la nature d'un attribut par consultation
;;; des attributs "nature" et "type" de ce dernier
(de :nature (schema nom-att)
    (if (eq nom-att 'lui-meme)
        (list '$un (vref schema %nom-sch))
        (let ((attrib (:attrib-he schema nom-att)))
             (if attrib
                 (list (vref attrib %nat)
                       (vref attrib %type))))))

;;; union des listes de  "l1" et "l2" dans cet ordre
(de :union (l1 l2)
    (ifn l1 
         l2
         (append l1
                 (mapcan '(lambda (x)
                                  (ifn (member x l1) (list x)))
                         l2))))

;;; liste des noms de tous les attributs (propres et herites) de "schema" 
(dmd :l-nom-att-he (schema)
     `(prog1 (delete 'lui-meme
                     (append '(est-un util-par nom-inst)
                             (:ramener-att ,schema)))
             (:demarquer ,schema)))

(de :ramener-att (schema)
    (unless (eq (:nom-sch schema) 'objet)
            (let ((l-att nil))
                 (mapc '(lambda (pere)
                                (unless (getprop pere 'traite)
                                        (putprop pere t 'traite)
                                        (setq l-att
                                              (:union (:ramener-att (:schema pere))
                                                      l-att))))                             
                       (:peres schema))
                 (:union l-att (:l-nom-att schema)))))

;;; liste des noms de tous les attributs (propres et herites) de "schema"
;;; en tenant compte des classes d'attributs 
(de :l-nom-att-he-class (schema classes-att sauf-classes-att)
    (if (and (not classes-att) (not sauf-classes-att))
        (prog1 (delete 'lui-meme
                       (append '(est-un util-par nom-inst)
                               (:ramener-att schema)))
               (:demarquer schema))
        ;prise en compte des classes d'attributs specifiees
        (prog1 (delete 'lui-meme
                       (:ramener-att-class schema 
                               classes-att sauf-classes-att))
               (:demarquer schema))))

(de :ramener-att-class (schema classes-att sauf-classes-att)
    (let ((l-att nil))
         (mapc '(lambda (pere)
                        (unless (getprop pere 'traite)
                                (putprop pere t 'traite)
                                (setq l-att
                                      (:union (:ramener-att-class 
                                                      (:schema pere)
                                                      classes-att sauf-classes-att)
                                              l-att))))                             
               (:peres schema))
         (:union l-att (:l-nom-att-class schema classes-att sauf-classes-att))))

;;; vrai si le schema de nom "nom-sch-inf" est 
;;; une sorte de schema de nom "nom-sch-sup"
(de :sorte-de? (nom-sch-inf nom-sch-sup)
    (prog1 
           (tag trouve
                (letn :parcours ((nom-sch-inf nom-sch-inf))
                      (if (eq nom-sch-inf nom-sch-sup)
                          (exit trouve nom-sch-sup)
                          (mapc '(lambda (pere)
                                         (unless (getprop pere 'traite)
                                                 (putprop pere t 'traite)
                                                 (:parcours pere)))
                                (:peres (:schema nom-sch-inf))))))
           (:demarquer (:schema nom-sch-inf))))

;;; liste des noms d'attributs instanciables de "schema", propres et herites,
;;; avec mention de leur numero d'ordre en valeur de propriete "position"
;;; et de leur nature en propriete "nature"
(de :l-nom-att-inst-he (schema)
    (prog1 (let ((l-att (delete 'lui-meme
                                (append 
                                        '(est-un util-par nom-inst)
                                        (:ramener-att-inst schema))))
                 ; a chaque nom d'attribut est associee son numero d'ordre et son type
                 (index -1))
                (mapc '(lambda (nom-att) 
                               (putprop nom-att (incr index) 'position))
                      l-att)
                l-att)
           (:demarquer schema)))

(de :ramener-att-inst (schema)
    (let ((nom-sch (:nom-sch schema)))
         (unless (eq nom-sch 'objet)
                 (let ((l-att (getprop nom-sch 'l-att)))
                      (unless l-att
                              ;attributs herites
                              (mapc '(lambda (pere)
                                             (unless (getprop pere 'traite)
                                                     (putprop pere t 'traite)
                                                     (setq l-att
                                                           (:union 
                                                                   (:ramener-att-inst 
                                                                           (:schema pere))
                                                                   l-att))))
                                    (:peres schema))
                              ; attributs propres
                              ; seuls les attributs depourvus de la facette 
                              ; $valeur sont retenus
                              (mapc '(lambda (att)
                                             (ifn (vectorp att)
                                                  (setq att (:schema att)))
                                             (slet ((nom-att (:nom-att att))
                                                    (bapparait (memq nom-att l-att)))
                                                   (ifn (eq nom-att 'lui-meme)
                                                        (if  (:val-fac att '$valeur)
                                                            (when bapparait
                                                                  (setq l-att (delete nom-att l-att)))
                                                            (unless bapparait
                                                                    (setq l-att (nconc1 l-att nom-att)))))))
                                    (:l-att schema)))  
                      (putprop nom-sch l-att 'l-att)
                      ))))
                   
;;; liste des noms d'attributs instanciables de "schema", propres et herites,
;;; avec prise en compte de leur classe d'instanciation
(de :l-nom-att-inst-he-class (schema pour-classes sauf-classes) 
    (let ((l-att (delete 'lui-meme
                         (:ramener-att-inst-class schema
                                 pour-classes sauf-classes))))
         (:demarquer schema)
         l-att))

(de :ramener-att-inst-class (schema pour-classes sauf-classes)
    (let ((nom-sch (:nom-sch schema))
          (l-att))
         ; attributs herites
         (mapc '(lambda (pere)
                        (unless (getprop pere 'traite)
                                (putprop pere t 'traite)
                                (setq l-att
                                      (:union 
                                              (:ramener-att-inst-class 
                                                      (:schema pere)
                                                      pour-classes sauf-classes)
                                              l-att))))
               (:peres schema))
         ; attributs propres
         ; seuls les attributs depourvus de la facette 
         ; $valeur sont retenus
         (mapc '(lambda (att)
                        (ifn (vectorp att)
                             (setq att (:schema att)))
                        (slet ((nom-att (:nom-att att))
                               (bapparait (memq nom-att l-att)))
                              (ifn (eq nom-att 'lui-meme)
                                   (if  (:val-fac att '$valeur)
                                       (when bapparait
                                             (setq l-att (delete nom-att l-att)))
                                       (when (and (not bapparait)
                                                  (:considerer-classe-att
                                                          (:classe-att-he 
                                                                  schema nom-att) 
                                                          pour-classes 
                                                          sauf-classes))
                                             (setq l-att 
                                                   (nconc1 l-att nom-att)))))))
               (:l-att schema))  
         l-att))

;;; position d'un attribut nomme apres appel a "l-nom-att-inst-he"
(dmd :position (nom-att)
     `(getprop ,nom-att 'position))

;;; position d'un attribut nomme de "schema"
(de :pos-nom-att (schema nom-att)
    (let ((l-att (:l-nom-att-inst-he schema)))
         (if (eq nom-att 'lui-meme)
             ; dans une instance de filtre, la derniere case est
             ; reservee a l'attribut lui-meme
             (length l-att)
             (if (member nom-att l-att)
                 (:position nom-att)))))

;;; nature d'un attribut nomme  apres appel a "l-nom-att-inst-he"
;(de nature-att (nom-att)
;(getprop nom-att 'nature)) 

;;; valeur d'attribut d'"instance" donne par sa position "pos-att"
(dmd :val-inst (instance pos-att)
     `(vref ,instance ,pos-att))

;;; vrai si l'attribut de nom "nom-att" de "schema" est instanciable
(dmd :att-inst? (schema nom-att)
     `(memq ,nom-att (:l-nom-att-inst-he ,schema)))

;;; classe de l'attribut nomme nom-att dans schema
(de #:shirka:noyau:classe-att (schema nom-att)
    (let ((attrib (:attrib schema nom-att)))
         (when attrib  
               (vref attrib %est-un))))

;;; determine par heritage la classe d'instanciation d'un attribut nomme
(de #:shirka:noyau:classe-att-he (schema nom-att)
    (tag trouve
         (protect 
                 (:chercher-classe-att schema nom-att)
                 (:demarquer-bis schema))))))

(de :chercher-classe-att (schema nom-att)
    (let ((nom-classe-att (#:shirka:noyau:classe-att schema nom-att)))
         (if (and nom-classe-att (nequal nom-classe-att 'attribut)) 
             (exit trouve nom-classe-att)
             (mapc '(lambda (pere)
                            (unless (getprop pere 'examine) 
                                    (putprop pere t 'examine)
                                    (:chercher-classe-att 
                                            (#:shirka:noyau:schema pere)
                                            nom-att)))
                   (:peres schema))
             (exit trouve 'attribut))))

(de :demarquer-bis (schema)
    (mapc '(lambda (pere)
                   (when (getprop pere 'examine)
                         (:demarquer-bis (:schema pere))
                         (putprop pere nil 'examine)))
          (:peres schema)))

;;; vrai si "schema" est un schema de classe
;;; c'est-a-dire est une instance de schema 
;;; ou d'une specialisation de schema
(de :sch-classe? (schema)
    (let ((est-un (:val-inst schema %est-un)))
         (or (eq est-un 'Schema)
             (:sorte-de? est-un 'Schema))))

;;; liste des schemas situes entre "nom-sch-inf" et "nom-sch-sup"
;;; si ceux ci sont resp. les elements min. et max. d'un sous-treillis
;;; nil dans le cas contraire
(de :l-sch-entre (nom-sch-inf nom-sch-sup)
    (let ((sch-inf (:schema nom-sch-inf)))
         (prog1 
                (tag rejet
                     (letn :parcours ((sch-inf sch-inf))
                           (nconc1 
                                   (mapcan '(lambda (pere)
                                                    (unless (getprop pere 'traite)
                                                            (putprop pere t 'traite)
                                                            (if (eq pere 'objet)
                                                                (exit rejet)
                                                                (if (eq pere nom-sch-sup)
                                                                    (list nom-sch-sup)
                                                                    (:parcours 
                                                                            (:schema 
                                                                                    pere))))))
                                           (:peres sch-inf))
                                   (:nom-sch sch-inf))))
                (:demarquer sch-inf))))

;;;***********
;;; attention: cette fonction modifiee par C.Capponi
;;; pour les tests sur entiers et reels
;;;***********
;;; vrai si "valeur" de type simple appartient 
;;; a l'un des intervalles de la liste "l-inter"
(de :app-inter? (valeur type l-inter)
    (if l-inter
        (tag app
             (mapc 
                   '(lambda ((min max))
                            (if (selectq type
                                        (entier  (and (or (eq min '-inf)
                                                          (eq valeur '+inf)
                                                          (ge valeur min))
                                                      (or (eq max '+inf)
                                                          (eq valeur '-inf)
                                                          (and (not (eq valeur '+inf))
                                                               (le valeur max)))))
                                        (reel    (and (or (eq min '-inf)
                                                          (eq valeur '+inf)
                                                          (>= valeur min))
                                                      (or (eq max '+inf)
                                                          (eq valeur '-inf)
                                                          (<= valeur max))))
                                        (chaine  (and (alphalessp min valeur)
                                                      (alphalessp valeur max)))
                                        (booleen (or  (eq valeur min)  ;pour etre complet...
                                                     (eq valeur max))))
                                (exit app t))) 
                   l-inter))
        t))

;;; vrai si "valeur" appartient a "domaine", le "type" est quelconque
(de :app-dom? (valeur type domaine)
    (if domaine
        (if (:type-simple? type)
            (member valeur domaine)
            (tag trouve
                 (mapc '(lambda (x)
                                (if (:egal-inst? valeur x)
                                    (exit trouve valeur)))
                       domaine)))
        t))

;;; vrai si "valeur" appartient a "sauf", le "type" est quelconque
(de :app-sauf? (valeur type sauf)
    (if sauf
        (if (:type-simple? type)
            (member valeur sauf)
            (tag trouve
                 (mapc '(lambda (x)
                                (if (:egal-inst? valeur x)
                                    (exit trouve valeur)))
                       sauf)))
        ))

;;; vrai si "type" est un type simple
(dmd :type-simple? (type)
     `(memq ,type '(entier reel chaine booleen 
                           symbole nombre objet-simple)))

;;; vrai si "inst1" et "inst2" sont deux instances identiques
(de :egal-inst? (inst1 inst2)
    (if (and (symbolp inst1)
             (symbolp inst2))
        (or (eq inst1 inst2)
            ;cas particulier des schemas de classe:
            ;inst1 et inst2 sont consideres egaux
            ;si il existe entre eux une relation de specialisation
            (and (:sch-classe? (:schema inst1))
                 (:sch-classe? (:schema inst2))
                 (:sorte-de? inst1 inst2)))
        (if (symbolp inst1)
            (eqvector (:schema inst1) inst2)
            (if (symbolp inst2)
                (eqvector (:schema inst2) inst1)
                (eqvector inst1 inst2)))))

;;; transforme la donnee d'un intervalle comme instance
;;; du schema "intervalle" sous la forme de la liste (min max)
(dmd :tr-inter (inter)
     `(if ,inter
          (list (:val-inst ,inter 3)
                (:val-inst ,inter 4))))

;;; liste des intervalles heritee par l'attribut "nom-att" de "schema"
(dmd :intervalle (schema nom-att type)
     `(prog1 (:ramener-int ,schema ,nom-att ,type)
             (:demarquer ,schema)))

(de :ramener-int (schema nom-att type)
    (slet ((attrib (:attrib schema nom-att))
           (l-inter (let ((val-fac-inter (:val-fac attrib '$intervalle)))
                         (if val-fac-inter
                             (mapcar ':tr-inter 
                                     val-fac-inter)))))
          (or l-inter
              (:comb-inter l-inter         ; combiner les intervalles courants avec la
                      (mapcan              ; liste des intervalles herites
                              '(lambda (pere)
                                       (unless (getprop pere 'traite)
                                               (putprop pere t 'traite)
                                               (let ((sch-pere (:schema pere))) 
                                                    (:ramener-int sch-pere nom-att type))))
                              (:peres schema))
                      type))))

;;; intersection de la liste d'intervalles "inter1" avec la liste "inter2"
(de :comb-inter (inter1 inter2 type)
    (ifn inter1 
         inter2
         (ifn inter2 
              inter1
              (mapcan 
                      '(lambda ((min1 max1))
                               (mapcan 
                                       '(lambda (inter)
                                                (let ((min (car inter))
                                                      (max (cadr inter)))
                                                     (if (:app-inter? min1 type inter)
                                                         (setq min min1))
                                                     (if (:app-inter? max1 type inter)
                                                         (setq max max1))
                                                     (list (list min max))))
                                       inter2))
                      inter1))))

;;; domaine herite par l'attribut "nom-att" de "schema"
(dmd :domaine (schema nom-att type)
     `(prog1 (:ramener-dom ,schema ,nom-att ,type)
             (:demarquer ,schema)))

(de :ramener-dom (schema nom-att type)
    (slet ((attrib (:attrib schema nom-att)) ;l'attribut est-il present?
           (dom (if attrib (:val-fac attrib '$domaine))))
          (or dom 
              (mapcan 
                      '(lambda (pere)
                               (unless (getprop pere 'traite)
                                       (putprop pere t 'traite)
                                       (let ((sch-pere (:schema pere)))
                                            (:comb-dom dom
                                                    (:ramener-dom sch-pere nom-att type)
                                                    type))))
                      (:peres schema)))))

;;; intersection de "domaine1" et "domaine2"
(de :comb-dom (domaine1 domaine2 type)
    (ifn domaine1
         domaine2
         (ifn domaine2
              domaine1
              (mapcan 
                      '(lambda (x)
                               (if (:app-dom? x type domaine2)
                                   (list x)))
                      domaine1))))

;;; union de toutes les facettes sauf heritees pour 
;;;l'attribut "nom-att" de "schema"
(dmd :sauf (schema nom-att type)
     `(prog1 (:ramener-sauf ,schema ,nom-att ,type)
             (:demarquer ,schema)))

(de :ramener-sauf (schema nom-att type)
    (slet ((attrib (:attrib schema nom-att));l'attribut est-il present?
           (sauf (if attrib (:val-fac attrib '$sauf))))
          (append sauf 
                  (mapcan 
                          '(lambda (pere)
                                   (unless (getprop pere 'traite)
                                           (putprop pere t 'traite)
                                           (let ((sch-pere (:schema pere)))
                                                (:ramener-sauf 
                                                        sch-pere nom-att type))))
                          (:peres schema)))))

;;; vrai si "nom-att" denomme un attribut apparaissant pour la
;;; premiere fois dans "schema"
(de :nouv-att? (schema nom-att)
    (let ((attrib (:attrib schema nom-att)))
         (ifn (and attrib  ;cas de la presence de l'attribut dans
                   ;schema sans indication de type: l'attribut
                   ;est defini plus haut dans le treillis
                   (not (:fac attrib '$un)) 
                   (not (:fac attrib '$liste-de))) 
              (ifn (:exist-type schema nom-att)
                   (and attrib
                        (or (:fac attrib '$un)
                            (:fac attrib '$liste-de)))))))

(de :exist-type (schema nom-att)
    (tag exist-type             
         (mapc                   
               '(lambda (pere)     
                        (if (:nature (:schema pere) nom-att)
                            (exit exist-type t)))
               (:peres schema))))

;;; liste des specialisations directes de "schema"
(dmd :l-spec (schema)
     `(vref ,schema %spec))

;;; liste de toutes les specialisations de "schema"
(de :l-spec-t (schema)
    (prog1 
           (:l-spec-t1 schema)
           (:demarquer-fils schema)))

(de :l-spec-t1 (schema)
    (mapcan '(lambda (spec)
                     (unless (getprop spec 'traite)
                             (putprop spec t 'traite)
                             (cons spec (:l-spec-t1 (:schema spec)))))
            (:l-spec schema)))

;;; liste des specialisations vraies de "schema":
;;; ni les filtres ou attachements proceduraux, 
;;; ni les specialisations locales
(de :l-spec-t-vraies (schema)
    (prog1 
           (:l-spec-t-vraies1 schema)
           (:demarquer-fils schema)))

(de :l-spec-t-vraies1 (schema)
    (mapcan '(lambda (spec)
                     (unless (or (getprop spec 'est-un-filtre)
                                 (getprop spec 'spec-loc)
                                 (getprop spec 'traite))
                             (putprop spec t 'traite)
                             (cons spec 
                                   (:l-spec-t-vraies1 
                                           (:schema spec)))))
            (:l-spec schema)))

;;; liste des instances propres de "schema"
(dmd :l-inst (schema)
     `(vref ,schema %inst))

;;; liste de toutes les instances de "schema"
(de :l-inst-t (schema)
    (prog1 
           (:l-inst-t1 schema)
           (:demarquer-fils schema)))

(de :l-inst-t1 (schema)
    (append (:l-inst schema)
            (mapcan '(lambda (spec)
                             (unless (or (getprop spec 'est-un-filtre)
                                         (getprop spec 'spec-loc)
                                         (getprop spec 'traite))
                                     (putprop spec t 'traite)
                                     (:l-inst-t1  (:schema spec))))
                    (:l-spec schema))))

;;; demarquage apres parcours des fils de "schema"
(de :demarquer-fils (schema)
    (mapc '(lambda (fils)
                   (when (getprop fils 'traite)
                         (:demarquer-fils (:schema fils))
                         (putprop fils nil 'traite)))
          (:l-spec schema)))

;;; creation et initialisation d'une instance "nom-inst"
;;; de la classe "nom-schema"
(de :make-inst (nom-schema nom-inst)
    (slet ((nb-att (length (:l-nom-att-inst-he (:schema nom-schema))))
           (nom-inst (if nom-inst nom-inst 
                         (#:shirka:comp:gen-symbole nom-schema)))
           (inst (makevector nb-att nil)))
          (vset inst %nom-inst nom-inst)
          (#:shirka:moteur:creer-est-un inst nom-schema)
          (putprop nom-inst inst 'schema)
          (putprop nom-inst 0 'nb-indet)
          inst))
