(defvar #:sys-package:colon '#:shirka:vi)

;;; visualisation d'une instance en utilisant les facettes
;;; $pour-vi, $avant-vi, $en-vi et $apres-vi. 
;;; la visualisation s'arrete au niveau "niveau"
;;; si "binf" les valeurs des attributs sont inferees, 
;;; sinon, les valeurs sont seulement prises dans "instance"
(de :visualiser (instance pour-classes sauf-classes niveau binf)
    (if pour-classes
        (setq pour-classes (nconc1 pour-classes 'Attribut-Shirka-est-un))
        (setq sauf-classes (nconc1 sauf-classes 'Attribut-Shirka-sauf-est-un)))
    (slet ((schema (#:shirka:noyau:schema 
                           (#:shirka:noyau:val-inst instance %est-un)))
           (nom-inst (#:shirka:noyau:nom-sch instance)))
          ; methodes de visualisation attachees globalement
          ; au schema au niveau de l'attribut lui-meme
          (unless 
                  (:appel-meth-vi nom-inst schema 'lui-meme '$avant-vi)
                  (prin "{ " nom-inst)
                  (lmargin (+ (lmargin) 5)))
          (unless (:appel-meth-vi nom-inst schema 'lui-meme '$pour-vi)
                  ; visualisation de chaque attribut
                  (mapc 
                        '(lambda (nom-att)
                                 (:vi-att instance pour-classes sauf-classes
                                         schema nom-att niveau binf))
                        (if binf
                            (#:shirka:noyau:l-nom-att-he-class schema
                                    pour-classes sauf-classes)
                            (#:shirka:noyau:l-nom-att-inst-he-class schema 
                                    pour-classes sauf-classes))))
          (unless 
                  (:appel-meth-vi nom-inst schema 'lui-meme '$apres-vi)
                  (lmargin (- (lmargin) 5))(terpri)
                  (prin "} "))
          (#:shirka:noyau:nom-sch instance)))

;;; recherche par heritage et appelle la methode de visualisation
;;; de la facette "nom-fac" pour l'attribut "nom-att"
;;; de "instance" de "schema"
(de :appel-meth-vi (nom-inst schema nom-att nom-fac)
    (let (((meth . nom-sch) 
           (#:shirka:noyau:val-fac-he-sch schema nom-att nom-fac)))
         (when meth
               ; execution de la methode
               (#:shirka:moteur:lier nom-sch nom-inst)
               (#:shirka:moteur:attach-proc meth)
               (#:shirka:moteur:delier nom-sch))))

;;; visualisation d'un attribut nomme "nom-att"
(de :vi-att (instance pour-classes sauf-classes schema nom-att niveau binf)
    (ifn (eq nom-att 'lui-meme)
         (let (((nature type) (#:shirka:noyau:nature schema nom-att))
               (nom-inst (#:shirka:noyau:nom-sch instance))
               (valeurs))
              ; facette $avant-vi
              (unless (:appel-meth-vi nom-inst schema nom-att '$avant-vi)
                      (setq valeurs 
                            (if binf
                                ; la valeur est inferee
                                (#:shirka:moteur:valeur? 
                                        nom-att instance)
                                ; la valeur est prise dans l'instance
                                (#:shirka:noyau:val-inst instance 
                                        (#:shirka:noyau:pos-nom-att 
                                                schema nom-att))))
                      (when valeurs
                            (terpri)
                            (prin nom-att " = ")))
              ; impression des valeurs
              (unless (:appel-meth-vi nom-inst schema nom-att '$pour-vi)
                      (if valeurs
                          (:vi-valeurs nom-inst pour-classes sauf-classes schema 
                                  nom-att valeurs niveau binf)))                         
              ; facette $apres-vi
              (unless (:appel-meth-vi nom-inst schema nom-att '$apres-vi)
                      (when valeurs
                            (prin '|;|))))))

;;; visualisation d'une valeur ou d'une liste de valeurs
(de :vi-valeurs (nom-inst pour-classes sauf-classes
                        schema nom-att valeurs niveau binf)
    (let (((nature type) (#:shirka:noyau:nature schema nom-att)))
         (if (or (eq nature '$un) (atom valeurs))
             (:vi-val schema pour-classes sauf-classes 
                     nom-att valeurs type niveau binf)
             ; traitement d'une liste
             (:vi-val schema pour-classes sauf-classes 
                     nom-att (car valeurs) type niveau binf)
             (mapc 
                   '(lambda (valeur)
                            (unless 
                                    (:appel-meth-vi instance schema nom-att '$en-vi)
                                    (prin " "))
                            (:vi-val schema pour-classes sauf-classes
                                    nom-att valeur type niveau binf))
                   (cdr valeurs)))))

;;; visualisation d'une valeur scalaire
(de :vi-val (schema pour-classes sauf-classes nom-att valeur type niveau binf)
    (if (#:shirka:noyau:type-simple? type)
        (if (eq type 'chaine)
            (prin """" valeur """")
            (prin valeur " "))
        (if (and (eq type 'objet)
                 (or (not (symbolp valeur))
                     (and (symbolp valeur)
                          (not (getprop valeur 'schema)))))
            (if (stringp valeur)
                (prin """" valeur """")
                (prin valeur " "))
            ; instance ou reference a une instance
            (let ((niveau (sub1 niveau)))
                 (if (or (vectorp valeur)
                         (and (atom valeur)
                              (getprop valeur 'non-part)
                              (setq valeur 
                                    (#:shirka:noyau:schema valeur))))
                     ; instance proprement dite
                     ; c'est une instance non partageable
                     ; elle est visualisee quel que soit le niveau
                     (:visualiser valeur pour-classes sauf-classes niveau binf)
                     ; reference a une instance
                     (if (and (atom valeur)
                              (getprop valeur 'schema))
                         (if (minusp niveau)
                             (prin valeur " ")
                             (:visualiser 
                                     (#:shirka:noyau:schema valeur)
                                     pour-classes sauf-classes 
                                     niveau binf))
                         ; objet de nature indeterminee
                         (prin " " valeur " ")))))))
