(defvar #:sys-package:colon '#:shirka:comp)

;;; genere un nouveau symbole a partir du parametre "nom"
;;; le nouveau symbole aura la forme %"nom"-index ou index
;;; correspond a l'index associe a la fonction GENSYM de le-lisp
 
(de :gen-symbole (nom)
    (setq #:SYSTEM:GENSYM-STRING (concat "%" nom "-"))
    (prog1 (gensym)
           (setq #:SYSTEM:GENSYM-STRING "g")))

;;; definition de la variable globale %l-fac-filtre qui contient
;;; le nom de toutes les facettes propres au filtrage ou aux
;;; attachements proceduraux

(defvar %l-fac-filtre 
        (#:shirka:noyau:l-spec-t (#:shirka:noyau:schema 'f-fil-att)))
                      
;;; initialisation a nil de la liste de schema deja traites 
;;; pour la compilation. Cette liste %l-sch-tr vaut nil tant
;;; au debut qu'a la fin de la compilation d'un schema

(defvar %l-sch-tr nil)
(defvar %nom-schema-en-cours nil)

;;; traduction en representation interne d'un schema recu 
;;; dans une representation externe sous la forme d'une 
;;; liste d'associations a 3 niveaux. Ensuite, compilation
;;; de cette representation interne.

(de def-sh (schema)
    (tag erreur
         (let ((sch-ri (#:shirka:comp:ext-int schema)))
              (when (and (#:shirka:noyau:sch-classe? sch-ri)
                         (not %deja-compile))
                    ; --> pre-traitement de facettes variables : $var<-, $var-liste<-, $var->
                    (:tr-var sch-ri)
                    ; --> compilation : construction du reseau pour chaque filtre
                    ;                   associe a la facette $sib-filtre 
                    (:compilation sch-ri))
              (setq %deja-compile nil)
              sch-ri)))

;;; ramene la liste des attributs qui apparaissent pour la
;;; premiere fois dans un schema

(de :ram-nouv-att (schema)
    (let ((l-nom-att (#:shirka:noyau:l-nom-att schema)))
         (mapcan '(lambda (nom-att)
                          (if (#:shirka:noyau:nouv-att? schema nom-att)
                              nom-att))
                 l-nom-att)))

;;; compilation d'un schema par rapport aux filtres
;;; a tous niveaux d'imbrication. La compilation est faite
;;; attribut par attribut. Seules les facettes contenues
;;; dans la variable globale %l-fac-filtre sont traitees.
;;; La compilation consiste a chercher les filtres et a construire  
;;; le reseau propre a chaque filtre, puis a compiler le filtre lui meme

(de :compilation (schema)
    (mapc '(lambda (att)
                   (mapc '(lambda (fac-f)
                                  (let ((val-fac-f 
                                                (#:shirka:noyau:val-fac att fac-f)))
                                       ; --> si la facette $sib-filtre est presente
                                       ;    les filtres sont traites particulierement
                                       (if (and (eq fac-f '$sib-filtre)
                                                val-fac-f)
                                           (:compilation-f val-fac-f)
                                           ; --> appel recursif pour la compilation
                                           ;    sur les autres facettes
                                           (mapc '(lambda (nom-filt)
                                                          (:compilation 
                                                                  (#:shirka:noyau:schema nom-filt)))
                                                 val-fac-f))))
                         %l-fac-filtre))
          (#:shirka:noyau:l-att schema)))

;;; compilation particuliere a un ensemble de filtres donnes 
;;; par leurs noms 
;;; construction et attachement du reseau pour chaque filtre
;;; ajout du nom du filtre dans la propriete 'filtres 
;;; du schema specialise 

(de :compilation-f (nom-filtres)
    (mapc '(lambda (nom-filt)
                   (let ((schema (#:shirka:noyau:schema nom-filt)))
                        (putprop nom-filt
                                (:const-reseau schema)
                                'reseau)
                        ; --> obtention du nom du schema specialise par le filtre
                        ;     et ajout du nom du filtre dans la propriete 'filtres  
                        (let ((sorte-de 
                                      (car 
                                           (#:shirka:noyau:val-inst 
                                                   schema %sorte-de))))
                             ; prefiltrage constant des instances existantes
                             ; de la classe de nom sorte-de
                             (mapc 
                                   '(lambda (nom-inst)
                                            (#:shirka:moteur:filtrage:filt-const-inst
                                                    nom-inst nom-filt))
                                   (#:shirka:noyau:l-inst-t 
                                           (#:shirka:noyau:schema sorte-de)))
                             (putprop sorte-de 
                                     (nconc1 (getprop sorte-de 'filtres)
                                             nom-filt)
                                     'filtres))
                        ; --> appel recursif sur le filtre lui meme
                        (:compilation schema)))
          nom-filtres))

;;; construction du reseau pour un filtre 
;;; le reseau est construit attribut par attribut a partir de 
;;; la liste d'attributs propres au filtre.
;;; le reseau est une liste d'associations dont les clefs sont
;;; les noms des attributs et la valeur associee un vecteur

(de :const-reseau (filtre)
    (#:shirka:noyau:l-nom-att-inst-he 
            (#:shirka:noyau:schema (car (vref filtre %sorte-de))))
    (mapcar '(lambda (att)
                     (cons (#:shirka:noyau:nom-att att)
                           (:const-vect att filtre)))
            (#:shirka:noyau:l-att filtre)))

;;; construction du vecteur pour un attribut
;;; tout vecteur comporte 7 positions, dans l'ordre :
;;;                     - information sur les facettes constantes
;;;                       ($valeur, $domaine et $intervalle)
;;;                     - references pour les facettes variables
;;;                       ($var<-, $var-liste<- et $var->)
;;;                     - pour la facette $sib-filtre, liste de noms de filtres
;;;                     - pour la facette $sib-exec, liste de noms
;;;                       des schemas methodes correspondants
;;;                     - idem pour la facette $sib-choix
;;;                     - pour la facette $defaut , la valeur associee
;;;                     - pour la facette $a-verifier, liste de noms
;;;                       des schemas predicats correspondants

(de :const-vect (attribut filtre)
    (slet ((const (:ext-part-const attribut))
           (nom-att (#:shirka:noyau:nom-att attribut))
           (var (:ext-part-var nom-att 
                        (#:shirka:noyau:nom-sch filtre)))
           (type (cadr (#:shirka:noyau:nature filtre nom-att)))
           (pos-att (nconc1 
                            (list (#:shirka:moteur:filtrage:pos-att-filtre? 
                                          filtre nom-att))
                            (if (eq nom-att 'lui-meme) 
                                -1 
                                (#:shirka:noyau:position nom-att)))))  
          (vector const var
                  (#:shirka:noyau:val-fac attribut '$sib-filtre)
                  (#:shirka:noyau:val-fac attribut '$sib-exec)
                  (#:shirka:noyau:val-fac attribut '$sib-choix)
                  (#:shirka:noyau:val-fac attribut '$defaut)
                  (#:shirka:noyau:val-fac attribut '$a-verifier)
                  type
                  pos-att)))

;;; extraction de l'information sur les facettes constantes 
;;; d'un attribut
;;; attribution d'un code pour identifier ces facettes :
;;;                     - 0 pour $valeur
;;;                     - 1 pour $intervalle
;;;                     - 2 pour $domaine
;;; le cas echeant on obtient au maximum une paire sous la forme 
;;; (code . valeur)

(de :ext-part-const (attribut)
    (let ((valeur (#:shirka:noyau:val-fac attribut '$valeur))
          (int (#:shirka:noyau:val-fac attribut '$intervalle))
          (dom (#:shirka:noyau:val-fac attribut '$domaine)))
         (cond (valeur (cons 0 valeur))
               (int (cons 1 (mapcar '#:shirka:noyau:tr-inter int)))
               (dom (cons 2 dom)))))

;;; extraction de l'information sur les references a des variables
;;; pour un attribut
;;; 2 types de reference sont permises : - assignation 
;;;                           (facettes $var<- et $var-liste<-)
;;;                                      - resultat (facette $var->)
;;; On peut les trouver simultanement.
;;; Les references et les codes correspondants ont ete construits
;;; lors du traitement des variables par la fonction "tr-var"

(de :ext-part-var (nom-att nom-filtre)
    ; --> la propriete 'ass pour un filtre contient l'ensemble
    ;    des references propres au filtre (donnees par les facettes
    ;    $var<- et $var-liste<-)
    ; --> la propriete 'dem pour un filtre contient la reference
    ;    donnee par la facette $var->
    (let ((ass (cassq nom-att (getprop nom-filtre 'ass)))
          (dem (cassq nom-att (getprop nom-filtre 'dem))))
         (if (and ass dem)
             (nconc1 (list ass) dem)
             (let ((res (or ass dem)))
                  (if res (list res))))))

(defvar %const 0)
(defvar %var 1)
(defvar %sib-filt 2)
(defvar %sib-exec 3)
(defvar %sib-choix 4)
(defvar %defaut 5)
(defvar %a-verif 6)
(defvar %type-att 7)
(defvar %pos-att 8)

;;; obtention de la valeur du reseau pour un filtre

(dmd :val-reseau (nom-filtre)
     `(getprop ,nom-filtre 'reseau))

;;; obtention du vecteur qui contient les informations
;;; pertinentes a un attribut d'un filtre

(dmd :val-att-reseau (nom-filtre nom-att)
     `(let ((val-reseau (:val-reseau ,nom-filtre)))
           (cassq ,nom-att val-reseau)))

;;; obtention de la valeur pour une facette sur un vecteur
;;; propre a un attribut d'un filtre

(dmd :val-fac-att-reseau (val-att-reseau nom-fac)
     `(vref ,val-att-reseau ,nom-fac))

;;; obtention de la valeur d'une facette pour un attribut d'un filtre

(dmd :val-fac-reseau (nom-filtre nom-att nom-fac)
     `(let ((val-att-reseau (:val-att-reseau ,nom-filtre ,nom-att)))
           (if val-att-reseau
               (:val-fac-att-reseau val-att-reseau ,nom-fac))))
