(defvar #:sys-package:colon '#:shirka:moteur)

(defvar #:exp:trace nil)

;;; demande de la valeur d'un attribut de nom "nom-att"
;;; pour une instance de nom "instance"
;;; Si la valeur n'est pas obtenue directement de l'instance
;;; un appel au moteur d'inference est produit

(de :valeur? (%%nom-att nom-inst)
    (tag sortie
         (let ((sch-inst (#:shirka:noyau:schema nom-inst)))
              (when (eq %%nom-att 'lui-meme)
                    (when #:exp:trace
                          (#:exp:cons-tr (list 'but %%nom-att nom-inst))
                          (#:exp:cons-tr 
                                  (list 'tr-vs nom-inst %%nom-att nom-inst))
                          (#:exp:cons-tr (list 'succes))
                          )
                    (exit sortie nom-inst))
              (if #:exp:trace
                  (#:exp:cons-tr (list 'but %%nom-att nom-inst)))
              (slet ((sch-pere 
                             (#:shirka:noyau:schema 
                                     (#:shirka:noyau:val-inst 
                                             sch-inst %est-un)))
                     (pos (#:shirka:noyau:pos-nom-att 
                                  sch-pere %%nom-att)))
                    ; l'attribut existe-t-il dans le schema de classe?
                    (ifn pos 
                         (if (not (memq %%nom-att
                                        (#:shirka:noyau:l-nom-att-he 
                                                sch-pere)))
                             (exit erreur ;attribut  inconnu
                                   (#:shirka:erreur:traiter-err
                                           (vector 'erreur-att-non-exist 
                                                   () () %%nom-att  
                                                   (#:shirka:noyau:val-inst 
                                                           sch-inst %est-un))
                                           nil))))
                    (let ((val (if pos 
                                   (or (let ((w-inst 
                                                     (getprop   
                                                             (#:shirka:noyau:nom-sch sch-pere)
                                                             'w-inst)))
                                            (if (and w-inst pos)
                                                (#:shirka:noyau:val-inst 
                                                        w-inst pos)))
                                       (#:shirka:noyau:val-inst 
                                               sch-inst pos))))
                          (nature (car (#:shirka:noyau:nature 
                                               sch-pere %%nom-att))))
                         (ifn val
                              (:moteur %%nom-att sch-pere 
                                      sch-inst nature)
                              (when #:exp:trace
                                    (#:exp:cons-tr 
                                            (list 'tr-vs
                                                  val %%nom-att nom-inst))
                                    (#:exp:cons-tr '(succes)))
                              val))))))

;;; liaison d'un schema avec une instance

(dmd :lier (nom-sch nom-inst)
     `(prog1 
             (addprop ,nom-sch ,nom-inst 'lie)         
             (addprop ,nom-sch (:make-w-inst ,nom-sch) 'w-inst)))

(dmd :make-w-inst (nom-sch)
     `(makevector 
              (length 
                      (#:shirka:noyau:l-nom-att-inst-he 
                              (#:shirka:noyau:schema ,nom-sch))) nil))

;;; delier un schema avec la derniere instance liee
;;; definie explicitement

(dmd :delier (nom-sch)
     `(prog1 
             (remprop ,nom-sch 'lie)
             (remprop ,nom-sch 'w-inst)))

;;; premiere couche du moteur d'inference
;;; obtention de la valeur pour un attribut de nom "nom-att"
;;; d'un schema "schema" sur une instance "instance" 
;;; On essaie d'abord de l'obtenir a partir des methodes 
;;; d'inference propres a l'attribut (fonction trouver-val). 
;;; Le cas echeant, la sortie du moteur prend lieu a travers un 
;;; tag sortie appele dans la fonction reusite; 
;;; seulement en cas d'echec, le retour au moteur est effectue

(de :moteur (nom-att schema instance nature)
    (let ((nom-sch (#:shirka:noyau:nom-sch schema))
          (nom-inst (#:shirka:noyau:nom-sch instance))
          (att (#:shirka:noyau:attrib schema nom-att)))
         (when att
               (:lier nom-sch nom-inst)
               ; --> essai pour trouver la valeur dans l'attribut
               (:trouver-val att schema instance nature)
               ; traitement de la facette $si-echec
               (:attach-proc 
                       (#:shirka:noyau:val-facette 
                               schema nom-att '$si-echec))
               ; --> retour en cas d'echec
               (:delier nom-sch)))
    ; --> essai d'heritage
    (let ((val (:essayer-peres nom-att schema instance nature)))
         (when val
               (if #:exp:trace 
                   (#:exp:cons-tr '(succes)))
               (exit sortie val)))
    )

;;; essai d'obtenir la valeur de l'attribut a partir
;;; des methodes d'inference propres au peres du schema
;;; Une premiere sortie est effectuee a ce niveau la en
;;; cas de reusite (lien avec l'explication)

(de :essayer-peres (nom-att schema instance nature)
    (tag sortie
         (mapc '(lambda (pere)
                        (when #:exp:trace
                              (#:exp:cons-tr (list 'tr-vh nom-att nom-inst pere))
                              (#:exp:cons-tr (list 'but nom-att nom-inst)))
                        (:moteur nom-att (#:shirka:noyau:schema pere)
                                instance nature)
                        (if #:exp:trace
                            (#:exp:cons-tr '(echec))))
               (#:shirka:noyau:peres schema))))

;;; essai sur toutes les facettes a travers lesquelles
;;; on peut obtenir une valeur pour un attribut
;;; "attribut" du schema "schema"

(de :trouver-val (attribut schema instance nature)
    ; --> facette $valeur
    (let ((val (#:shirka:noyau:val-fac attribut '$valeur)))
         (when val
               (if #:exp:trace
                   (#:exp:cons-tr 
                           (list 'tr-vv val 
                                 (#:shirka:noyau:nom-att attribut)
                                 (#:shirka:noyau:nom-sch instance)
                                 (#:shirka:noyau:nom-sch schema))))
               (:reusite val schema)))
    ; --> facettes $var<-, $var-liste<-,$sib-exec, $defaut, $sib-filtre
    ;
    ; ==> NOTE : A rajouter traitement de la facette $sib-choix
    ;
    ;     Pour les variables, la methode consiste a trouver la reference
    ;     pour l'attribut en question, ensuite a essayer de resoudre cette
    ;     reference. Dans les autres cas, des methodes propres a chacun
    ;     sont appliquees
    (let ((var (:obt-ref-var (#:shirka:noyau:nom-att attribut) 
                       schema))
          (sibfilt (#:shirka:noyau:val-fac attribut '$sib-filtre))
          (sibex (#:shirka:noyau:val-fac attribut '$sib-exec))
          (def (#:shirka:noyau:val-fac attribut '$defaut)))
         (when var
               (if #:exp:trace
                   (#:exp:cons-tr 
                           (list 'tr-vr (#:shirka:noyau:nom-att attribut)
                                 (cdr var))))
               (let ((val (:obt-val-ref? (cdr var))))
                    (if val
                        ; --> Si la reference est simple, l'admissibilite
                        ;    est testee pour toute la valeur
                        ;     sinon, l'admissibilite est testee pour chaque
                        ;     valeur en particulier
                        (if (:ref-simple? var)
                            (if (:admissible? val 
                                        (#:shirka:noyau:nom-att attribut)
                                        (#:shirka:noyau:schema 
                                                (#:shirka:noyau:val-inst instance %est-un))
                                        instance)
                                (:reusite val schema))
                            (let ((valeurs 
                                          (:test-adm-lval val 
                                                  (#:shirka:noyau:nom-att attribut)
                                                  (#:shirka:noyau:schema 
                                                          (#:shirka:noyau:val-inst 
                                                                  instance %est-un))
                                                  instance)))
                                 (if valeurs
                                     (:reusite valeurs schema)))))))
         (when sibfilt
               (if #:exp:trace
                   (#:exp:cons-tr 
                           (list 'tr-vf 
                                 (#:shirka:noyau:nom-att attribut)
                                 (#:shirka:noyau:nom-sch instance)
                                 (#:shirka:noyau:nom-sch schema))))
               (:filtrage:sib-filtre sibfilt schema nature))
         (if sibex
             (:sib-exec sibex schema 
                     (#:shirka:noyau:nom-att attribut) 
                     (#:shirka:noyau:nom-sch instance)))
         (when def
               (if #:exp:trace
                   (#:exp:cons-tr 
                           (list 'tr-vd def 
                                 (#:shirka:noyau:nom-att attribut)
                                 (#:shirka:noyau:nom-sch instance)
                                 (#:shirka:noyau:nom-sch schema))))
               (:reusite def schema))))

;;; sortie du moteur d'inference avec un resultat
;;; seul point de sortie pour l'instant

(de :reusite (val schema)
    (if #:exp:trace (#:exp:cons-tr '(succes)))
    ; rangement de la valeur obtenue dans l'instance de travail
    (let ((pos (#:shirka:noyau:pos-nom-att schema %%nom-att)))
         (if pos
             (vset (getprop (#:shirka:noyau:nom-sch schema) 'w-inst)
                   pos
                   val)))
    ; traitement de la facette $si-succes recherchee sans heritage
    (:attach-proc 
            (#:shirka:noyau:val-facette schema %%nom-att '$si-succes))
    (:delier (#:shirka:noyau:nom-sch schema))
    (exit sortie val))

;;; obtention d'une reference pour un attribut donne par son nom

(dmd :obt-ref-var (nom-att schema)
     `(cassq ,nom-att
             (getprop (#:shirka:noyau:nom-sch ,schema) 'ass)))

;;; determine si une reference est simple
;;; c'est-a-dire : $var<- a $un
;;;                $var-liste<- a $liste-de
(dmd :ref-simple? (ref)
     `(or (eq %var-fixe-simple (car ,ref))
          (eq %var-semifixe-simple (car ,ref))))

;;; solution d'une reference de la forme (nom-sch nom-att-ref)
;;; ou (nom-sch suite-de-nom-att-ref)
;;; La reference est resolue en parcourant toute la suite
;;; composee par nom-sch et les noms des attributs references

(de :obt-val-ref? (ref)
    (if (atom (cdr ref))
        (#:shirka:moteur:filtrage:obt-val-ref ref)
        (let ((sch-int (#:shirka:moteur:filtrage:obt-val-ref
                               (cons (car ref) (cadr ref)))))
             (if sch-int
                 (:obt-val-ref1? (cons sch-int (cddr ref)))))))

(de :obt-val-ref1? (ref)
    (if (atom (cdr ref))
        (:valeur? (cdr ref) (car ref))
        (let ((sch-int (:valeur? (cadr ref) (car ref))))
             (if sch-int
                 (:obt-val-ref1? (cons sch-int (cddr ref)))))))

;;; test d'admissibilite pour une liste d'elements
;;; Renvoie comme resultat l'ensemble d'elements satisfaisant
;;; les conditions d'admissibilite

(de :test-adm-lval (valeurs nom-att schema instance)
    (if (consp valeurs)
        (mapcan '(lambda (val)
                         (if (:admissible? val nom-att schema instance)
                             (list val)))
                valeurs)
        (:admissible? valeurs nom-att schema instance)))

;;; determine si un schema est une instance ou un schema de classe

(dmd :instance? (nom-sch)
     `(neq (#:shirka:noyau:val-inst 
                   (#:shirka:noyau:schema ,nom-sch) %est-un)
           'schema))

;;; traitement d'un filtre procedural du type $sib_exec
;;; Les trois cas de l'attachement procedural sont traites

(de :sib-exec (nom-filtres schema nom-att nom-inst)
    (mapc '(lambda (nom-filt)
                   (tag echec-att-proc
                        (let ((type (getprop nom-filt 'type))
                              (nom-fct (#:shirka:noyau:val-fac-he 
                                               (#:shirka:noyau:schema nom-filt) 
                                               'nom-fct '$valeur)))
                             (if #:exp:trace
                                 (#:exp:cons-tr (list 'tr-vp nom-fct 
                                                      nom-att nom-inst)))
                             (selectq type
                                     (0
                                        (:traiter-se-sans-filtr 
                                                (#:shirka:noyau:schema nom-filt)
                                                schema nom-fct))
                                     (1
                                        (:traiter-se-simple 
                                                (#:shirka:noyau:schema nom-filt)
                                                schema nom-fct))
                                     (2
                                        (:traiter-se-complexe 
                                                (#:shirka:noyau:schema nom-filt)
                                                schema nom-fct)))))
                   (if #:exp:trace
                       (#:exp:cons-tr '(echec))))
          nom-filtres))

;;; traitement d'un filtre procedural sans filtrage
;;; Le schema methode est instancie et l'appel a la fonction
;;; est fait effectivement si l'instanciation reussit

(de :traiter-se-sans-filtr (filtre schema nom-fct)
    (let ((inst-filtre (:instantier filtre)))
         (funcall nom-fct inst-filtre)
         (let ((val (:cherch-val-renv 
                            (#:shirka:noyau:nom-sch filtre) inst-filtre)))
              (if val
                  (:reusite val schema)))))

;;; Le traitement est le meme que dans le premier cas
;;; a la difference pres que le filtrage est essaye avant
;;; de faire effectivement l'appel a la fonction
 
(de :traiter-se-simple (filtre schema nom-fct)
    (let ((inst-filtre (:instantier filtre)))
         ;(when #:exp:trace
         ;(#:exp:cons-tr (list 'but 'filtrer (nom-sch filtre))))
         (let ((val (:lancer-filtrage filtre inst-filtre)))
              (when val 
                    ;(if #:exp:trace
                    ;(#:exp:cons-tr '(succes)))
                    (:reusite val schema)))
         ;(if #:exp:trace
         ;(#:exp:cons-tr '(echec)))
         ; --> ajout de l'instance dans la liste d'instances du filtre 
         (:aj-inst filtre (#:shirka:noyau:nom-sch inst-filtre))
         (when #:exp:trace
               (#:exp:cons-tr (list 'but 'appel-fct nom-fct))
               (#:exp:cons-tr (list 'tr-appel nom-fct))
               )
         (funcall nom-fct inst-filtre)
         (let (
               ; --> recherche de la valeur a renvoyer comme resultat
               ;    de l'appel a la fonction 
               (val (:cherch-val-renv 
                            (#:shirka:noyau:nom-sch filtre) 
                            inst-filtre)))
              (when #:exp:trace
                    (#:exp:cons-tr (list 'but 'retour-fct nom-fct))
                    (#:exp:cons-tr (list 'tr-retour nom-fct val))
                    (#:exp:cons-tr '(succes))
                    )
              (ifn val
                   (if #:exp:trace
                       (#:exp:cons-tr '(echec)))
                   (if #:exp:trace
                       (#:exp:cons-tr '(succes)))
                   (:reusite val schema)))))

;;; traitement d'un filtre procedural complexe
;;; On construit autant d'instances qu'il le faut (fonction
;;; exploser-inst) a partir de references complexes existantes
;;; dans le filtre ($var<- a $liste-de)
;;; Ensuite le meme processus suivi pour le sib-exec-simple
;;; est applique a chaque instance obtenue

(de :traiter-se-complexe (filtre schema nom-fct)
    (lets ((l-inst (:exploser-inst (:instantier filtre)
                           (#:shirka:noyau:l-nom-att-inst-he filtre)))
           (l-val (:obt-val l-inst filtre nom-fct)))
          (if l-val
              (:reusite l-val schema))))

(de :obt-val (l-inst filtre nom-fct)
    (mapcan '(lambda (inst)
                     (let ((val (:lancer-filtrage filtre inst)))
                          (if val
                              (list val)
                              (:aj-inst filtre 
                                      (#:shirka:noyau:nom-sch inst))
                              (funcall nom-fct inst)
                              (let ((val-v (:cherch-val-renv 
                                                   (#:shirka:noyau:nom-sch filtre)
                                                   inst)))
                                   (if val-v
                                       (list val-v))))))
            l-inst))

;;; essai de filtrage pour une instance d'un filtre procedural
;;; Toute l'information pertinente pour le filtrage est mise a jour
;;; ou initialisee avant de lancer le filtrage proprement dit
;;; Si le filtrage reussit, la fonction rend la valeur obtenue
;;; comme resultat
 
(de :lancer-filtrage (filtre inst-filtre)
    (tag echec-filtrage
         (let ((nom-inst (#:shirka:noyau:nom-sch inst-filtre))
               (nom-filt (#:shirka:noyau:nom-sch filtre))
               (marqueur (:obt-marqueur-se filtre)))
              (putprop nom-inst 
                      (list (getprop nom-filt 'ordre-inst-att)  nil)
                      'ordre-att)
              (putprop nom-inst nil 'f-imb)
              (putprop nom-inst marqueur 'marq-inst)
              (let ((nom-inst-tr 
                            (#:shirka:comp:gen-symbole 
                                    (concat "inst-" nom-filt))))
                   (putprop nom-filt nom-inst-tr 'lie))
              (#:shirka:moteur:filtrage:filtrage nom-inst nom-filt))))

;;; cree une instance de schema a partir de sa liste
;;; d'attributs instantiables en utilisant le moteur pour
;;; calculer la valeur de chaque attribut
;;; Cette fonction d'instanciation est propre au $sib-exec

(de :instantier (schema)
    (when #:exp:trace
          (#:exp:cons-tr (list 'but 'instancier (nom-sch schema)))
          (#:exp:cons-tr (list 'tr-inst (nom-sch schema)))
          )
    (lets ((nom-sch (#:shirka:noyau:nom-sch schema))
           (l-att-inst (getprop nom-sch 'ordre-inst-att))
           (nom-inst (#:shirka:moteur:filtrage:creer-inst-vide
                             nom-sch
                             (add1 (length l-att-inst))))
           (instance (#:shirka:noyau:schema nom-inst)))
          (mapc '(lambda (nom-att)
                         ;
                         ; ==> NOTE : Une optimisation possible consiste a passer par
                         ;            le moteur directement au lieu d'appeller la fonction
                         ;            valeur?. L'interet est d'economiser un appel a la
                         ;            fonction pos-nom-att
                         ;
                         (let ((val-att (:valeur? nom-att nom-inst))
                               (pos-att (#:shirka:noyau:pos-nom-att schema nom-att)))
                              (if val-att
                                  (vset instance pos-att val-att)
                                  (if #:exp:trace
                                      (#:exp:cons-tr '(echec)))
                                  (exit echec-att-proc))))
                l-att-inst)
          (if #:exp:trace
              (#:exp:cons-tr '(succes)))
          (getprop nom-inst 'schema)))

;;; Cree une instance de schema a partir de sa liste
;;; d'attributs instantiables en utilisant le moteur pour
;;; calculer la valeur de chaque attribut.
;;; Cette fonction sert a instancier tous les schema methodes
;;; qui correspondent a des facettes filtre sauf $sib-exec.
;;; La differentiation est faite a cause de l'existence
;;; d'un attribut "resultat" dans tous les schemas methodes
;;; propres a la facette $sib-exec

(de :instancier (schema)
    (slet ((l-att-inst (getprop 
                               (#:shirka:noyau:nom-sch schema) 
                               'ordre-inst-att))
           (nom-inst (#:shirka:moteur:filtrage:creer-inst-vide 
                             (#:shirka:noyau:nom-sch schema)
                             (length l-att-inst)))
           (instance (#:shirka:noyau:schema nom-inst)))
          (mapc '(lambda (nom-att)
                         (let ((val-att (:valeur? nom-att nom-inst))
                               (pos-att 
                                       (#:shirka:noyau:pos-nom-att 
                                               schema nom-att)))
                              (if val-att
                                  (vset instance pos-att val-att)
                                  (exit echec-att-proc))))
                l-att-inst)
          (getprop nom-inst 'schema)))

;;; ajoute le nom d'une instance dans la liste d'instances
;;; propres a un schema donne

(dmd :aj-inst (schema nom-inst)
     `(let ((l-inst (#:shirka:noyau:l-inst ,schema)))
           (vset ,schema %inst (nconc1 l-inst ,nom-inst))))

;;; renvoie la valeur resultat dans l'instance "instance"
;;; qui est en fait une instance d'un filtre procedural

(de :cherch-val-renv (nom-filtre inst-filtre)
    (let ((nom-att-dem (:obt-att-dem nom-filtre)))
         (when nom-att-dem
               (#:shirka:noyau:val-inst inst-filtre 
                       (#:shirka:noyau:pos-nom-att 
                               (#:shirka:noyau:schema nom-filtre) 
                               nom-att-dem)))))

;;; obtention du nom de l'attribut qui contient
;;; le resultat a envoyer pour un filtre ou un
;;; attachement procedural

(dmd :obt-att-dem (nom-filtre)
     `(caar (getprop ,nom-filtre 'dem)))

(de :obt-marqueur-se (filtre)
    (let ((l-inst (#:shirka:noyau:l-inst filtre)))
         (ifn l-inst
              (exit echec-filtrage))
         l-inst))

(de :exploser-inst (instance l-att-inst)
    (let ((index 3)
          (l-ref-c (getprop 
                           (#:shirka:noyau:val-inst instance %est-un) 
                           'l-ref-c))
          (l-val (list (makevector (vlength instance) nil))))
         (mapc '(lambda (nom-att)
                        (let ((valeurs 
                                      (#:shirka:noyau:val-inst instance index)))
                             (ifn (memq nom-att l-ref-c)
                                  (mapc '(lambda (inst)
                                                 (vset inst index valeurs))
                                        l-val)
                                  (setq l-val
                                        (:exploser-att l-val valeurs index))))
                        (incr index))
               (cdddr l-att-inst))
         (let ((nom-sch (#:shirka:noyau:val-inst instance %est-un))
               (util-par (#:shirka:noyau:val-inst instance %util-par)))
              (mapcar '(lambda (inst)
                               (vset inst %est-un nom-sch)
                               (vset inst %util-par util-par)
                               (let ((nom-inst 
                                             (#:shirka:comp:gen-symbole 
                                                     (concat "inst-" nom-sch))))
                                    (vset inst %nom-inst nom-inst)
                                    (putprop nom-inst inst 'schema)))
                      l-val))))

(de :exploser-att (l-inst valeurs index)
    (if valeurs
        (let ((nb-copies (length valeurs)))
             (mapcan '(lambda (inst)
                              (let ((index-v 0)
                                    (nouv-l-inst 
                                            (:const-list nb-copies inst)))
                                   (mapc '(lambda (n-inst)
                                                  (let ((val (nth index-v valeurs)))
                                                       (vset n-inst index val))
                                                  (incr index-v))
                                         nouv-l-inst)
                                   nouv-l-inst))
                     l-inst))
        l-inst))

(de :const-list (nb-copies inst)
    (cond ((eq 0 nb-copies) nil)
          (t (let ((elem (copy inst)))
                  (cons elem (:const-list (1- nb-copies) inst))))))

;;; traitement de tous les attachements proceduraux autres que $sib-exec
;;; Le principe de fonctionnement est le meme :
;;; instanciation du schema methode et appel a la fonction.
;;; Le filtrage n'est jamais essaye.

(de :attach-proc (nom-filtres)
    (mapc '(lambda (nom-filt)
                   (tag echec-att-proc
                        (:essayer-attach-proc 
                                (#:shirka:noyau:schema nom-filt))))
          nom-filtres))

(de :essayer-attach-proc (filtre)
    (let ((inst-filtre (:instancier filtre))
          (nom-fct 
                  (#:shirka:noyau:val-fac-he filtre 'nom-fct '$valeur)))
         (funcall nom-fct inst-filtre)))

;;; traitement de la facette $a-verifier
;;; Le principe general est le meme que celui de
;;; l'attachement procedural

(de :a-verif (nom-filtres)
    (tag echec-a-verif
         (mapc '(lambda (nom-filt)
                        (let ((val-pred 
                                      (:essayer-a-verif
                                              (#:shirka:noyau:schema nom-filt))))
                             (unless val-pred
                                     (setq %%NOM-PRED 
                                           (car (vref (#:shirka:noyau:schema nom-filt) 
                                                      %sorte-de)))
                                     (exit echec-a-verif))))
               nom-filtres)
         t) )

(de :essayer-a-verif (filtre)
    (tag echec-att-proc
         (let ((inst-filtre (:instancier filtre))
               (nom-fct 
                       (#:shirka:noyau:val-fac-he filtre 'nom-fct '$valeur)))
              (funcall nom-fct inst-filtre))))
