(defvar #:sys-package:colon '#:shirka:moteur:filtrage)

(de :sib-filtre (l-nom-filt schema nature)
    (let ((val (:sib-filtre1 l-nom-filt nature)))
         (when val
               (#:shirka:moteur:reusite val schema))))

;;; instanciation des filtres susceptibles d'etre utilises
;;; lors d'un filtrage et appel a la fonction de filtrage
(de :sib-filtre1 (l-nom-filt nature)
    (mapcan '(lambda (nom-filt)
                     (tag echec-filtrage
                          (let ((inst-filt (:inst-filt nom-filt))
                                (l-val-filt nil))
                               (setq val-filt 
                                     (:filtrage inst-filt nom-filt))
                               (when (and val-filt (eq nature '$un))
                                     (#:shirka:moteur:reusite val-filt schema))
                               (while val-filt
                                      (setq l-val-filt 
                                            (nconc1 l-val-filt val-filt))
                                      (setq val-filt 
                                            (:filtrage inst-filt nom-filt)))
                               l-val-filt)))
            l-nom-filt))

;;; instanciation d'un filtre
;;; seul les attributs "fixes" seront instancies
;;; c'est-a-dire : -references a une variable de plus haut niveau
;;;                -fixation de la valeur de l'attribut avec la facette $valeur
;;;                -si le premier cas echoue, existence de la facette $defaut
;;;                 et non existence d'aucun autre moyen d'inference
(de :inst-filt (nom-filt)
    (when #:exp:trace
          (#:exp:cons-tr (list 'but 'instancier nom-filt))
          (#:exp:cons-tr (list 'tr-inst nom-filt))
          )
    (let ((marqueur (:obt-marqueur nom-filt))
          (nom-inst-tr (concat "%inst-" nom-filt))
          (ordre-att (getprop nom-filt 'ordre-att))
          (f-imb (getprop nom-filt 'f-imb)))
         (let ((nom-inst (:creer-inst-vide nom-filt (1+ (length ordre-att)))))
              (putprop nom-filt nom-inst-tr 'lie)
              (:inst-att-fixes nom-inst 
                      nom-filt ordre-att)
              (let ((f-imbr (:f-imbr f-imb)))
                   (putprop nom-inst f-imbr 'f-imb))
              (putprop nom-inst marqueur 'marq-inst)
              (if #:exp:trace
                  (#:exp:cons-tr '(succes))
                  )
              nom-inst)))

;;; initialisation du marqueur pour un filtre
;;; le marqueur est initialise a la liste d'instances
;;; potentielles qui peuvent satisfaire le filtre
(de :obt-marqueur (nom-filt)
    (let ((l-inst (#:shirka:noyau:l-inst (#:shirka:noyau:schema nom-filt))))
         ;si le filtre n'est pas imbrique, echec de filtrage s'il n'y a pas d'instances potentielles
         
         ;;;(if (:filtre-plus-haut-niveau? nom-filt)
         ;;;(ifn l-inst
         ;;;(exit echec-filtrage)))
         l-inst))

;;; creation d'une instance vide d'un schema a partir de son nom
;;; et du nombre des attributs
(de :creer-inst-vide (nom-sch nb-att)
    (let ((val-inst (makevector (+ nb-att 3) nil))
          (nom-inst (#:shirka:comp:gen-symbole nom-sch)))
         (putprop nom-inst val-inst 'schema)
         (:aj-val-inst nom-sch %est-un nom-inst)
         (:aj-val-inst nom-inst %nom-inst nom-inst)
         nom-inst))

;;; instantiation des attributs fixes d'un filtre
;;; les attributs seront instancies dans l'ordre donne
;;; par la liste ordre-att
(de :inst-att-fixes (nom-inst nom-filt ordre-att)
    (let ((l-suc nil)
          (l-ech (list (#:shirka:moteur:obt-att-dem nom-filt))))
         (mapc '(lambda (nom-att)
                        (slet ((val-att-reseau (#:shirka:comp:val-att-reseau nom-filt nom-att))
                               (val-att 
                                       (:obt-val-att-fixes 
                                               val-att-reseau
                                               nom-inst 
                                               nom-filt 
                                               nom-att)))
                              (ifn val-att
                                   (setq l-ech (nconc1 l-ech nom-att))
                                   (:aj-valeur-inst-filtre val-att nom-att nom-inst)
                                   (setq l-suc (nconc1 l-suc nom-att)))))
               ordre-att)
         (putprop nom-inst (list l-suc l-ech) 'ordre-att)))

;;; obtention de la valeur d'un attribut d'un filtre
;;; a partir de l'information contenue dans le reseau
;;; et qui est propre a l'attribut
(de :obt-val-att-fixes (val-att-reseau nom-inst nom-filt nom-att)
    (tag sortie
         (let ((val-fac-const (:val-const? (#:shirka:comp:val-fac-att-reseau val-att-reseau %const))
                       )
               (val-fac-var (#:shirka:comp:val-fac-att-reseau val-att-reseau %var)))
              (if val-fac-const
                  (exit sortie val-fac-const))
              (if val-fac-var
                  (:res-ref-var
                          val-fac-var nom-inst nom-filt nom-att)))))

;;; essai d'obtention de la valeur d'une reference a une variable
;;; l'essai sera effectue uniquement dans le cas ou la reference
;;; concerne un attribut du schema de plus haut niveau
(de :res-ref-var (val-fac-var nom-inst nom-filt nom-att)
    (let ((ref-fixe? (:ref-fixe? val-fac-var)))
         (when ref-fixe?
               (when #:exp:trace
                     (#:exp:cons-tr (list 'but nom-att nom-filt))
                     (#:exp:cons-tr (list 'tr-vr nom-att ref-fixe?))
                     )             
               (let ((val (#:shirka:moteur:admissible?
                                  (#:shirka:moteur:obt-val-ref? ref-fixe?)
                                  nom-att
                                  (#:shirka:noyau:schema nom-filt)
                                  (#:shirka:noyau:schema nom-inst))))
                    (ifn  val
                         ;(admis-const-a-verif? nom-filt nom-att val))
                         (:essayer-def nom-filt nom-att)
                         (if #:exp:trace
                             (#:exp:cons-tr '(succes))
                             )             
                         (exit sortie val)))
               (exit sortie))))

;;; extraction des references fixes (a un schema de plus haut niveau)
(dmd :ref-fixe? (val-fac-var)
     `(or (:ref-var-type 0 ,val-fac-var)
          (:ref-var-type 1 ,val-fac-var)))

;;; extraction d'une reference de type type sur une val-fac-var
(de :ref-var-type (type val-fac-var)
    (tag succes
         (mapc '(lambda (elem)
                        (if (eq type (car elem))
                            (exit succes (cdr elem))))
               val-fac-var)))

;;; obtention de la valeur d'un attribut fixe a partir de sa reference
(de :obt-val-ref (ref)
    (let ((nom-sch (car ref))  ;nom du schema de classe
          (att-ref (cdr ref))) ;nom de l'attribut
         ;on cherche d'abord la valeur dans l'instance de travail attachee
         ;au schema reference, puis dans l'instance lie a ce schema
         (or (unless (eq att-ref 'lui-meme)
                     (let ((pos (#:shirka:noyau:pos-nom-att 
                                        (#:shirka:noyau:schema nom-sch) att-ref)) 
                           (w-inst (getprop nom-sch 'w-inst)))   
                          (when (and w-inst pos)
                                (#:shirka:noyau:val-inst w-inst pos))))
             (#:shirka:moteur:valeur? att-ref (:inst-cour nom-sch)))))

(de :obt-val-ref-f (nom-inst-f ref)
    (let ((att-ref (cdr ref)))
         (:valeur-att-f? nom-inst-f att-ref)))

;;; obtention de l'instance courante lie a un schema de nom nom-sch
(dmd :inst-cour (nom-sch)
     `(getprop ,nom-sch 'lie))

;;; essai d'obtention de valeur d'un attribut filtre
;;; a partir de l'existence de la facette $defaut
(de :essayer-def (nom-filt nom-att)
    (let ((val-fac-sibf (#:shirka:comp:val-fac-reseau nom-filt nom-att %sib-filt))
          (val-fac-sibe (#:shirka:comp:val-fac-reseau nom-filt nom-att %sib-exec))
          (val-fac-sibc (#:shirka:comp:val-fac-reseau nom-filt nom-att %sib-choix))
          (val-fac-def (#:shirka:comp:val-fac-reseau nom-filt nom-att %defaut)))
         (ifn (or val-fac-sibf
                  val-fac-sibe
                  val-fac-sibc)
              (if val-fac-def
                  (exit sortie 
                        (when #:exp:trace
                              (#:exp:cons-tr (list 'but nom-att nom-filt))
                              (#:exp:cons-tr (list 'tr-vd val-fac-def
                                                   nom-att inst-cour (nom-filt) nom-filt))
                              (#:exp:cons-tr '(succes))
                              )             
                        val-fac-def)
                  (if #:exp:trace
                      (#:exp:cons-tr '(echec))
                      )
                  (exit echec-filtrage))
              (exit sortie))))

;;; extraction de la partie constante $valeur
(dmd :val-const? (val-fac-const)
     `(if (eq 0 (car ,val-fac-const))
          (cdr ,val-fac-const)))

;;; instanciation de filtres imbriques
(de :f-imbr (f-imb)
    (mapcar '(lambda (filtre)
                     (let ((nom-inst-f (:inst-filt filtre)))
                          (cons filtre nom-inst-f)))
            f-imb))

;;; obtention de la valeur d'un attribut sur une instance
(de :valeur-att? (nom-inst-tr nom-att)
    (let ((nom-sch (:val-att? nom-inst-tr %est-un)))
         (if (eq nom-att 'lui-meme)
             (#:shirka:noyau:nom-sch (#:shirka:noyau:schema nom-inst-tr))
             (let ((pos-att (#:shirka:noyau:pos-nom-att (#:shirka:noyau:schema nom-sch) nom-att)))
                  (if pos-att
                      (:val-att? nom-inst-tr pos-att))))))

(de :valeur-att-f? (nom-inst-f nom-att)
    (let ((nom-filtre (:val-att? nom-inst-f %est-un)))
         (if (eq nom-att 'lui-meme)
             (#:shirka:noyau:nom-sch (#:shirka:noyau:schema (:inst-cour nom-filtre)))
             (let ((pos-att (:pos-att-filtre? nom-filtre nom-att)))
                  (if pos-att
                      (:val-att? nom-inst-f pos-att))))))

(dmd :val-att? (nom-inst pos-att)
     `(#:shirka:noyau:val-inst 
              (#:shirka:noyau:schema ,nom-inst) ,pos-att))

(de :filtre? (schema)
    (tag sortie
         (let ((sorte-de (#:shirka:noyau:val-inst schema %sorte-de))
               (nom-sch (#:shirka:noyau:nom-sch schema)))
              (mapc '(lambda (nom-sorte-de)
                             (if (memq nom-sch
                                       (getprop nom-sorte-de 'filtres))
                                 (exit sortie t)))
                    sorte-de))))

;;; obtention de la position d'un attribut dans une instance de filtre
(de :pos-att-filtre? (nom-f nom-att)
    (lets ((schema (#:shirka:noyau:schema nom-f))
           (l-nom-att (#:shirka:noyau:l-nom-att schema)))
          (+ 3 
             (- (length l-nom-att)
                (length (memq nom-att l-nom-att))))))

;;; ajout d'une valeur dans une instance de filtre
(de :aj-valeur-inst-filtre (val-att nom-att nom-inst-f)
    (lets ((nom-f (:val-att? nom-inst-f %est-un))
           (pos-att (:pos-att-filtre? nom-f nom-att)))
          (:aj-val-inst val-att pos-att nom-inst-f)))

(dmd :aj-val-inst (val-att pos-att nom-inst)
     `(let ((schema (#:shirka:noyau:schema ,nom-inst)))
           (vset schema ,pos-att ,val-att)))
