(defvar #:sys-package:colon '#:shirka:comp)

;;; compilation du schema par rapport aux variables
;;; a tous niveaux d'imbrication
(de :tr-var (schema)
    (let ((l-att (#:shirka:noyau:l-att schema))
          (methode (#:shirka:noyau:sorte-de? 
                           (#:shirka:noyau:nom-sch schema) 'attach-proc))
          (nom-sch (#:shirka:noyau:nom-sch schema)))
         ; traitement de la facette $var-nom (variables definies explicitement)  
         (let ((l-var-def (:ram-var-def l-att)))
              (putprop nom-sch l-var-def 'def))
         ; empilement du schema dans la liste de schemas deja traites
         (newl %l-sch-tr nom-sch)
         ; si attachement procedural, avec ou sans filtrage?
         (if methode
             (if (or (#:shirka:noyau:sorte-de? nom-sch 'methode-sans)
                     (#:shirka:noyau:sorte-de? nom-sch 'predicat-sans))
                 (putprop nom-sch 0 'type)))
         ; traitement des autres facettes variables ($var<- et $var->)
         ; appel recursif sur les facettes "filtre"
         (let ((l-var-ass nil)
               (l-var-dem nil)
               (l-dep nil)
               (l-f-imb nil))
              (mapc '(lambda (att)
                             (let ((var-ass (:tr-var-ass nom-sch att))
                                   (var-dem (:tr-var-dem nom-sch att)))
                                  (if var-ass
                                      (setq l-var-ass
                                            (nconc1 l-var-ass var-ass)))
                                  (if var-dem
                                      (setq l-var-dem
                                            (nconc1 l-var-dem var-dem))))
                             (let ((l-dep-att (:tr-var-fac-filtre att)))
                                  (setq l-dep
                                        (nconc1 l-dep
                                                (:constr-dep-att 
                                                        (#:shirka:noyau:nom-att att)
                                                        l-dep-att))))
                             (setq l-f-imb
                                   (append l-f-imb 
                                           (#:shirka:noyau:val-fac 
                                                   att '$sib-filtre))))
                    l-att) 
              (putprop nom-sch l-f-imb 'f-imb)
              (putprop nom-sch l-var-ass 'ass)
              (putprop nom-sch l-var-dem 'dem)
              (let ((elem-dem (assq (caar l-var-dem) l-dep)))
                   (setq l-dep (delete elem-dem l-dep)))
              (putprop nom-sch (:ord-dep l-dep) 'ordre-att)
              (if methode
                  (putprop nom-sch (:obt-ord-inst-att schema) 'ordre-inst-att)))
         ; depilement du schema de la liste de schemas deja traites
         (nextl %l-sch-tr)))

;;; construit la liste de paires (nom-var nom-att)
;;; pour une liste d'attributs propres a un schema quelconque
(de :ram-var-def (l-attribut)
    (mapcan '(lambda (att)
                     (let ((vec-att (if (vectorp att)
                                        att
                                        (#:shirka:noyau:schema att))))
                          (let ((var-def (:obt-var-def vec-att))
                                (spec-loc (:spec-loc vec-att)))
                               (if var-def
                                   (nconc1 spec-loc var-def)
                                   spec-loc))))
            l-attribut))

(dmd :obt-var-def (att)
     `(let ((var (#:shirka:noyau:val-fac ,att '$var-nom)))
           (if var
               (cons var (#:shirka:noyau:nom-att ,att)))))

(de :spec-loc (att)
    (let ((type (or (#:shirka:noyau:val-fac att '$un)
                    (#:shirka:noyau:val-fac att '$liste-de))))
         (if (:spec-loc? type)
             (let ((ram-var-def 
                           (:ram-var-def 
                                   (#:shirka:noyau:l-att  
                                           (#:shirka:noyau:schema type)))))
                  (mapcar '(lambda (ref)
                                   (cons (car ref)
                                         (cons (#:shirka:noyau:nom-att 
                                                       att) 
                                               (cdr ref))))
                          ram-var-def)))))

;;; determine si un schema donne par son nom
;;; correspond a une specialisation locale
(dmd :spec-loc? (nom-sch)
     `(getprop ,nom-sch 'spec-loc))

;;; traitement des facettes $var<- et $var-liste<- pour un attribut
(de :tr-var-ass (nom-sch attribut)
    (let ((var (#:shirka:noyau:val-fac attribut '$var<-))
          (var-liste (#:shirka:noyau:val-fac attribut '$var-liste<-))
          (nom-att (#:shirka:noyau:nom-att attribut)))
         (cond (var (:tr-var-att nom-sch nom-att  var '$var<-))
               (var-liste (:tr-var-att nom-sch nom-att  var-liste
                                  '$var-liste<-)))))

;;; construction de la paire (nom-att reference)
;;; pour un attribut donne par son nom
(de :tr-var-att (nom-sch nom-att att-ref type-var)
    (let ((ref (:res-ref att-ref type-var)))
         (if ref
             (cons nom-att ref) 
             ; reference a un attribut indefini
             (#:shirka:erreur:traiter-err 
                     (vector 'erreur-ref-indef () () 
                             att-ref nom-att nom-sch)
                     t))))

;;; recherche dans la liste de schema deja traites
;;; d'une reference a un attribut
(de :res-ref (att-ref type-var)
    (tag sortie                                  
         (mapc '(lambda (nom-sch)
                        (let ((ref (:chercher-ref nom-sch att-ref)))
                             (if ref
                                 (exit sortie (:constr-ref 
                                                      (#:shirka:noyau:schema nom-sch) 
                                                      ref type-var)))))
               %l-sch-tr)))

;;; recherche de la reference a un attribut de nom att-ref
;;; dans un schema de nom nom-sch
(de :chercher-ref (nom-sch att-ref)
    (let ((ref (or (cassq att-ref (getprop nom-sch 'def))
                   (if (memq att-ref 
                             (#:shirka:noyau:l-nom-att 
                                     (#:shirka:noyau:schema nom-sch)))
                       att-ref))))
         (if ref
             (cons nom-sch ref))))

;;; determination du type d'une reference
;;; et construction complete de la reference
;;; avec la paire (type reference)
(de :constr-ref (schema ref type-var)
    (let ((nat-ref (:obt-nature ref))
          (niv-ref (:niv-ref (car ref))))
         (selectq type-var
                 ($var<- (selectq niv-ref
                                 (0 (if (eq nat-ref '$un)
                                        (cons 0 ref)   
                                        (cons 1 ref)))
                                 (1 (cons 6 ref))
                                 (2 (if (eq nat-ref '$un)
                                        (cons 2 ref)
                                        (cons 3 ref)))))
                 ($var-liste<- (if (eq nat-ref '$liste-de)
                                   (selectq niv-ref
                                           (0 (cons 0 ref))
                                           (1 (cons 6 ref))
                                           (2 (cons 2 ref)))
                                   (#:shirka:erreur:traiter-err nil t)))                               
                 ($var-> (if (eq niv-ref 0)
                             (cons 4 ref)
                             (cons 5 ref))))))
 
(de :obt-nature (ref)
    (if (atom (cdr ref))
        (car (#:shirka:noyau:nature 
                    (#:shirka:noyau:schema (car ref)) (cdr ref)))
        (lets ((nom-sch (cadr (#:shirka:noyau:nature 
                           (#:shirka:noyau:schema (car ref)) (cadr ref))))
               (sch-classe (if (:spec-loc? nom-sch)
                               (car (#:shirka:noyau:val-inst 
                                      (#:shirka:noyau:schema nom-sch)
                                      %sorte-de))
                               nom-sch)))
             (:obt-nature (cons sch-classe (cddr ref))))))

;;; traitement de la facette $var->
(de :tr-var-dem (nom-sch attribut)
    (let ((var (#:shirka:noyau:val-fac attribut '$var->))
          (nom-att (#:shirka:noyau:nom-att attribut)))
         (if var
             (:tr-var-att nom-sch nom-att var '$var->))))

;;; determine le niveau d'une reference
;;; rend 0 s'il s'agit d'une reference au schema de plus haut niveau
(de :niv-ref (nom-sch)
    (cond ((eq nom-sch (car (last %l-sch-tr))) 0)
          ((eq nom-sch (car %l-sch-tr)) 1)
          (t 2)))

;;; donne la valeur de l'attribut reference 
;;; s'il existe une reference interne dans la
;;; liste de references %l-var-ass pour l'attribut de nom nom-att
(de :ref-int? (nom-att l-var-ass)
    (let ((ref (cassq nom-att l-var-ass)))
         (if (eq 6 (car ref))
             (cddr ref))))

;;; compilation de variables pour les facettes pouvant
;;; contenir un filtre d'un attribut particulier
(de :tr-var-fac-filtre (attribut)
    (let ((l-dep-att nil))
         (mapc '(lambda (fac-filtre)
                        (let ((nom-filtres 
                                      (#:shirka:noyau:val-fac attribut 
                                              fac-filtre))
                              (sib-exec (eq fac-filtre '$sib-exec))
                              (constr-dep (or (eq fac-filtre '$sib-filtre)
                                              (eq fac-filtre '$sib-exec))))
                             (mapc '(lambda (nom-f)
                                            (:tr-var 
                                                    (#:shirka:noyau:schema nom-f))
                                            (if sib-exec
                                                (:typer-se nom-f))
                                            (if constr-dep
                                                (setq l-dep-att
                                                      (nconc l-dep-att
                                                             (:constr-dep nom-f)))))
                                   nom-filtres)))
               %l-fac-filtre)
         l-dep-att))

(de :typer-se (nom-f)
    (ifn (getprop nom-f 'type) ; si l'attachement n'est pas deja type
         (let ((lass (getprop nom-f 'ass)))
              (ifn lass
                   (putprop nom-f 0 'type)
                   (let ((l-ref-c (:det-ref-c lass)))
                        (ifn l-ref-c
                             (putprop nom-f 1 'type)
                             (putprop nom-f 2 'type)
                             (putprop nom-f l-ref-c 'l-ref-c))))))) 

(de :det-ref-c (lass)
    (mapcan '(lambda (ref)
                (ifn (#:shirka:moteur:ref-simple? (cdr ref))
                     (list (car ref))))
            lass))

;;; construit les relations de dependences pour un filtre
;;; qui peut ramener une valeur entre l'attribut qui recoit la valeur
;;; et les autres attributs du meme niveau
(de :constr-dep (nom-filtre)
    (let ((sch-dem (caddar (getprop nom-filtre 'dem)))
          (l-ref (getprop nom-filtre 'ass)))
         (mapcan '(lambda (ref)
                     (let ((sch-ref (caddr ref)))
                          (if (eq sch-ref sch-dem)
                              (list (cdddr ref)))))
                 l-ref)))

(de :constr-dep-att (nom-att l-dep)
    (if l-dep
        (list nom-att (:elim-doubles l-dep))
        (list nom-att)))

;;; elimine les elements qui se trouvent en plus 
;;; d'un exemplaire dans une liste
(de :elim-doubles (liste)
    (unless (null (cdr liste))
            (delq (car liste) (cdr liste))
            (:elim-doubles (cdr liste))))

;;; construit une liste qui determine l'ordre de calcul des
;;; divers attributs d'un filtre a partir de ses relations de dependance
(de :ord-dep (liste-dep)
    (ifn (null liste-dep)
         (let ((sources (:obt-sources liste-dep))
               (l-dep nil))
              (setq sources
                   (delq nil sources))
              ; elimination des relations de dependance une fois
              ; les sources obtenus
              (mapc '(lambda (elem)
                        (ifn (member (car elem) sources)
                             (let ((nouv-elem (:obt-nouv-dep elem sources)))
                                  (setq l-dep 
                                        (nconc1 l-dep nouv-elem)))))
                    liste-dep)
              (nconc sources (:ord-dep l-dep)))))

(de :obt-nouv-dep (elem sources)
    (let ((l-nouv-dep (:elim-sources (cadr elem) sources)))
         (setq l-nouv-dep 
               (delq nil l-nouv-dep))
         (if l-nouv-dep
             (list (car elem) l-nouv-dep)
             (list (car elem)))))

;;; elimination des sources dans %l-dep
(de :elim-sources (l-elem sources)
    (mapcar '(lambda (elem)
                     (ifn (member elem sources)
                          elem))
            l-elem))

;;; obtention des sources dans le graphe de dependances
;;; c'est-a-dire des attributs qui ne dependent de personne
(de :obt-sources (l-dep)
    (mapcar '(lambda (elem)
                     (if (null (cdr elem))
                         (car elem)))
            l-dep))

(de :obt-ord-inst-att (schema)
    (let ((l-ord-att (getprop (#:shirka:noyau:nom-sch schema) 'ordre-att))
          (l-ord-inst-att-he (:obt-ord-inst-att-he schema)))
         (mapc '(lambda (nom-att)
                        (setq l-ord-inst-att-he
                              (delq nom-att l-ord-inst-att-he)))
               l-ord-att)
         (let ((att-dem (#:shirka:moteur:obt-att-dem 
                                (#:shirka:noyau:nom-sch schema))))
              (setq l-ord-inst-att-he 
                    (delq att-dem l-ord-inst-att-he)))
         (append l-ord-inst-att-he
                 (mapcan '(lambda (nom-att)
                                  (ifn (#:shirka:noyau:val-facette 
                                               schema nom-att '$valeur)
                                       (list nom-att)))
                         l-ord-att))))

(de :obt-ord-inst-att-he (schema)
    (let ((peres (#:shirka:noyau:peres schema)))
         (mapcan '(lambda (pere)
                          (append (getprop pere 'ordre-inst-att)
                                  (:obt-ord-inst-att-he 
                                          (#:shirka:noyau:schema pere))))
                 peres)))
