;;; Fonctions utilitaires du verifieur-formateur

;;;(setq #:system:read-case-flag t)
(defvar #:sys-package:colon '#:shirka:verif)

;;;-- specialisations sur <sortie-lexicale>
;;;   -------------------------------------
(de :sortie-lex (token chaine)
    (let (
          (in-spe (:chercher-nom token  #:shirka:tables:classes-spe))
          (in-cle (:chercher-nom chaine #:shirka:tables:classes-cle))
          (in-val (:chercher-val chaine #:shirka:tables:classes-val))
          )
         (tag sortir
              (when (member token #:shirka:tables:classes-spe)
                    ; l'US possede une specialisation "interne":
                    ; *** construire suivant les criteres de l'utilisateur ***
                    (setq :spec-US 'special)
                    (print "--> sortie-lex: specialiser " token)
                    (exit sortir))
              ;
              (when (nequal (car in-spe) '- )
                    ; l'US fait partie d'une liste de specialisations:
                    (setq :spec-US (car in-spe) :valeur-US (cadr in-spe))
                    (exit sortir))
              ;
              (when (nequal (car in-cle) '- )
                    ; l'US fait partie d'une liste de mots-cles:
                    (setq :spec-US (car in-cle) :valeur-US (cadr in-cle))
                    (exit sortir))
              ;
              (when (nequal (car in-val) '- )
                    ; l'US fait partie d'une liste de mots-cles avec valeur:
                    (setq :spec-US (car in-val) :valeur-US (cadr in-val)))))
    )

;;;-- chercher un nom dans classes specialisations
;;;   --------------------------------------------
(de :chercher-nom (clef classes)
    (let ((ptr 0) (liste-spes ()) (lgmax (length classes))
          (index 0) (lg-lspes 0) (trouve (list '- '-)) (nom '- )
          )
         (while (lt ptr lgmax)
                (setq nom (nth ptr classes))
                (setq liste-spes (nth (incr ptr) classes))
                (setq lg-lspes (+ (length liste-spes) 1))
                (when (member clef liste-spes)
                      (setq index (length (member clef liste-spes)))
                      (setq trouve (list nom (- lg-lspes index)))
                      (setq ptr lgmax))
                (incr ptr))
         trouve)
    )

;;;-- chercher <mot-clef> dans classes valeur
;;;   ---------------------------------------
(de :chercher-val (clef classes)
    (let ((ptr 0) (liste-cles ()) (lgmax (length classes))
          (index 0) (lg-lcles 0) (trouve (list '- '-)) (nom '- )
          )
         (while (lt ptr lgmax)
                (setq nom (nth ptr classes))
                (setq liste-cles (nth (incr ptr) classes))
                (setq lg-lcles (+ (length liste-cles) 1))
                (when (member clef liste-cles)
                      (setq index (- lg-lcles (length (member clef liste-cles))))
                      (setq trouve (list nom (nth index liste-cles)))
                      (setq ptr lgmax))
                (incr ptr))
         trouve)
    )

;;;-- lire <cara-courant> dans <fsource>
;;;   ----------------------------------
(de :new-curcar ()
    (let ( (deb 0) (fin) (lgligne)
          )
         (when (> (incr :ptcurc) :lgbuff-in)
               ; --- lire une nouvelle ligne: 
               (setq :ptcurc 0)
               (if (= :err-inline 0) ()
                   ; --- lister erreurs + marqueurs de la ligne precedente:
                   (:lister-marques)
                   (:lister-erreurs :liste-errs))
               ; --- reinitialiser le traitement d'erreurs:
               (setq :liste-errs () :err-inline 0 :marqueurs (makevector 50 1))
               (setq :ligne-marq (makestring :lgmax-liste 32))
               ; --- preparation de la ligne lue:
               (setq :buff-in (readline))
               (setq :lgbuff-in (length :buff-in))
               (setq ligne-lue (string :buff-in))
               (setq lgligne (slength ligne-lue))
               (setq fin (min lgligne :lgmax-liste))
               (incr :nlignes-src)
               ; --- imprimer la ligne lue avec continuations si besoin:
               (:listing 'source (substring ligne-lue 0 fin))
               (decr lgligne fin)
               ;
               (while (> lgligne 0)
                      (incr deb fin)
                      (setq fin (min lgligne :lgmax-liste))
                      (:listing 'trace (substring ligne-lue deb fin) )
                      (decr lgligne fin))
               ;
               ; --- ajouter le <end-of-line>:
               (setq :buff-in (append1 :buff-in :e-of-l)))
         ; --- caractere courant <curcar>:
         (setq :curcar (nth :ptcurc :buff-in))
         ; --- valider cara lu en fonction de "typcar" active:
         (if (> :curcar (vref #:shirka:tables:symbs-ascii 0)) (setq :curcar 1) :curcar) )
    )

;;;-- lister les marques d'erreurs
;;;   ----------------------------
(de :lister-marques ()
    (let ((new-marq 1) (ptmarq (vref :marqueurs 1))
          )
         ;
         (when (> :nlignes-src 1)
               (repeat :err-inline
                       (chrset (min ptmarq :nbcars-eff) :ligne-marq #/! )
                       (setq ptmarq (vref :marqueurs (incr new-marq))))
               (:listing 'marque :ligne-marq)))
    )

;;;-- lister toutes les erreurs de <ligne courante>
;;;   ---------------------------------------------
(de :lister-erreurs (liste)
    (if (null liste)
        ()
        (:listing (car liste) (cadr liste))
        (:lister-erreurs (cddr liste)))
    )

;;;-- recherche <type-lexical> dans <instruction-lexicale>
;;;   ----------------------------------------------------
(de :match-type (instr)
    (let ((l-types (car instr))
          )
         ;
         (setq :exit-autre (equal l-types '(AUTRE)))
         (or :exit-autre (member :type-cc l-types)))
    )

;;;-- rendre <index> de <nom> dans <liste-un-niveau>
;;;   ----------------------------------------------
(de :index-of (nom liste-1)
    (- (+ (length liste-1) 1) (length (member nom liste-1)))
    )

;;;-- empiler <curcar> dans le tampon <Unite Syntaxique>
;;;   --------------------------------------------------
(de :get-cc ()
    (setq :buf-US (append1 :buf-US :curcar))
    (setq :oldcurcar :curcar)
    )

;;;-- empiler <valeur> dans <pile-lexicale>
;;;   -------------------------------------
(de :pushlex (valeur)
    (lets ((top-stack (vref :pile-lex 0))
           (ptr (vref :pile-lex top-stack)) )
          ;
          (if (equal top-stack ptr)
              (error ':pushlex valeur 'Pile-lexicale-pleine))
          (vset :pile-lex ptr valeur)
          (vset :pile-lex top-stack (incr ptr)))
    )

;;;-- empiler <valeur> dans <pile-syntaxique>
;;;   ---------------------------------------
(de :pushsynt (valeur)
    (lets ((top-stack (vref :pile-synt 0))
           (ptr (vref :pile-synt top-stack)) )
          ;
          (if (equal top-stack ptr)
              (error ':pushsynt valeur 'Pile-syntaxique-pleine))
          (vset :pile-synt ptr valeur)
          (vset :pile-synt top-stack (incr ptr)))
    )

;;;-- depiler pile lexicale
;;;   ---------------------
(de :poplex ()
    (lets ((top-stack (vref :pile-lex 0))
           (ptr (vref :pile-lex top-stack)) )
          ;
          (if (equal ptr 1)
              (error ':poplex :pile-lex 'Pile-lexicale-vide))
          (vset :pile-lex top-stack (decr ptr))
          (vref :pile-lex ptr))
    )

;;;-- depiler pile syntaxique
;;;   -----------------------
(de :popsynt ()
    (lets ((top-stack (vref :pile-synt 0))
           (ptr (vref :pile-synt top-stack)) )
          ;
          (if (equal ptr 1)
              (error ':popsynt :pile-synt 'Pile-syntaxique-vide))
          (vset :pile-synt top-stack (decr ptr))
          (vref :pile-synt ptr))
    )

;;;-- 'reset' pile lexico
;;;    ------------------
(de :resetlex ()
    (vset :pile-lex (vref :pile-lex 0) 1)
    )

;;;-- 'reset' pile syntaxe
;;;    -------------------
(de :resetsynt ()
    (vset :pile-synt (vref :pile-synt 0) 1)
    )

;;;-- test sur pile interne vide
;;;   --------------------------
(de :pile-vide? (pile)
    (let ((top (vref pile 0)) )
         (= (vref pile top) 1))
    )

;;;-- execution <action-lex> puis <etat-suivant>
;;;   ------------------------------------------
(de :alex-and-branch (instr)
    (let ((al (cadr instr))
          (branch (car (nth 2 instr)))
          (adresse (cadr (nth 2 instr)))
          )
         ; appeler action lexicale:
         (:actions-lex al)
         ;;; *** modif speciale pour shunter les vecteurs <lisp> ***
         (when :retour-en-1
               (setq branch '*GOTO  adresse 1 :retour-en-1 () ))
         ;;; ***                fin modif speciale               ***
         (selectq branch
                 ;
                 ; --- empiler <curcar> et allera <adresse>:
                 (GOTO
                       (setq :etat adresse)
                       (:get-cc) (setq :type-cc (vref #:shirka:tables:typcar (:new-curcar))))
                 ;
                 ; --- ignorer <curcar> et allera <adresse>:
                 (*GOTO
                        (setq :etat adresse)
                        (setq :type-cc (vref #:shirka:tables:typcar (:new-curcar))))
                 ;
                 ; --- sortie lexicale <adresse>:
                 (OUT
                      (setq :US adresse :reconnue t)
                      (when (= :etat 1) (:get-cc) (setq :exit-autre ()))
                      (setq :nom-US (concat (string :buf-US)))
                      (if (= (length :buf-US) 1)
                          (setq :symb-ascii-US (vref #:shirka:tables:symbs-ascii :oldcurcar)))
                      ; Recherche de specialisation sur US reconnue:
                      (:sortie-lex :US :nom-US)
                      ; Avancer si sortie OUT sur <type> distinct de AUTRE:
                      (ifn :exit-autre
                           (setq :type-cc (vref #:shirka:tables:typcar (:new-curcar)))))
                 ;
                 ; --- AUTRE: erreur systeme
                 (t
                    (error ':alex-and-branch branch 'inconnue))))
    )

;;;-- moteur lexical
;;;   --------------
(de :analyse-lex ()
    (let ((instr-lex) (ptautolex)
          )
         ;
         (setq :etat 1 :buf-US () :exit-autre () :reconnue () )
         (setq :US '- :nom-US '- :symb-ascii-US '- :spec-US '- :valeur-US '-)
         ;
         ; boucle d'analyse
         ; ----------------
         (until :reconnue
                (setq ptautolex (vref #:shirka:tables:hash-lex :etat))
                (setq instr-lex (vref #:shirka:tables:auto-lex ptautolex))
                (until (:match-type instr-lex)
                       (setq instr-lex (vref #:shirka:tables:auto-lex (incr ptautolex))))
                (:alex-and-branch instr-lex))
         ;
         (list :spec-US :symb-ascii-US :US :nom-US :valeur-US))
    )

;;;-- recherche <US> dans <instruction-syntaxique>
;;;   --------------------------------------------
(de :match-US (instr)
    ; US::= (<spec-US> <symb-ascii-US> <US> <nom-US> <valeur-US>)
    (let ((test (car instr)) (err (caar instr))
          )
         ;
         (setq :cas-autre (equal test '(AUTRE)))
         (setq :cas-erreur (member err '(WARNING ERREUR FATALE)))
         (or :cas-autre :cas-erreur
             (member :spec-US test)
             (member :symb-ascii-US test)
             (member :US test)))
    )

;;;-- emission des messages d'erreurs
;;;   -------------------------------
(de :msg-err (erreur)
    (let ((type (car erreur)) (message (cdr erreur))
          )
         (incr :err-inline)
         (vset :marqueurs :err-inline :ptcurc )
         ;
         ; on n'empile que les nouveaux messages:
         (if (member message :liste-errs)
             ()
             (selectq type
                     ;
                     ; --- messages attention:
                     (WARNING
                             (setq :liste-errs (append1 :liste-errs 'wng)))
                     ;
                     ; --- messages erreur:
                     (ERREUR
                             (setq :liste-errs (append1 :liste-errs 'err))
                             (incr :nb-errors)
                             (if (> :nb-errors :maxerrfat)
                                 (setq :fatal t)))
                     ;
                     ; --- message fatal:
                     (FATALE
                             (setq :liste-errs (append1 :liste-errs 'fatal))
                             (setq :fatal t)
                             (incr :nb-errors))
                     ;
                     ; --- AUTRE: erreur systeme
                     (t
                        (error ':msg-err erreur 'inconnue)) )
             ; ajouter le nouveau message dans la liste d'erreurs:
             (setq :liste-errs (append1 :liste-errs message))))
    )

;;;-- execution action-semantique et instruction suite
;;;   ------------------------------------------------
(de :asem-and-branch (instr)
    (lets ((asem (cadr instr)) (adresses (nth 2 instr)) (branch (car adresses))
           (go1 (cadr adresses)) (go2 (cdr adresses))
           )
          ;=== DEBUG instr
          ;;;(print "-> " instr)
          ;=== FIN DEBUG
          ; imprimer message d'erreur si besoin
          ; -----------------------------------
          (if :cas-erreur (:msg-err (car instr)))
          ;
          ; appeler action semantique
          ; -------------------------
          (:actions-sem asem)
          ;
          ; executer instruction suite
          ; --------------------------
          ;;; *** modif speciale pour shunter les <schemas> invalides ***
          (when :reset-syntaxe
                (setq branch 'GOTO go1 (car #:shirka:tables:liste-etiqs) :reset-syntaxe () ))
          ;;; ***                 fin  modif  speciale                ***
          (selectq branch
                  ;
                  ; --- allera <adresse> avec US suivante:
                  (GOTO
                        (setq :etiq go1)
                        (:analyse-lex)
                        )
                  ;
                  ; --- allera <adresse> en gardant US courante:
                  (*GOTO
                         (setq :etiq go1))
                  ;
                  ; --- appeler sous-automate avec US suivante:
                  (GOSUB
                         (:call-automate go2)
                         (:analyse-lex))
                  ;
                  ; --- appeler sous-automate en gardant US courante:
                  (*GOSUB
                          (:call-automate go2))
                  ;
                  ; --- retour depuis sous-auto avec US suivante:
                  (RET
                       (setq :etiq (:popsynt))
                       (:analyse-lex))
                  ;
                  ; --- retour depuis sous-auto en gardant US courante:
                  (*RET
                        (setq :etiq (:popsynt)))
                  ;
                  ; --- arret automate:
                  (STOP
                        (:lister-marques)
                        (:lister-erreurs :liste-errs)
                        (:listing 'trace (catenate "...STOP... " :nb-errors " erreur(s)") )
                        (setq :flag-stop 'on)
                        (exit compile)
                        )
                  ;
                  ; --- AUTRE: erreur systeme
                  (t   (error ':asem-and-branch branch 'inconnue))))
    )

;;;-- decode (<etiq-sous-auto> <etiq-retour>) des "GOSUB"
;;;   ---------------------------------------------------
(de :call-automate (etiquettes)
    (let ((adr-auto (car etiquettes))
          (adr-ret (cadr etiquettes))
          )
         (ifn (and (member adr-auto #:shirka:tables:liste-etiqs)
                   (member adr-ret #:shirka:tables:liste-etiqs))
              (error ':call-automate etiquettes 'inconnues)
              (setq :etiq adr-auto)
              (:pushsynt adr-ret)))
    )

;;;-- moteur syntaxique
;;;   -----------------
(de :analyse-synt ()
    (let ((instr-synt ()) (ptauto 1) (canal)
          )
         ;
         ; boucle d'analyse
         ; ----------------
         (until (or :fatal :flag-stop :b-include)
                (setq :type-cc (vref #:shirka:tables:typcar (:new-curcar))
                      :etiq (car #:shirka:tables:liste-etiqs)
                      :b-include () )
                (:analyse-lex)
                ;
                (untilexit eof
                        (ifn (member :etiq #:shirka:tables:liste-etiqs)
                             (error ':analyse-synt branch 'etiquette-inconnue))
                        (setq ptauto (vref #:shirka:tables:hash-syntax (:index-of :etiq #:shirka:tables:liste-etiqs)))
                        (setq instr-synt (vref #:shirka:tables:auto-syntax ptauto))
                        (until (:match-US instr-synt)
                               (setq instr-synt (vref #:shirka:tables:auto-syntax (incr ptauto))))
                        (:asem-and-branch instr-synt))
                ; depiler le canal-courant sur <eof>
                (setq canal (:pop :pile-include))
                (if (numberp canal) (inchan canal)
                    (setq :b-include t)))
         ; sortie en <eof> final: on simule une instruction 'STOP'
         (:asem-and-branch '((AUTRE) exit-eof (STOP)) ))
    )
