(defvar #:sys-package:colon '#:shirka:classif)

;;; verification de l'admissibilite de(s) valeur(s) de l'attribut 
;;; nom-att de l'instance nom-inst dans "classe"
;;; rend le couple (sur, possible, impossible . valeur) 
(de :verif-val (nom-inst nom-sch nom-att valeur classe dom)
    (slet ((val (#:shirka:moteur:admissible? valeur nom-att classe 
                        (#:shirka:noyau:schema nom-inst)))
           (testval 'sur))
          (if (and (not val) %%EXPL-NEG-TYPE)
              (progn 
                     (setq testval 'impossible)
                     (:ecr-expl nom-inst
                             (#:shirka:noyau:nom-sch classe)
                             nom-att
                             valeur
                             'impossible
                             (list 't %%EXPL-NEG-TYPE)))
              (if (and (not val) %%EXPL-NEG-PRED)
                  (progn 
                         (setq testval 'impossible)
                         (:ecr-expl nom-inst
                                 (#:shirka:noyau:nom-sch classe)
                                 nom-att
                                 valeur
                                 'impossible
                                 (list 'p %%EXPL-NEG-PRED)))
                  (ifn val     ; toutes les valeurs doivent etre admissibles
                       (progn                      
                              (setq testval 'impossible)
                              (:ecr-expl nom-inst (#:shirka:noyau:nom-sch classe) 
                                      nom-att
                                      valeur 'impossible dom)))))
          (cons testval val)))

;;; obtention d'une valeur d'un attribut nom-att pour l'instance de nom
;;; nom-inst de la classe de nom nom-sch
;;; si binf avec inference suivie en cas d'echec d'une question si bdial
;;; si non binf question uniquement
;;; pas d'essai d'obtention si on a deja essaye ou si les domaines de
;;; validite de l'attribut sont incompatibles dans classe et nom-sch  
(de :obtien-val (nom-inst nom-sch nom-att classe binf bdial)
    (slet ((instance (#:shirka:noyau:schema nom-inst))
           (schema (#:shirka:noyau:schema nom-sch))
           (att (#:shirka:noyau:attrib classe nom-att))
           (type (#:shirka:noyau:val-inst att %type))
           (nature (#:shirka:noyau:val-inst att %nat))
           (dej-rep (member nom-att (getprop nom-inst 'deja-repondu)))
           (prop-dom (:dom-adm schema nom-att classe))
           (prop (car prop-dom))
           (dom (cdr prop-dom))
           ; obtention de la valeur de l'attribut
           (valeur (if (and 
                            (if bdial (not dej-rep) t)
                            (eq prop 'possible)
                            (getprop nom-inst 'a-repondre))
                       (if binf
                           ; obtention par inference 
                           (let ((val
                                      (:infer-val schema instance 
                                              classe att nature nom-att)))
                                (if val val
                                    (if (and bdial (not dej-rep))
                                        (:dem-val schema classe nom-att 
                                                instance bguide t))))
                           ; obtention par question
                           (:dem-val schema classe 
                                   nom-att instance bguide t))))
           (testval prop))
          (if (null valeur)
              (if (neq testval 'sur)
                  (:ecr-expl nom-inst 
                          (#:shirka:noyau:nom-sch classe) 
                          nom-att nil testval dom))
              (let ((resul
                           (:verif-val nom-inst nom-sch nom-att 
                                   valeur classe dom)))
                   (setq testval (car resul))
                   (setq valeur
                         (cdr resul))))
          (cons testval valeur)))

;;; inference de valeur sans heritage en cas d'echec
(de :infer-val (schema instance  classe attribut nature %%nom-att)
    (slet ((nom-sch (#:shirka:noyau:nom-sch schema))
           (nom-inst (#:shirka:noyau:nom-sch instance))
           (dej-rep (member %%nom-att 
                            (getprop nom-inst 'deja-repondu))))
          (let ((val
                     (tag sortie
                          (#:shirka:moteur:lier 
                                  (#:shirka:noyau:nom-sch classe) nom-inst)
                          (#:shirka:moteur:trouver-val 
                                  attribut classe instance nature)
                          ; si succes, il y a sortie par (exit sortie)
                          ; traitement de la facette $si-echec
                          (#:shirka:moteur:attach-proc 
                                  (#:shirka:noyau:val-facette 
                                          schema %%nom-att '$si-echec))
                          nil)))
               (when val                                 
                     (#:shirka:moteur:aj-valeur
                             instance %%nom-att val t)
                     (ifn dej-rep
                          (let ((l-dej-rep 
                                        (getprop nom-inst 'deja-repondu)))
                               (setq 
                                     l-dej-rep 
                                     (nconc1 l-dej-rep %%nom-att))
                               (putprop nom-inst 
                                       l-dej-rep 'deja-repondu))))
               (#:shirka:moteur:delier (#:shirka:noyau:nom-sch classe))
               val)))

;;; intersection d'un intervalle et d'un domaine
(de :comb-i-d (i d type)
    (cond ((null i) (cons 'd d))
          ((null d) (cons 'i i))
          ((or (eq i 'vide)(eq d 'vide)) 'vide)
          (t (let ((l)(elem))
                  (while (nextl d elem)
                         (if (:app-inter? elem type i)
                             (setq l (nconc1 l elem))))
                  (if l (cons 'd l) 'vide)))))

;;; rend le couple : ( possibilite . domaine-general-des-valeurs)
;;; de l'attribut nom-att admissibles a la fois dans schema et classe
(de :dom-adm (schema nom-att classe)
    (slet ((att (#:shirka:noyau:attrib classe nom-att))
           (type (#:shirka:noyau:val-inst att %type))
           (i)(d)(i-d-v)(v)(p1)
           (p2)(pp)(p3))
          (tag fin
               
               ;---------------- intervalle---------------------------------
               (slet ((inter-bid (:pintervalle schema nom-att type))
                      (inter (:pintervalle classe nom-att type))
                      (intersec (:pcomb-inter inter-bid inter type))
                      (prop
                            (if (or inter-bid inter)
                                (if (eq intersec 'vide)
                                    'impossible
                                    (if (equal intersec inter-bid)
                                        'sur
                                        'possible))
                                'sur)))
                     (if (eq prop 'impossible)
                         (exit fin (cons 'impossible 'vide))
                         (progn 
                                (setq p1 prop)(setq i intersec))))
               
               ;--------------------domaine---------------------------------
               (slet ((dom-bid (:pdomaine schema nom-att type))
                      (dom (:pdomaine classe nom-att type))
                      (intersec (:pcomb-dom dom-bid dom type))
                      (prop        (if (or dom-bid dom)
                                       (if (eq intersec 'vide)
                                           'impossible
                                           (if (equal intersec dom-bid)
                                               'sur
                                               'possible))
                                       'sur)))
                     (if (eq prop 'impossible)
                         (exit fin (cons 'impossible 'vide))
                         (progn 
                                (setq p2 prop)(setq d intersec))))

               ;-----------fusion domaine intervalle---------------------------
               (ifn (or i d)
                    (progn (setq pp 'possible)(setq i-d-v ()))
                    (let ((fus (:comb-i-d i d type)))
                         (if (eq fus 'vide)
                             (exit fin (cons 'impossible 'vide))
                             (let 
                                  ((prop 'possible))
                                  (setq pp prop)
                                  (setq i-d-v fus)))))
               (cons pp i-d-v))))

;;; liste des intervalles heritee par l'attribut "nom-att" de "schema"
(de :pintervalle (schema nom-att type)
    (prog1 (:pramener-int schema nom-att type)
           (#:shirka:noyau:demarquer schema)))

;;; expression simplifiee de l'intersection de listes d'intervalles
(de :c-inter (inter type)
    (if (atom inter)
        inter
        (:pcomb-inter (car inter) (:c-inter (cdr inter) type)
                type)))

;;; ramener la liste d'intervalles de validite de l'attribut nom-att
;;; pour le "schema"
(de :pramener-int (schema nom-att type)
    (slet ((attrib (#:shirka:noyau:attrib schema nom-att))
           (l-inter (let ((val-fac-inter (#:shirka:noyau:val-fac attrib '$intervalle)))
                         (if val-fac-inter
                             (mapcar '#:shirka:noyau:tr-inter 
                                     val-fac-inter)))))
          ;*** suppression du 26/03/86 :(or l-inter
          (:pcomb-inter l-inter    
                  ; combiner les intervalles courants avec la
                  ; liste des intervalles herites combines entre eux
                  ;*** modif du 26/03/86
                  (:c-inter (mapcar '(lambda (pere)
                                             (unless (getprop pere 'traite)
                                                     (putprop pere t 'traite)
                                                     (let ((sch-pere 
                                                                   (#:shirka:noyau:schema pere))) 
                                                          (:pramener-int sch-pere nom-att 
                                                                  type))))
                                    (#:shirka:noyau:peres schema))
                          type)        type)))

;;; intersection de la liste d'intervalles "inter1" avec la liste "inter2"
(de :pcomb-inter (inter1 inter2 type)
    ; si une liste d'intervalle est vide, elle a pour valeur 'vide
    ; si elle est non restrictive, elle a pour valeur nil
    (cond ((null inter1) inter2)
          ((null inter2) inter1)
          ((or (eq inter1 'vide)(eq inter2 'vide)) 'vide)
          (t (:simpl-inter       ; rend l'intersection simplifiee
                     (mapcan 
                             '(lambda ((min1 max1))
                                      (mapcar 
                                              '(lambda (inter)
                                                       (let ((min (car inter))
                                                             (max (cadr inter)))
                                                            (if (or (:super min1 max type)
                                                                    (:super min max1 type))
                                                                'vide       ; intervalles disjoints
                                                                (progn 
                                                                       (if (:app-inter? min1 type 
                                                                                   (list inter))
                                                                           (setq min min1))
                                                                       (if (:app-inter? max1 type
                                                                                   (list inter))
                                                                           (setq max max1))
                                                                       (list min max)))))
                                              inter2))
                             inter1) type))))

;;; domaine herite par l'attribut "nom-att" de "schema"
(de :pdomaine (schema nom-att type)
    (prog1 (:pramener-dom schema nom-att type)
           (#:shirka:noyau:demarquer schema)))

;;; le domaine ramene est le domaine reel de definition
(de :pramener-dom (schema nom-att type)
    (slet ((attrib (#:shirka:noyau:attrib schema nom-att)) 
           ; l'attribut est-il present?
           (dom (if attrib (#:shirka:noyau:val-fac attrib '$domaine))))
          ;******modification du 26/03/86  (or dom :supprime
          (:pcomb-dom dom           ; intersection avec les domaines
                  (:c-dom           ; possibles pour les peres intersectes
                          (mapcar   ; entre eux
                                  '(lambda (pere)
                                           (unless (getprop pere 'traite)
                                                   (putprop pere t 'traite)
                                                   (:pramener-dom 
                                                           (#:shirka:noyau:schema pere) 
                                                           nom-att type)
                                                   )  )                      
                                  (#:shirka:noyau:peres schema))
                          type)
                  type)))

;;; fonction de simplification de liste de domaines
(de :c-dom (l-domaine type)
    ; intersection d'une liste de domaine entre eux
    (if (atom l-domaine)
        l-domaine
        (:pcomb-dom (car l-domaine)
                (:c-dom (cdr l-domaine) type) type)))

;;; intersection de "domaine1" et "domaine2"
(de :pcomb-dom (domaine1 domaine2 type)
    (cond ((null domaine1) domaine2)
          ((null domaine2) domaine1)
          ((or (eq domaine1 'vide)(eq domaine2 'vide)) 'vide)
          (t (let ((inter-dom
                           (mapcan 
                                   '(lambda (x)
                                            (if (#:shirka:noyau:app-dom? x type domaine2)
                                                (list x)))
                                   domaine1)))
                  (ifn inter-dom 'vide inter-dom)))))

;;; union d'intervalles dont la liste est en parametre
(de :simpl-inter (l-inter type)
    (if (atom l-inter)        ; intervalle vide ou l'espace entier
        l-inter
        (:simpl-int (:tri-int l-inter type) type)))

;;; l-inter trie par min croissants
(de :simpl-int (l-inter type)
    (if (null l-inter) 'vide
        (let (( int1 (car l-inter) )
              (reste (:simpl-int (cdr l-inter) type)))  ; reste simplifie
             (cond ( (eq int1 'vide) reste )            ; rend le reste
                   ( (or (null reste)(null int1)) nil ) ; rend tout
                   ( (eq reste 'vide) (list int1)) 
                   ( t (tag resul
                            (let (( (min  max ) int1))
                                 ; on sait que min <= min1
                                 (while (nextl reste int2)
                                        ; tant que min1 compris entre min et max on prend le maximum de 
                                        ; max et max  
                                        (let (((mi ma) int2))
                                             (cond ((:super mi max type)(exit resul
                                                                              (cons (list min max)
                                                                                    (cons int2 reste))))
                                                   ((:super ma max type) (setq max ma)))))
                                 ; sinon on ne fait rien
                                 (exit resul (list (list min max))))))))))

(de :tri-int (l-inter type)
    ; classement par min croissant
    (let ((liste (mapcan '(lambda (int)
                                  (if (eq int 'vide) nil (list int)))
                         l-inter)))
         (tag fin-tri
              (selectq type
                      (entier (exit fin-tri (sort ':inf-min-ent liste)))
                      (reel   (exit fin-tri (sort ':inf-min-reel liste)))
                      (chaine (exit fin-tri (sort ':inf-min-chaine liste))))
              liste)))

;;; predicat de classement d'intervalle par min croissant
(de :inf-min-ent (int1 int2)
    (:super (car int2)(car int1) 'entier))

(de :inf-min-reel (int1 int2)
    (:super (car int2)(car int1) 'reel))

(de :inf-min-chaine (int1 int2)
    ( alphalessp (car int1) (car int2)))

;;; relation d'ordre "superieur strict" pour tous les types
(de :super (x y type)     ; x > y
    (selectq type
            (entier 
                    (cond ((or (eq x '-inf)(eq y '+inf)) nil)
                          ((or (eq x '+inf)(eq y '-inf)) t)
                          (t (gt x y))))
            (reel
                  (cond ((or (eq x '-inf)(eq y '+inf)) nil)
                        ((or (eq x '+inf)(eq y '-inf)) t)
                        (t (> x y))))
            (chaine (not (alphalessp x y)))
            (booleen (and (eq x 'vrai)(eq y 'faux)))))

;;; vrai si "valeur" de type simple appartient 
;;; a l'un des intervalles de la liste "l-inter"
(de :app-inter? (valeur type l-inter)
    (if l-inter
        (tag app
             (mapc 
                   '(lambda ((min max))
                            (if (or  (equal valeur min)
                                    (equal valeur max)
                                    (and (:super max valeur type)
                                         (:super valeur min type)))
                                (exit app t))) 
                   l-inter))
        t))

;;; domaine general de validite (intervalle et domaine) de l'attribut
;;; "nom-att" pour "schema"
(de :union-int-dom (schema nom-inst nom-att bguide)
    (slet ((chef (:chef-fam schema))
           (l-int)(dom-g)(typ)(nom-sch)
           (l (list chef)))
          (while (nextl l nom-sch)
                 (slet ((schem (#:shirka:noyau:schema nom-sch))
                        (attrib (#:shirka:noyau:attrib schem nom-att)) )
                       (if (and attrib                        
                                ;definition pour schem
                                (or (not bguide)
                                    (not (member nom-inst 
                                                 (getprop nom-sch 'impossible)))))
                           (slet ((type (vref attrib %type))
                                  (int (:pintervalle schem nom-att type))
                                  (dom (:pdomaine schem nom-att type)))
                                 (setq typ type)
                                 (if int (setq l-int (append l-int int)))
                                 (if dom (setq dom-g (#:shirka:noyau:union dom-g dom)))))
                       (setq l (append l (:filles schem)))))
          (cons (:simpl-inter l-int typ) dom-g)))

;;; verifie les predicats attaches a l'attribut lui-meme
;;; cette fonction est appelee en fin de verification s'il y a l'attribut
;;; lui-meme
(de :verif-lui-meme (schema instance)
    (let ((nom-sch (#:shirka:noyau:nom-sch schema))
          (nom-inst (#:shirka:noyau:nom-sch instance))
          (nom-filtres (#:shirka:noyau:val-facette schema 'lui-meme '$a-verifier))
          (resultat 'sur))
         (#:shirka:moteur:lier nom-sch nom-inst)
         (tag echec 
              (mapc '(lambda (filt)
                             (slet ((filt (#:shirka:noyau:schema filt))
                                    (inst-filt (:instancier filt)) 
                                    (fct (#:shirka:noyau:val-fac-he filt 'nom-fct '$valeur)))
                                   (ifn inst-filt        ; toutes les valeurs ne sont pas connues
                                        (setq resultat 'possible)
                                        (ifn (funcall fct inst-filt)
                                             (exit echec 'impossible)))))
                    nom-filtres) resultat)))

;;; cree une instance de schema a partir de sa liste
;;; d'attributs instantiables en utilisant le moteur pour
;;; calculer la valeur de chaque attribut
;;; Cette fonction sert a instancier tous les schema methodes
;;; qui correspondent a des facette filtres sauf $sib-exec
;;; La differentiation est faite a cause de l'existence
;;; d'un attribut "resultat" dans tous les schemas methodes
;;; propres a la facette $sib-exec
(de :instancier (schema)
    (lets ((l-att-inst (getprop 
                               (#:shirka:noyau:nom-sch schema) 
                               'ordre-inst-att))
           (nom-inst (#:shirka:moteur:filtrage:creer-inst-vide 
                             (#:shirka:noyau:nom-sch schema)
                             (length l-att-inst)))
           (instance (#:shirka:noyau:schema nom-inst)))
          (tag echec-att-proc
               (mapc 
                     '(lambda (nom-att)
                              (let ((val-att (#:shirka:moteur:valeur? nom-att nom-inst))
                                    (pos-att (#:shirka:noyau:pos-nom-att schema nom-att)))
                                   (if val-att
                                       (vset instance pos-att val-att)
                                       (exit echec-att-proc))))
                     l-att-inst)
               (getprop nom-inst 'schema))))

;;; obtention de la valeur de l'attribut nom-att de l'instance nom-inst
;;; apres classification ou specialisation
(de :v? (nom-inst nom-att)
    (slet ((inst-bid (#:shirka:noyau:schema (getprop nom-inst 'inst-bid)))
           (classe-bid (#:shirka:noyau:schema (vref inst-bid %est-un)))
           (pos-att (#:shirka:noyau:pos-nom-att classe-bid nom-att)))
          (vref inst-bid pos-att)))

;;; impression de la valeur de tous les attributs
(de :vi-c (nom-inst)
    (slet ((inst-bid (#:shirka:noyau:schema (getprop nom-inst 'inst-bid)))
           (classe-bid (#:shirka:noyau:schema (vref inst-bid %est-un)))
           (list-att (#:shirka:noyau:l-nom-att-inst-he classe-bid)))
          (mapc '(lambda (nom-att)
                         (let ((pos-att (#:shirka:noyau:pos-nom-att classe-bid nom-att)))
                              (print nom-att " = " 
                                     (let ((val (vref inst-bid pos-att)))
                                          (if val val "-")) " ;")))
                (cdddr list-att))))

;;; ajout de valeur dans l'attribut nom-att de l'instance nom-inst
;;; apres classification ou specialisation
(de :a-v (nom-inst nom-att valeur)
    (slet ((inst-bid (#:shirka:noyau:schema (getprop nom-inst 'inst-bid)))
           (nom-classe (vref (#:shirka:noyau:schema nom-inst) %est-un))
           (nom-inst-bid (#:shirka:noyau:nom-sch inst-bid)))
          (remprop nom-inst-bid 'a-repondre)
          (remprop nom-inst-bid 'deja-repondu)
          (:sup-prop (:chef-fam nom-classe) (list nom-inst-bid))
          (#:shirka:moteur:aj-valeur inst-bid nom-att valeur t)))

;;; modification de valeur dans l'attribut nom-att de l'instance nom-inst
;;; apres classification ou specialisation
(de :m-v (nom-inst nom-att valeur nouv-valeur)
    (slet ((inst-bid (#:shirka:noyau:schema (getprop nom-inst 'inst-bid)))
           (nom-classe (vref (#:shirka:noyau:schema nom-inst) %est-un))
           (nom-inst-bid (#:shirka:noyau:nom-sch inst-bid)))
          (remprop nom-inst-bid 'a-repondre)
          (remprop nom-inst-bid 'deja-repondu)
          (:sup-prop (:chef-fam nom-classe) (list nom-inst-bid))
          (#:shirka:moteur:mod-valeur inst-bid nom-att nouv-valeur valeur)))

;;; suppression de valeur dans l'attribut nom-att de l'instance nom-inst
;;; apres classification ou specialisation
(de :s-v (nom-inst nom-att valeur)
    (slet ((inst-bid (#:shirka:noyau:schema (getprop nom-inst 'inst-bid)))
           (nom-classe (vref (#:shirka:noyau:schema nom-inst) %est-un))
           (nom-inst-bid (#:shirka:noyau:nom-sch inst-bid)))
          (remprop nom-inst-bid 'a-repondre)
          (remprop nom-inst-bid 'deja-repondu)
          (:sup-prop (:chef-fam nom-classe) (list nom-inst-bid))
          (#:shirka:moteur:sup-valeur inst-bid nom-att valeur)))
