;;; Repertoire: ~LeShirka/LeIvan/CodeIace/
;;;
;     +----------------------------------------+-----------------------------------------+
;     | fichier :  fc-classif.ll               | package : #:iace:fc-classif             |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface d'Aide a la Classification et a l'Explication              |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Francois Cruypenninck    1991-1992                        |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     | descriptif du package :                                                          |
;     |                                                                                  |
;     | La fonction principale :moteur de classification shirka modifie pour le rendre   |
;     | compatible avec l'interface IACE .                                               |
;     | Interactivite entre les deux modules pour que l'arbre des classes se modifie en  |
;     | meme temps que la classification de l'instance .                                 |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+

;;;***************************************************************************************
;;; personnalisation du package
;;;***************************************************************************************

(defvar #:sys-package:colon '#:iace:fc-classif)

;;;***************************************************************************************
;;; action sur le bouton lancer une classification

(de :action-bouts-lancer (b)
    (ifn %mode (#:iace:fc-expli:action-bouts-mode b))
    (ifn (getprop (#:ivan:ed-util:symbole (send 'get-line (component 'edl-instance b))) 'schema)
         (#:iace:fc-interf:action-sa-warn b nil t (list (row (font #:ivan:in-global:fstd #M#:txt-const:schema)
                                                             (font #:ivan:in-global:fstdb
                                                                   (send 'get-line (component 'edl-instance b))
                                                                   )
                                                             (font #:ivan:in-global:fstd #M#:txt-const:indef)
                                                             )
                                                        )
                 )
         (#:iace:fc-interf:inhiber-com b)
         (setq select-instance selection-instance)
         (setq selection-instance nil)
         (lets ((noeud (send 'get-current (component 'ed-arbre b)))
                (instance (if (equal (send 'nom-inst noeud) "")
                              (#:iace:fc-util:schema
                                      (#:ivan:ed-util:symbole (send 'get-line (component 'edl-instance b)))
                                      )
                              (#:iace:fc-util:schema (#:ivan:ed-util:symbole (send 'nom-inst noeud)))
                              )
                        )
                (bouton b)
                (nom (send 'get-current (component 'ed-arbre b)))
                (nom-sch-tete (#:ivan:ed-util:symbole (if nom (string (send 'nom nom)) "")))
                (pour-types (if (send 'checkedp (component 'case-pour-type b)) (:action-pour-types b) nil))
                (sauf-types (if (send 'checkedp (component 'case-sauf-type b)) (:action-sauf-types b) nil))
                (bguide t)
                (binf (if (send 'checkedp (component 'case-inference b)) t nil))
                (bdial t)
                (bresult (tag arret-classif
                              (:class-inst bouton instance nom-sch-tete pour-types sauf-types bguide binf bdial)
                              )
                        )
                )
               (if (equal bresult "Familles incoherentes")
                   (#:iace:fc-interf:action-sa-warn
                           b nil t (list (row (font #:ivan:in-global:fstd #M#:txt-const:icoh-fam)))
                           )
                   (send 'set-current (component 'ed-arbre b) nil)
                   (ifn %arret ; teste si la classification de l'instance est complete sinon ...
                        (#:iace:fc-interf:action-sa-warn
                                b nil nil (list (row (font #:ivan:in-global:fstd #M#:txt-const:voila1)
                                                     (font #:ivan:in-global:fstdb
                                                           (send 'get-line (component 'edl-instance b))
                                                           )
                                                     (font #:ivan:in-global:fstd #M#:txt-const:voila2)
                                                     )
                                                )
                                )
                        (setq %arret ())
                        (#:iace:fc-interf:action-sa-warn
                                b nil nil (list (row (font #:ivan:in-global:fstd #M#:txt-const:stop)
                                                     (font #:ivan:in-global:fstdb
                                                           (send 'get-line (component 'edl-instance b))
                                                           )
                                                     )
                                                )
                                )
                        )
                   (setq selection-instance select-instance)
                   (#:iace:fc-interf:autoriser-com b)
                   (setq %valider t)
                   (authorize-application (component 'bouts-valider b))
                   (authorize-application (component 'bouts-modifier b))
                   (authorize-application (component 'bouts-reinit b))
                   )
               )
         )
    )

(de :action-pour-types (bouton)
    (let ((list-cl-attr (:list-attr bouton)))
         (if list-cl-attr
             (mapcar '#:ivan:ed-util:symbole 
                     (ask-strings (column (font #:ivan:in-global:fstdb  #M#:txt-const:choix-cl-att)
                                          (font #:ivan:in-global:fmicro #M#:txt-const:faire)
                                          )
                             (mapcar 'string list-cl-attr)
                             (mapcar 'string list-cl-attr)
                             1
                             (length list-cl-attr)
                             )
                     )
             )
         )
    )

(de :action-sauf-types (bouton)
    (let ((list-cl-attr (:list-attr bouton)))
         (if list-cl-attr
             (mapcar '#:ivan:ed-util:symbole 
                     (ask-strings (column (font #:ivan:in-global:fstdb #M#:txt-const:choix-cl-att-non)
                                          (font #:ivan:in-global:fmicro #M#:txt-const:faire)
                                          )
                             (mapcar 'string list-cl-attr)
                             nil
                             1
                             (length list-cl-attr)
                             )
                     )
             )
         )
    )

;;;***************************************************************************************
;;; action du bouton [reinit] dans le cas d'objets complexes

(de :action-objets-complexes (b sans-control)
    ; dans le cas d'objets complexes sur un seul concept, on transforme l'atome list-concept en une liste
    (if (atom list-concept)
        (progn (setq list-concept (list list-concept))
               (setq list-instance (getprop 'liste-inst 'classif))
               )
        )
    (lets ((concept ({fc-interf}:ch-list-concepts (component 'top b)))
           (longueur (length concept))
           (profondeur (length (member (car list-concept) concept)))
           (pos (- longueur profondeur))
           (list-init (if sans-control
                          list-instance
                          (ask-strings (column (font #:ivan:in-global:fstdb #M#:txt-const:init)
                                               (font #:ivan:in-global:fmicro #M#:txt-const:faire)
                                               )
                                  (mapcar 'string list-instance)
                                  (mapcar 'string list-instance)
                                  1
                                  (length list-instance)
                                  )
                          )
                   )
           )
          (ifn list-init
               (progn (authorize-application (component 'bouts-valider b))
                      (authorize-application (component 'bouts-modifier b))
                      (authorize-application (component 'bouts-reinit b))
                      (#:iace:fc-interf:autoriser-com b)
                      )
               (#:iace:fc-interf:action-sa-warn b nil nil
                       (list (row (font #:ivan:in-global:fstd #M#:txt-const:re-init-insts)
                                  (font #:ivan:in-global:fstd " ...")
                                  )
                             )
                       )
               (if (eq (length list-instance) (length list-init))
                   (progn (setq list-instance ())
                          (#:iace:fc-interf:action-menu-concept (component 'top b) pos nil)
                          )
                   (let ((list-inst-sup (#:ivan:ed-util:diff-liste (mapcar '#:ivan:ed-util:symbole list-instance)
                                                (mapcar '#:ivan:ed-util:symbole list-init)
                                                )
                                 )
                         )
                        (mapc '(lambda (instance)
                                       (lets ((classe (vref (#:iace:fc-util:schema instance) 0))
                                              (concept (:recherche-cl classe))
                                              )
                                             (if (equal (string instance) (send 'get-line (component 'edl-instance b)))
                                                 (progn (send 'set-current (component 'ed-arbre b)
                                                              (getprop concept 'noeud-iace)
                                                              )
                                                        (send 'clear (component 'ed-arbre b))
                                                        )
                                                 (send 'set-current (component 'ed-arbre b)
                                                       (getprop concept instance)
                                                       )
                                                 (send 'clear (component 'ed-arbre b))
                                                 )
                                             )
                                       )
                              list-inst-sup
                              )
                        )
                   )
               (setq %liste ())
               (setq %liste-expli ())
               (#:iace:fc-interf:reinit-sm (component 'scroll-attribut b) (component 'sm-attribut b) ())
               ;hy------------------------
               ; reinit editeur hypertexte
               ;               (#:iace:fc-interf:reinit-sm (component 'scroll-explication b) (component 'sm-explication b) ())
               (#:iace:fc-interf:reinit-sm-hy (component 'scroll-explication b) (component 'sm-explication b) ())
               ;--------------------------
               (#:iace:fc-util:fin-cl (mapcar '#:ivan:ed-util:symbole list-init))
               (send 'set-current (component 'ed-arbre b)
                     (#:ivan:ed-util:symbole (send 'get-line (component 'edl-instance b)))
                     )
               (send 'mapnodes (component 'ed-arbre b) (lambda (node)
                                                               (send 'img node "interro")
                                                               (send 'set-image (send 'editordata node)
                                                                     (send 'title node)
                                                                     )
                                                               )
                     )
               (send 'set-line (component 'edl-instance b) "")
               (setq selection-instance nil)
               (inhibit-application (component 'bouts-reinit b))
               (setq %valider ())
               (if (and selection-classe selection-instance)
                   (when t (authorize-application (component 'bouts-lancer b))
                         (authorize-application (component 'bouts-reinit b))
                         )
                   (inhibit-application (component 'bouts-lancer b))
                   (inhibit-application (component 'bouts-reinit b))
                   )
               (#:iace:fc-interf:action-sa-warn
                       b nil nil (list (row (font #:ivan:in-global:fstd #M#:txt-const:fait)))
                       )
               (#:iace:fc-interf:autoriser-com b)
               )
          ) 
    )

;;;***************************************************************************************
;;; recherche du concept connaissant la classe

(de :recherche-cl (classe)
    (let ((pere (#:iace:fc-util:peres (#:iace:fc-util:schema classe))))
         (if (eq (car pere) 'objet)
             classe
             (:recherche-cl (car pere))
             )
         )
    )

;;;***************************************************************************************
;;; action sur le bouton reinit l'instance

(de :action-bout-reinit (b)
    (:action-bouts-reinit b ())
    (setq %cl-modif ())
    )

(de :action-bouts-reinit (b sans-control)
    (#:iace:fc-interf:inhiber-com b)
    (ifn %mode (#:iace:fc-expli:action-bouts-mode b))
    (if (> (length (getprop 'liste-inst 'classif)) 1)
        ;(if (atom list-instance)
        (:action-objets-complexes b sans-control) ; je dois reinitialiser des objets complexes
        (lets ((concept ({fc-interf}:ch-list-concepts (component 'top b)))
               (longueur (length concept))
               (profondeur (length (member list-concept concept)))
               (pos (- longueur profondeur))
               )
              (#:iace:fc-interf:action-sa-warn b nil nil
                      (list (row (font #:ivan:in-global:fstd #M#:txt-const:re-init-inst)
                                 (font #:ivan:in-global:fstdb (send 'get-line (component 'edl-instance b)))
                                 (font #:ivan:in-global:fstd " ...")
                                 )
                            )
                      )
              (#:iace:fc-util:fin-cl (getprop 'liste-inst 'classif))
              ;(#:iace:fc-util:fin-cl (list (#:ivan:ed-util:symbole (send 'get-line (component 'edl-instance b)))))
              (setq list-instance ())
              (#:iace:fc-interf:action-menu-concept (component 'top b) pos nil)
              (setq %liste ())
              (setq %liste-expli ())
              (#:iace:fc-interf:reinit-sm (component 'scroll-attribut b) (component 'sm-attribut b) ())
              ;hy------------------------
              ; reinit editeur hypertexte
              ;               (#:iace:fc-interf:reinit-sm (component 'scroll-explication b) (component 'sm-explication b) ())
              (#:iace:fc-interf:reinit-sm-hy (component 'scroll-explication b) (component 'sm-explication b) ())
              ;--------------------------
              (send 'set-current (component 'ed-arbre b)
                    (#:ivan:ed-util:symbole (send 'get-line (component 'edl-instance b)))
                    )
              (send 'mapnodes (component 'ed-arbre b) (lambda (node)
                                                              (send 'img node "interro")
                                                              (send 'set-image (send 'editordata node)
                                                                    (send 'title node)
                                                                    )
                                                              )
                    )
              (send 'set-line (component 'edl-instance b) "")
              (setq selection-instance nil)
              (inhibit-application (component 'bouts-reinit b))
              (setq %valider ())
              (if (and selection-classe selection-instance)
                  (when t (authorize-application (component 'bouts-lancer b))
                        (authorize-application (component 'bouts-reinit b))
                        )
                  (inhibit-application (component 'bouts-lancer b))
                  (inhibit-application (component 'bouts-reinit b))
                  )
              (#:iace:fc-interf:autoriser-com b)
              (#:iace:fc-interf:action-sa-warn b nil nil
                      (list (row (font #:ivan:in-global:fstd #M#:txt-const:fait)))
                      )
              )
        )
    )

;;;***************************************************************************************
;;; action sur le bouton ? quand on ne connait pas le nom de l'instance

(de :action-bouts-? (b)
    (ifn (send 'get-current (component 'ed-arbre b))
         (when t (#:iace:fc-interf:action-sa-warn b nil t (list (font #:ivan:in-global:fstd
									#M#:txt-const:no-cl-sel)
                                                                )
                         )
               (setq selection-classe nil)
               (setq selection-instance nil)
               (send 'set-line (component 'edl-instance b) "")
               )
         (#:iace:fc-interf:inhiber-com b)
         (lets ((classe (send 'nom (send 'get-current (component 'ed-arbre b))))
                (liste (#:iace:fc-util:l-inst-t (#:iace:fc-util:schema (#:ivan:ed-util:symbole classe))))
                (classe-sel-sym (#:ivan:ed-util:symbole classe))
                (nom-pere (#:iace:fc-util:peres (#:iace:fc-util:schema classe-sel-sym)))
                (classe-orig (getprop classe-sel-sym 'noeud-iace))
                (choix (if liste 
			   (ask-string #M#:txt-const:choix-inst (#:iace:fc-interf:list-alpha liste) nil)
                           (#:iace:fc-interf:action-sa-warn
                                   b nil t (list (row (font #:ivan:in-global:fstd #M#:txt-const:no-inst)
                                                      (font #:ivan:in-global:fstdb (string classe))
                                                      )
                                                 )
                                   )
                           nil
                           )
                       )
                )
               (if (or (not choix) (equal choix ""))
                   (progn (setq selection-instance nil)
                          (send 'set-line (component 'edl-instance b) "")
                          )
                   (send 'set-line (component 'edl-instance b) choix)
                   (#:iace:fc-interf:action-edl-instance (component 'edl-instance b))
                   (setq selection-instance t)
                   (setq selection-classe t)       
                   (:action-sur-arbre b 'sur classe choix)
                   (:classe-sur nom-pere b choix)
                   (send 'set-current (component 'ed-arbre b) classe-orig)
                   (#:iace:fc-interf:action-bouts-repos b)
                   )
               (#:iace:fc-interf:autoriser-com b)
               )
         )
    )

;;;***************************************************************************************
;;; passe toutes les surclasses de la classe courante en status 'sur

(de :classe-sur (peres b instance)
    (ifn (cdr peres)
         (ifn (equal peres '(objet))
              (progn (setq classe-sel-sym (#:ivan:ed-util:symbole (car peres)))
                     (:action-sur-arbre b 'sur classe-sel-sym instance)
                     (setq peres (#:iace:fc-util:peres (#:iace:fc-util:schema classe-sel-sym)))
                     (:classe-sur peres b instance)
                     )
              )
         (:classe-sur (list (car peres)) b instance)
         (:classe-sur (cdr peres) b instance)
         )
    )

;;;***************************************************************************************
;;; action sur le bouton valider quand on veut valider le classement de ou des instances

(de :action-bouts-valider (b)
    (ifn (send 'get-current (component 'ed-arbre b))
         (#:iace:fc-interf:action-sa-warn
                 b nil t (list (font #:ivan:in-global:fstd #M#:txt-const:no-cl-sel))
                 )
         (let ((instance-orig (#:ivan:ed-util:symbole (send 'get-line (component 'edl-instance b))))
               (classe-dest-orig (#:ivan:ed-util:symbole (send 'nom (send 'get-current (component 'ed-arbre b)))))
               )
              (ifn (:ratt-inst (#:iace:fc-util:schema instance-orig) classe-dest-orig b)
                   (#:iace:fc-interf:action-sa-warn
                           b nil t (list (row (font #:ivan:in-global:fstd #M#:txt-const:ratt-imp1)
                                              (font #:ivan:in-global:fstdb (string instance-orig))
                                              (font #:ivan:in-global:fstd #M#:txt-const:ratt-imp2)
                                              (font #:ivan:in-global:fstdb (string classe-dest-orig))
                                              (font #:ivan:in-global:fstd " ...")
                                              )
                                         )
                           )
                   (ifn (atom list-concept)
                        (while (confirm (column (row (font #:ivan:in-global:fstd #M#:txt-const:l-inst)
                                                     (font #:ivan:in-global:fstdb (string instance-orig))
                                                     (font #:ivan:in-global:fstd #M#:txt-const:est-att)
                                                     (font #:ivan:in-global:fstd #M#:txt-const:ratt-imp2)
                                                     (font #:ivan:in-global:fstdb (string classe-dest-orig))
                                                     )
                                                (font #:ivan:in-global:fstd #M#:txt-const:cpt-prin)
                                                " "
                                                (font #:ivan:in-global:fstd #M#:txt-const:sel-compl-cl1)
                                                (row (font #:ivan:in-global:fstd #M#:txt-const:sel-compl-cl2)
                                                     (font #:ivan:in-global:fstdb #M#:txt-const:d-accord)
                                                     )
                                                " "
                                                (row (font #:ivan:in-global:fstd #M#:txt-const:sinon)
                                                     (font #:ivan:in-global:fstdb #M#:txt-const:annuler)
                                                     )
                                                )
                                       )
                               (let ((instance (#:ivan:ed-util:symbole
                                                       (send 'nom-inst (send 'get-current (component 'ed-arbre b)))
                                                       )
                                             )
                                     (classe-dest (#:ivan:ed-util:symbole
                                                          (send 'nom (send 'get-current (component 'ed-arbre b)))
                                                          )
                                             )
                                     )
                                    (if instance
                                        (if (:ratt-inst (#:iace:fc-util:schema instance) classe-dest b)
                                            (#:iace:fc-interf:action-sa-warn
                                                    b nil nil (list (row (font #:ivan:in-global:fstd 
										#M#:txt-const:l-inst)
                                                                         (font #:ivan:in-global:fstdb (string instance))
                                                                         (font #:ivan:in-global:fstd
										(catenate #M#:txt-const:est-att
											  #M#:txt-const:ratt-imp2))
                                                                         (font #:ivan:in-global:fstdb (string classe-dest))
                                                                         )
                                                                    )
                                                    )
                                            (#:iace:fc-interf:action-sa-warn
                                                    b nil t (list (row (font #:ivan:in-global:fstd
										#M#:txt-const:ratt-imp1)
                                                                       (font #:ivan:in-global:fstdb (string instance))
                                                                       (font #:ivan:in-global:fstd
										#M#:txt-const:ratt-imp2)
                                                                       (font #:ivan:in-global:fstdb (string classe-dest))
                                                                       (font #:ivan:in-global:fstd " ...")
                                                                       )
                                                                  )
                                                    )
                                            )
                                        (#:iace:fc-interf:action-sa-warn
                                                b nil t (list (row (font #:ivan:in-global:fstd 
									#M#:txt-const:sel-imp1)
                                                                   (font #:ivan:in-global:fstdb (string classe-dest))
                                                                   (font #:ivan:in-global:fstd
									#M#:txt-const:sel-imp2)
                                                                   )
                                                              )
                                                )
                                        )
                                    )
                               )
                        )
                   (#:iace:fc-interf:action-sa-warn
                           b nil nil (list (row (font #:ivan:in-global:fstd #M#:txt-const:l-inst)
                                                (font #:ivan:in-global:fstdb (string instance-orig))
                                                (font #:ivan:in-global:fstd 
						     (catenate #M#:txt-const:est-att #M#:txt-const:ratt-imp2))
                                                (font #:ivan:in-global:fstdb (string classe-dest-orig))
                                                )
                                           )
                           )
                   (setq selection-classe nil)
                   (setq selection-instance nil)
                   (send 'set-line (component 'edl-instance b) "")
                   (:action-bout-reinit b)
                   )
              )
         )
    )

;;;***************************************************************************************
;;; rattacher l'instance temporaire a la classe destination en gerant le cas des listes

(de :ratt-inst (instance nom-sch-dest b)
    (slet ((prop (:spec-inst instance nom-sch-dest nil nil nil nil b))
           (nom-inst (#:iace:fc-util:nom-inst instance))
           (nom-inst-bid (getprop nom-inst 'inst-bid))
           )
          (if (not (memq prop '(sur possible)))
              (progn 
(when #:trace:cla
		(print "Rattachement refuse.")
)
		(#:iace:fc-util:expl-cl nom-inst nom-sch-dest)
		nil )
              (slet ((inst-bid (#:iace:fc-util:schema nom-inst-bid))
                     (nom-sch-bid (#:iace:fc-util:val-inst inst-bid %est-un))
                     (p (#:iace:fc-util:sup-instance (#:iace:fc-util:schema nom-inst)))
                     (inst (#:iace:fc-util:make-inst nom-sch-dest nom-inst))
                     )
                    (mapc '(lambda (nom-att)
                                   (lets ((pos-att (#:iace:fc-util:pos-nom-att (#:iace:fc-util:schema nom-sch-bid)
                                                           nom-att
                                                           )
                                                  )
                                          (list-valeur (#:iace:fc-util:val-inst inst-bid pos-att))
                                          )
                                         (if (> pos-att %nom-inst)
                                             (if (atom list-valeur) 
                                                 (#:iace:fc-util:aj-valeur inst nom-att
                                                         (#:iace:fc-util:val-inst inst-bid pos-att)
                                                         t
                                                         )
                                                 (while list-valeur
                                                        (let ((une-valeur (nextl list-valeur)))
                                                             (#:iace:fc-util:aj-valeur inst nom-att une-valeur t)
                                                             )
                                                        )
                                                 )
                                             )
                                         )
                                   )
                          (#:iace:fc-util:l-nom-att-he (#:iace:fc-util:schema nom-sch-dest))
                          )
                    (#:iace:fc-util:aj-valeur inst 'nom-inst nom-inst t)
                    (#:iace:fc-util:creer-est-un inst nom-sch-dest)
                    )
              nom-sch-dest
              )
          )
    )

;;;***************************************************************************************
;;; classification de l'instance <instance> dans la classe <nom-sch-tete>
;;; recuperation de "PP-classif1.ll"

(de :class-inst ( bouton instance nom-sch-tete pour-types sauf-types bguide binf bdial)
    (setq %pour-types pour-types
          %sauf-types sauf-types
          %binf binf
          %bdial bdial
          %bguide bguide
          )
    (tag erreur
         (slet ((nom-sch-orig (#:iace:fc-util:nom-sch (#:iace:fc-util:assoc-schema instance)))
                (chef-sch-inst (#:iace:fc-util:chef-fam nom-sch-orig))
                (nom-sch-tete (if (null nom-sch-tete)
                                  chef-sch-inst
                                  nom-sch-tete
                                  )
                        )
                (schema-orig (#:iace:fc-util:schema nom-sch-orig))
                (nom-inst (#:iace:fc-util:nom-inst instance))
                (nom-inst-bid (getprop nom-inst 'inst-bid))
                (nom-cl)
                (li-sur)
                (li-pos)
                (li-imp)
                (liatrait (list nom-sch-tete))
                )
               (when bdial
                     (#:iace:fc-interf:action-sa-warn
                             bouton
                             nil 
                             nil
                             (list (row (font #:ivan:in-global:fstd #M#:txt-const:cla-de)
                                        (font #:ivan:in-global:fstdb (send 'get-line (component 'edl-instance bouton)))
                                        (font #:ivan:in-global:fstd #M#:txt-const:a-part)
                                        (font #:ivan:in-global:fstdb (send 'get-line (component 'edl-classe bouton)))
                                        )
                                   )
                             )
                     )
               (if (or (null chef-sch-inst) (neq (#:iace:fc-util:chef-fam nom-sch-tete) chef-sch-inst))
                   (exit erreur 
(when #:trace:cla
			 (print "Familles incoherentes")
)
				"Familles incoherentes")
                   )
               (ifn nom-inst-bid
                    (setq nom-inst-bid (#:iace:fc-util:crebid-cl-inst instance))
                    )
               (#:iace:fc-util:marquer-sur-anc schema-orig nom-inst-bid)
               (while (nextl liatrait nom-cl)
                      (ifn (member nom-cl (append li-sur li-pos li-imp))
                           (lets ((classe-sel-sym (#:ivan:ed-util:symbole nom-cl))
                                  (noeud-classe (getprop classe-sel-sym 'noeud-iace))
                                  )
                                 (#:iace:fc-interf:reinit-browser bouton noeud-classe classe-sel-sym)
                                 (let ((trait (:specialiser-vers 
                                                      nom-inst-bid nom-cl pour-types sauf-types bguide binf bdial bouton
                                                      )
                                              )
                                       )
                                      (cond ((eq trait 'sur) (setq li-sur (nconc1 li-sur nom-cl)))
                                            ((eq trait 'possible) (setq li-pos (nconc1 li-pos nom-cl)))
                                            ( t (setq li-imp (nconc1 li-imp nom-cl)))
                                            )
                                      (unless (eq trait 'impossible)
                                              (mapc '(lambda (fille)
                                                             (ifn (member fille liatrait)
                                                                  (setq liatrait (nconc1 liatrait fille))
                                                                  )
                                                             )
                                                    (#:iace:fc-util:filles (#:iace:fc-util:schema nom-cl))
                                                    )
                                              )
                                      )
                                 )
                           )
                      )
               (list li-sur li-pos li-imp)
               )
         )
    )

;;;***************************************************************************************
;;; repositionnement du tableau par rapport a la classification

(de :action-sur-arbre (bouton trait classe instance)
    ; gere dynamiquement la largeur de la representation des objets a l'ecran quand on change de concept
    (ifn (member (#:ivan:ed-util:symbole classe) ({fc-interf}:ch-list-classes (component 'top bouton)))
         (let ((list-tempo ({fc-interf}:ch-list-classes (component 'top bouton))) 
               (pos (- (length ({fc-interf}:ch-list-concepts (component 'top bouton))) 1))
               (class (#:ivan:ed-util:symbole classe))
               )
              (while (and (>= pos 0)(not (member class list-tempo)))
                     (setq list-tempo (cadr (vref ({fc-interf}:ch-vect-classes (component 'top bouton)) pos)))
                     (setq pos (- pos 1))
                     )
              (if (member class list-tempo)
                  (let ((cur-font (current-font)))
                       (current-font #:ivan:in-global:fmicro)
                       (setq #:ivan:in-global:larg (#:ivan:ed-util:wc (#:ivan:ed-util:larg-max list-tempo)))
                       (current-font cur-font)
                       )
                  )
              (setq tempo t)
              )
         (setq tempo nil)
         )
    (lets ((classe-sel-sym (#:ivan:ed-util:symbole classe))
           (nom-pere (car (#:iace:fc-util:peres (#:iace:fc-util:schema classe-sel-sym))))
           (noeud-classe (if (equal (#:ivan:ed-util:symbole instance)
                                    (#:ivan:ed-util:symbole (send 'get-line (component 'edl-instance bouton)))
                                    )
                             (getprop classe-sel-sym 'noeud-iace)
                             (if (getprop classe-sel-sym instance) (getprop classe-sel-sym instance)
                                 (getprop classe-sel-sym 'noeud-iace)
                                 )
                             )
                   )
           (noeud-pere (if (equal nom-pere 'objet)
                           (getprop classe-sel-sym 'noeud-iace)
                           (getprop nom-pere 'noeud-iace)
                           )
                   )
           (traite (if (and (equal trait 'sur) (equal (send 'img noeud-pere) "possible"))
                       'possible
                       trait
                       )
                   )
           )
          (when classe-sel-sym
                (send 'set-current (component 'ed-arbre bouton) noeud-classe)
                (#:iace:fc-interf:action-bouts-repos bouton)
                (if (eq (#:ivan:ed-util:symbole (send 'get-line (component 'edl-concept bouton)))
                        (#:shirka:classif:chef-fam classe-sel-sym)
                        ) 
                    ; marque le nom de l'instance associee au noeud si l'on fait une specialisation local
                    (send 'nom-inst noeud-classe instance)
                    )
                (cond ((eq traite 'sur) (send 'img noeud-classe "sur"))
                      ((eq traite 'possible) (send 'img noeud-classe "possible"))
                      ((eq traite 'complexe) (send 'img noeud-classe "complexe"))
                      ((eq traite 'impossible)
                       (send 'mapnodes (component 'ed-arbre bouton) (lambda (node)
                                                                            (send 'img node "impossible")
                                                                            (send 'set-image 
                                                                                  (send 'editordata node)
                                                                                  (send 'title node)
                                                                                  )
                                                                            )
                             )
                       )
                      )
                (ifn (eq traite 'impossible) (send 'set-image (send 'editordata noeud-classe)
                                                   (send 'title noeud-classe)
                                                   )
                     )
                )
          (if tempo (let ((cur-font (current-font)))
                         (current-font #:ivan:in-global:fmicro)
                         (setq #:ivan:in-global:larg
                               (#:ivan:ed-util:wc (#:ivan:ed-util:larg-max
                                                          ({fc-interf}:ch-list-classes (component 'top bouton))
                                                          )
                                       )
                               )
                         (current-font cur-font)
                         )
              )
          )
    )

;;;***************************************************************************************
;;; specialisation de l'instance <nom-inst> dans la classe <nom-sch-dest>
;;; recup de "PP-classif1.ll"

(de :specialiser-vers (nom-inst nom-sch-dest pour-types sauf-types bguide binf bdial b)
    (lets ((nom-sch (#:iace:fc-util:schema nom-sch-dest))
           (l-att nil)
           (l-type-att
                   (if %cl-modif
                       (progn (setq valeur nil)
                              (mapc (lambda (modif)
                                            (let ((result
                                                          (member modif
                                                                  (#:iace:fc-interf:list-plate
                                                                          (mapcar (lambda (attrib)
                                                                                          (setq l-att
                                                                                                (cons l-att
                                                                                                      (cadr (#:iace:fc-util:nature
                                                                                                                    nom-sch
                                                                                                                    (#:iace:fc-util:nom-att attrib)
                                                                                                                    )
                                                                                                            )
                                                                                                      )
                                                                                                )
                                                                                          )
                                                                                  (#:iace:fc-util:l-att nom-sch)
                                                                                  )
                                                                          )
                                                                  )
                                                          )
                                                  )
                                                 (if result (setq valeur t))
                                                 )
                                            )
                                    %cl-modif
                                    )
                              valeur
                              )
                       nil
                       )
                   )
           ; s'il y a une modif, controler si la classe modifiee a un attribut complexe; si c'est le cas,
           ; si c'est le cas: retester le status de la classe <nom-sch-dest>
           (prop (ifn l-type-att
                      (#:iace:fc-util:prop-inst nom-inst nom-sch-dest)
                      (remprop nom-sch-dest 'sur)
                      (remprop nom-sch-dest 'possible)
                      (remprop nom-sch-dest 'impossible)
                      nil
                      )
                 )
           )
          ; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
          ; ++++++ il y a un probleme l'objet complexe n'est pas reteste meme si on va dessus. +++++++
          ; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
          (if prop
              prop
              (let ((peres (#:iace:fc-util:peres (#:iace:fc-util:schema nom-sch-dest))))
                   (let ((l (getprop 'liste-cl-vues 'classif)))
                        (setq l (append l (list nom-sch-dest)))
                        )
                   (while (and (nextl peres pere) (not (member nom-inst (getprop nom-sch-dest 'impossible)))
                               (not (#:iace:fc-util:chef pere))
                               )
                          (if (eq pere 'objet)
                              (#:iace:fc-util:trait-imposs nom-inst nom-sch-dest)
                              (:specialiser-vers nom-inst pere pour-types sauf-types bguide binf bdial b)
                              )
                          )
                   (if (member nom-inst (getprop nom-sch-dest 'impossible))
                       'impossible
                       (let ((valcase (:instpp-de? (#:iace:fc-util:schema nom-inst)
                                              (#:iace:fc-util:schema nom-sch-dest)
                                              pour-types sauf-types bguide binf bdial nom-inst b
                                              )
                                     )
                             )
                            (:action-sur-arbre b valcase nom-sch-dest (vref (#:iace:fc-util:schema nom-inst) 1))
                            (cond ((eq valcase 'possible) (#:iace:fc-util:trait-poss nom-inst nom-sch-dest))
                                  ((eq valcase 'impossible) (#:iace:fc-util:trait-imposs nom-inst nom-sch-dest))
                                  ( t (if (#:iace:fc-util:sur-peres? nom-sch-dest nom-inst)
                                          (#:iace:fc-util:trait-sur nom-inst nom-sch-dest)
                                          (#:iace:fc-util:trait-poss nom-inst nom-sch-dest)
                                          )
                                     )
                                  )
                            )
                       )
                   )
              )
          )
    )

;;;***************************************************************************************
;;; execute la fonction decrite dans PP-classif1.ll et permet de positionner le texte
;;; de la question dans l'editeur de iace

(de :instpp-de? (instance classe pour-types sauf-types bguide binf bdial nom-inst-bid b)
    (let ((nom-inst (#:iace:fc-util:nom-inst instance))
          (nom-sch (#:iace:fc-util:val-inst instance %est-un))
          (possibilite 'sur)
          (lui-meme)
          (l-nouv-att)
          )
         (tag echec
              (mapc '(lambda (attrib)
                             (slet ((nom-att (#:iace:fc-util:nom-att attrib))
                                    (type-att (#:iace:fc-util:classe-att-he classe nom-att))
                                    )
                                   (when (#:iace:fc-util:considerer-classe-att type-att pour-types sauf-types)
                                         (if (eq nom-att 'lui-meme)
                                             (setq lui-meme 'a-traiter)
                                             (slet ((pos-att (#:iace:fc-util:pos-nom-att (#:iace:fc-util:schema nom-sch)
                                                                     nom-att
                                                                     )
                                                            )
                                                    (nou-att (#:iace:fc-util:nouv-att? classe nom-att))
                                                    (att-a-trait (or nou-att
                                                                     (#:iace:fc-util:val-fac attrib '$domaine)
                                                                     (#:iace:fc-util:val-fac attrib '$valeur)
                                                                     (#:iace:fc-util:val-fac attrib '$sauf)
                                                                     (#:iace:fc-util:val-fac attrib '$intervalle)
                                                                     (#:iace:fc-util:val-fac attrib '$a-verifier)
                                                                     (#:iace:fc-util:val-fac attrib '$un)
                                                                     (#:iace:fc-util:val-fac attrib '$liste-de)
                                                                     (#:iace:fc-util:val-fac attrib '$cardmin)
                                                                     (#:iace:fc-util:val-fac attrib '$cardmax)
                                                                     )
                                                            )
                                                    )
                                                   (cond ((and pos-att att-a-trait)
                                                          (slet ((instance (#:iace:fc-util:schema nom-inst))
                                                                 (valeur (#:iace:fc-util:val-inst instance pos-att))
                                                                 (resul (when valeur
                                                                              (slet ((att (#:iace:fc-util:attrib
                                                                                                  classe
                                                                                                  nom-att
                                                                                                  )
                                                                                          )
                                                                                     (type (#:iace:fc-util:val-inst
                                                                                                   att
                                                                                                   %type
                                                                                                   )
                                                                                           )
                                                                                     (i (#:iace:fc-util:pintervalle
                                                                                                classe
                                                                                                nom-att
                                                                                                type
                                                                                                )
                                                                                        )
                                                                                     (d (#:iace:fc-util:pdomaine
                                                                                                classe
                                                                                                nom-att
                                                                                                type
                                                                                                )
                                                                                        )
                                                                                     (dom (#:iace:fc-util:comb-i-d
                                                                                                  i d 
                                                                                                  type
                                                                                                  )
                                                                                          )
                                                                                     )
                                                                                    (:verif-val
                                                                                            nom-inst nom-sch
                                                                                            nom-att valeur     
                                                                                            classe dom b
                                                                                            )
                                                                                    )
                                                                              )
                                                                        )
                                                                 (testval (car resul))
                                                                 (val (cdr resul))
                                                                 )
                                                                (cond ((null resul)
                                                                       (setq l-nouv-att (nconc1 l-nouv-att nom-att))
                                                                       )
                                                                      ((eq testval 'impossible)
                                                                       (exit echec 'impossible)
                                                                       )
                                                                      ((eq testval 'possible)
                                                                       (setq possibilite 'possible)
                                                                       )
                                                                      )
                                                                )
                                                          )
                                                         ((and (not pos-att) att-a-trait)
                                                          (setq l-nouv-att (nconc1 l-nouv-att nom-att))
                                                          )
                                                         )  
                                                   )
                                             )
                                         )
                                   )
                             )
                    (#:iace:fc-util:l-att classe)
                    )
              (mapc '(lambda (nom-att)
                             (slet ((posit (#:iace:fc-util:pos-nom-att (#:iace:fc-util:schema nom-sch) nom-att))
                                    (pos-att (if posit posit (#:iace:fc-util:allonge nom-inst nom-sch
                                                                     nom-att classe
                                                                     )
                                                 )
                                            )
                                    (instance (#:iace:fc-util:schema nom-inst))
                                    (resul (:obtien-val nom-inst nom-sch nom-att
                                                   classe binf bdial nom-inst-bid b
                                                   )
                                           )
                                    (testval (car resul))
                                    (val (cdr resul))
                                    )
                                   (cond ((eq testval 'impossible)(exit echec 'impossible))
                                         ((eq testval 'possible)(setq possibilite 'possible))
                                         )
                                   )
                             )
                    l-nouv-att
                    )
              (let ((instance (#:iace:fc-util:schema nom-inst)))
                   (if lui-meme
                       (let ((verif-lui (#:iace:fc-util:verif-lui-meme classe instance)))
                            (cond ((eq verif-lui 'impossible) 
                                   (progn (setq possibilite 'impossible)
                                          (#:iace:fc-util:ecr-expl nom-inst 
                                                  (#:iace:fc-util:nom-sch classe)
                                                  'lui-meme nil 'impossible nil
                                                  )
                                          )
                                   )
                                  ((eq verif-lui 'possible)
                                   (progn (setq possibilite 'possible)
                                          (#:iace:fc-util:ecr-expl nom-inst
                                                  (#:iace:fc-util:nom-sch classe)
                                                  'lui-meme nil 'possible nil
                                                  )
                                          )
                                   )
                                  )
                            )
                       )
                   )
              possibilite
              )
         )
    )

;;;***************************************************************************************
;;; verification de l'admissibilite de(s) valeur(s) de l'attribut <nom-att> de l'instance
;;; <nom-inst> dans <classe>

(de :verif-val (nom-inst nom-sch nom-att valeur classe dom b)
    (slet ((val (:admissible? valeur nom-att classe (#:iace:fc-util:schema nom-inst) b))
           (testval 'sur)
           )
          (if (and (not val) %%EXPL-NEG-TYPE)
              (progn (setq testval 'impossible)
                     (#:iace:fc-util:ecr-expl nom-inst (#:iace:fc-util:nom-sch classe)
                             nom-att valeur 'impossible
                             (list 't %%EXPL-NEG-TYPE)
                             )
                     )
              (if (and (not val) %%EXPL-NEG-PRED)
                  (progn (setq testval 'impossible)
                         (#:iace:fc-util:ecr-expl nom-inst (#:iace:fc-util:nom-sch classe)
                                 nom-att valeur 'impossible
                                 (list 'p %%EXPL-NEG-PRED)
                                 )
                         )
                  (ifn val (progn (setq testval 'impossible)
                                  (#:iace:fc-util:ecr-expl nom-inst (#:iace:fc-util:nom-sch classe) 
                                          nom-att valeur 'impossible dom
                                          )
                                  )
                       )
                  )
              )
          (cons testval val)
          )
    )

(de :admissible? (valeur nom-att schema instance b)
    (if (atom valeur) (if (:test-admissible valeur nom-att schema instance b) valeur)
        (if (every '(lambda (val)
                            (:test-admissible val nom-att schema instance b)
                            )
                   valeur
                   )
            valeur
            )
        )
    )

(de :test-admissible (valeur nom-att schema instance b)
    (setq %%EXPL-NEG-TYPE nil)
    (setq %%EXPL-NEG-PRED nil)
    (tag verif
         (let (((nature type) (#:iace:fc-util:nature schema nom-att))
               (nom-inst (#:iace:fc-util:nom-sch instance))
               (nom-sch  (#:iace:fc-util:nom-sch schema))
               (attrib (#:iace:fc-util:attrib-he schema nom-att))
               )
              (if (selectq type (objet-simple (or (symbolp valeur)(numberp valeur)(stringp valeur)))               
                          (symbole (symbolp valeur))
                          (nombre (numberp valeur))
                          (entier (fixp valeur))
                          (reel (floatp valeur))
                          (chaine (stringp valeur))
                          (booleen (member valeur '(vrai faux oui non)))
                          (objet t)
                          ( t ;type complexe
                             (:action-sur-arbre b 'complexe nom-sch (vref instance 1))
                             (if (or (symbolp valeur)(vectorp valeur))                         
                                 (let (((complexe pos) (:test-complexe type b))
                                       (instance (if (vectorp valeur)
                                                     valeur
                                                     (let ((sch (tag erreur (#:iace:fc-util:schema valeur))))
                                                          (if sch
                                                              sch
                                                              (let ((occurrences (assq valeur %liste-reprise)))
                                                                   (ifn occurrences
                                                                        (setq %liste-reprise 
                                                                              (acons valeur
                                                                                     (list (list nom-sch
                                                                                                 nom-inst
                                                                                                 nom-att
                                                                                                 )
                                                                                           )
                                                                                     %liste-reprise
                                                                                     )
                                                                              )
                                                                        (rplacd occurrences
                                                                                (cons (list nom-sch
                                                                                            nom-inst
                                                                                            nom-att
                                                                                            )
                                                                                      (cdr occurrences)
                                                                                      )
                                                                                )
                                                                        )
                                                                   )
                                                              (putprop nom-inst
                                                                      (add1 (getprop nom-inst 'nb-indet))
                                                                      'nb-indet
                                                                      )
(when #:trace:cla
                                                              (prin "Attention: l'admissibilite de la valeur "
                                                                    valeur
                                                                    )
                                                              (print " ne peut etre verifiee pour l'attribut ")
                                                              (prin nom-att " de l'instance " nom-inst)
                                                              (print " de la classe " nom-sch)
)
                                                              nil
                                                              )
                                                          )
                                                     )
                                               )
                                       )
                                      (if complexe (#:iace:fc-complex:action-arbre-complexe
                                                           (#:iace:fc-util:nom-sch instance)
                                                           b type pos
                                                           )
                                          )
                                      (if instance (let ((sortie (:spec-inst? instance type b))
                                                         ((complexe pos)
                                                          (:test-complexe (#:iace:fc-util:nom-sch schema) b)
                                                          )
                                                         )
                                                        (if complexe
                                                            (#:iace:fc-complex:action-arbre-complexe
                                                                    (#:iace:fc-util:nom-sch instance) b
                                                                    (#:iace:fc-util:nom-sch schema) pos
                                                                    )
                                                            )
                                                        sortie
                                                        )
                                          (exit verif t)
                                          )
                                      )
                                 )
                             )
                          )                           
                  (unless (#:iace:fc-util:app-sauf? valeur type (vref attrib %sauf))
                          (when (#:iace:fc-util:app-dom? valeur type (vref attrib %dom))
                                (if (#:iace:fc-util:type-simple? type)
                                    (if (#:iace:fc-util:app-inter? valeur type (vref attrib %int))
                                        (let ((pred (#:iace:fc-util:a-verifier? valeur nom-att schema instance)))
                                             (if pred pred (setq %%EXPL-NEG-PRED %%NOM-PRED) nil))
                                        ) 
                                    (let ((pred (#:iace:fc-util:a-verifier? valeur nom-att schema instance)))
                                         (if pred pred (setq %%EXPL-NEG-PRED %%NOM-PRED) nil)
                                         )
                                    )
                                )
                          )
                  (setq %%EXPL-NEG-TYPE type)
                  nil
                  )
              )
         )
    )

;;;***************************************************************************************
;;; permet d'executer une specialisation dans le cas d'un objet complexe

(de :spec-inst? (instance nom-sch-dest b)
    (let ((resultat (:spec-inst instance nom-sch-dest %pour-types %sauf-types %binf %bdial b)))
         (memq resultat '(sur possible))
         )
    )

;;;***************************************************************************************
;;; demande de specialisation d'une instance vers une classe

(de :spec-inst (instance nom-sch-dest pour-types sauf-types binf bdial b)
    (tag erreur (slet ((nom-sch-orig (#:iace:fc-util:val-inst instance %est-un))
                       (schema-orig (#:iace:fc-util:schema nom-sch-orig))
                       (nom-inst (#:iace:fc-util:nom-inst instance))
                       (nom-inst-bid (getprop nom-inst 'inst-bid))
                       )
                      (ifn (eq (#:iace:fc-util:chef-fam nom-sch-orig)(#:iace:fc-util:chef-fam nom-sch-dest))
                           (exit erreur 
(when #:trace:cla
				(print "familles incoherentes")
)
				"familles incoherentes")
                           )
                      (ifn nom-inst-bid
                           (setq nom-inst-bid (#:iace:fc-util:crebid-cl-inst instance))
                           )
                      (#:iace:fc-util:marquer-sur-anc schema-orig nom-inst-bid)
                      (:specialiser-vers nom-inst-bid nom-sch-dest pour-types
                              sauf-types %bguide binf bdial b
                              )
                      )
         )
    )

;;;***************************************************************************************
;;; execute la fonction decrite dans PP-classif2.ll et permet d'obtenir le texte 
;;; de la question dans l'editeur de iace.

(de :obtien-val (nom-inst nom-sch nom-att classe binf bdial nom-inst-bid b)
    (slet ((instance (#:iace:fc-util:schema nom-inst))
           (schema (#:iace:fc-util:schema nom-sch))
           (att (#:iace:fc-util:attrib classe nom-att))
           (type (#:iace:fc-util:val-inst att %type))
           (nature (#:iace:fc-util:val-inst att %nat))
           (dej-rep (member nom-att (getprop nom-inst 'deja-repondu)))
           (prop-dom (#:iace:fc-util:dom-adm schema nom-att classe))
           (prop (car prop-dom))
           (dom (cdr prop-dom))
           (valeur (if (and (if bdial (not dej-rep) t) (eq prop 'possible) (getprop nom-inst 'a-repondre))
                       (if binf (let ((val (#:iace:fc-util:infer-val schema instance 
                                                   classe att nature nom-att
                                                   )
                                           )
                                      )
                                     (if val val (if (and bdial (not dej-rep))
                                                     (:dem-val schema classe nom-att instance %bguide t nom-inst-bid b)
                                                     )
                                         )
                                     )
                           (:dem-val schema classe nom-att instance %bguide t nom-inst-bid b)
                           )
                       )
                   )
           (testval prop)
           )
          (if (null valeur)
              (if (neq testval 'sur)
                  (#:iace:fc-util:ecr-expl nom-inst (#:iace:fc-util:nom-sch classe) nom-att nil testval dom)
                  )
              (let ((resul (:verif-val nom-inst nom-sch nom-att valeur classe dom b)))
                   (setq testval (car resul))
                   (setq valeur (cdr resul))
                   )
              )
          (cons testval valeur)
          )
    )

;;;***************************************************************************************
;;; execute la fonction decrite dans PP-classif.ll et permet d'obtenir la reponse a la 
;;; question dans l'editeur de iace.

(de :dem-val (schema classe nom-att instance bguide branger nom-inst-bid b)
    (with ((prompt (catenate (makestring (lmargin) #/ ) nom-att "? ")))
          (slet (((nature type) (#:iace:fc-util:nature schema nom-att))
                 (nom-inst (#:iace:fc-util:nom-sch instance))
                 (bliste-de (eq nature '$liste-de))
                 (classe-sym (#:ivan:ed-util:symbole (#:iace:fc-util:nom-sch classe)))
                 (noeud-cl (if (equal (#:ivan:ed-util:symbole (vref instance %util-par))
                                      (#:ivan:ed-util:symbole (send 'get-line (component 'edl-instance b)))
                                      )
                               (getprop classe-sym 'noeud-iace)
                               (if (getprop classe-sym (vref instance %util-par))
                                   (getprop classe-sym (vref instance %util-par))
                                   (getprop classe-sym 'noeud-iace)
                                   )
                               )
                         )
                 )
                (let ((brepeter t)
                      (dej-rep (member nom-att (getprop nom-inst 'deja-repondu)))
                      (res)
                      (rep)
                      )
                     (tag fin-val
                          (while brepeter
                                 (lets ((erreur nil)
                                        ((int . dom) (#:iace:fc-util:union-int-dom schema
                                                             (#:iace:fc-util:nom-sch instance) nom-att bguide
                                                             )
                                         )
                                        )
                                       (send 'set-current (component 'ed-arbre b) noeud-cl)
                                       (setq rep (#:iace:fc-edatt:edit nom-inst (vref instance %util-par)
                                                         nom-att nom-inst-bid b int dom schema classe nil
                                                         )
                                             )
                                       (setq repon rep)
                                       (setq rep (if (or (equal rep '(-)) (equal rep '(<)) bliste-de)
                                                     (:traitement rep bliste-de erreur branger
                                                             instance nom-att schema b type
                                                             ) 
                                                     (if (or (equal type 'reel) (equal type 'entier))
                                                         (#:ivan:ed-util:symbole (implode (explode rep)))
                                                         (if (equal type 'chaine)
                                                             (if (or (equal rep "-") (equal rep "<"))
                                                                 (#:ivan:ed-util:symbole rep)
                                                                 rep
                                                                 )
                                                             (#:ivan:ed-util:symbole rep)
                                                             )
                                                         )
                                                     )
                                             )
                                       (selectq rep (-  ;fin des valeurs de l'attribut 
                                                       (exit fin-val)
                                                       ) 
                                               (<  ;fin de l'instanciation courante
                                                  (progn (remprop (#:iace:fc-util:nom-sch instance)
                                                                 'a-repondre
                                                                 )
                                                         (exit fin-val)
                                                         )
                                                  )
                                               ( t )
                                               )
                                       (when (and rep (not erreur))
                                             (setq rep (if branger 
                                                           (#:iace:fc-util:aj-valeur instance nom-att rep t)
                                                           (:admissible? rep nom-att schema instance b)
                                                           )
                                                   )
                                             (if rep (setq res (if bliste-de (nconc1 res rep) rep))
                                                 (#:iace:fc-interf:action-sa-warn
                                                         b nil t (list (row (font #:ivan:in-global:fstd
										#M#:txt-const:valeur-non-admiss)
                                                                            )
                                                                       )
                                                         )
                                                 (terpri)
                                                 (setq erreur t)
                                                 )
                                             )
                                       (setq brepeter (or erreur bliste-de))
                                       )
                                 )
                          )
                     (setq %liste (if rep
                                      (cons %liste
                                            (#:ivan:ed-util:symbole
                                                    (concat "[" (string classe-sym) "/" (vref instance %util-par) "]"
                                                            (string nom-att) "="
                                                            (string (if bliste-de
                                                                        (if (or (equal repon '(-)) (equal repon '(<)))
                                                                            "?,"
                                                                            (:const-liste repon)
                                                                            )
                                                                        (if (or (equal repon "-") (equal repon "<"))
                                                                            "?"
                                                                            repon
                                                                            )
                                                                        )
                                                                    )
                                                            )
                                                    )
                                            )
                                      )
                           )
                     (#:iace:fc-interf:reinit-sm (component 'scroll-attribut b)
                             (component 'sm-attribut b)
                             (#:iace:fc-interf:list-plate %liste)
                             )
                     ({stringmenu}:selection-limit (component 'sm-attribut b)
                             (length (#:iace:fc-interf:list-plate %liste))
                             )
                     (#:ivan:ed-util:positionne (component 'scroll-attribut b) t)
                     (ifn dej-rep (let ((l-dej-rep (getprop nom-inst 'deja-repondu)))
                                       (setq l-dej-rep (nconc1 l-dej-rep nom-att))
                                       (putprop nom-inst l-dej-rep 'deja-repondu)
                                       )
                          )
                     res
                     )
                )
          )
    )

;;;***************************************************************************************
;;; permet de traiter le cas des listes de choix multiples

(de :traitement (list-rep blist berreur branger instance nom-att schema bouton type)
    (if (equal list-rep '(-))
        (#:ivan:ed-util:symbole "-")
        (if (equal list-rep '(<))
            (#:ivan:ed-util:symbole "<")
            (tag fin-err
                 (mapcar '(lambda (reponse)
                                  (ifn (equal type 'chaine)
                                       (setq reponse (#:ivan:ed-util:symbole reponse))
                                       )
                                  (when (and reponse (not berreur))
                                        (setq reponse (if branger 
                                                          (#:iace:fc-util:aj-valeur instance nom-att reponse t)
                                                          (:admissible? reponse nom-att schema instance bouton)
                                                          )
                                              )
                                        (if reponse (setq res (if blist (nconc1 res reponse) reponse))
                                            (#:iace:fc-interf:action-sa-warn
                                                    b nil t (list (row (font #:ivan:in-global:fstd
									#M#:txt-const:valeur-non-admiss)
                                                                       )
                                                                  )
                                                    )
                                            (terpri)
                                            (exit fin-err)
                                            )
                                        )
                                  )
                         list-rep
                         )
                 (#:ivan:ed-util:symbole "-")  
                 )
            )
        )
    )

;;;***************************************************************************************
;;; construction d'une chaine de caracteres correspondant a <liste>

(de :const-liste (liste)
    (ifn liste ""
         (let liste (concat (string (car liste)) "," (:const-liste (cdr liste))))
         )
    )

;;;***************************************************************************************
;;; teste si le nom de la classe traitee existe dans l'arbre principal 
;;; sinon retourne le concept a charger

(de :test-complexe (classe b)
    (lets ((classes ({fc-interf}:ch-vect-classes (component 'top b)))
           (concepts ({fc-interf}:ch-list-concepts (component 'top b)))
           (membre (member classe concepts))
           (liste-classes-courantes ({fc-interf}:ch-list-classes (component 'top b)))
           (nbr nil)
           )
          (if (member classe liste-classes-courantes)
              (list nil nil) ;pas de changement de concept
              (if membre ;alors il faut trouver le nouveau concept si il existe
                  (list (car membre) (- (length concepts) (length membre)))                 
                  (let ((pos (- (length concepts) 1)));sinon la classe est dans la liste vecteur classes
                       (while (>= pos 0)
                              (lets ((list-tempo (vref classes pos))
                                     (list-plate-tempo (#:iace:fc-interf:list-plate list-tempo))
                                     )
                                    (ifn (member classe list-plate-tempo)
                                         (setq pos (- pos 1))
                                         (setq nbr pos)
                                         (setq pos -1)
                                         )
                                    )
                              )
                       (if nbr (lets ((list-tempo (vref classes nbr))
                                      (list-plate-tempo (#:iace:fc-interf:list-plate list-tempo))
                                      )
                                     (list (car list-plate-tempo) nbr)
                                     )
                           )
                       )
                  )
              )
          )
    )

;;;***************************************************************************************
;;; renvoie la liste des classes d'attributs du concept courant

(de :list-attr (bouton)
    (let (list)
         (mapc '(lambda (nom-classe)
                        (let ((schema (#:iace:fc-util:schema nom-classe))) 
                             (mapc '(lambda (att)
                                            (let ((classe-att (#:iace:fc-util:classe-att-he schema att)))
                                                 (ifn (or (eq classe-att 'attribut-shirka-est-un) 
                                                          (eq classe-att 'attribut-shirka-sauf-est-un)
                                                          (eq classe-att 'attribut)
                                                          )
                                                      (ifn (member classe-att list)
                                                           (setq list (cons classe-att list))
                                                           )
                                                      )
                                                 )
                                            )
                                   (#:iace:fc-util:l-nom-att-he schema)
                                   )
                             )
                        )
               ({fc-interf}:ch-list-classes (component 'top bouton))
               )
         (sortl list)
         )
    )

;;;***************************************************************************************
