;;; Repertoire: ~LeShirka/LeIvan/CodeIace/
;;;
;     +----------------------------------------+-----------------------------------------+
;     | fichier :  fc-complex.ll               | package : #:iace:fc-complex             |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface d'Aide a la Classification et a l'Explication              |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Francois Cruypenninck    1991-1992                        |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     | descriptif du package :                                                          |
;     |                                                                                  |
;     | La fonction principale :interface cree une instance de la structure {fc-complex} |
;     | Cette structure permet de construire interactivement une suite d'arbres en       |
;     | cascade pour permettre de gerer le cas des objets complexes                      |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+

;;;***************************************************************************************
;;; personnalisation du package
;;;***************************************************************************************


(defvar #:sys-package:colon '#:iace:fc-complex)

;;;***************************************************************************************
;;; affichage du nouvel arbre dans le cas d'objets complexes
;;;***************************************************************************************

;;;***************************************************************************************
;;; le systeme a detecte un concept different du concept courant

(de :action-arbre-complexe (instance b classe pos)
    (:action-menu-concept (component 'top b) pos instance)
    (lets ((classe-sym (#:ivan:ed-util:symbole classe))
           (noeud-classe (if (equal (#:ivan:ed-util:symbole instance)
                                    (#:ivan:ed-util:symbole (send 'get-line (component 'edl-instance b)))
                                    )
                             (getprop classe-sym 'noeud-iace)
                             (getprop classe-sym instance)
                             )
                   )
           )
          (send 'set-current (component 'ed-arbre b) noeud-classe)
          )
    )

;;;***************************************************************************************
;;; changer de concept: il faut charger sa representation graphique

(de :action-menu-concept (appli pos instance)
    (lets ((concept (nth pos ({fc-interf}:ch-list-concepts appli)))
           (list-classes (vref ({fc-interf}:ch-vect-classes appli) pos))
           (list-classes-courante ({fc-interf}:ch-list-classes appli))
           (concept-courant ({fc-interf}:ch-concept appli))
           (list-hierarchie-courante ({fc-interf}:ch-list-hierarchie appli))
           nbcl
           )
          (if (member concept list-concept)
              (if (member instance list-instance)
                  (setq existe t)
                  (setq list-instance (#:iace:fc-interf:list-plate (cons list-instance instance)))
                  (setq existe nil)
                  )                  
              (setq list-concept (#:iace:fc-interf:list-plate (cons list-concept concept)))
              (setq list-instance (#:iace:fc-interf:list-plate (cons list-instance instance)))
              (setq existe nil)
              )
          (if existe
              nil ;le concept existe deja
              (#:iace:fc-interf:action-sa-warn appli
                      nil nil (list (row (font #:ivan:in-global:fstd #M#:txt-const:compl-aj)
                                         (font #:ivan:in-global:fstdb (string concept))
                                         (font #:ivan:in-global:fstd "...")
                                         )
                                    )
                      )
              (#:iace:fc-interf:reinit-edl-classe (component 'edl-classe appli) "")
              ({fc-interf}:ch-concept appli concept)
              ({fc-interf}:ch-list-hierarchie appli (car list-classes))
              ({fc-interf}:ch-list-classes appli (cadr list-classes))
              (setq nbcl (length ({fc-interf}:ch-list-classes appli)))
              (#:iace:fc-interf:demarquer-ed-arbre appli instance)
              (inhibit-application (component 'ed-arbre appli))
              (lets ((scroll-arbre (component 'scroll-arbre appli))
                     (nouv-ed-arbre (:creer-arbre appli (component 'ed-arbre appli) instance))
                     nouv-scroll-arbre
                     inside
                     (nom-classe (#:iace:fc-util:valeur? 'est-un instance))
                     (nom-pere (#:iace:fc-util:peres (#:iace:fc-util:schema nom-classe)))
                     )
                    (send 'set-current (component 'ed-arbre appli) nil)
                    (send 'pretty nouv-ed-arbre)
                    (setq inside (send 'image nouv-ed-arbre))
                    (send 'grow nouv-ed-arbre (+ (send 'x inside) (* 2 (send 'width  inside)))
                          (+ {scroller}:scrollbarheight (* 2 (send 'height inside)))
                          )
                    (setq nouv-scroll-arbre (scroller 0 0 (send 'width scroll-arbre)
                                                    (send 'height scroll-arbre)
                                                    nouv-ed-arbre
                                                    t t
                                                    )
                          )
                    (authorize-application (component 'ed-arbre appli))
                    (set-component appli 'ed-arbre nouv-ed-arbre)
                    (set-action nouv-ed-arbre '#:iace:fc-interf:action-ed-arbre)
                    (send 'set-image scroll-arbre nouv-scroll-arbre)
                    (set-component appli 'scroll-arbre nouv-scroll-arbre)
                    (#:iace:fc-interf:action-bouts-prettytree (component 'bouts-prettytree appli))
                    (#:iace:fc-classif:action-sur-arbre appli 'sur nom-classe instance)
                    (#:iace:fc-classif:classe-sur nom-pere appli instance)
                    )
              ({fc-interf}:ch-concept appli concept-courant)
              ({fc-interf}:ch-list-hierarchie appli list-hierarchie-courante)
              ({fc-interf}:ch-list-classes appli list-classes-courante)
              (#:iace:fc-interf:action-sa-warn appli
                      nil nil (list (row (font #:ivan:in-global:fstd #M#:txt-const:fin-cha)
                                         )
                                    )
                      )
              )
          )
    )

;;;***************************************************************************************
;;; construction de l'arbre a partir de la liste des classes du concept + objet

(de :creer-arbre (appli ancien-arbre instance)
    (let ((list-hierar ({fc-interf}:ch-list-hierarchie appli))
          (list-classe ({fc-interf}:ch-list-classes appli))
          (cur-font (current-font))
          )
         (current-font #:ivan:in-global:fmicro)
         (setq #:ivan:in-global:larg (#:ivan:ed-util:wc (#:ivan:ed-util:larg-max list-classe)))
         (current-font cur-font)
         (let ((ed-arbre ancien-arbre)
               noeud-pere
               )
              (if (null list-hierar)
                  ed-arbre
                  (setq noeud-pere (if instance (omakeq {fc-noeud} nom (string (car list-classe))
                                                        nom-inst (string instance)
                                                        concept "vrai"
                                                        titre (string (car list-classe))
                                                        img "interro"
                                                        larg #:ivan:in-global:larg
                                                        )
                                       (omakeq {fc-noeud} nom (string (car list-classe))
                                               nom-inst (string instance)
                                               concept "faux"
                                               titre (string (car list-classe))
                                               img "interro"
                                               larg #:ivan:in-global:larg
                                               )
                                       )
                        )
                  (send 'set-current ed-arbre nil)
                  (send 'insert ed-arbre noeud-pere)
                  (putprop (car list-hierar) noeud-pere 'noeud-iace)
                  (putprop (car list-hierar) noeud-pere instance)
                  (#:iace:fc-interf:creer-noeud ed-arbre list-hierar instance)
                  )
              )
         )
    )

;;;***************************************************************************************
