;;; Repertoire: ~LeShirka/LeIvan/CodeIace/
;;;
;     +----------------------------------------+-----------------------------------------+
;     | fichier :  iace-interf.ll              | package : #:iace:fc-interf              |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface d'Aide a la Classification et a l'Explication              |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Francois Cruypenninck    1991-1992                        |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     | descriptif du package :                                                          |
;     |                                                                                  |
;     | La fonction principale :interface cree une instance de la structure {fc-interf}. |
;     | Cette structure contient les champs de representation de l'interface de          |
;     | classification.                                                                  |
;     | L'interface est une fenetre composee de 5 parties fonctionnelles :               |
;     |                                                                                  |
;     |        |-------------------------------------------------+-----------|           |
;     |        |           LIGNE D'ENTETE                        | 'quitter' |           |
;     |        |---------------+---------------------------------+-----------|           |
;     |        |   COMMANDES   |               REPRESENTATION GRAPHIQUE      |           |
;     |        |---------------+---------------------------------------------|           |
;     |        |          TABLEAU DE BORD DE CLASSIF ET D'EXPLICATION        |           |
;     |        |-------------------------------------------------------------|           |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+

;;;***************************************************************************************
;;; personnalisation du package
;;;***************************************************************************************

(defvar #:sys-package:colon '#:iace:fc-interf)

;;;***************************************************************************************
;;; mise en place de l'image graphique de l'application
;;;***************************************************************************************

;;;***************************************************************************************
;;; fonction principale de construction de l'interface

(de :interface ()
    (let ((appli (omakeq {fc-interf} x 0 y 0 w 0 h 0))
          (bouts-quitter (standardbutton #M#:txt-const:fin))
          (bouts-retour  (standardbutton #M#:txt-const:revenir-ivan))
          (bouts-iace    (standardbutton (font #:ivan:in-global:fstd #M#:txt-const:infos)))
          (image-appli)
          )
         (set-action bouts-quitter ':action-bouts-quitter)
         (set-action bouts-retour  ':action-bouts-retour)
         (set-action bouts-iace    ':action-bouts-iace)
         ({application}:components appli (acons 'top appli ()))
         (add-component appli 'bouts-quitter bouts-quitter)
         (add-component appli 'bouts-retour  bouts-retour)
         ({fc-interf}:ch-list-concepts appli (:const-ch-list-concepts appli))
         ({fc-interf}:ch-vect-classes appli  (:const-ch-vect-classes appli))
         ({fc-interf}:ch-concept appli "")
         ({fc-interf}:ch-list-hierarchie appli nil)
         ({fc-interf}:ch-list-classes appli nil)
         ({fc-interf}:ch-ed-arbre appli (:const-ch-ed-arbre  appli))
         ({fc-interf}:ch-classes appli (:const-ch-classes appli))
         ({fc-interf}:ch-scrollers appli (:const-ch-scrollers appli))
         ({fc-interf}:ch-commandes appli (:const-ch-commandes appli))
         ({fc-interf}:ch-edl-concept appli (:const-ch-edl-concept appli))
         ({fc-interf}:ch-edl-classe appli (:const-ch-edl-classe appli))
         ({fc-interf}:ch-sa-action appli (:const-ch-sa-action  appli))
         (setq image-appli (column (row (column ({fc-interf}:ch-commandes appli)
                                                ""
                                                )
                                        ({fc-interf}:ch-ed-arbre appli)
                                        )
                                   (row ({fc-interf}:ch-edl-concept appli) 
                                        ({fc-interf}:ch-sa-action appli) 
                                        )
                                   (row (column ({fc-interf}:ch-edl-classe appli)
                                                ({fc-interf}:ch-classes appli)
                                                )
                                        ({fc-interf}:ch-scrollers appli)
                                        )
                                   )
               )
         (send 'grow image-appli #:ivan:in-global:larg-appli #:ivan:in-global:haut-appli)
         (send 'set-image appli (header logo
                                        (row "    " bouts-iace " " bouts-retour)
                                        bouts-quitter
                                        (view)
                                        image-appli      
                                        t
                                        nil
                                        )
               )
         ;----------------------------------------------------------
         (send 'fit-to-contents appli)
         (send 'set-title appli "IACE")
         appli
         )
    )

;;;***************************************************************************************
;;;    Fonction de construction du champ des commandes = colonne des boutons repartie
;;;    en 3 groupes : 
;;;      1. Fonction sur une base Shirka
;;;      2. Fonctions sur l'arbre
;;;      3. Aide

(de :const-ch-commandes (appli)
    (lets ((titre-clu1 #M#:txt-const:gest-bases)
           (titre-clu2 #M#:txt-const:visu)
           (menu-concept (create-menu #M#:txt-const:choisir))
           (bouts-concept	(menubutton #M#:txt-const:cpts 0 0 0 0 menu-concept))
           (bouts-chemin	(standardbutton #M#:txt-const:rep))       
           (bouts-charger	(standardbutton #M#:txt-const:charg))
           (bouts-sauver	(standardbutton #M#:txt-const:sauv))
           (bouts-prettytree	(standardbutton #M#:txt-const:redessin))
           (bouts-standardtree	(standardbutton #M#:txt-const:taille-norm))
           (bouts-reduct1	(standardbutton #M#:txt-const:red-1))
           (bouts-reduct2	(standardbutton #M#:txt-const:red-2))
           (bouts-minitree	(standardbutton #M#:txt-const:vue-gen))
           (bouts-repos		(standardbutton #M#:txt-const:repos)) 
           (bouts-aller		(standardbutton #M#:txt-const:select)) 
           (clu1 (#:ivan:ed-util:clusterimage (column bouts-chemin
                                                      bouts-charger
                                                      bouts-sauver
                                                      bouts-concept
                                                      )
                         titre-clu1
                         )
                 )
           (clu2 (#:ivan:ed-util:clusterimage (column bouts-prettytree 
                                                      bouts-standardtree
                                                      bouts-reduct1
                                                      bouts-reduct2
                                                      bouts-minitree
                                                      bouts-aller
                                                      bouts-repos
                                                      )
                         titre-clu2
                         )
                 )
           (clu3 (#:ivan:ed-util:clusterimage (column " "
                                                      (row " " (icon 0 0 ic-interro)
                                                           " " (icon 0 0 ic-arrow-right)
                                                           #M#:txt-const:interro
                                                           )
                                                      " "
                                                      (row " " (icon 0 0 ic-sur)
                                                           " " (icon 0 0 ic-arrow-right)
                                                           #M#:txt-const:sur
                                                           )
                                                      " "
                                                      (row " " (icon 0 0 ic-possible)
                                                           " " (icon 0 0 ic-arrow-right)
                                                           #M#:txt-const:poss
                                                           )
                                                      " "
                                                      (row " " (icon 0 0 ic-impossible)
                                                           " " (icon 0 0 ic-arrow-right)
                                                           #M#:txt-const:imposs
                                                           )
                                                      " "
                                                      )
                         #M#:txt-const:legende
                         )
                 )
           )
          (add-component appli 'menu-concept menu-concept)
          (add-component appli 'bouts-concept bouts-concept)
          (add-component appli 'bouts-charger bouts-charger)
          (add-component appli 'bouts-chemin bouts-chemin)
          (add-component appli 'bouts-sauver bouts-sauver)
          (add-component appli 'bouts-prettytree bouts-prettytree)
          (add-component appli 'bouts-standardtree bouts-standardtree)
          (add-component appli 'bouts-reduct1 bouts-reduct1)
          (add-component appli 'bouts-reduct2 bouts-reduct2)
          (add-component appli 'bouts-minitree bouts-minitree)
          (add-component appli 'bouts-aller bouts-aller)
          (add-component appli 'bouts-repos bouts-repos)
          (set-action bouts-charger      ':action-bouts-charger)
          (set-action bouts-chemin       ':action-bouts-chemin)    
          (set-action bouts-sauver       ':action-bouts-sauver)
          (set-action bouts-prettytree   ':action-bouts-prettytree)
          (set-action bouts-standardtree ':action-bouts-standardtree)
          (set-action bouts-reduct1      ':action-bouts-reduct1)
          (set-action bouts-reduct2      ':action-bouts-reduct2)
          (set-action bouts-minitree     ':action-bouts-minitree)
          (set-action bouts-aller        ':action-bouts-aller)
          (set-action bouts-repos        ':action-bouts-repos)
          (:creer-menu appli)
          (:inhiber-com appli) 
          (:autoriser-com appli) 
          (column clu1 clu2 clu3)
          )
    )
     
;;;***************************************************************************************
;;; champs graphiques de la representation de la classification et de l'explication
;;;***************************************************************************************

;;;***************************************************************************************
;;;construction des scrollers = attributs, classes d'attributs et instances

(de :const-ch-scrollers (appli)
    (lets ((larg (div (- #:ivan:in-global:larg-appli 48) 4))
           (titre-classif  #M#:txt-const:classif)
           (titre-expli    #M#:txt-const:expli)
           (titre-instance #M#:txt-const:inst)
	   (titre-attribut #M#:txt-const:history)
           (titre-config   #M#:txt-const:config)
           (sm-attribut (stringmenu nil))
           ;hy-------------------------------------------
           ; editeur de texte dans le cas d'un hypertexte
           ;           (sm-explication (stringmenu nil))
           (sm-explication (textedit 0 0 #240wc #60hc))
           ;---------------------------------------------
           (bouts-lancer   (standardbutton #M#:txt-const:lancer))
           (bouts-reinit   (standardbutton #M#:txt-const:reinit))
           (bouts-modifier (standardbutton #M#:txt-const:modification))
           (bouts-valider  (standardbutton #M#:txt-const:valider))
           (bouts-mode     (standardbutton #M#:txt-const:mode))
           (bouts-pourquoi (standardbutton #M#:txt-const:pourquoi))
           (case-inference (checkbox 0 0 90 #hc  (font #:ivan:in-global:fstd #M#:txt-const:inf)))
           (case-pour-type (checkbox 0 0 73 #hc  (font #:ivan:in-global:fstd #M#:txt-const:class)))
           (case-sauf-type (checkbox 0 0 85 #hc (font #:ivan:in-global:fstd #M#:txt-const:sauf)))
           (bouts-? (standardbutton #M#:txt-const:liste))
           (scroll-attribut    (scroller 0 0 larg #6hc sm-attribut t t))
           (scroll-explication (scroller 0 0 (- (* 2 larg) 8) #15hc sm-explication t t))         
           (edl-instance (lineedit 0 0 (* 13 (div larg 16)) ""))
           (edl-expli (lineedit 0 0 (- (* 2 larg) 68) #M#:txt-const:mode-clas))
           (clu-attribut (#:ivan:ed-util:clusterimage scroll-attribut titre-attribut))
           (clu-instance (#:ivan:ed-util:clusterimage (row edl-instance bouts-?) titre-instance))
           (clu-config (#:ivan:ed-util:clusterimage (row " " case-inference case-pour-type "  " case-sauf-type) titre-config))
           (clu-classif (#:ivan:ed-util:clusterimage (column (centeredview (rectangle 0 0 larg 0) clu-instance)
                                                             (centeredview (rectangle 0 0 larg 0) clu-config)
                                                             " "
                                                             (row bouts-lancer bouts-reinit
                                                                  bouts-modifier bouts-valider
                                                                  )
                                                             " "
                                                             clu-attribut
                                                             )
                                titre-classif
                                )
                   )
           (clu-expli (#:ivan:ed-util:clusterimage (column (row bouts-pourquoi
                                                                " "
                                                                bouts-mode
                                                                " "
                                                                (icon 0 0 ic-arrow-right)
                                                                " "
                                                                edl-expli
                                                                )
                                                           " "
                                                           scroll-explication
                                                           )
                              titre-expli
                              )
                   )
           )
          ;hy-------------------------------------
          ; positionne les parametres de 'textedit'
          (te-hide-cursor sm-explication)
          (te-reset sm-explication)
          (te-refresh sm-explication t)
          (te-readonly sm-explication t)
          ;---------------------------------------
          ({medite}:readonly-flag edl-expli t)
          (send 'checked case-inference t)
          (add-component appli 'bouts-lancer bouts-lancer)
          (add-component appli 'bouts-reinit bouts-reinit)
          (add-component appli 'bouts-modifier bouts-modifier)
          (add-component appli 'bouts-valider bouts-valider)
          (add-component appli 'bouts-? bouts-?)
          (add-component appli 'bouts-mode bouts-mode)
          (add-component appli 'bouts-pourquoi bouts-pourquoi)
          (add-component appli 'edl-instance edl-instance)
          (add-component appli 'edl-expli edl-expli)
          (add-component appli 'sm-attribut sm-attribut)
          (add-component appli 'sm-explication sm-explication)
          (add-component appli 'scroll-attribut scroll-attribut)
          (add-component appli 'scroll-explication scroll-explication)
          (add-component appli 'case-inference case-inference)
          (add-component appli 'case-pour-type case-pour-type)
          (add-component appli 'case-sauf-type case-sauf-type)
          (set-action edl-instance   ':action-edl-instance)
          (set-action bouts-lancer   '#:iace:fc-classif:action-bouts-lancer)
          (set-action bouts-reinit   '#:iace:fc-classif:action-bout-reinit)
          (set-action bouts-modifier '#:iace:fc-modif:action-bouts-modifier)
          (set-action bouts-valider  '#:iace:fc-classif:action-bouts-valider)
          (set-action bouts-?        '#:iace:fc-classif:action-bouts-?)
          (set-action bouts-mode     '#:iace:fc-expli:action-bouts-mode)
          (set-action bouts-pourquoi '#:iace:fc-expli:action-bouts-pourquoi)
          (inhibit-application edl-instance)
          (inhibit-application bouts-pourquoi)
          (row 
               clu-classif 
               clu-expli
               )
          )
    )

;;;***************************************************************************************
;;; inhiber les boutons de commande

(de :inhiber-com (appli)
    (inhibit-application (component 'bouts-quitter appli))
    (inhibit-application (component 'bouts-chemin appli))
    (inhibit-application (component 'bouts-retour appli))
    (inhibit-application (component 'bouts-charger appli))
    (inhibit-application (component 'bouts-sauver appli))
    (inhibit-application (component 'bouts-concept appli))
    (inhibit-application (component 'bouts-prettytree appli))
    (inhibit-application (component 'bouts-standardtree appli))
    (inhibit-application (component 'bouts-reduct1 appli))
    (inhibit-application (component 'bouts-reduct2 appli))
    (inhibit-application (component 'bouts-minitree appli))
    (inhibit-application (component 'bouts-aller appli))
    (inhibit-application (component 'bouts-repos appli))
    (inhibit-application (component 'bouts-lancer appli))
    (inhibit-application (component 'bouts-reinit appli))
    (inhibit-application (component 'bouts-modifier appli))
    (inhibit-application (component 'bouts-valider appli))
    (inhibit-application (component 'bouts-? appli))
    (inhibit-application (component 'bouts-mode appli))
    (inhibit-application (component 'bouts-pourquoi appli))
    )

;;;*****************************************************************************************
;;; initialiser l'etat de Ivan quand des bases on ete chargees en Shirka (sans passer par IVAN)
;;  (active en particulier le bouton qui permet de choisir un concept)
(de :initialise-boutons ()
    (let ((menu-concept (create-menu #M#:txt-const:l-cpts))
	  (old-menu-concept (component 'menu-concept #:iace:appli))
	 )
	 (setq mots-cle (cdr (#:systeme:unix:catalogue (pathname ""))))
         ; mettre a jour la liste des concepts
         ({fc-interf}:ch-list-concepts #:iace:appli (:const-ch-list-concepts #:iace:appli))
         ({fc-interf}:ch-vect-classes #:iace:appli  (:const-ch-vect-classes #:iace:appli))
         (kill-menu old-menu-concept)
         (set-component #:iace:appli 'menu-concept menu-concept)
	 (send 'menu (component 'bouts-concept #:iace:appli) menu-concept)
         (:creer-menu #:iace:appli)
))

;;;***************************************************************************************
;;; autoriser les boutons de commandes

(de :autoriser-com (a)
    (let ((appli (component 'top a)))
         (if (and selection-classe selection-instance)
             (progn (authorize-application (component 'bouts-lancer appli))
                    (authorize-application (component 'bouts-reinit appli))
                    (if %valider (progn (authorize-application (component 'bouts-valider appli))
                                        (authorize-application (component 'bouts-modifier appli))
                                        )
                        )
                    )
             )
         (if ({fc-interf}:ch-list-concepts appli)
             (if (#:ivan:ed-util:symbole ({fc-interf}:ch-concept appli))
                 (progn (authorize-application (component 'edl-instance appli))
                        (authorize-application (component 'bouts-? appli))
                        (if (componentp 'ivan appli)
                            (authorize-application (component 'bouts-retour appli))
                            (authorize-application (component 'bouts-quitter appli))
                            )
                        (authorize-application (component 'bouts-charger appli))
                        (authorize-application (component 'bouts-chemin appli))
                        (authorize-application (component 'bouts-sauver appli))
                        (authorize-application (component 'bouts-concept appli))
                        (authorize-application (component 'bouts-prettytree appli))
                        (authorize-application (component 'bouts-mode appli))
                        (if #:ivan:in-global:bminitree
                            (authorize-application (component 'bouts-standardtree appli))        
                            (inhibit-application   (component 'bouts-standardtree appli))
                            )
                        (if #:ivan:in-global:breduct1      
                            (authorize-application (component 'bouts-standardtree appli))        
                            (authorize-application (component 'bouts-reduct1 appli))
                            )
                        (if #:ivan:in-global:breduct2       
                            (authorize-application (component 'bouts-standardtree appli))        
                            (authorize-application (component 'bouts-reduct2 appli))
                            )
                        (authorize-application (component 'bouts-minitree appli))
                        (authorize-application (component 'bouts-aller appli))
                        (when (#:ivan:ed-util:symbole (send 'get-line (component 'edl-classe appli)))
                              (authorize-application (component 'bouts-repos appli))
                              )
                        )
                 (if (componentp 'ivan appli)
                     (authorize-application (component 'bouts-retour appli))
                     (authorize-application (component 'bouts-quitter appli))
                     
                     )
                 (authorize-application (component 'bouts-charger appli))
                 (authorize-application (component 'bouts-chemin appli))
                 (inhibit-application   (component 'bouts-sauver appli))
                 (authorize-application (component 'bouts-concept appli))
                 (inhibit-application   (component 'bouts-prettytree appli))
                 (inhibit-application   (component 'bouts-standardtree appli))        
                 (inhibit-application   (component 'bouts-reduct1 appli))
                 (inhibit-application   (component 'bouts-reduct2 appli))
                 (inhibit-application   (component 'bouts-minitree appli))
                 (inhibit-application   (component 'bouts-aller appli))
                 (inhibit-application   (component 'bouts-repos appli))
                 (inhibit-application   (component 'bouts-mode appli))
                 )
             (if (componentp 'ivan appli)
                 (authorize-application (component 'bouts-retour appli))
                 (authorize-application (component 'bouts-quitter appli))
                 )
             (authorize-application (component 'bouts-charger appli))
             (authorize-application (component 'bouts-chemin appli))
             (inhibit-application (component 'bouts-sauver appli))
             (inhibit-application (component 'bouts-concept appli))
             (inhibit-application (component 'bouts-prettytree appli))
             (inhibit-application (component 'bouts-standardtree appli))        
             (inhibit-application (component 'bouts-reduct1 appli))
             (inhibit-application (component 'bouts-reduct2 appli))
             (inhibit-application (component 'bouts-minitree appli))
             (inhibit-application (component 'bouts-aller appli))
             (inhibit-application (component 'bouts-repos appli))
             (inhibit-application (component 'bouts-mode appli))
             )
         ) 
    )

;;;***************************************************************************************
;;; champs graphiques de la representation de l'arbre
;;;***************************************************************************************

;;;***************************************************************************************
;;; construction de la representation graphique des classes

(de :const-ch-ed-arbre (appli)
    (lets ((titre-clu #M#:txt-const:arbre-spec)
           (ed-arbre (view))
           (scroll-arbre (scroller 0 0 #80wd #50hd ed-arbre t t))
           (clu (#:ivan:ed-util:clusterimage scroll-arbre titre-clu))
           )
          (add-component appli 'ed-arbre ed-arbre) 
          (add-component appli 'scroll-arbre scroll-arbre)
          clu
          )
    )

;;;***************************************************************************************
;;; champs graphiques de la representation de la hierarchie des classes et sous-classes
;;;***************************************************************************************


;;;***************************************************************************************
;;; construction des scrollers = sur-classes et sous-classes

(de :const-ch-classes (appli)
    (lets ((larg (div (- #:ivan:in-global:larg-appli 48) 4))
           (titre-surclasse  #M#:txt-const:sur-cl)
           (titre-sousclasse #M#:txt-const:sous-cl)
           (sm-surclasse  (stringmenu nil))
           (sm-sousclasse (stringmenu nil))
           (scroll-surclasse  (scroller 0 0 larg #4hc sm-surclasse t t))
           (scroll-sousclasse (scroller 0 0 larg #9hc sm-sousclasse t t))
           (clu-surclasse  (#:ivan:ed-util:clusterimage scroll-surclasse  titre-surclasse))
           (clu-sousclasse (#:ivan:ed-util:clusterimage scroll-sousclasse titre-sousclasse))
           )
          (add-component appli 'sm-surclasse  sm-surclasse)
          (add-component appli 'sm-sousclasse sm-sousclasse)
          (add-component appli 'scroll-surclasse  scroll-surclasse)
          (add-component appli 'scroll-sousclasse scroll-sousclasse)
          (set-action sm-surclasse ':action-sm-surclasse)
          (set-action sm-sousclasse ':action-sm-sousclasse)
          (column clu-surclasse clu-sousclasse)
          )
    )

;;;***************************************************************************************
;;; construction de l'editeur de ligne contenant le nom du concept selectionne

(de :const-ch-edl-concept (appli)
    (let ((edl-concept (lineedit 0 0 (send 'width (component 'scroll-surclasse appli)) "")))
         (add-component appli 'edl-concept edl-concept)
         (#:ivan:ed-util:clusterimage edl-concept #M#:txt-const:cpt-sel)
         )
    )

;;;***************************************************************************************
;;; construction de l'editeur de ligne contenant le nom de la classe courante

(de :const-ch-edl-classe (appli)
    (let ((edl-classe (lineedit 0 0 (send 'width (component 'scroll-surclasse appli)) "")))
         (add-component appli 'edl-classe edl-classe)
         (set-action edl-classe ':action-edl-classe)
         (#:ivan:ed-util:clusterimage edl-classe #M#:txt-const:cl-sel)
         )
    )

;;;***************************************************************************************
;;; champs graphiques de la representation des messages utilisateur
;;;***************************************************************************************

;;;***************************************************************************************
;;; construction du champ d'affichage de messages d'information/d'alerte

(de :const-ch-sa-action (appli)
    (let ((sa-warn (selectorapplication 0 0
                           (* 3 (send 'width (component 'scroll-surclasse appli)))
                           (send 'height (component 'edl-classe appli))
                           nil
                           )
                  ))
         (add-component appli 'sa-warn sa-warn)
         (#:ivan:ed-util:clusterimage sa-warn #M#:txt-const:message)
         )
    )

;;;***************************************************************************************
;;; deroulement des actions quitter, iace, ... 
;;;***************************************************************************************

;;;***************************************************************************************
;;; action du bouton quitter = fermer l'application apres une confirmation

(de :action-bouts-quitter (b)
    (when (confirm #M#:txt-const:sur-de-quitter)
          (remove-application (send 'father b))
          )
    )

;;;***************************************************************************************
;;; action du bouton retour = retour a l'application IVAN

(de :action-bouts-retour (b)
    (when (and (componentp 'ivan b) (activatedp (component 'ivan b)))
          (#:ivan:in-interf:action-edl-classe (component 'edl-classe (component 'ivan b)))
          (pop-application (component 'ivan b))
          )
    )

;;;***************************************************************************************
;;; action du bouton iace = afficher une fenetre d'information

(de :action-bouts-iace (b)
    (let ((list-doc (list (font #:ivan:in-global:fstdb #M#:txt-const:expl-iace)
                          ""
                          (row "      "
                               (font #:ivan:in-global:fstdb "I")
                               (font #:ivan:in-global:fstd "nterface d'")
                               )
                          (row "      "
                               (font #:ivan:in-global:fstdb "A")
                               (font #:ivan:in-global:fstd "ide  la")
                               )
                          (row "      "
                               (font #:ivan:in-global:fstdb "C")
                               (font #:ivan:in-global:fstd "lassification et  l'")
                               )
                          (row "      "
                               (font #:ivan:in-global:fstdb "E")
                               (font #:ivan:in-global:fstd "xplication")
                               )
                          ""
                          (row "(C) IRIMAG"
                               (font #:ivan:in-global:fmicro "(*)")
                               "-INRIA, Projet Sherpa, 1991-1992"
                               )
                          (row (font #:ivan:in-global:fstd #M#:txt-const:vers-util-1)
                               (font #:ivan:in-global:fstdb "1.93.12 ")
                               (font #:ivan:in-global:fstd #M#:txt-const:vers-util-2)
                               )
                          ""
                          (font #:ivan:in-global:fstdb "Auteur : Franois CRUYPENNINCK (CNAM Grenoble)")
                          ""
                          (font #:ivan:in-global:fstd "N'hsitez pas  nous contacter pour toute demande ")
                          (font #:ivan:in-global:fstd "de renseignement, aussi bien sur le projet SHERPA ")
                          (font #:ivan:in-global:fstd "que sur les logiciels Shirka, IVAN et IACE.")
                          ""
                          (font #:ivan:in-global:fstd "Contact: ")
                          ""
                          (font #:ivan:in-global:fstdb "     Francois RECHENMANN")
                          (font #:ivan:in-global:fstd  "     LIFIA / IMAG Projet SHERPA")
                          (font #:ivan:in-global:fstd  "     46, avenue Flix Viallet")
                          (font #:ivan:in-global:fstd  "     38031 GRENOBLE Cedex 1")
                          (font #:ivan:in-global:fstdb "     e_mail : Francois.Rechenmann@imag.fr")
                          (font #:ivan:in-global:fstdb "-----------------------------------------------")
                          (font #:ivan:in-global:fmicro "(*) Accord INRIA, CNRS, UJF (Grenoble 1), INPG")
                          )
                  ))
         (:affichage-doc list-doc)
         )
    )

;;;***************************************************************************************
;;; affichage dans une fenetre de la documentation contenue dans la liste 'list'

(de :affichage-doc (list)
    (lets ((bouts-daccord  (standardbutton #M#:txt-const:retour-iace))
           (doc (header (icon 0 0 ic-artemis)
                        #M#:txt-const:doc
                        bouts-daccord
                        (view)
                        (apply 'column list)
                        t
                        t
                        )
                )
           )
          (set-action bouts-daccord ':action-bouts-daccord)
          (add-application doc)
          )
    )

;;;***************************************************************************************
;;; action du bouton d'accord = fermer l'application

(de :action-bouts-daccord (b)
    (remove-application (send 'father b))
    )

;;;***************************************************************************************
;;; deroulement des actions charger, sauver, concept, ...
;;;***************************************************************************************

;;;***************************************************************************************
;;; permet de modifier le repertoire courant (variable #:shirka:*bases*)

(de :action-bouts-chemin (b)
    (lets
          (
           (edl-chemin (lineedit 0 0 (+ #5wc (send 'width #:shirka:*bases*))
                               #:shirka:*bases*))
           (img-chemin
                   (#:ivan:ed-util:boxedimage 
                           (row 
                                (font #:ivan:in-global:fstd #M#:txt-const:chemin)
                                edl-chemin
                                )))
           (list-dir 
                   (car (#:systeme:unix:catalogue (pathname #:shirka:*bases*))))
           (nom-chemin t)
           )
          
          (set-action edl-chemin 
                  (lambda (edl) 
                          (setq #:shirka:*bases* (send 'get-line edl))
                          ))
          
          (while nom-chemin
                 (setq edl-chemin (lineedit 0 0 (+ #5wc (send 'width #:shirka:*bases*)) #:shirka:*bases*) 
                       img-chemin (#:ivan:ed-util:boxedimage (row (font #:ivan:in-global:fstd #M#:txt-const:chemin)
                                                                  edl-chemin
                                                                  )
                                          )
                       list-dir 
                       (car (#:systeme:unix:catalogue (pathname #:shirka:*bases*)))
                       nom-chemin (ask-string (column (font #:ivan:in-global:fstdb #M#:txt-const:changer-rep)
                                                      ""
                                                      img-chemin
                                                      ""
                                                      (font #:ivan:in-global:fstd
                                                            #M#:txt-const:pos-rep
                                                            )
                                                      )
                                          list-dir
                                          t
                                          )
                       )  
                 (cond ((null nom-chemin) nil)
                       ((equal nom-chemin "..") (#:systeme:unix:aller-dans nom-chemin)
                        (if (equal (plength (current-directory)) 2)
                            (setq #:shirka:*bases* "/")
                            (setq #:shirka:*bases* (current-directory))
                            )
                        )
                       ( t (catenate #:shirka:*bases* nom-chemin "/")
                          (#:systeme:unix:aller-dans (pathname (catenate #:shirka:*bases* nom-chemin "/")))
                          (setq #:shirka:*bases* (current-directory))
                          )
                       )
                 )
          )
    )

;;;***************************************************************************************
;;; action du bouton charger = ouvrir une fenetre pour saisir le nom de la base a charger

(de :action-bouts-charger (b)
    (lets ((edl-chemin (lineedit 0 0 (+ #5wc (send 'width #:shirka:*bases*)) #:shirka:*bases*))
           (img-chemin (#:ivan:ed-util:boxedimage (row (font #:ivan:in-global:fstd #M#:txt-const:chemin)
                                                       (font #:ivan:in-global:fstdb edl-chemin)
                                                       )
                               )
                   )
           (list-files (cdr (#:systeme:unix:catalogue (pathname #:shirka:*bases*))))
           (nom-base (ask-strings (column (font #:ivan:in-global:fstdb #M#:txt-const:charger)
                                          ""
                                          img-chemin
                                          ""
                                          (row (font #:ivan:in-global:fstd #M#:txt-const:pour-charger)
                                               (font #:ivan:in-global:fstd #M#:txt-const:faire)
                                               )
                                          )
                             list-files nil 1 (length list-files) nil
                             )
                   )
           (chem (send 'get-line edl-chemin))
           )
          (when nom-base
                (mapc (lambda (nom-f)
                              (lets ((nom-fichier (catenate chem nom-f))
                                     (nbel (length ({fc-interf}:ch-list-concepts (component 'top b))))
                                     )
                                    (:inhiber-com b)
                                    (ifn (:charger-fic nom-fichier b)
                                         (progn (:action-sa-warn b nil t
                                                        (list (row (font #:ivan:in-global:fstdb #M#:txt-const:echec)
                                                                   (font #:ivan:in-global:fstd
									(catenate #M#:txt-const:le-fic
										  #M#:txt-const:pas-ouv)
                                                                         )
                                                                   )
                                                              )
                                                        )
                                                (:autoriser-com (component 'top b))
                                                ) 
                                         (:maj-concept-charger nbel b)
                                         (if (and (componentp 'ivan b) (activatedp (component 'ivan b)))
                                             (#:ivan:in-interf:maj-concept-charger nbel (component 'bout-charger
                                                                                                (component 'ivan b)
                                                                                                )
                                                     )
                                             )
                                         ;hy--------------------------------------------------------------------------
                                         ; mettre a jour le reseau hypertexte dans le cas ou il n'y a que iace actif.
                                         (#:ivan:hy-interf:hy-build-web #:ivan:hy-global:web nom-fichier)
                                         ;----------------------------------------------------------------------------
                                         (:autoriser-com (component 'top b)) 
                                         (:action-sa-warn
                                                 b nil nil
                                                 (list (row (font #:ivan:in-global:fstd #M#:txt-const:sel-cp)
                                                            )
                                                       )
                                                 ) 
                                         )
                                    )
                              )
                      nom-base 
                      )
                )
          )
    )

;;;***************************************************************************************
;;; mise a jour du concept pour le raffraichissement de l'ecran <ivan>

(de :maj-concept-charger (nbel b)
    (lets ((i)
           ;hy--------------------------------------------------------
           ; recherche du repertoire hypertexte et liste des fichiers
           (repertoire nom-fichier) ; attention variable initialisee par ivan et iace dans "action-bouts-charger".
           (reptext (catenate repertoire "-hytext/"))
           )
          (for (i 0 1 (- nbel 1) 'ok)
               (menu-delete-item (component 'menu-concept b) 0 0)
               )
          (setq mots-cle (cdr (#:systeme:unix:catalogue (pathname reptext))))
          ;----------------------------------------------------------
          ({fc-interf}:ch-list-concepts (component 'top b) (:const-ch-list-concepts (component 'top b)))
          ({fc-interf}:ch-vect-classes  (component 'top b) (:const-ch-vect-classes  (component 'top b)))
          (:creer-menu (component 'top b))
          
          )
    )

;;;***************************************************************************************
;;; chargement de schemas inscrits dans un fichier (repris de FR-fct-commandes.ll)

(de :charger-fic (nom-fichier b)
    (if (probefile nom-fichier)
        (progn (:action-sa-warn b nil nil (list (row (font #:ivan:in-global:fstd #M#:txt-const:pat-charg-fic)
                                                     (font #:ivan:in-global:fstdb nom-fichier)
                                                     )
                                                )
                       )
(when #:trace:cla
               (print "Chargement de la base " nom-fichier)
)
               (with ((inchan (openi nom-fichier))
                      (typech '|{| 'cmsymb)
                      (typech '|}| 'cmsymb)
                      (typech '|&| 'cmsymb)
                      (typech '|[| 'cmsymb)
                      (typech '|]| 'cmsymb)
                      (typech '|;| 'cmsymb)
                      )
                     (#:iace:fc-util:impr-res (tag erreur-descr (untilexit eof
                                                                        (let ((caract (read)))
                                                                             (setq %deja-compile ())
                                                                             (if (consp caract)
                                                                                 (eval caract)
                                                                                 (def-sh (#:iace:fc-util:trad-sch caract ()))
                                                                                 )
                                                                             )
                                                                        )
                                                   nom-fichier
                                                   )
                             )
                     )
               (:action-sa-warn b nil nil (list (row (font #:ivan:in-global:fstd #M#:txt-const:sel-cp)
                                                     )
                                                )
                       )
               t
               )
(when #:trace:cla
        (print "Erreur : la base " nom-fichier " n'existe pas.")
)
        (#:iace:fc-util:impr-res '())
        nil
        )
    
    )

;;;***************************************************************************************
;;; renvoyer la liste plate contenant les atomes de list
;;; (on ne considere qu'une occurence pour chaque atome)

(de :list-plate (list)
    (let ((l (:list-plate2 list))
          li
          )
         (mapc '(lambda (atom) 
                        (unless (member atom li)       
                                (setq li (cons atom li))
                                )
                        )
               l
               )
         (reverse li)
         )
    )

(de :list-plate2 (list)
    (cond ((null list) nil)
          ((atom list) (cons list nil))
          ( t (append (:list-plate2 (car list))
                      (:list-plate2 (cdr list)) 
                      )
             )
          )
    )

;;;***************************************************************************************
;;; Creation du menu deroulant associe au bouton de choix d'un concept

(de :creer-menu (appli)
    (let ((list-concept ({fc-interf}:ch-list-concepts appli))
          (pos 0)
          (menu (component 'menu-concept appli))
          )
         (while list-concept (let ((concept (car list-concept)))
                                  (menu-insert-item menu
                                          0
                                          pos
                                          (format () "~18@A" (string concept))
                                          1
                                          `(lambda (a)
                                                   (:action-menu-concept (quote ,appli) (quote ,pos) t)
                                                   a
                                                   )
                                          )
                                  (setq pos (+ pos 1))
                                  (nextl list-concept)
                                  )
                )
         )
    )

;;;***************************************************************************************
;;; extraction des concepts d'une base Shirka

(de :const-ch-list-concepts (appli)
    (lets ((schema-objet (#:iace:fc-util:schema 'objet))
           (list-concepts (#:ivan:ed-util:diff-liste
                                  (:l-spec-vraies schema-objet)
                                  '(objet-simple schema attribut facette intervalle attach-proc erreur)
                                  )
                   )
           )
          (when #:ivan:in-global:bobjet (newr list-concepts 'objet))
          (sortl list-concepts)
          )
    )

;;;***************************************************************************************
;;; liste de toutes les specialisations directes de "schema"

(de :l-spec-vraies (schema)
    (prog1 (:l-spec-vraies1 schema)
           (#:iace:fc-util:demarquer-fils schema)
           )
    )

(de :l-spec-vraies1 (schema)
    (mapcan '(lambda (spec)
                     (unless (or (getprop spec 'est-un-filtre)
                                 (getprop spec 'spec-loc)
                                 (getprop spec 'traite)
                                 )
                             (putprop spec t 'traite)
                             (list spec)
                             )
                     )
            (#:iace:fc-util:l-spec schema)
            )
    )

;;;***************************************************************************************
;;; construire, pour chaque concept, la hierarchie de ses classes et la liste plate
;;; de ses classes

(de :const-ch-vect-classes (appli)
    (lets ((list-concepts ({fc-interf}:ch-list-concepts appli))
           (vect (makevector (length list-concepts) ()))
           list-h
           list-c
           (rg 0)
           )
          (while list-concepts 
                 (lets ((concept (car list-concepts))
                        (list-h (:list-hierarchie concept))
                        (list-c (:list-plate list-h))
                        )
                       (vset vect rg (list list-h list-c))
                       (setq rg (+ rg 1))
                       (nextl list-concepts)
                       )
                 )
          vect
          )
    )

;;;***************************************************************************************
;;; renvoyer la hierarchie des sous-classes de "classe-mere"
;;;
;;;    La liste (p f1 (f2 pf1 (pf2 apf1 apf2) pf3) (f3 pf4)) est la representation de :
;;;                               p
;;;                               |
;;;               |---------------|---------------|
;;;               f1              f2              f3
;;;                               |               |
;;;                    |----------|----------|    |
;;;                   pf1        pf2        pf3  pf4
;;;                               |
;;;                         |-----|-----|
;;;                         apf1        apf2

(de :list-hierarchie (classe-mere)
    (let ((list-classe-mere (sortl (:l-spec-vraies (#:iace:fc-util:schema classe-mere))))
          (list-resultat (list classe-mere))
          )
         (while list-classe-mere
                (lets ((sous-classe (car list-classe-mere))
                       (list-sous-classe-trav (:list-hierarchie sous-classe))
                       (list-sous-classe (if (= (length list-sous-classe-trav) 1)
                                             list-sous-classe-trav
                                             (list list-sous-classe-trav)
                                             )
                               )
                       )
                      (setq list-resultat (append list-resultat list-sous-classe))
                      (nextl list-classe-mere)
                      )
                )
         list-resultat
         )
    )

;;;***************************************************************************************
;;; action du bouton sauver = ouvrir une fenetre pour saisir le concept + classes a sauver

(de :action-bouts-sauver (b)
    (lets ((sm-concepts     (stringmenu (mapcar 'string ({fc-interf}:ch-list-concepts (component 'top b)))))
           (sm-classes      (stringmenu nil))
           (scroll-concepts (scroller 0 0 #20wc #10hc sm-concepts t t))
           (scroll-classes  (scroller 0 0 #20wc #10hc sm-classes t t))
           (clu-concepts    (#:ivan:ed-util:clusterimage scroll-concepts #M#:txt-const:cpts))
           (clu-classes     (#:ivan:ed-util:clusterimage scroll-classes #M#:txt-const:classes))
           (titre-toutes    (font #:ivan:in-global:fstd #M#:txt-const:toutes-cl))
           (bouts-toutes    (standardbutton (centeredview (rectangle 0 0 (send 'width clu-classes) 0) titre-toutes)))
           (edl-chemin      (lineedit 0 0 (+ #5wc (send 'width #:shirka:*bases*)) #:shirka:*bases*))
           (img-concepts    (column clu-concepts (rectangle 0 0 0 (send 'height bouts-toutes))))
           (img-classes     (column clu-classes bouts-toutes))
           (fleche-droite   (centeredview (rectangle 0 0 (send 'width (icon 0 0 ic-arrow-right))
                                                (send 'height img-classes)
                                                ) 
                                  (icon 0 0 ic-arrow-right)
                                  )
                   )
           (fleche-bas (centeredview (rectangle 0 0 (send 'width img-classes) 0) (icon 0 0 ic-arrow-down)))
           (img-scrollers (row img-concepts fleche-droite (column img-classes fleche-bas)))
           (img-chemin (#:ivan:ed-util:boxedimage (view (rectangle 0 0 (send 'width img-scrollers) 0)
                                                        (row (font #:ivan:in-global:fstd #M#:txt-const:chemin)
                                                             (font #:ivan:in-global:fstdb edl-chemin)
                                                             )
                                                        )
                               )
                   )
           nom-base
           nom-fichier
           (bsauver t)
           bfini
           )
          (inhibit-application bouts-toutes)
          (add-component (component 'top b) 'bouts-toutes bouts-toutes)
          (add-component (component 'top b) 'sm-concepts sm-concepts)
          (add-component (component 'top b) 'scroll-classes scroll-classes)
          (add-component (component 'top b) 'sm-classes  sm-classes)
          ({application}:userdata sm-concepts (list ({fc-interf}:ch-vect-classes (component 'top b)) 
                                                    (component 'scroll-classes b)
                                                    (component 'sm-classes b)
                                                    (component 'bouts-toutes b)
                                                    )
                  )
          ({application}:userdata bouts-toutes (component 'sm-classes b))
          (set-action sm-concepts  ':action-sm-concepts)
          (set-action bouts-toutes ':action-bouts-toutes)
          (:inhiber-com b)
          (while (not bfini)
                 (setq nom-base (ask-string (column (font #:ivan:in-global:fstdb #M#:txt-const:r-sauv)
                                                    ""
                                                    img-scrollers
                                                    img-chemin
                                                    (centeredview (rectangle 0 0 (send 'width img-chemin) 0)
                                                            (icon 0 0 ic-arrow-down)
                                                            )
                                                    (font #:ivan:in-global:fstd #M#:txt-const:nom-fic-sau)
                                                    )
                                        ({fc-interf}:ch-fichiers (component 'top b))
                                        t
                                        )
                       )
                 (ifn nom-base
                      (setq bfini t
                            bsauver nil)
                      ({fc-interf}:ch-fichiers (component 'top b)
                              (:list-plate (cons nom-base 
                                                 ({fc-interf}:ch-fichiers (component 'top b))
                                                 )
                                      )
                              )
                      (setq nom-fichier (catenate (send 'get-line edl-chemin) nom-base)  
                            #:shirka:*bases* (send 'get-line edl-chemin)
                            )
                      (when (probefile nom-fichier)
                            (setq bsauver
                                  (confirm (column (font #:ivan:in-global:fstdb #M#:txt-const:attention)
                                                   (row (font #:ivan:in-global:fstd #M#:txt-const:le-fic)
                                                        (font #:ivan:in-global:fstdb (string nom-base))
                                                        (font #:ivan:in-global:fstd #M#:txt-const:ecrase)
                                                        )
                                                   (row (font #:ivan:in-global:fstd #M#:txt-const:clic-on)
                                                        (font #:ivan:in-global:fstdb #M#:txt-const:d-accord)
                                                        (font #:ivan:in-global:fstd #M#:txt-const:pour-sauver)
                                                        )
                                                   (row (font #:ivan:in-global:fstdb #M#:txt-const:annuler)
                                                        (font #:ivan:in-global:fstd #M#:txt-const:pour-aband)
                                                        )
                                                   )
                                          )
                                  )
                            )
                      (when bsauver
                            (if (null (send 'get-selected-strings sm-classes))
                                (:action-sa-warn b nil t 
                                        (list (row (font #:ivan:in-global:fstdb #M#:txt-const:sauver-imp)
                                                   (font #:ivan:in-global:fstd #M#:txt-const:sel-cl-ou-clic)
                                                   (font #:ivan:in-global:fstdb #M#:txt-const:annuler)
                                                   (font #:ivan:in-global:fstd #M#:txt-const:pour-aband)
                                                   )
                                              )
                                        )
                                (setq bfini t)
                                )
                            )
                      )
                 
                 )
          (ifn  bsauver
               (:action-sa-warn b nil nil nil)
               (:sauver-fic nom-fichier b (mapcar '#:ivan:ed-util:symbole (send 'get-selected-strings sm-classes)))
               )
          (:autoriser-com b) 
          )
    )

;;;***************************************************************************************
;;; action du sm de choix d'un concept lors d'une sauvegarde

(de :action-sm-concepts (sm)
    (lets ((concept (send 'get-selected-string sm))
           (pos     (send 'get-selected-rank sm))
           (list-param ({application}:userdata sm))
           (vect           (nth 0 list-param))
           (scroll-classes (nth 1 list-param))
           (sm-classes     (nth 2 list-param))
           (bouts-toutes   (nth 3 list-param))
           list-classes 
           )
          (ifn pos
               (inhibit-application bouts-toutes)
               (setq list-classes  (vref vect pos))
               (authorize-application bouts-toutes)
               )
          (send 'set-strings sm-classes (mapcar 'string (cadr list-classes)))
          (send 'selection-limit sm-classes (length (cadr list-classes)))
          (send 'reinitialise scroll-classes)
          )
    )

;;;***************************************************************************************
;;; action du bouton de selection de toutes les classes a sauvegarder lors d'une sauvegarde

(de :action-bouts-toutes (b)
    (let ((sm-classes ({application}:userdata b))
          (list-classes (send 'get-strings sm-classes))
          )
         (send 'set-selected-strings sm-classes list-classes)
         )
    )

;;;***************************************************************************************
;;; sauvegarde des schemas d'instances dans un fichier

(de :sauver-fic (nom-fichier b l-nom-classe)
    (:action-sa-warn b nil nil (list (row (font #:ivan:in-global:fstd #M#:txt-const:pat-sauve)
                                          (font #:ivan:in-global:fstdb nom-fichier)
                                          )
                                     )
            )
    (with ((outchan (openo nom-fichier)))
          (protect (progn (mapc '(lambda (nom-classe)
                                         (if (getprop nom-classe 'schema)
                                             (mapc '(lambda (nom-sch)
                                                            (#:iace:fc-util:visualiser (#:iace:fc-util:schema nom-sch)
                                                                    nil
                                                                    nil
                                                                    0
                                                                    nil
                                                                    )
                                                            (terpri)
                                                            )
                                                   (#:iace:fc-util:l-inst (#:iace-fc-util:schema nom-classe))
                                                   )
                                             )
                                         )
                                l-nom-classe
                                )
                          (terpri)
                          )
                  (close (outchan))
                  )
          )
    (:action-sa-warn b nil nil (list (row (font #:ivan:in-global:fstd #M#:txt-const:fait))))  
    )

;;;***************************************************************************************
;;; menu deroulant : on a choisi un concept, il faut charger sa representation graphique

(de :action-menu-concept (appli pos bfois)
    (if (member 'objet ({fc-interf}:ch-list-concepts appli))
        ; teste si "objet" est un concept de la liste: alors pas chargement dans iace
        (warn (column (font #:ivan:in-global:fstd #M#:txt-const:imp-dans-iace)
                      (font #:ivan:in-global:fstd #M#:txt-const:utiliser)
                      (row (font #:ivan:in-global:fstdb #M#:txt-const:visu-ob)
                           (font #:ivan:in-global:fstd #M#:txt-const:puis)
                           )
                      )
              )
        (if (and list-instance (if (or (equal " " (send 'get-line (component 'edl-instance appli)))
                                       (equal ""  (send 'get-line (component 'edl-instance appli)))
                                       )
                                   t nil
                                   )
                 )
            (:action-sa-warn appli nil t (list (row (font #:ivan:in-global:fstd #M#:txt-const:nommer)
                                                    )
                                               )
                    )
            (ifn %mode (#:iace:fc-expli:action-bouts-mode appli))
            (lets ((concept (nth pos ({fc-interf}:ch-list-concepts appli)))
                   (list-classes (vref ({fc-interf}:ch-vect-classes appli) pos))
                   (position 0)
                   nbcl
                   )
                  (:action-sa-warn
                          appli nil nil (list (row (font #:ivan:in-global:fstd #M#:txt-const:pat-cpt)
                                                   (font #:ivan:in-global:fstdb (string concept))
                                                   (font #:ivan:in-global:fstd "...")
                                                   )
                                              )
                          )
                  (:inhiber-com appli)
                  ;-hy-------------------------------------------------------------------------------------
                  ; permet de ne pas reinitialiser les bouton de classif quand on revient de l'hypertexte.
                  (if chg-concept-hy
                      ()
                      (setq selection-classe ())
                      (setq selection-instance ())
                      )
                  ;----------------------------------------------------------------------------------------
                  (inhibit-application (component 'bouts-lancer appli))
                  (when (and (componentp 'ivan appli) (activatedp (component 'ivan appli)) bfois)
                        (#:ivan:in-interf:action-menu-concept #:ivan:appli pos nil)
                        )
                  (ifn list-instance
                       (while (nth position ({fc-interf}:ch-list-concepts appli)) ; permet de demarquer tous les arbres
                              (lets ((demarquer-concept (nth position ({fc-interf}:ch-list-concepts appli)))
                                     (demarquer-list-classes (vref ({fc-interf}:ch-vect-classes appli) position))
                                     (list-inst (#:iace:fc-util:l-inst-t
                                                        (#:iace:fc-util:schema (caadr demarquer-list-classes))
                                                        )
                                             )
                                     )
                                    ({fc-interf}:ch-list-classes appli (cadr demarquer-list-classes))
                                    (setq position (+ position 1))
                                    (:demarquer-ed-arbre appli nil)
                                    (mapc (lambda (instance) (:demarquer-ed-arbre appli instance)) list-inst)
                                    )
                              )
                       (lets ((classe (car ({fc-interf}:ch-list-classes appli)))
                              (classe-sel-sym (#:ivan:ed-util:symbole classe))
                              (noeud-classe (getprop classe-sel-sym 'noeud-iace))
                              (instance (#:ivan:ed-util:symbole (send 'get-line (component 'edl-instance appli))))
                              )
                             (send 'set-current (component 'ed-arbre appli) noeud-classe)
                             (send 'nom-inst noeud-classe (string instance))
                             (send 'concept noeud-classe "vrai")
                             (send 'set-image (send 'editordata noeud-classe) (send 'title noeud-classe))
                             (putprop classe-sel-sym noeud-classe 'noeud-iace)
                             (putprop classe-sel-sym noeud-classe instance)
                             (send 'mapnodes (component 'ed-arbre appli)
                                   (lambda (node)
                                           (send 'nom-inst node (string instance))
                                           (send 'set-image (send 'editordata node) (send 'title node))
                                           )
                                   )
                             (mapc '(lambda (classe)
                                            (lets ((classe-sel-sym (#:ivan:ed-util:symbole classe))
                                                   (noeud-classe (getprop classe-sel-sym 'noeud-iace))
                                                   )
                                                  (putprop classe-sel-sym noeud-classe 'noeud-iace)
                                                  (putprop classe-sel-sym noeud-classe instance)
                                                  )
                                            )
                                   ({fc-interf}:ch-list-classes appli)
                                   )
                             (:redessiner-arbre appli)
                             )
                       )
                  (ifn chg-concept-hy (send 'set-line (component 'edl-instance appli) ""))
                  (setq #:ivan:in-global:bminitree nil) 
                  (setq #:ivan:in-global:breduct1  nil)
                  (setq #:ivan:in-global:breduct2  nil)
                  (:reinit-edl-classe  (component 'edl-classe appli) "")
                  ({fc-interf}:ch-concept         appli concept)
                  ({fc-interf}:ch-list-hierarchie appli (car list-classes))
                  ({fc-interf}:ch-list-classes    appli (cadr list-classes))
                  (:reinit-edl-concept (component 'edl-concept appli) concept)
                  (setq nbcl (length ({fc-interf}:ch-list-classes appli)))
                  (:reinit-sm (component 'scroll-attribut appli) (component 'sm-attribut appli) ())
                  ;hy------------------------
                  ; reinit editeur hypertexte
                  ; (:reinit-sm  (component 'scroll-explication appli) (component 'sm-explication appli) ())
                  (:reinit-sm-hy (component 'scroll-explication appli) (component 'sm-explication appli) ())
                  ;--------------------------
                  (setq %liste ())
                  (setq %liste-expli ())
                  (if list-instance (inhibit-application (component 'ed-arbre appli)))
                  (lets ((scroll-arbre (component 'scroll-arbre appli))
                         (bconcept (getprop concept 'noeud-iace))
                         (nouv-ed-arbre (ifn list-instance
                                             (:creer-arbre appli nil)
                                             (ifn bconcept
                                                  (#:iace:fc-complex:creer-arbre appli (component 'ed-arbre appli) nil)
                                                  )
                                             ) 
                                 )
                         nouv-scroll-arbre
                         inside         
                         )
                        (ifn bconcept
                             (progn (send 'pretty nouv-ed-arbre)
                                    (setq inside (send 'image nouv-ed-arbre))
                                    (send 'grow nouv-ed-arbre (+ (send 'x inside) (* 2 (send 'width  inside)))
                                          (+ {scroller}:scrollbarheight (* 2 (send 'height inside)))
                                          )
                                    (setq nouv-scroll-arbre (scroller 0 0 (send 'width scroll-arbre)
                                                                    (send 'height scroll-arbre)
                                                                    nouv-ed-arbre
                                                                    t t
                                                                    )
                                          )
                                    (set-component appli 'ed-arbre nouv-ed-arbre)
                                    (set-action nouv-ed-arbre ':action-ed-arbre)
                                    (send 'set-image scroll-arbre nouv-scroll-arbre)
                                    (set-component appli 'scroll-arbre nouv-scroll-arbre)
                                    (send 'set-current nouv-ed-arbre nil)
                                    (:action-bouts-prettytree (component 'bouts-prettytree appli))
                                    )
                             )
                        (if list-instance (authorize-application (component 'ed-arbre appli)))
                        (ifn list-instance (setq list-concept concept)
                             (ifn bconcept
                                  (setq list-concept (cons concept list-concept))
                                  )
                             )
                        (:autoriser-com appli)
                        (setq chg-concept-hy ())
                        (:action-sa-warn appli nil nil
                                (list (row (font #:ivan:in-global:fstd #M#:txt-const:sel-cl)
                                           )
                                      )
                                )
                        )
                  )
            )
        )
    )

;;;***************************************************************************************
;;; creation du "node" specifique au noeud

(de {fc-noeud}:title (noeud-iace)
    (let ((img-courant (cond ((equal ({fc-noeud}:img noeud-iace) "sur")        (icon 0 0 ic-sur))
                             ((equal ({fc-noeud}:img noeud-iace) "possible")   (icon 0 0 ic-possible))
                             ((equal ({fc-noeud}:img noeud-iace) "impossible") (icon 0 0 ic-impossible))
                             ((equal ({fc-noeud}:img noeud-iace) "complexe")   (icon 0 0 ic-complexe))
                             ( t (icon 0 0 ic-interro))
                             )
                  )
          )
         (cond (#:ivan:in-global:breduct1
                       (if (equal ({fc-noeud}:concept noeud-iace) "faux")
                           (row (font #:ivan:in-global:freduct1 ({fc-noeud}:titre noeud-iace))
                                "-"
                                (boxedimage 0 0 (+ 2 (send 'width (icon 0 0 ic-interro))) #hc img-courant)
                                )
                           (row (font #:ivan:in-global:freduct1 ({fc-noeud}:titre noeud-iace))
                                "-"
                                (font #:ivan:in-global:freduct1 ({fc-noeud}:nom-inst noeud-iace))
                                "-"
                                (boxedimage 0 0 (+ 2 (send 'width (icon 0 0 ic-interro))) #hc img-courant)
                                )
                           )
                       )
               (#:ivan:in-global:breduct2
                       (boxedimage 0 0 (+ 2 (send 'width (icon 0 0 ic-interro))) #hc img-courant)
                       )
               ( t (if (equal ({fc-noeud}:concept noeud-iace) "faux")
                       (row (boxedimage 0 0 (+ 6 ({fc-noeud}:larg noeud-iace)) (+ 4 #hc)
                                    (font #:ivan:in-global:fmicro ({fc-noeud}:titre noeud-iace))
                                    )
                            (boxedimage 0 0 (+ 6 (send 'width (icon 0 0 ic-interro))) (+ 4 #hc) img-courant)
                            )
                       (row (boxedimage 0 0 (+ 6 ({fc-noeud}:larg noeud-iace)) (+ 14 #hc)
                                    (column
                                            (font #:ivan:in-global:fmicro ({fc-noeud}:titre noeud-iace))
                                            (font #:ivan:in-global:fmicro ({fc-noeud}:nom-inst noeud-iace))
                                            )
                                    )
                            (boxedimage 0 0 (+ 6 (send 'width (icon 0 0 ic-interro))) (+ 14 #hc) img-courant)
                            )
                       ) 
                  )      
               )
         )
    )

;;;***************************************************************************************
;;; creation du "node" specifique au noeud

;(defabbrev fc-noeud2 {node}:fc-noeud2)
;
;(defstruct {fc-noeud2}
;           nom
;           titre
;)
;
;(de {fc-noeud2}:title (noeud)
;               (let ((img (row (icon 0 0 ic-peigne)
;                               (font #:ivan:in-global:fmicro ({fc-noeud2}:titre noeud))
;                          )
;                     )
;                    )
;                    (cond (#:ivan:in-global:breduct1 (font #:ivan:in-global:fmicro ({fc-noeud2}:titre noeud)))
;                          (#:ivan:in-global:breduct2 (boxedimage 0 0 #wc #hc ""))
;                          ( t (boxedimage 0 0 (+ 6 (send 'width img))
;                                              (+ 6 #hc)
;                                              img
;                              )
;                          )
;                    )
;               )
; 
;)
;

;;;***************************************************************************************
;;; supprimer la propriete 'noeud' des noms de classe lors du changement de concept

(de :demarquer-ed-arbre (appli instance)
    (mapc (lambda (classe) (when (getprop classe 'noeud-iace)
                                 (remprop classe 'noeud-iace)
                                 )
                  )
          ({fc-interf}:ch-list-classes appli)
          )
    (if instance (mapc (lambda (classe) (when (getprop classe instance)
                                              (remprop classe instance)
                                              )
                               )
                       ({fc-interf}:ch-list-classes appli)
                       )
        )
    )

;;;***************************************************************************************
;;; construction de l'arbre a partir de la liste des classes du concept

(de :creer-arbre (appli nom-inst)
    (let ((list-hierar ({fc-interf}:ch-list-hierarchie appli))
          (list-classe ({fc-interf}:ch-list-classes appli))
          (cur-font (current-font))
          )
         (current-font #:ivan:in-global:fmicro)
         (setq #:ivan:in-global:larg (#:ivan:ed-util:wc (#:ivan:ed-util:larg-max list-classe)))
         (current-font cur-font)
         (let ((ed-arbre (treeeditor 0 0 0 0 nil))
               noeud-pere
               )
              (if (null list-hierar)
                  ed-arbre
                  (setq noeud-pere (omakeq {fc-noeud} nom (string (car list-classe))
                                           nom-inst (string nom-inst)
                                           concept "faux"
                                           titre (string (car list-classe))
                                           img "interro"
                                           larg #:ivan:in-global:larg
                                           )
                        )
                  (send 'set-current ed-arbre nil)
                  (send 'insert ed-arbre noeud-pere)
                  (putprop (car list-hierar) noeud-pere 'noeud-iace)
                  (if nom-inst (putprop (car list-hierar) noeud-pere nom-inst))
                  (:creer-noeud ed-arbre list-hierar nom-inst)
                  )
              )
         )
    )

;;;***************************************************************************************
;;; creation recursive des noeuds

(de :creer-noeud (ed-arbre list-classe nom-inst)
    (let ((pere (car list-classe))
          (list-fils (cdr list-classe))
          )
         (while list-fils
                (let ((car-list-fils (car list-fils)))
                     (if (atom car-list-fils)
                         (:inserer-fils ed-arbre pere car-list-fils nom-inst)
                         (:inserer-fils ed-arbre pere (car car-list-fils) nom-inst)
                         ;noeud avec plus de 10 classes
                         ;                         (if (< (length car-list-fils) 10)
                         (:creer-noeud ed-arbre car-list-fils nom-inst)
                         ;                             (:inserer-fils2 ed-arbre (car car-list-fils) (cdr car-list-fils))
                         ;                         )
                         (send 'set-current ed-arbre pere)
                         )
                     (nextl list-fils)
                     )
                )
         ed-arbre
         )
    )

;;;***************************************************************************************
;;; inserer dans l'arbre le fils si le nombre total de fils ne depasse pas 10

(de :inserer-fils (ed-arbre pere fils nom-inst)
    (lets ((noeud-fils (getprop fils 'noeud-iace))
           (noeud-pere (getprop pere 'noeud-iace))
           nouv-noeud
           )
          (if noeud-fils
              (progn (send 'set-current ed-arbre noeud-pere)
                     (send 'link ed-arbre noeud-pere noeud-fils)
                     (send 'set-current ed-arbre noeud-fils)
                     )
              (setq nouv-noeud (omakeq {fc-noeud} nom (string fils)
                                       nom-inst (string nom-inst)
                                       concept "faux"
                                       titre (string fils)
                                       img "interro"
                                       larg #:ivan:in-global:larg
                                       )
                    )
              (send 'set-current ed-arbre noeud-pere)
              (send 'insert ed-arbre nouv-noeud)
              (putprop fils nouv-noeud 'noeud-iace)
              (if nom-inst (putprop fils nouv-noeud nom-inst))
              )
          )
    )

;;;***************************************************************************************
;;; inserer dans l'arbre le fils si le nombre total de fils depasse 10

;(de :inserer-fils2 (ed-arbre pere list-fils) 
;    (lets ((titre-noeud (length list-fils))
;           (nouv-noeud  (omakeq {fc-noeud2} nom "" 
;                                            titre (catenate " " (string titre-noeud) " classes") 
;                        )
;           )
;           (noeud-pere (getprop pere 'noeud-iace))
;          )
;          (send 'set-current ed-arbre noeud-pere)
;          (send 'insert ed-arbre nouv-noeud)
;          (mapc '(lambda (classe)
;                         (putprop classe nouv-noeud 'noeud-iace)
;                 )
;                 (:list-plate list-fils)
;          )
;    )
;)
;

;;;***************************************************************************************
;;; bouton redessiner

(de :action-bouts-prettytree (b)
    (let ((ed-arbre (component 'ed-arbre b)))
         (send 'pretty ed-arbre)
         )
    )

;;;***************************************************************************************
;;; editeur d'arbre = on a clique un noeud: 
;;; transmettre le nom du noeud a l'editeur de ligne

(de :action-ed-arbre (ea)
    (ifn %mode (#:iace:fc-expli:explication ea))
    (lets ((noeud-courant (send 'get-current ea))
           (nom-noeud (if noeud-courant (string (send 'nom noeud-courant)) ""))
           )
          (if (equal nom-noeud "") (setq selection-classe nil) (setq selection-classe t))
          (:reinit-edl-classe (component 'edl-classe ea) nom-noeud)
          ;cas du pere qui a plus de 10 fils
          ;         (if (typep noeud-courant '{node}:fc-noeud2)
          ;             (:action-sa-warn ea
          ;                              nil nil
          ;                              (list (row (font #:ivan:in-global:fstdb #M#:txt-const:attention)
          ;                                         (font #:ivan:in-global:fstd #M#:txt-const:noeud-ens)
          ;                                    )
          ;                              )
          ;             )
          ;             (:action-sa-warn ea nil nil nil)
          ;         )
          )
    )

;;;***************************************************************************************
;;; action de l'editeur de ligne du choix d'instance

(de :action-edl-instance (edl)
    (ifn (send 'get-current (component 'ed-arbre edl))
         (when t (:action-sa-warn edl nil t (list (font #:ivan:in-global:fstd #M#:txt-const::no-cl-sel)))
               (setq selection-classe nil)
               (setq selection-instance nil)
               (let ((choix ""))
                    (send 'set-line edl choix)
                    )
               )
         (setq %liste ())
         (setq %liste-expli ())
         (:reinit-sm (component 'scroll-attribut edl) (component 'sm-attribut edl) ())
         ;hy------------------------
         ; reinit editeur hypertexte
         ;         (:reinit-sm (component 'scroll-explication edl) (component 'sm-explication edl) ())
         (:reinit-sm-hy (component 'scroll-explication edl) (component 'sm-explication edl) ())
         ;--------------------------
         (ifn (equal (send 'get-line edl) "") 
              (progn (setq selection-instance t)
                     (lets ((classe         (send 'nom (send 'get-current (component 'ed-arbre edl))))
                            (classe-sel-sym (#:ivan:ed-util:symbole classe))
                            (noeud-classe   (getprop classe-sel-sym 'noeud-iace))
                            (instance       (send 'get-line edl))
                            )
                           (send 'set-line edl (string instance))
                           (if (atom list-concept)
                               (setq list-instance instance)
                               (ifn (or (member instance list-instance)
                                        (member (#:ivan:ed-util:symbole instance) list-instance)
                                        )
                                    (setq list-instance (if (cdr (:list-plate (cons instance list-instance)))
                                                            (:list-plate (cons instance list-instance))
                                                            (car (:list-plate (cons instance list-instance)))
                                                            )
                                          )
                                    )
                               )
                           (setq selection-instance t)
                           (send 'set-current (component 'ed-arbre edl) noeud-classe)
                           )                   
                     )
              (setq selection-instance nil)
              )
         (if (and selection-classe selection-instance)
             (when t (authorize-application (component 'bouts-lancer edl))
                     (authorize-application (component 'bouts-reinit edl))
                   )
             (inhibit-application (component 'bouts-lancer edl))
             (inhibit-application (component 'bouts-reinit edl))
             )
         )
    )

;;;***************************************************************************************
;;; action de l'editeur de ligne : mettre a jour les differents scrollers

(de :action-edl-classe (edl)
    (lets ((nom-classe-ch (send 'get-line edl))
           (nom-classe-sym (#:ivan:ed-util:symbole nom-classe-ch))
           list-surclasse
           list-sousclasse
           schema-clas
           )
          (ifn nom-classe-sym
               (inhibit-application (component 'bouts-lancer edl))
               (setq schema-clas (#:iace:fc-util:schema nom-classe-sym)
                     list-surclasse (#:iace:fc-util:peres schema-clas)
                     list-sousclasse (:l-spec-vraies schema-clas)
                     selection-classe t
                     )
               (if (and selection-classe selection-instance)
                   (when t (authorize-application (component 'bouts-lancer edl))
                         (authorize-application (component 'bouts-reinit edl))
                         )
                   (inhibit-application (component 'bouts-lancer edl))
                   (inhibit-application (component 'bouts-reinit edl))
                   )
               )
          (:reinit-sm (component 'scroll-surclasse edl) 
                  (component 'sm-surclasse edl) 
                  (sortl list-surclasse)
                  )
          (:reinit-sm (component 'scroll-sousclasse edl)
                  (component 'sm-sousclasse edl)
                  (sortl list-sousclasse)
                  )
          (if  nom-classe-sym
              (authorize-application (component 'bouts-repos edl))
              (inhibit-application (component 'bouts-repos edl))
              )
          )
    )

;;;***************************************************************************************
;;; applications standard, reduite1, reduite2, generale, selection, repositionner, ...
;;;***************************************************************************************

;;;***************************************************************************************
;;; bouton taille standard

(de :action-bouts-standardtree (b)
    (unless (inhibitedp b)
            (setq #:ivan:in-global:bminitree nil) 
            (setq #:ivan:in-global:breduct1 nil) 
            (setq #:ivan:in-global:breduct2 nil)
            (:redessiner-arbre b)
            )
    )

;;;***************************************************************************************
;;; bouton taille reduite = construction des noeuds avec un texte sans les cadres

(de :action-bouts-reduct1 (b)
    (unless (inhibitedp b)
            (setq #:ivan:in-global:bminitree nil)
            (setq #:ivan:in-global:breduct1 t)
            (setq #:ivan:in-global:breduct2 nil)
            (:redessiner-arbre b)
            )
    )

;;;***************************************************************************************
;;; bouton taille reduite = construction des noeuds dans un petit rectangle

(de :action-bouts-reduct2 (b)
    (unless (inhibitedp b)
            (setq #:ivan:in-global:bminitree nil)
            (setq #:ivan:in-global:breduct1 nil)
            (setq #:ivan:in-global:breduct2 t)
            (:redessiner-arbre b)
            )
    )

;;;***************************************************************************************
;;; bouton minitree = c'est une bascule

(de :action-bouts-minitree (b)
    (lets ((ed-arbre (component 'ed-arbre b)))
          (:inhiber-com b)
          (if #:ivan:in-global:bminitree
              (progn (send 'standardtree ed-arbre)
                     (setq #:ivan:in-global:bminitree nil)
                     (:autoriser-com b)
                     (if (and selection-classe selection-instance)
                         (when t (authorize-application (component 'bouts-lancer b))
                                 (authorize-application (component 'bouts-reinit b))
                               )
                         (inhibit-application (component 'bouts-lancer b))
                         (inhibit-application (component 'bouts-reinit b))
                         )
                     )
              (send 'minitree ed-arbre)
              (setq #:ivan:in-global:bminitree t)
              (authorize-application (component 'bouts-minitree b))
              (authorize-application (component 'bouts-quitter b))
              )
          
          )
    )

;;;***************************************************************************************
;;; repositionner l'arbre sur la classe selectionnee

(de :action-bouts-aller (b)
    (lets ((classe (if (send 'get-current (component 'ed-arbre b))
                       (#:ivan:ed-util:symbole (send 'nom (send 'get-current (component 'ed-arbre b))))
                       )
                   )
           ((complexe pos) (if classe (#:iace:fc-classif:test-complexe classe b)))
           (list-classes (if complexe (cadr (vref ({fc-interf}:ch-vect-classes (component 'top b)) pos))
                             ({fc-interf}:ch-list-classes (component 'top b))
                             )
                   )
           (nbclasses (length list-classes))
           (classe-sel-ch (ask-string (column (row (ifn complexe
                                                        (font #:ivan:in-global:fstd #M#:txt-const:arb)
                                                        (font #:ivan:in-global:fstd #M#:txt-const:arb-com)
                                                        )
                                                   (font #:ivan:in-global:fstdb (string nbclasses))
                                                   (font #:ivan:in-global:fstd #M#:txt-const:cla)
                                                   )
                                              (font #:ivan:in-global:fstd #M#:txt-const:sel-cl-rep)
                                              )
                                  (:list-alpha list-classes)
                                  nil
                                  )
                   )
           (classe-sel-sym (#:ivan:ed-util:symbole classe-sel-ch))
           )
          (when classe-sel-sym
                (:reinit-browser b (getprop classe-sel-sym 'noeud-iace) classe-sel-sym)
                )
          )
    )

;;;***************************************************************************************
;;; renvoyer  la liste 'list' triee par ordre alphabetique

(de :list-alpha (list)
    (sortl (mapcar 'string list))
    )

;;;***************************************************************************************
;;; repositionner l'arbre sur le noeud courant

(de :action-bouts-repos (b)
    (lets ((noeud-courant (send 'get-current (component 'ed-arbre b)))
           (image-noeud (send 'editordata noeud-courant))
           (x (send 'x image-noeud))
           (y (:calc-y noeud-courant b))
           (w (send 'width  image-noeud))
           (h (send 'height image-noeud))
           )
          (send 'ensure-visible (component 'scroll-arbre b) x y)
          (send 'ensure-visible (component 'scroll-arbre b) (+ x w) (+ y h))
          (send 'pretty (component 'ed-arbre b))
          )
    )

;;;***************************************************************************************
;;; renvoyer l'ordonnee de l'image du noeud selectionne

(de :calc-y (noeud-classe c)
    (let ((y (send 'y (send 'editordata noeud-classe))))
         (when (<= (- (send 'height (component 'scroll-arbre c)) y) {scroller}:scrollbarheight)
               (setq y (+ y {scroller}:scrollbarheight))
               )
         y
         )
    )

;;;***************************************************************************************
;;; actions sur les applications surclasse et sousclasse
;;;***************************************************************************************

;;;***************************************************************************************
;;; repositionnement du tableau par rapport a l'arbre des classes, vue de la surclasse

(de :action-sm-surclasse (sm)
    (let ((classe-sel-sym (#:ivan:ed-util:symbole (send 'get-selected-string sm)))
          noeud-classe
          )
         (when classe-sel-sym
               (if (and (equal classe-sel-sym 'objet) (null (getprop 'objet 'noeud-iace)) (not #:ivan:in-global:bobjet))
                   (progn (:action-sa-warn sm nil t (list (font #:ivan:in-global:fstd #M#:txt-const:obj)
                                                          )
                                  )
                          (send 'set-selected-string sm "")
                          )
                   (setq noeud-classe (getprop classe-sel-sym 'noeud-iace))
                   (:reinit-browser sm noeud-classe classe-sel-sym)
                   )
               )
         )
    )

;;;***************************************************************************************
;;; repositionnement du tableau par rapport a l'arbre des classes, vue des sousclasse

(de :action-sm-sousclasse (sm)
    (let ((classe-sel-sym (#:ivan:ed-util:symbole (send 'get-selected-string sm)))
          noeud-classe
          )
         (when classe-sel-sym
               (setq noeud-classe (getprop classe-sel-sym 'noeud-iace))
               (:reinit-browser sm noeud-classe classe-sel-sym)
               )
         )
    )

;;;***************************************************************************************
;;; utilitaires
;;;***************************************************************************************

;;;***************************************************************************************
;;; cette fonction est declenchee pour representer l'arbre sous sa nouvelle forme

(de :redessiner-arbre (b)
    (:inhiber-com b)
    (:action-sa-warn b nil nil (list (row (font #:ivan:in-global:fstd #M#:txt-const:pat-mod-gr)
                                          )
                                     )
            )
    (send 'mapnodes (component 'ed-arbre b) (lambda (node)
                                                    (send 'set-image (send 'editordata node)
                                                          (send 'title node)
                                                          )
                                                    )
          )
    (send 'standardtree (component 'ed-arbre b))
    (:action-sa-warn b nil nil (list (row (font #:ivan:in-global:fstd #M#:txt-const:sel-cl)
                                          )
                                     )
            )
    (:autoriser-com b)
    )

;;;***************************************************************************************
;;; reinitialiser le browser dans le cas ou l'on est parti d'une classe

(de :reinit-browser (comp noeud-iace classe)
    (send 'set-current  (component 'ed-arbre comp) noeud-iace)
    (:action-bouts-repos (component 'bouts-repos comp))
    (when (typep noeud-iace '{node}:fc-noeud2)
          (:reinit-edl-classe (component 'edl-classe comp) classe)
          )
    )

;;;***************************************************************************************
;;; reinitialiser les stringmenu (menu de chaines de caracteres) des scrollers

(de :reinit-sm (scroll sm list)
    (send 'set-strings sm (mapcar 'string list))
    (#:ivan:ed-util:positionne scroll nil)
    )

;;;***************************************************************************************
;;; remise a zero de l'editeur d'hypertexte

(de :reinit-sm-hy (scroll sm list)
    (te-hide-cursor sm)
    ;    (te-show-cursor sm)        
    (te-reset sm)
    (te-delimiter sm 45 nil) ; le caractere - n'est pas un separateur
    (ifn list
         ()
         (mapcar '(lambda (texte)
                          (te-string-insert sm (string texte))
                          (te-line-break sm)
                          )
                 list
                 )
         )
    (:sel-zone sm)
    (te-refresh sm t)
    (te-readonly sm t)
    (#:ivan:ed-util:positionne scroll nil)
    )

;;;***************************************************************************************
;;; reinitialiser l'editeur de ligne (contenant le nom du concept)

(de :reinit-edl-concept (edl concept)
    ({medite}:readonly-flag edl nil)
    (send 'set-line edl (string concept))
    ({medite}:readonly-flag edl t) 
    )

;;;***************************************************************************************
;;; reinitialiser l'editeur de ligne (contenant le nom de la classe)

(de :reinit-edl-classe (edl msg)
    ({medite}:readonly-flag edl nil)
    (send 'set-line edl (string msg))
    ({medite}:readonly-flag edl t) 
    (:action-edl-classe edl)
    )

;;;***************************************************************************************
;;; envoyer le message list-msg au composant sa-warn de l'application appli.

(de :action-sa-warn (appli bcouleur bbeep list-msg)
    (lets ((col-msg (apply 'column list-msg))
           (message (if bcouleur (background wheat col-msg) col-msg))
           )
          (send 'set-image (component 'sa-warn appli) message)
          (when bbeep (tybeep))
          )
    )

;;;***************************************************************************************
;;; action sur textedit qui permet de charger le module hypertexte si le mot selectionne existe

(de :appel-hyper (sm)
    (te-mouse-set-cursor-position sm)
    (if (te-word-under sm) (progn (setq var-hyper t)
                                  (hy-move-to #:ivan:hy-global:tv (te-word-under sm))
                                  )
        )
    )

;;;***************************************************************************************
;;; permet de definir les zones sensibles dans le texte edite

(de hilite-zone (sm)
    (te-extend sm (te-selection sm) te-select)
    )

(de :sel-zone (sm)
    (te-define-style sm 'config 'background (make-named-color "blue") 'foreground (make-named-color "yellow"))
    (setq te-select (te-create-selection sm 'config))
    (te-cursor-top sm)
    (te-search-word-and-map sm mots-cle 'hilite-zone)
    (te-set-selection-style sm 'te-hidden-style te-select)
    (te-set-selection-style sm 'config te-select)
    )

;;;***************************************************************************************
