;;; Repertoire: ~LeShirka/LeIvan/CodeIace/
;;;
;     +----------------------------------------+-----------------------------------------+
;     | fichier :  fc-modif.ll                 | package : #:iace:fc-modif               |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface d'Aide a la Classification et a l'Explication              |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Francois Cruypenninck    1991-1992                        |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     | descriptif du package :                                                          |
;     |                                                                                  |
;     | La fonction principale, contient les fonctions principales permettant de gerer   |
;     | les modifications pendant et apres la classification.                            |
;     | Ce module est inspire des fonctions de modification qui existe deja dans SHIRKA. |
;     | La modification interactive est un nouveau module.                               |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+

;;;***************************************************************************************
;;; personnalisation du package
;;;***************************************************************************************

(defvar #:sys-package:colon '#:iace:fc-modif)

;;;***************************************************************************************
;;; permet de modifier la valeur d'un attribut d'une instance
;;; relance la classification apres modification

(de :action-bouts-modifier (b)
    (ifn (send 'get-selected-strings (component 'sm-attribut b))
         (if (send 'get-current (component 'ed-arbre b))
             (lets ((noeud (send 'get-current (component 'ed-arbre b)))
                    (classe (send 'nom noeud))
                    (instance (if (equal (send 'nom-inst noeud) "")
                                  (send 'get-line (component 'edl-instance b))
                                  (send 'nom-inst noeud)
                                  )
                            )
                    (list-att (#:ivan:ed-util:diff-liste (#:iace:fc-util:l-nom-att-inst-he
                                                                 (#:iace:fc-util:schema (#:ivan:ed-util:symbole classe))
                                                                 )
                                      '(est-un util-par nom-inst)
                                      )
                            )
                    (attribut (ask-string #M#:txt-const:choix-att (#:iace:fc-interf:list-alpha list-att) nil))
                    )
                   (if attribut
                       (lets ((val (string (concat "[" classe "/" instance "]" attribut)))
                              (val-fin (:sup-double val))
                              )
                             (#:iace:fc-interf:reinit-sm (component 'scroll-attribut b)
                                     (component 'sm-attribut b)
                                     (#:iace:fc-interf:list-plate %liste)
                                     )
                             (send 'set-selected-string (component 'sm-attribut b) val-fin)
                             )
                       )
                   )
             (#:iace:fc-interf:action-sa-warn
                     b nil t
                     (list
                           (font #:ivan:in-global:fstd #M#:txt-const:comment-choix)
                           )
                     )
             )
         )
    (if (send 'get-selected-strings (component 'sm-attribut b))
        (progn (#:iace:fc-interf:inhiber-com b)
               (lets ((nom-cl-orig (vref (#:iace:fc-util:schema (#:ivan:ed-util:symbole
                                                                        (send 'get-line (component 'edl-instance b))
                                                                        )
                                                 )
                                         %est-un
                                         )
                              )
                      (noeud-classe (getprop nom-cl-orig 'noeud-iace))
                      )
                     (mapc (lambda (modif-att)
                                   (lets ((list-modif-att (:decomposer modif-att))
                                          (nom-clas (#:ivan:ed-util:symbole (car list-modif-att)))
                                          (nom-inst (#:ivan:ed-util:symbole (cadr list-modif-att)))
                                          (nom-att (#:ivan:ed-util:symbole (caddr list-modif-att)))
                                          (list-valeur (cdddr list-modif-att))
                                          (inst-bid (getprop nom-inst 'inst-bid))
                                          (sch-inst-bid (#:iace:fc-util:schema inst-bid))
                                          (val-inst-bid (#:iace:fc-util:val-inst sch-inst-bid %est-un))
                                          (schema (#:iace:fc-util:schema val-inst-bid))
                                          (sch-clas (#:iace:fc-util:schema nom-clas))
                                          ((int . dom) (#:iace:fc-util:union-int-dom schema val-inst-bid nom-att t))
                                          (modif (#:iace:fc-edatt:edit
                                                         inst-bid nom-inst nom-att inst-bid b int dom schema sch-clas t
                                                         )
                                                 )
                                          (list-nature (#:ivan:ed-util:nature schema nom-att))
                                          (bliste (eq (car list-nature) '$liste-de))
                                          (type-attr-temp (cadr list-nature))
                                          (bspec-loc (equal (ascii (sref (string type-attr-temp) 0)) '%))
                                          (type-attr (if bspec-loc
                                                         (car (#:iace:fc-util:peres (#:iace:fc-util:schema
                                                                                            type-attr-tem
                                                                                            )
                                                                      )
                                                              )
                                                         type-attr-temp
                                                         )
                                                  )
                                          )
                                         (setq %cl-modif (#:iace:fc-interf:list-plate (cons %cl-modif nom-clas)))
                                         (if bliste ; l'attribut est de type liste
                                             (if (eq (car list-valeur) '?)
                                                 ; il n'y a pas de valeur pour cette attribut
                                                 (if (and (not (equal modif '(<))) (not (equal modif '(-))))
                                                     (progn (mapc (lambda (car-modif) 
                                                                          ; la touche valider est actionnee, il faut ajouter
                                                                          (:ajouter-valeur
                                                                                  type-attr nom-inst nom-att car-modif
                                                                                  )
                                                                          )
                                                                  modif
                                                                  )
                                                            (:maj-histo modif-att nom-clas nom-inst nom-att
                                                                    modif bliste "avl" b
                                                                    )
                                                            )
                                                     )
                                                 ; l'attribut contient deja une valeur ou une liste
                                                 ; on modifie ou on supprime
                                                 (if (not (equal modif '(<)))
                                                     (if (not (equal modif '(-)))
                                                         ; il existe une ou plusieurs valeurs
                                                         ;alors on veut remplacer par une ou plusieurs valeur
                                                         (progn (mapc (lambda (car-list)
                                                                              (:supprimer-valeur
                                                                                      type-attr nom-inst
                                                                                      nom-att bliste car-list
                                                                                      )
                                                                              )
                                                                      list-valeur
                                                                      )
                                                                (mapc (lambda (car-modif)
                                                                              ; la touche valider est actionnee, il faut ajouter
                                                                              (:ajouter-valeur
                                                                                      type-attr nom-inst
                                                                                      nom-att car-modif
                                                                                      )
                                                                              )
                                                                      modif
                                                                      )
                                                                (:maj-histo modif-att nom-clas nom-inst nom-att
                                                                        modif bliste "mvl" b
                                                                        )
                                                                )
                                                         ;sinon on supprime toutes les valeurs de l'attributs
                                                         (mapc  (lambda (car-list)
                                                                        (:supprimer-valeur
                                                                                type-attr nom-inst nom-att
                                                                                bliste car-list
                                                                                )
                                                                        )
                                                               list-valeur
                                                               )
                                                         (:maj-histo modif-att nom-clas nom-inst nom-att
                                                                 modif bliste "svl" b
                                                                 )
                                                         )
                                                     )
                                                 )
                                             ; ce n'est pas une liste
                                             (if (eq (car list-valeur) '?)
                                                 ; il n'y a pas de valeur pour cette attribut
                                                 (if (and (not (equal modif "<")) (not (equal modif "-")))
                                                     ; ("<" = Annuler) , ("-" = Supprimer)
                                                     ; la touche valider est actionnee, il faut ajouter
                                                     (progn (:ajouter-valeur type-attr nom-inst nom-att modif)
                                                            (:maj-histo modif-att nom-clas nom-inst nom-att
                                                                    modif bliste "av" b
                                                                    )
                                                            )
                                                     )
                                                 ; l'attribut contient deja une valeur, on modifie ou on supprime
                                                 (if (not (equal modif "<"))
                                                     (if (not (equal modif "-"))
                                                         (progn (:modifier-valeur type-attr nom-inst nom-att modif)
                                                                (:maj-histo modif-att nom-clas nom-inst nom-att
                                                                        modif bliste "mv" b
                                                                        )
                                                                )
                                                         (:supprimer-valeur
                                                                 type-attr nom-inst nom-att bliste list-valeur
                                                                 )
                                                         (:maj-histo modif-att nom-clas nom-inst nom-att
                                                                 modif bliste "sv" b
                                                                 )
                                                         )
                                                     )
                                                 )
                                             )
                                         ; je redonne la valeur t pour que la classification s'effectue sur les nouveaux attributs
                                         ; voir s'il n'y a pas de contradiction avec shirka dans le module de classification car
                                         ; cette valeur est remise a nil dans #:shirka:classif:a-v m-v s-v
                                         (putprop inst-bid t 'a-repondre)
                                         )
                                   )
                           (send 'get-selected-strings (component 'sm-attribut b))
                           )
                     (send 'set-current (component 'ed-arbre b) noeud-classe)
                     (#:iace:fc-classif:action-bouts-lancer b)
                     (setq %cl-modif ())
                     (#:iace:fc-interf:autoriser-com b)
                     )
               )
        )
    )

;;;***************************************************************************************
;;; ajoute une valeur dans un attribut avec controle automatique du type

(de :ajouter-valeur (type-attr nom-inst nom-att modif)
    (lets ((valeur (if (or (equal type-attr 'reel) (equal type-attr 'entier))
                       (#:ivan:ed-util:symbole (implode (explode modif)))
                       (if (equal type-attr 'chaine)
                           modif
                           (#:ivan:ed-util:symbole modif)
                           )
                       )
                   )
           )
          (#:iace:fc-util:a-v nom-inst nom-att valeur)
          )
    )

;;;***************************************************************************************
;;; modifie une valeur dans un attribut avec controle automatique du type

(de :modifier-valeur (type-attr nom-inst nom-att modif)
    (lets ((valeur (if (or (equal type-attr 'reel) (equal type-attr 'entier))
                       (#:ivan:ed-util:symbole (implode (explode modif)))
                       (if (equal type-attr 'chaine)
                           modif
                           (#:ivan:ed-util:symbole modif)
                           )
                       )
                   )
           )
          (#:iace:fc-util:m-v nom-inst nom-att nil valeur)
          )
    )

;;;***************************************************************************************
;;; supprime une valeur dans un attribut avec controle automatique du type

(de :supprimer-valeur (type-attr nom-inst nom-att bliste modif)
    (if bliste
        (lets ((valeur (if (or (equal type-attr 'reel) (equal type-attr 'entier))
                           (#:ivan:ed-util:symbole (implode (explode modif)))
                           (if (equal type-attr 'chaine)
                               modif
                               (#:ivan:ed-util:symbole modif)
                               )
                           )
                       )
               )
              (#:iace:fc-util:s-v nom-inst nom-att valeur)
              )
        (#:iace:fc-util:s-v nom-inst nom-att nil)
        )
    )

;;;***************************************************************************************
;;; mise a jour de la liste d'historique

(de :maj-histo (modif-att nom-clas nom-inst nom-att modif bliste action b)
    (lets ((ligne (#:ivan:ed-util:symbole modif-att))
           (list-hist (#:iace:fc-interf:list-plate %liste))
           (ligne-modif (concat "[" nom-clas "/" nom-inst "]" nom-att "="
                                (if bliste
                                    (if (or (equal action "avl") (equal action "mvl"))
                                        (#:iace:fc-classif:const-liste modif)
                                        "?,"
                                        )
                                    (if (or (equal action "av") (equal action "mv"))
                                        (string modif)
                                        "?"
                                        )
                                    )
                                )
                   )
           )
          (setq %liste ())
          (mapc (lambda (list)
                        (if (eq ligne list)
                            (setq %liste (cons %liste ligne-modif))
                            (setq %liste (cons %liste list))
                            )
                        )
                list-hist
                )
          (#:iace:fc-interf:reinit-sm (component 'scroll-attribut b)
                  (component 'sm-attribut b)
                  (#:iace:fc-interf:list-plate %liste)
                  )
          )
    )

;;;***************************************************************************************
;;; permet de decomposer la chaine modif-att en une liste a quatre elements
;;; (classe, instance, attribut, liste de valeur)

(de :decomposer (modif-att)
    (lets ((pos1 (chrpos #/] modif-att))
           (pos2 (chrpos #/= modif-att))
           (pos3 (chrpos #/, modif-att))
           (pos4 (chrpos #// modif-att))
           (nom-clas (#:ivan:ed-util:symbole (substring modif-att 1 (- pos4 1))))
           (nom-inst (#:ivan:ed-util:symbole (substring modif-att (+ pos4 1) (- pos1 pos4 1))))
           (nom-att (#:ivan:ed-util:symbole (substring modif-att (+ pos1 1) (- pos2 pos1 1))))
           (list-valeur-temp (#:ivan:ed-util:symbole (substring modif-att (+ pos2 1)
                                                             (if pos3 (- (slength modif-att) pos2 2)
                                                                 (- (slength modif-att) pos2 1)
                                                                 )
                                                             )
                                     )
                   )
           (list-valeur (if (chrpos #/, list-valeur-temp)
                            (lets ((pos1 (chrpos #/, list-valeur-temp))
                                   (valeur (#:ivan:ed-util:symbole (substring list-valeur-temp 0 pos1)))
                                   )
                                  (setq valeur-temp (substring list-valeur-temp (+ pos1 1) (slength list-valeur-temp)))
                                  (while (chrpos #/, valeur-temp)
                                         (setq pos1 (chrpos #/, valeur-temp))
                                         (setq valeur (cons valeur (#:ivan:ed-util:symbole
                                                                           (substring valeur-temp 0 pos1)
                                                                           )
                                                            )
                                               )
                                         (setq valeur-temp (substring valeur-temp (+ pos1 1) (slength valeur-temp)))
                                         )
                                  (setq valeur (cons valeur valeur-temp))
                                  (#:iace:fc-interf:list-plate (cons valeur valeur-temp))
                                  )
                            list-valeur-temp
                            )
                   )
           )
          (#:iace:fc-interf:list-plate (mcons nom-clas nom-inst nom-att list-valeur))
          )
    )

;;;***************************************************************************************
;;; permet d'effacer les lignes en double dans l'editeur d'historique

(de :sup-double (ligne)
    (let (status)
         (mapc (lambda (liste)
                       (lets ((pos (chrpos #/= liste))
                              (valeur (substring liste 0 pos))
                              )
                             (if (equal valeur ligne)
                                 (setq status liste)
                                 )
                             )
                       )
               (#:iace:fc-interf:list-plate %liste)
               )
         (ifn status
              (progn (setq status (concat ligne "=?"))
                     (setq %liste (cons %liste status))
                     )
              )
         status
         )   
    )

;;;***************************************************************************************
;;; construction du champ de validation de l'application editeur d'attribut
;;; cas d'une modification
;;;***************************************************************************************

;;;***************************************************************************************
;;; construction de la ligne de validation

(de :const-ch-ligne-valid (appli nom-inst-trav nom-attr bliste btype-simple type-attr valeur-prec bvaleur)
    (lets ((bout-supprimer  (standardbutton #M#:txt-const:supprimer))
           (bout-valider (standardbutton #M#:txt-const:valider))
           (bout-annuler (standardbutton #M#:txt-const:annuler))
           (ligne (#:ivan:ed-util:centeredimage (row bout-valider "  " bout-supprimer "  " bout-annuler)))
           )
          (send 'w ligne (send 'width (component 'img-com appli)))
          ({application}:userdata bout-annuler (list nom-inst-trav nom-attr bliste type-attr valeur-prec))
          ({application}:userdata bout-supprimer (list nom-inst-trav nom-attr bliste type-attr valeur-prec))
          (set-action bout-annuler ':action-bout-arreter)
          (add-component appli 'bout-valider bout-valider)
          ({application}:userdata bout-valider)
          (set-action bout-valider '#:iace:fc-edatt:action-bout-valider)
          (add-component appli 'bout-supprimer bout-supprimer)
          (set-action bout-supprimer '#:iace:fc-edatt:action-bout-indefini)
          (#:ivan:ed-util:clusterimage ligne #M#:txt-const:commande)
          )
    )

;;;***************************************************************************************
;;; action du bouton standard arreter dans une modification

(de :action-bout-arreter (b)
    (lets ((list-param ({application}:userdata b))
           (blist (nth 2 list-param))
           )
          (if blist  (setq sortie '(<))
              (setq sortie "<")
              )    
          (remove-application (component 'top b))
          )
    )

;;;***************************************************************************************
