;;; repertoire: ~/LeShirka/LeIvan/CodeIvan/Editeur/
;
;     +----------------------------------------+-----------------------------------------+
;     | fichier :  ivan-ed-edattr.ll           | package : #:ivan:ed-edattr              |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface de visualisation et d'Aide a la Navigation                 |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Sylvain Grivaud          1991                             |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     | descriptif du package :                                                          |
;     |                                                                                  |
;     | La fonction :edit permet l'edition d'un attribut d'une instance Shirka.          |
;     | Une instance de la structure AIDA {in-edattr} composee de 8 champs est creee.    |
;     | La taille de la fenetre finale est fixe mais les composants sont crees dynami-   |
;     | quement en fonction de l'attribut.                                               |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+

;;;*****************************************************************************************

(defvar #:sys-package:colon '#:ivan:ed-edattr)

;;;*****************************************************************************************

(defvar {scroller}:scrollbarwidth  20)     
(defvar {scroller}:scrollbarheight 20)

(defvar :%l-ea 45)  ; Largeur editeur attribut

(defvar :%h-max 4)  ; Hauteur max pour affichage du commentaire

;;;*****************************************************************************************
;;; fonction creant une instance de la structure {in-edattr} pour permettre l'edition
;;; de l'attribut nom-attr de l'instance nom-inst.

(de :edit (nom-inst nom-attr nom-inst-trav)
    (lets
          (   ; application generale
             (appli (omakeq {in-edattr} x 0 y 0 w 0 h 0) )
             
             ; vecteur du schema de classe
             (schema-clas   (#:ivan:ed-util:classe-de nom-inst-trav))
             
             ; facettes  $domaine , $sauf, $intervalle , $valeur
             (list-dom    (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$domaine))
             (list-sauf   (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$sauf))
             (list-inter  (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$intervalle))
             (list-valeur (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$valeur))
             (card-min    (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$cardmin))
             (card-max    (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$cardmax))
             (bvaleur     (not (null list-valeur)))
             ; sauvegarde valeur precedente de l'attribut
             (valeur-prec
                     (if (null (copy (#:ivan:ed-util:val-attr-sym nom-attr nom-inst-trav)))
                         list-valeur 
                         (copy (#:ivan:ed-util:val-attr-sym nom-attr nom-inst-trav)) ))
             ; nature shirka de l'attribut
             (list-nature (#:ivan:ed-util:nature schema-clas nom-attr))
             ; l'attribut est-il une liste ?
             (bliste (eq (car list-nature) '$liste-de))
             ; type shirka de l'attribut : 
             ; c'est un reel, un entier, une chaine, un symbole ou si c'est un objet 
             ; complexe sa classe. 
             (type-attr-temp (cadr list-nature))
             (bspec-loc (equal (ascii (sref (string type-attr-temp) 0)) '%))
             (type-attr
                     (if bspec-loc
                         ; le type de l'attribut commence par un %. C'est le cas
                         ; de la specialisation locale. Je cherche la classe mere.
                         (car (#:shirka:noyau:peres (#:shirka:noyau:schema type-attr-temp)))
                         ; sinon c'est le type simple ou la classe
                         type-attr-temp))
             ; est-ce un type simple ?
             (btype-simple (not (null (#:ivan:ed-util:type-simple? type-attr))))
             ; est-ce une chaine ?
             (bchaine (eq type-attr 'chaine))
             
             (edl-titre (lineedit 0 0 #40wc ""))
             image-ed
             )
          (unless (listp list-dom)    (setq list-dom    (list list-dom))    )
          (unless (listp list-sauf)   (setq list-sauf   (list list-sauf))   )
          (unless (listp list-inter)  (setq list-inter  (list list-inter))  )
          
(when #:trace:ed
          (PRINT "ENTREE EA ........................................")
          (PRINT "     nom-inst      " nom-inst " " (type-of nom-inst))
          (PRINT "     nom-inst-trav " nom-inst-trav " " (type-of nom-inst-trav))
          (PRINT "     nom-attr      " nom-attr    " " (type-of nom-attr))
          (PRINT "     valeur-prec   " valeur-prec)
          (PRINT "     type-attr     " type-attr) 
          (PRINT "     list-sauf  = " list-sauf)
          (PRINT "     list-inter = " list-inter)
          (PRINT "     card-max   = " card-max)
          (PRINT "     card-min   = " card-min)
)
          
          ; composant top pour reperer le pere de plus haut niveau
          ({application}:components appli (acons 'top appli ()))
          
          ; construction des champs de l'application
          ({in-edattr}:ch-scroll-com   appli  (:const-ch-scroll-com   appli
                                                      nom-attr
                                                      schema-clas))
          ({in-edattr}:ch-titre        appli  (:const-ch-titre        appli 
                                                      nom-attr
                                                      bliste
                                                      type-attr
                                                      bspec-loc))
          ({in-edattr}:ch-sa-action    appli  (:const-ch-sa-action    appli))
          ({in-edattr}:ch-edc-val      appli  (:const-ch-edc-val      appli 
                                                      nom-inst-trav
                                                      nom-attr
                                                      bliste
                                                      btype-simple
                                                      type-attr
                                                      valeur-prec
                                                      bvaleur))
          ({in-edattr}:ch-ligne-valid  appli  (:const-ch-ligne-valid  appli 
                                                      nom-inst-trav
                                                      nom-attr
                                                      bliste
                                                      btype-simple
                                                      type-attr
                                                      valeur-prec
                                                      list-dom
                                                      bspec-loc
                                                      bvaleur))
          ({in-edattr}:ch-scroll-val   appli  (:const-ch-scroll-val   appli
                                                      nom-inst-trav
                                                      nom-attr
                                                      schema-clas
                                                      bliste
                                                      type-attr
                                                      card-min
                                                      card-max
                                                      list-valeur)) 
          ({in-edattr}:ch-scroll-choix appli  (:const-ch-scroll-choix appli
                                                      bliste
                                                      btype-simple
                                                      type-attr
                                                      list-dom))
          ({in-edattr}:ch-contraintes  appli  (:const-ch-contraintes  appli
                                                      list-sauf
                                                      
                                                      list-inter))
          ; assemblage pour constituer l'image finale
          (setq image-ed
                (column
                        (#:ivan:ed-util:centeredimage
                                (view
                                      (filledbox 0 0
                                              ;0 0 0)
                                              (send 'width ({in-edattr}:ch-sa-action  appli)) 619 0)
                                      (column      
                                              ({in-edattr}:ch-titre appli)
                                              "" 
                                              ({in-edattr}:ch-scroll-com appli)
                                              ({in-edattr}:ch-sa-action  appli)
                                              ""  
                                              (row
                                                   ({in-edattr}:ch-scroll-val   appli)
                                                   ({in-edattr}:ch-scroll-choix appli) )
                                              "" 
                                              ({in-edattr}:ch-edc-val     appli)
                                              ""
                                              ({in-edattr}:ch-contraintes appli)
                                              ""
                                              )
                                      )
                                )
                        ({in-edattr}:ch-ligne-valid appli)
                        )
                )
          
          (if (>= (read-from-string aidaversion) 1.5403)
              (progn
                     (send 'new-image
                           appli
                           (header
                                   (rectangle 0 0 32 32) ; logo 
                                   edl-titre
                                   (view) ; pas de bouton
                                   (view) ; pas de commande 
                                   image-ed
                                   t
                                   t
                                   )
                           )
			(#:ivan:ed-util:titre-fenetre appli #M#:txt-const:titre-ed-att)
			(send 'set-line edl-titre (catenate nom-attr #M#:txt-const:de nom-inst))
                     )
              ; SINON
              (send 'new-image
                    appli
                    image-ed
                    )
		(#:ivan:ed-util:titre-fenetre appli (catenate nom-attr #M#:txt-const:de nom-inst))
              )
          
          (when bvaleur
                (:action-sa-warn appli 
                        (list
                              (font #:ivan:ed-global:fstd #M#:txt-const:non-modifiable)
					#M#:txt-const:non-modifiable
                              ""
                              (font #:ivan:ed-global:fstd #M#:txt-const:clique-annul)
			)))
          
          ; dimensionnement de l'application
          (send 'fit-to-contents appli)
          
          ;  l'application est activee
          (grab-application
                  (send 'translate
                        appli
                        #:ivan:ed-global:%x #:ivan:ed-global:%y
                        ))
          ; pas de trace a l'ecran
          't
          )
    )

;;;#########################################################################################
;;;
;;;                                    CONSTRUCTION DES CHAMPS 
;;;
;;;#########################################################################################

;;;*****************************************************************************************
;;; construction de l'image de visualisation du commentaire de l'attribut
;;; suivant la taille de l'image, elle sera affichee ou non dans un scroller

(de :const-ch-scroll-com (appli nom-attr schema-clas)
    (lets
          (  
           ; commentaire de la classe (liste de chaines) 
           (list-comm (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$com)) 
           (list-com
                   (mapcar '(lambda (chaine)
                                    (let ( (chaine-font (font #:ivan:ed-global:fstd chaine)))
                                         chaine-font
                                         ))
                           list-comm))
           ; colonne des lignes
           (col-com (apply 'column list-com))
           
           ; largeur max d'1 ligne en nombre de caracteres (attention #wc = 9)
           (larg (#:ivan:ed-util:larg-max list-comm))
           
           ; hauteur du commentaire (nombre de lignes)
           (haut (ifn (null list-com)
                      (length list-com)))
           
           ; hauteur de l'image commentaire si scroller
           (h-scroll (+ {scroller}:scrollbarheight
                        (#:ivan:ed-util:hc :%h-max)
                        2))
           ; largeur de l'image commentaire si scroller
           (w-scroll (+ {scroller}:scrollbarwidth
                        (#:ivan:ed-util:wc :%l-ea)
                        2))
           
           ; img du commentaire     
           (img-com 
                   (cond
                         ; pas de commentaire = 1 ligne vide
                         ((null list-com)
                          (rectangle 0 0 w-scroll 0)
                          )
                         
                         ; le commentaire tient en largeur et hauteur : boite rectangulaire
                         ((and (<= larg :%l-ea) (<= haut :%h-max))
                          (view
                                (rectangle 0 0 w-scroll (#:ivan:ed-util:hc haut))
                                col-com
                                )
                          )
                         
                         ; le commentaire est trop large et trop haut : scroller   
                         ((and (> larg :%l-ea) (> haut :%h-max))
                          (scroller 0 0
                                  w-scroll h-scroll
                                  col-com
                                  t t
                                  )
                          )
                         
                         ; le commentaire est trop large : scroller horizontal                   
                         ((> larg :%l-ea)
                          (horizontalscroller 0 0
                                  w-scroll h-scroll                               
                                  col-com
                                  )
                          )
                         
                         ; le commentaire est trop haut : scroller vertical              
                         (t         
                            (verticalscroller 0 0
                                    w-scroll h-scroll
                                    col-com
                                    )
                            )
                         ))
           )
          (add-component appli 'img-com img-com)
          
          ; image finale
          (if (null list-com)
              (rectangle 0 0 (+ 12 w-scroll) 0)
              ; sinon il faut sauter une ligne
              (column
		(#:ivan:ed-util:clusterimage img-com #M#:txt-const:comment)
                ""))
          ))
;;;********************************************************************
;;; construction du champ titre. Le type de l'attribut est affiche dans un cadre

(de :const-ch-titre (appli nom-attr bliste type-attr bspec-loc)
    (lets
          (
           (liste-de (if bliste 
			(font #:ivan:ed-global:fstd #M#:txt-const:liste-de)
                        " " ))
           (spec-loc (if bspec-loc
			(font #:ivan:ed-global:fstd #M#:txt-const:spec-loc)
                        "" ))
	   (titre-attr (catenate #M#:txt-const:att-type (string nom-attr)))
           (img (centeredview 
                        (rectangle 0 0 (send 'width (component 'img-com appli)) 0)
                        (row
                             liste-de
                             (font #:ivan:ed-global:fstdb (string type-attr))
                             " "
                             spec-loc)
                        ))
           
           )
          ; image du champ 
          (#:ivan:ed-util:clusterimage img titre-attr)
          ))
;;;*****************************************************************************************
;;; construction d'un editeur de chaine pour saisir la valeur de l'attribut
;;; NB : la structure stringedit permet d'associer des filtres de saisie , de validation
;;;      et un message d'erreur en fonction du type de l'attribut

(de :const-ch-edc-val 
    (appli nom-inst-trav nom-attr bliste btype-simple type-attr valeur-prec bvaleur)
    (lets
          (
           ; bouton d'effacement de l'editeur de chaine
           (bout-effacer (standardbutton #:ivan:ed-global:txt-effacer))
           ; valeur initiale de l'editeur de chaine
           (val-initiale (#:ivan:ed-util:nom-ch
                                 (if (or bliste
                                         (null valeur-prec))
                                     '"" 
                                     valeur-prec)) )
           (titre-edc (row
                           (font #:ivan:ed-global:fstdb
                                 (string nom-attr))
                           (font #:ivan:ed-global:fstdb
                                 " = ") ))
           ; largeur de l'editeur de chaine calculee dynamiquement
           (larg 
                 (- (send 'width (component 'img-com appli))
                    (send 'width (#:ivan:ed-util:centeredimage titre-edc))
                    7
                    #wc
                    (send 'width bout-effacer)
                    ))
           
           ; editeur de chaine pour saisir la valeur de l'attribut
           (edc-val (stringedit 0 0 larg val-initiale))
           
           (img-edc (row
                         (#:ivan:ed-util:centeredimage titre-edc)
                         (#:ivan:ed-util:boxedimage edc-val)
                         " "
                         bout-effacer))
           )
          
          ; fonctions de validation et filtre de saisie en fonction du type
          (cond
                ( (eq type-attr 'entier)
                 (send 'filter-func edc-val ':filtre-edc-entier)
                 (send 'validate-func edc-val ':valid-edc-entier) )
                ( (eq type-attr 'reel)
                 (send 'filter-func edc-val ':filtre-edc-reel)
                 (send 'validate-func edc-val ':valid-edc-reel) )
                ( t
                   (send 'filter-func edc-val ':filtre-edc)
                   (send 'validate-func edc-val ':valid-edc) )
                )
          ; fonction d'erreur de l'editeur de chaine
          (send 'error-func edc-val ':erreur-edc-val)
          
          (add-component appli 'edc-val edc-val)
          ({application}:userdata edc-val
                  (list bliste btype-simple type-attr nom-inst-trav nom-attr))
          (set-action edc-val ':action-edc-val)
          
          ({application}:userdata bout-effacer (list nom-inst-trav nom-attr bliste))
          (set-action bout-effacer ':action-bout-effacer)
          
          (when bvaleur (inhibit-application bout-effacer))
          
          ; afficher l'editeur de chaine
	  (#:ivan:ed-util:clusterimage img-edc #M#:txt-const:zone-edi)
          )
    )
;;;-----------------------------------------------------------------------------------------
;;;                               filtres de l'editeur de chaine
;;;-----------------------------------------------------------------------------------------

(de :filtre-edc-entier (edc char)
    ; ENTIER : char doit etre un chiffre, le signe - ou le signe +
    (let 
         ( (bok   (or
                      (digitp char)
                      (equal (cascii "-") char)
                      (equal (cascii "+") char)   
                      ))
          )
         (:filtre-msg bok)
         )
    )
;;;-----------------------------------------------------------------------------------------

(de :filtre-edc-reel (edc char)
    ; REEL : idem entier plus le point decimal
    (let 
         ( (bok   (or
                      (digitp char)
                      (equal (cascii "-") char)
                      (equal (cascii "+") char)   
                      (equal (cascii ".") char)
                      ))
          )
         (:filtre-msg bok)
         )
    )
;;;-----------------------------------------------------------------------------------------

(de :filtre-edc (edc char)
    ; CAS GENERAL : idem REEL plus les caracteres alphabetiques
    (let 
         ( (bok   (or
                      (letterp char)
                      (digitp char)
                      (equal (cascii "-") char)
                      (equal (cascii "+") char)   
                      (equal (cascii ".") char)
                      ;;;               (equal (cascii " ") char)
                      (equal (cascii ",") char) 
                      (equal (cascii "(") char)   
                      (equal (cascii ")") char)
                      (equal (cascii "[") char)
                      (equal (cascii "]") char) ))
          )
         (:filtre-msg bok)
         )
    )
;;;-----------------------------------------------------------------------------------------
;;; envoi d'un message d'erreur si la saisie est incorrecte

(de :filtre-msg (bok)
    (if bok
        (:action-sa-warn (component 'sa-warn edc) '(""))
	(:action-sa-warn (component 'sa-warn edc) '("" #M#:txt-const:caractere-invalide))
        (tybeep)
        )
    bok
    )
;;;-----------------------------------------------------------------------------------------
;;;                               fonctions de validation de l'editeur de chaine

(de :valid-edc-entier (edc)
    ; ENTIER
    (lets
          (
           (rep (send 'get-line edc))
           (val (when (> (slen rep) 0)
                      (read-from-string rep)
                      ))
           )
          (fixp val)
          ))
;;;-----------------------------------------------------------------------------------------

(de :valid-edc-reel (edc)
    ; REEL
    (lets
          (
           (rep (send 'get-line edc))
           (val (when (> (slen rep) 0)
                      (read-from-string rep)
                      ))
           )
          (floatp val)
          ))
;;;-----------------------------------------------------------------------------------------

(de :valid-edc (edc)
    ; CAS GENERAL : on a soit une chaine, soit un symbole
    (lets
          (
           (rep (send 'get-line edc))
           (val (when (> (slen rep) 0)
                      (read-from-string rep)
                      ))
           )
          (or (symbolp val) (stringp val))
          ))
;;;-----------------------------------------------------------------------------------------
;;;                              fonction d'erreur de l'editeur de chaine
;;;-----------------------------------------------------------------------------------------

(de :erreur-edc-val (edc)
    (:action-sa-warn
            (component 'sa-warn edc)
            (list
                  (font #:ivan:ed-global:fstdb #M#:txt-const:valid-impossible)
                  (font #:ivan:ed-global:fstd #M#:txt-const:incorrect-type) ))
    )
;;;*****************************************************************************************
;;; construction d'un selecteur d'application pour afficher l'aide textuelle
;;; sur l'action en cours. 
;;; les messages seront transmis au composant sa-warn par la fonction :action-sa-warn.

(de :const-ch-sa-action (appli)
    (let
         (
          (sa-warn (selectorapplication 0 0
                           (send 'width (component 'img-com appli))
                           (#:ivan:ed-util:hc 3)
                           nil))
          )
         
         (add-component appli 'sa-warn sa-warn)
         ; image finale centree
         (#:ivan:ed-util:clusterimage sa-warn #M#:txt-const:message)
    ))
;;;*****************************************************************************************
;;; construction du scroller de valeurs de l'attribut
;;; l'image ne sera activee que dans le cas d'une liste

(de :const-ch-scroll-val 
    (appli nom-inst-trav nom-attr schema-clas bliste type-attr card-min card-max
           list-valeur)
    (lets
          ( 
           ; liste des valeurs sous forme de symbole sans faire appel au moteur d'inference
           (list-val-sy (#:ivan:ed-util:val-attr-sym nom-attr nom-inst-trav) )
           (list-val-sym
                   (if (null list-val-sy)
                       list-valeur
                       list-val-sy))
           ; liste des valeurs de l'attribut sous forme de chaine
           (list-val-ch
                   (if (listp list-val-sym)
                       (mapcar 'string list-val-sym)
                       (list (string list-val-sym))))
           ; menu de chaine de caracteres des valeurs
           (sm-val (stringmenu list-val-ch))
           ; bouton radio pour insertion en-tete/en fin de liste
           (cb-entete (checkbox 0 0 #20wc #hc #M#:txt-const:ins-tete))
           
           ; largeur du scroller est fixe
           (larg (- (div (send 'width ({in-edattr}:ch-scroll-com appli)) 2) 12))
           
           ; scroller de valeur
           (scroll-val (scroller 0 0
                               larg #10hc 
                               sm-val
                               t t))
           ; titre de l'image
           (titre-img (if bliste #M#:txt-const:valeurs #M#:txt-const:valeur))
           ; image du selecteur de valeurs
           (img-sel-val 
                   (if bliste
                       (column
                               scroll-val
                               (if (or card-min card-max)
                                   ""
                                   (rectangle 0 0 0 0) )
                               (if card-min
                                   (row 
					#M#:txt-const:card-min
					(string card-min)) 
                                   (rectangle 0 0 0 0) 
                                   )
                               (if card-max
                                   (row 
					#M#:txt-const:card-max
					(string card-max)) 
                                   (rectangle 0 0 0 0) 
                                   )
                               ""
                               cb-entete)
                       ; sinon on ne peut pas inserer donc on n'affiche simplement le scroller
                       scroll-val
                       ))
           )
          
          (add-component appli 'scroll-val scroll-val)
          (add-component appli 'sm-val sm-val)
          (add-component appli 'img-sel-val img-sel-val)
          (add-component appli 'cb-entete cb-entete)
          
          ({application}:userdata sm-val (list nom-inst-trav nom-attr type-attr))
          (set-action sm-val ':action-sm-val)
          
          (set-action cb-entete ':action-cb-entete)
          
          ; image renvoyee  
          (if bliste
              (#:ivan:ed-util:clusterimage img-sel-val titre-img)
              (rectangle 0 0 (+ 12 larg) 0)
              )
          
          )
    )
;;;*****************************************************************************************
;;; construction du scroller des valeurs possibles ie soit instances, soit domaine

(de :const-ch-scroll-choix (appli bliste btype-simple type-attr list-dom)
    (lets
          (
           ; liste des valeurs a choisir sous forme de chaine
           (list-val-pos
                   (if (null list-dom)
                       ; construire la liste des instances dans le cas d'un objet
                       (#:ivan:ed-util:diff-liste (#:ivan:ed-util:l-inst-t-ch type-attr)
                               #:ivan:ed-global:%list-inst-trav)
                       ; sinon les valeurs du domaine sont affichees
                       list-dom))
           (img-sel-val  (component 'img-sel-val appli))
           
           (sm-choix     (stringmenu (mapcar 'string list-val-pos)))
           
           (larg (- (div (send 'width ({in-edattr}:ch-scroll-com appli)) 2) 12))
           
           ; scroller
           (scroll-choix (scroller 0 0
                                 larg #10hc
                                 sm-choix
                                 t t))
           ; titre de l'image
           (titre-img 
                   (if (null list-dom)
                       (row 
				#M#:txt-const:inst-de
                            	(string type-attr))
                       ; sinon 
				#M#:txt-const:domaine
			))
           (img-sel-inst
                   (view 
                         (rectangle 0 0 larg (send 'height img-sel-val))
                         scroll-choix
                         )
                   ) 
           )
          
          (add-component appli 'scroll-choix scroll-choix)
          (add-component appli 'sm-choix sm-choix)
          
          ({application}:userdata sm-choix bliste)
          (set-action sm-choix ':action-sm-choix)
          
          (if (and btype-simple (null list-dom))
              ; ce n'est pas un objet et il n'y a pas de domaine
              (rectangle 0 0 0 0)
              ; afficher le selectionneur
              (#:ivan:ed-util:clusterimage img-sel-inst titre-img)
              )
          )
    )
;;;*****************************************************************************************
;;; construction de la ligne de validation avec 2 boutons

(de :const-ch-ligne-valid
    (appli nom-inst-trav nom-attr bliste btype-simple type-attr
           valeur-prec list-dom bspec-loc bvaleur)
    (lets
          (
           (bout-editer  (standardbutton #:ivan:ed-global:txt-editer))
           (bout-valider (standardbutton #:ivan:ed-global:txt-valider))
           (bout-annuler (standardbutton #:ivan:ed-global:txt-annuler))
           (ligne
                  (if btype-simple
                      (#:ivan:ed-util:centeredimage
                              (row bout-valider "   " bout-annuler) )
                      (#:ivan:ed-util:centeredimage 
                              (row bout-valider "   " bout-annuler "   " bout-editer))
                      )
                  )
           )
          
          (send 'w ligne (send 'width (component 'img-com appli)))
          
          ; bouton annuler
          ({application}:userdata bout-annuler 
                  (list nom-inst-trav nom-attr bliste type-attr valeur-prec))
          (set-action bout-annuler ':action-bout-annuler)
          
          ; bouton valider
          (add-component appli 'bout-valider bout-valider)
          ({application}:userdata bout-valider) 
          (set-action bout-valider ':action-bout-valider)
          
          ; bouton editer 
          (add-component appli 'bout-editer bout-editer)
          ({application}:userdata bout-editer
                  (list nom-inst-trav nom-attr type-attr list-dom bspec-loc))
          (set-action bout-editer ':action-bout-editer)
          
          (when bvaleur
                (inhibit-application bout-valider)
                (inhibit-application bout-editer)
                )
          ; image
          (#:ivan:ed-util:clusterimage ligne #M#:txt-const:comande)
          )
    )
;;;*****************************************************************************************
;;; construction du champ des contraintes a partir des facettes "$sauf" et "$intervalle"

(de :const-ch-contraintes (appli list-sauf list-inter)
    (lets
          (
           ; CAS DE $intervalle
           (list-binf    (:list-bornes list-inter 3) )
           (list-bsup    (:list-bornes list-inter 4) )
           (sm-binf      (stringmenu (mapcar 'string list-binf)))
           (sm-bsup      (stringmenu (mapcar 'string list-bsup)))
	   (tit-binf     (font #:ivan:ed-global:fstd #M#:txt-const:borne-inf))
	   (tit-bsup     (font #:ivan:ed-global:fstd #M#:txt-const:borne-sup))
           (titre        (row
                              (rectangle 0 0 {scroller}:scrollbarwidth 0)
                              tit-binf
                              "  " 
                              tit-bsup) )
           (scroll-inter (scroller 0 0
                                 (send 'width titre) #5hc
                                 (row sm-binf "  " sm-bsup)
                                 t t))
           ; image de la facette $intervalle
           (img-inter (if (null list-inter)
                          (rectangle 0 0 (send 'width titre) 0)
                          (#:ivan:ed-util:clusterimage
                                (column
                                          titre
                                          scroll-inter)
				#M#:txt-const:intervalle) ))
           
           ; CAS DE $sauf
           (sm-sauf      (stringmenu (mapcar 'string list-sauf)))
           (larg-sauf    (- (send 'width ({in-edattr}:ch-scroll-com appli))
                            (send 'width img-inter)
                            12))
           (scroll-sauf  (scroller 0 0
                                 larg-sauf #5hc
                                 sm-sauf
                                 t t))
           ; image de la facette $sauf
           (img-sauf     (if (null list-sauf)
                             (rectangle 0 0 (+ larg-sauf 12) 0)
                             (#:ivan:ed-util:clusterimage
                                scroll-sauf
				#M#:txt-const:vals-interdites) ))
           
           ; image finale
           (img 
                (if (and (null list-sauf) (null list-inter) )
                    (rectangle 0 0 0 0)
                    ; sinon
                    (row img-sauf img-inter)
                    ))
           )
          
          (send 'w sm-binf (send 'width tit-binf))
          (send 'w sm-bsup (send 'width tit-bsup))
          
          (when (or list-sauf list-inter)
                (send 'resize img 
                      nil
                      (max (send 'height img-sauf) (send 'height img-inter))
                      ))
          
          (set-action sm-sauf ':action-reinit-sm)
          (set-action sm-binf ':action-reinit-sm)
          (set-action sm-bsup ':action-reinit-sm)
          
          ; image renvoyee
          img
          )
    )
;;;*****************************************************************************************
;;; construit la liste des bornes min ou max en fonction la variable pos

(de :list-bornes (int pos)
    (let
         (
          (list nil)
          )
         (cond
               ((null int) nil) 
               ((atom int) (newr list (#:ivan:ed-util:val-inst-ch int pos)))
               (t (append (:list-bornes (car int) pos)
                          (:list-bornes (cdr int) pos) )))))

;;;#########################################################################################
;;;                                   
;;;                                    ACTIONS DES CHAMPS 
;;;
;;;#########################################################################################

;;;*****************************************************************************************
;;; action du bouton standard de validation
;;; l'editeur de chaine est declenche et ajoute directement son contenu a l'attribut
;;; si l'ajout a ete possible, le booleen bok prend la valeur vrai et la fenetre est 
;;; fermee; sinon, un message d'action est envoye et la fenetre ne doit pas etre fermee

(de :action-bout-valider (b)
    (let
         (
          (bok (:action-edc-val (component 'edc-val b)) )
          )
         ;  la valeur est admissible et a ete rajoutee : on ferme la fenetre
         (when bok
               (remove-application (component 'top b))
(when #:trace:ed
               (PRINT "SORTIE EA ........................................")
)
               )
         )
    )
;;;*****************************************************************************************
;;; action du bouton d'annulation : il faut restaurer la valeur precedente de l'attribut

(de :action-bout-annuler (b)
    (lets
          (
           (list-param    ({application}:userdata b))
           (nom-inst-trav (nth 0 list-param))
           (nom-attr      (nth 1 list-param))
           (bliste        (nth 2 list-param))
           (type-attr     (nth 3 list-param))
           (val-prec      (nth 4 list-param))
           )
          (when (activatedp b)
                (if bliste
                    (progn
                           (#:ivan:ed-util:sup-toutes-valeurs nom-inst-trav nom-attr)
                           (mapc '#:ivan:ed-util:aj-valeur 
                                 (cirlist (#:ivan:ed-util:schema nom-inst-trav))
                                 (cirlist nom-attr)
                                 val-prec
                                 (cirlist 't) )
                           )
                    ; sinon
                    (if 
                        (let
                             ( (val (string val-prec)) )
                             (#:ivan:ed-util:sup-valeur
                                     (#:ivan:ed-util:schema nom-inst-trav) 
                                     nom-attr
                                     nil)
                             (#:ivan:ed-util:aj-valeur
                                     (#:ivan:ed-util:schema nom-inst-trav)
                                     nom-attr
                                     (#:ivan:ed-util:objet-simple val type-attr) 
                                     nil)
                             )
                        ; alors
(when #:trace:ed
                        (PRINT "     restauration ancienne valeur effectuee")
)
                        ; sinon
(when #:trace:ed
                        (PRINT "     probleme ancienne valeur >" 
                               (#:ivan:ed-util:objet-simple val-prec type-attr) "<")
)
                        )
                    )
                ; fermer la fenetre
                (remove-application (component 'top b))
(when #:trace:ed
                (PRINT "SORTIE EA ........................................")
)
                )
          )
    )
;;;*****************************************************************************************
;;; action du bouton standard d'edition d'une instance

(de :action-bout-editer (b)
    (lets
          (
           (list-param    ({application}:userdata b))
           (nom-inst-trav (nth 0 list-param))
           (nom-attr      (nth 1 list-param))
           (type-attr     (nth 2 list-param))
           (list-dom      (nth 3 list-param))
           (bspec-loc     (nth 4 list-param))
           
           (valedc-ch     (send 'get-line (component 'edc-val b))) ; valeur edc (chaine)
           (valedc-sym    (#:ivan:ed-util:symbole valedc-ch))      ; valeur edc (symbole)
           (nom-classe    (#:ivan:ed-util:symbole type-attr))      ; classe de l'attribut
           (list-val      (#:ivan:ed-util:l-inst-t-ch type-attr))  ; liste des instances
           classe-valedc
           )
          
          (when (activatedp b)
                (ifn valedc-sym
                     (progn
                            (:envoie-msg b
                                    (list (font #:ivan:ed-global:fstd "")
                                          (font #:ivan:ed-global:fstd
						#M#:txt-const:donner-nom
						) ))
                            (current-keyboard-focus-window
                                    ({application}:window (component 'edc-val b)) )
                            nil
                            )
                     ; sinon
                     (if (and 
                              bspec-loc
                              (not
                                   (member
                                           valedc-sym 
                                           (#:ivan:ed-util:l-inst-t-sym (#:shirka:noyau:schema nom-classe))))
                              )
                         (progn
                                (:envoie-msg b
                                        (list
                                              (font #:ivan:ed-global:fstdb
							#M#:txt-const:valeur-invalide)
                                              (font #:ivan:ed-global:fstd
							#M#:txt-const:choisir-inst-exist)
                                              (font #:ivan:ed-global:fstd
							#M#:txt-const:cas-spec-loc) ))
                                nil
                                ) 
                         ; sinon
                         ; modif du 6.8.91
                         ; la valeur dans l'edc peut etre une instance a creer,
                         ; dans ce cas, sa classe est la classe de l'attribut
                         (if (member valedc-sym
                                     (#:shirka:noyau:l-inst-t (#:shirka:noyau:schema 'objet)))
                             ; la valeur ds l'edc est une instance existante
                             (setq classe-valedc (#:shirka:noyau:nom-sch
                                                         (#:ivan:ed-util:classe-de valedc-sym)))
                             ; sinon, je dois creer cette instance !
                             (setq classe-valedc type-attr)
                             )
                         ; test d'admissibilite               
                         (if (and
                                  
                                  (not (#:shirka:moteur:admissible?
                                               valedc-sym
                                               (#:ivan:ed-util:ch-as-sym nom-attr)
                                               (#:shirka:noyau:schema
                                                       (#:ivan:ed-util:classe-de nom-inst-trav))
                                               (#:shirka:noyau:schema nom-inst-trav) ))
                                  (not (equal type-attr classe-valedc)) 
                                  )
                             ; alors erreur car la valeur dans l'edc est non admissible pour l'instance
                             ; et elle n'appartient pas a la classe de l'instance
                             (progn
                                    (:envoie-msg b
                                            (list
                                                  (row (font #:ivan:ed-global:fstdb
                                                             valedc-ch)
                                                       (font #:ivan:ed-global:fstd
								#M#:txt-const:classe)
                                                       (font #:ivan:ed-global:fstdb
                                                             (string classe-valedc))
                                                       (font #:ivan:ed-global:fstd ")" ) )
                                                  (font #:ivan:ed-global:fstd 
							#M#:txt-const:inst-incompatible)
                                                  (row (font #:ivan:ed-global:fstdb
                                                             (string nom-attr))
                                                       (font #:ivan:ed-global:fstd
                                                             " !")) ))
                                    nil
                                    )
                             
                             ; sinon
                             (if (member valedc-sym #:ivan:ed-global:%list-nom-inst)
                                 ; alors erreur
                                 (progn
                                        (:envoie-msg b
                                                (list
                                                      (font #:ivan:ed-global:fstd 
								#M#:txt-const:impossible)
                                                      (font #:ivan:ed-global:fstdb 
								#M#:txt-const:deja-editee) ))
                                        nil
                                        )
                                 
                                 ; sinon on peut enfin editer l'instance
                                 
                                 ; appel editeur d'instance
                                 ; la classe d'appel doit etre la classe de l'instance et non la classe de
                                 ; l'attribut (l'instance peut appartenir en effet a une sous-classe de la
                                 ; classe de l'attribut).
                                 (#:ivan:ed-edinst:edit
                                         classe-valedc
                                         valedc-sym
                                         nil
                                         nil
                                         nil
                                         nil
                                         nil)
                                 ; il faut reafficher la liste des instances et le nom de l'instance
                                 (lets
                                       (
                                        ; liste des valeurs a choisir sous forme de chaine
                                        (list-val-pos
                                                (if (null list-dom)
                                                    ; construire la liste des instances dans le cas d'un objet
                                                    (#:ivan:ed-util:diff-liste
                                                            (#:ivan:ed-util:l-inst-t-ch type-attr)
                                                            #:ivan:ed-global:%list-inst-trav)
                                                    ; sinon les valeurs du domaine sont affichees
                                                    list-dom)) 
                                        ; le nom de l'instance est retrouve par difference entre la
                                        ; liste des instances avant et apres l'appel a l'editeur
                                        ; d'instances.
                                        (val-edc  (#:ivan:ed-util:symbole
                                                          (car 
                                                               (#:ivan:ed-util:diff-liste
                                                                       (#:ivan:ed-util:l-inst-t-ch type-attr)
                                                                       list-val))))
                                        ; si val-edc est nil, il faut remettre la valeur de l'edc
                                        ; avant l'appel
                                        (val-ret (if val-edc val-edc valedc-ch))
                                        )
                                       ; raffraichir le selectionneur de choix
                                       (send 'set-strings (component 'sm-choix b) list-val-pos)
                                       ; reinitialiser le scroller
                                       (#:ivan:ed-util:positionne (component 'scroll-choix b) nil)
                                       ; renvoyer le nom de l'instance qui a peut etre ete modifie par
                                       ; l'editeur d'instance
                                       (send 'set-line (component 'edc-val b) (string val-ret))
                                       ;
                                       t
                                       ) ; lets
                                 ) ; if member
                             ) ; if and
                         ) ; if bspec
                     ) ; ifn
                ) ; when
          )
    )
;;;*****************************************************************************************
;;; action du bouton standard d'effacement de la zone d'edition

(de :action-bout-effacer (b)
    (lets
          (
           (list-param    ({application}:userdata b))
           (nom-inst-trav (nth 0 list-param))
           (nom-attr      (nth 1 list-param))
           (bliste        (nth 2 list-param))
           )
          (when (activatedp b)
                ; vider l'editeur de ligne de son contenu     
                (send 'set-line (component 'edc-val b) '"") 
                ; si l'attribut n'est pas une liste, supprimer sa valeur
                (unless bliste
                        (if (#:ivan:ed-util:sup-valeur (#:ivan:ed-util:schema nom-inst-trav)
                                    nom-attr
                                    nil)
(when #:trace:ed
                           (print "     la valeur de l'attribut " nom-attr " a ete supprimee !")
)
(when #:trace:ed
                           (print "     probleme pour supprimer la valeur de l'attribut " nom-attr)
)
                            )  
                        )
                ; raz de la zone de message
                (:action-sa-warn b
                        (list
                              (font #:ivan:ed-global:fstd
					#M#:txt-const:clic-valid-ou-annul-pour)
                              (font #:ivan:ed-global:fstd
					#M#:txt-const:fermer-fenetre) ))
                )
          )
    )
;;;*****************************************************************************************

(de :action-edc-val (edc)
    (lets
          (
           (list-param    ({application}:userdata edc))
           (bliste           (nth 0 list-param))
           (btype-simple     (nth 1 list-param))
           (type-attr        (nth 2 list-param))
           (nom-inst-trav    (nth 3 list-param))
           (nom-attr         (nth 4 list-param))
           
           (pere          (component 'top edc))
           (sm-val        (component 'sm-val edc))
           (scroll-val    (component 'scroll-val edc))
           (cb-entete     (component 'cb-entete edc))
           
           (nom-edc-ch    (send 'get-line edc))
           (nom-edc-sym   (#:ivan:ed-util:symbole nom-edc-ch))
           (nom-classe    (#:ivan:ed-util:symbole type-attr))
           valret
           )
          
          (:action-sa-warn edc '(""))
          
          (if (<= (slength nom-edc-ch) 0)
              ; alors
              t
              ; sinon
(when #:trace:ed
              (PRINT "     il y a une valeur")
)
              (if (and 
                       (not btype-simple)
                       (not (member nom-edc-sym (#:ivan:ed-util:l-inst-t-sym nom-classe))) )
                  ; alors
                  (progn
                         ; on est dans le cas ou l'on a saisi une instance inconnue : 
                         ; il faut appeler l'editeur d'instances sur cette instance
                         
                         ; message
                         (:action-sa-warn edc
                                 (list
                                       (font #:ivan:ed-global:fstd
						#M#:txt-const:inst-inconnue)
                                       (row (font #:ivan:ed-global:fstdb
                                                  (string type-attr))
                                            (font #:ivan:ed-global:fstd
                                                  " !")) ))
                         
                         ; pour lancer l'editeur d'instance, je declenche l'action du bouton editer
                         ; le bouton teste si la valeur est admissible et declencle l'action de
                         ;  l'editeur d'instance
                         (ifn (:action-bout-editer (component 'bout-editer edc))
                              ; renvoyer nil
                              nil
                              ; sinon
                              ; si j'ai clique sur annuler dans l'editeur d'instances, l'instance n'a
                              ; pas ete creee : il faut donc renvoyer nil pour que le bouton valider 
                              ; ne ferme pas la fenetre
                              (if (member nom-edc-sym (#:ivan:ed-util:l-inst-t-sym nom-classe))
                                  ; l'instance a ete rajoutee dans la liste des instances : 
                                  ; on peut donc tenter d'ajouter la valeur 
                                  (:bajout-val bliste nom-inst-trav nom-attr nom-edc-ch sm-val
                                          type-attr cb-entete pere)
                                  ; sinon du if member
                                  ; l'instance n'existe pas : on a annule sa creation, il faut
                                  ; renvoyer nil pour que le bouton valider ne puisse pas fermer
                                  ; la fenetre
                                  (unless
                                          (member (#:ivan:ed-util:symbole nom-edc-ch)
                                                  (#:ivan:ed-util:l-inst-t-sym 'objet))
                                          (:envoie-msg edc
                                                  (list
                                                        (font #:ivan:ed-global:fstdb 
								#M#:txt-const:inst-pas-cree)
                                                        (font #:ivan:ed-global:fstd
								#M#:txt-const:saisir-autre-ou-creer)
                                                        (font #:ivan:ed-global:fstd
								#M#:txt-const:clic-edit) ))
                                          )
                                  ; renvoyer nil
                                  nil
                                  ) ; endif member
                              ) ; ifn
                         ) ;progn
                  ; sinon
                  (:bajout-val bliste nom-inst-trav nom-attr nom-edc-ch sm-val type-attr
                          cb-entete pere)
                  ) ; endif and
              ) ; endif <=
          )
    )
;;;*****************************************************************************************

(de :bajout-val (bliste nom-inst-trav nom-attr valeur sm-val type-attr cb-entete pere)
    (if bliste
        ; alors : cas de l'attribut multi-value
        (:ajoute-val-sm
                nom-inst-trav
                nom-attr
                valeur
                sm-val
                type-attr
                (#:ivan:ed-util:ins-en-fin? cb-entete) ; vrai = ins en fin  faux = ins en tete
                )
        ; sinon : cas de la valeur simple
        (:ajoute-val
                nom-inst-trav
                nom-attr
                valeur
                pere
                type-attr
                )
        ) ; endif bliste
    )
;;;*****************************************************************************************
;;; cas de l'attribut multi-value : on ajoute la chaine valeur a la liste des valeurs
;;; du selecteur de chaines sm-val en ins-en-fin de la liste pour l'attribut nom-attr
;;; de l'instance nom-inst-trav
;;; si l' ajout est possible , la fonction prend la valeur vrai (t) , faux (nil) sinon

(de :ajoute-val-sm (nom-inst-trav nom-attr valeur sm-val type-attr ins-en-fin)
    (lets
          (
           (old (send 'get-strings sm-val))
           (new (if ins-en-fin
                    (reverse (cons valeur (reverse old)))
                    (cons valeur old)))
           )
          (if (#:shirka:moteur:aj-valeur
                      (#:ivan:ed-util:schema nom-inst-trav)
                      nom-attr 
                      (#:ivan:ed-util:objet-simple valeur type-attr)
                      ins-en-fin)
              ; alors la valeur est admissible et a ete rajoutee
              (progn
                     ; ajouter la valeur dans la liste des valeurs
                     (send 'set-strings sm-val new)         
                     ; reinitialiser le scroller au debut en cas d'insertion en-tete
                     (#:ivan:ed-util:positionne (component 'scroll-val sm-val) ins-en-fin)
                     ; raz de l'editeur de ligne
                     (send 'set-line edc "")
                     ; raz de la zone de message
                     (:action-sa-warn sm-val
				(list (font #:ivan:ed-global:fstd #M#:txt-const:clic-valid-ou-annul-pour)
					(font #:ivan:ed-global:fstd #M#:txt-const:fermer-fenetre))
			)
                     ; la modif a ete possible : renvoyer vrai (bouton valider)
                     t
                     )
              ; sinon la valeur est non admissible
              (:envoie-msg sm-val
                      (list
                            (font #:ivan:ed-global:fstdb
				#M#:txt-const:valeur-non-admiss)
                            (font #:ivan:ed-global:fstd 
				#M#:txt-const:changer-val)
                            (font #:ivan:ed-global:fstd 
				#M#:txt-const:effacer-avant) ))
              ; la modif n'a pas ete possible : renvoyer faux (bouton valider)
              nil
              )
          )
    )
;;;*****************************************************************************************
;;; cas de l'attribut mono-value :
;;; il faut d'abord supprimer l'ancienne valeur avant de rajouter la nouvelle valeur
;;; si l' ajout est possible , la fonction prend la valeur vrai (t) , faux (nil) sinon

(de :ajoute-val (nom-inst-trav nom-attr valeur appli type-attr)
    (let
         ()
         (if 
             (let
                  ()
                  (#:ivan:ed-util:sup-valeur (#:ivan:ed-util:schema nom-inst-trav) nom-attr nil)
                  (#:ivan:ed-util:aj-valeur  (#:ivan:ed-util:schema nom-inst-trav)
                          nom-attr
                          (#:ivan:ed-util:objet-simple valeur type-attr)
                          nil)
                  )
             ; alors
             (progn
                    ; raz de la zone de message
                    (:action-sa-warn appli
                            (list
                                  (font #:ivan:ed-global:fstd 
					#M#:txt-const:clic-valid-ou-annul-pour)
                                  (font #:ivan:ed-global:fstd
					#M#:txt-const:fermer-fenetre)))
                    
                    ; la modif a ete possible : renvoyer vrai (bouton valider)
                    t
                    )
             ; sinon la valeur est non admissible
             (:envoie-msg appli
                     (list
                           (font #:ivan:ed-global:fstdb 
				#M#:txt-const:valeur-non-admiss)
                           (font #:ivan:ed-global:fstd 
				#M#:txt-const:changer-val)
                           (font #:ivan:ed-global:fstd
				#M#:txt-const:effacer-avant) ))
             ; la modif n'a pas ete possible : renvoyer faux (bouton valider)
             nil
             )
         )
    )
;;;*****************************************************************************************
;;; envoi du message list-msg au composant sa-warn de l'application appli

(de :action-sa-warn (appli list-msg)
    (let
         (
          (col-msg (apply 'column list-msg)) 
          )
         (send 'set-image (component 'sa-warn appli) col-msg)
         )
    )
;;;*****************************************************************************************
;;; envoi le message list-msg en declenchant 2 beeps

(de :envoie-msg (comp list-msg)
    (tybeep)
    (tybeep)
    (:action-sa-warn comp list-msg)
    )
;;;*****************************************************************************************
;;; action du selecteur de menus sm (scroller de valeurs)
;;; envoi la chaine selectionnee (valeur) vers l'editeur de chaine
;;; supprime la valeur de la liste des valeurs.

(de :action-sm-val (sm)
    (lets
          ( 
           (valeur        (send 'get-selected-string sm))
           (rang          (if valeur (+ (send 'get-selected-rank sm) 1)))
           (list-param    ({application}:userdata sm))
           (nom-inst-trav (nth 0 list-param))
           (nom-attr      (nth 1 list-param))
           (type-attr     (nth 2 list-param))
           
           )
          (ifn valeur
               ; alors rien
               't
               ; sinon
               ; supprimer la valeur selectionnee de la liste des valeurs
               (ifn
                    (#:shirka:moteur:sup-valeur
                            (#:ivan:ed-util:schema nom-inst-trav)
                            (#:ivan:ed-util:symbole nom-attr)
                            (#:ivan:ed-util:objet-simple valeur type-attr) )
                    ; alors [ca ne devrait pas arriver !]
                    (:action-sa-warn sm 
                            (list
                                  (font #:ivan:ed-global:fstd 
					#M#:txt-const:supp-imposs
					)))
                    ; sinon
                    ; supprimer la valeur de la liste du selectionneur
                    (send 'set-strings sm (#:ivan:ed-util:supp-nl rang (send 'get-strings sm)))
                    ; envoyer la valeur a l'editeur de chaine
                    (send 'set-line (component 'edc-val sm) valeur)
                    )
               )
          )
    )
;;;*****************************************************************************************
;;; action de la case a cocher dans le cas d'une liste de valeurs

(de :action-cb-entete (cb)
    (let
         (
          (choix (send 'checkedp cb))       
          )
         (if choix
             (:action-sa-warn cb
                     (list
                           (font #:ivan:ed-global:fstd
				(catenate #M#:txt-const:ins-tete #M#:txt-const:de-liste))))
             (:action-sa-warn cb 
                     (list
                           (font #:ivan:ed-global:fstd 
				(catenate #M#:txt-const:ins-fin #M#:txt-const:de-liste))))
             ))
    )
;;;*****************************************************************************************
;;; action du selecteur de menus sm (scroller de choix des valeurs possibles)
;;; envoi de la chaine selectionnee vers l'editeur de ligne

(de :action-sm-choix (sm)
    (lets
          ( 
           (valeur (send 'get-selected-string sm))
           (valeur-sym (#:ivan:ed-util:symbole valeur))
           (bliste ({application}:userdata sm))
           (list-val (send 'get-strings (component 'sm-val sm)))
           )
          (ifn valeur-sym
               ; alors rien (cas de la reinitialisation du sm)
               't
               ; sinon
               ; reinitialiser le selecteur
               (send 'set-selected-string sm "")
               ; envoyer la valeur a l'editeur de ligne
               (send 'set-line (component 'edc-val sm) valeur)
               ; message d'action
               (if bliste
                   (:action-sa-warn sm
                           (list
                                 (font #:ivan:ed-global:fstd 
					#M#:txt-const:faire-rc)
                                 (font #:ivan:ed-global:fstd 
					#M#:txt-const:pour-aj-val)))
                   ; sinon
                   (:action-sa-warn sm 
                           (list 
                                 (font #:ivan:ed-global:fstd 
					#M#:txt-const:faire-rc)
                                 (font #:ivan:ed-global:fstd
					#M#:txt-const:ou-valid)))
                   )
               )
          )
    )
;;;*****************************************************************************************
;;; reinitialise le scroller sm pour desselectionner la valeur qui a ete cliquee
;;; sert pour les sm des facettes $sauf et $intervalle (contraintes)

(de :action-reinit-sm (sm)
    (let
         ( 
          (valeur (send 'get-selected-string sm))
          )
         (ifn valeur
              ; alors rien (cas de la reinitialisation du sm)
              't
              ; sinon
              ; reinitialiser le selecteur
              (send 'set-selected-string sm "")
              )
         )
    )
;;;*****************************************************************************************
