;;; repertoire: ~/LeShirka/LeIvan/CodeIvan/Editeur/
;
;     +----------------------------------------+-----------------------------------------+
;     | fichier :  ivan-ed-edinst.ll           | package : #:ivan:ed-edinst              |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface de Visualisation et d'Aide a la Navigation                 |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Sylvain Grivaud          1991                             |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     | descriptif du package :                                                          |
;     |                                                                                  |
;     | La fonction :edit permet l'edition d'une instance Shirka.                        |
;     | Une instance de la structure AIDA {in-edinst} composee de 7 champs est creee.    |
;     | La taille de la fenetre finale est fixe mais les composants sont crees dynami-   |
;     | quement en fonction de la classe de l'instance.                                  |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+

;;;*****************************************************************************************

(defvar #:sys-package:colon '#:ivan:ed-edinst)

;;;*****************************************************************************************

(defvar {scroller}:scrollbarwidth  20)     
(defvar {scroller}:scrollbarheight 20)
(defvar :%l-ei 45)   ; Largeur editeur instance en pixels (45 caracteres X 9)
(defvar :%h-ei 20)   ; Hauteur editeur instance (scroller de visualisation attributs)
(defvar :%h-max 4)   ; Hauteur max d'affichage du commentaire (en nombre de ligne)

;;;*****************************************************************************************

;;;*****************************************************************************************
;;; fonction creant une instance de la structure {in-edinst} pour permettre l'edition
;;; de l'instance nom-instance de la classe nom-classe. classes-attr est la liste des
;;; attributs a visualiser, sauf-classes-attr est la liste des attributs a ignorer.
;;; L'argument bappel permet de parametrer les boutons actifs de la ligne de validation.

(de :edit (nom-classe
                  nom-instance
                  classes-attr
                  sauf-classes-attr
                  attr-mere 
                  inst-mere
                  bappel)
    
    (lets
          (   ; application generale
             (appli (omakeq {in-edinst} x 0 y 0 w 0 h 0) )
             
             ; variable de travail
             (nom-inst nom-instance)
             ; vecteur du schema de classe
             (schema-clas   (#:ivan:ed-util:schema nom-classe))
             ; booleen de creation 
             (bcreation (or
                            (null nom-inst)
                            (not
                                 (member (#:ivan:ed-util:ch-as-sym nom-inst)
                                         (#:ivan:ed-util:l-inst-t-sym schema-clas))))
                     )
             ; booleen de visualisation
             (bvisu (or
                        (equal bappel 'visu)
                        (equal bappel 'visusup) ))
             ; creation instance de travail
             (nom-inst-trav (#:ivan:ed-util:ch-as-sym (catenate "instrav-" (gensym))))
             ; inst. temporaire
             (inst-trav     (#:ivan:ed-util:cr-inst nom-classe nom-inst-trav))
             ; famille d'attributs
             (sauf-classes      (if #:ivan:ed-global:%battribut-shirka
                                    (cons 'attribut-shirka sauf-classes-attr)))
             (sauf-classes-attr (if #:ivan:ed-global:%bmeta-attribut
                                    (cons 'meta-attribut sauf-classes)))     
             ; message d'action
             (list-msg-clic
                     (if bvisu
                         (list (font #:ivan:ed-global:fstd #M#:txt-const:mode-visu)
                               ""
                               (font #:ivan:ed-global:fstd #M#:txt-const:sel-att-com) )
                         ; sinon               
                         (list ""
                               (font #:ivan:ed-global:fstd #M#:txt-const:sel-att-com) )
                         ))
             (list-msg-crea
                     (if bvisu
                         (list ""
                               (font #:ivan:ed-global:fstd #M#:txt-const:visu-att) )
                         ; sinon
                         (list ""
                               (font #:ivan:ed-global:fstd #M#:txt-const:donner-nom-inst) )
                         ))
             (edl-titre (lineedit 0 0 #40wc ""))
             (titre-fen "")
             image-ed
             )
          
(when #:trace:ed
          (PRINT "ENTREE EI ======================================")
          (PRINT "     nom-classe    " nom-classe " " (type-of nom-classe))
          (PRINT "     nom-instance  " nom-instance " " (type-of nom-instance))
)
          
          ; initialisation de l'instance de travail dans le cas de la modification
          (when (not bcreation)
                (#:ivan:ed-util:copie-inst (#:ivan:ed-util:schema nom-inst) inst-trav))
          ; insertion du nom de l'instance de travail
          (setq #:ivan:ed-global:%list-inst-trav 
                (newr #:ivan:ed-global:%list-inst-trav (string nom-inst-trav)))
          ; insertion de l'instance
          (when (#:ivan:ed-util:symbole nom-inst)
                (setq #:ivan:ed-global:%list-nom-inst
                      (newr #:ivan:ed-global:%list-nom-inst (#:ivan:ed-util:symbole nom-inst)))
                )
          
          ; composant top pour reperer le pere de plus haut niveau
          ({application}:components appli (acons 'top appli ()))
          (add-component appli 'edl-titre edl-titre)
          
          ; liste des parametres utilises dans les fonctions
          ({application}:userdata appli
                  (list
                        nom-classe
                        bcreation
                        nom-inst-trav 
                        inst-trav 
                        classes-attr
                        sauf-classes-attr
                        nom-inst 
                        bvisu
                        bappel))
          ; construction des champs de l'application
          ({in-edinst}:ch-scroll-com    appli (:const-ch-scroll-com    appli
                                                      schema-clas))
          ({in-edinst}:ch-titre         appli (:const-ch-titre         appli
                                                      nom-classe
                                                      nom-inst))
          ({in-edinst}:ch-sa-action     appli (:const-ch-sa-action     appli))
          ({in-edinst}:ch-edl-inst      appli (:const-ch-edl-inst      appli
                                                      nom-inst
                                                      nom-classe
                                                      bcreation 
                                                      bvisu))
          ({in-edinst}:ch-scroll-attr   appli (:const-ch-scroll-attr   appli
                                                      nom-inst-trav 
                                                      schema-clas
                                                      bcreation
                                                      classes-attr
                                                      sauf-classes-attr))
          ({in-edinst}:ch-ligne-valid   appli (:const-ch-ligne-valid   appli 
                                                      nom-classe
                                                      nom-inst-trav 
                                                      bcreation
                                                      attr-mere 
                                                      inst-mere
                                                      bappel))
          ({in-edinst}:ch-list-attr-inf appli (:const-ch-list-attr-inf appli
                                                      nil
                                                      t))
          ; assemblage pour constituer l'image finale
          (setq image-ed
                (column
                        (#:ivan:ed-util:centeredimage
                                (view
                                      (filledbox 0 0
                                              (send 'width ({in-edinst}:ch-sa-action  appli)) 619 0)
                                      (column      
                                              ({in-edinst}:ch-titre  appli)
                                              "" 
                                              ({in-edinst}:ch-scroll-com    appli)
                                              ({in-edinst}:ch-sa-action     appli)
                                              ""  
                                              ({in-edinst}:ch-scroll-attr   appli)
                                              "" 
                                              ({in-edinst}:ch-edl-inst      appli) 
                                              ""
                                              )
                                      )
                                )             
                        ({in-edinst}:ch-ligne-valid   appli)
                        )
                )
          
          (if (null nom-inst)
              ; alors
              (progn
                     ; titre de la fenetre
                     (if bvisu
                         (setq titre-fen #M#:txt-const:visu-att)
                         ; sinon
                         (setq titre-fen #M#:txt-const:creation-inst)
                         (inhibit-application (component 'scroll-attr appli))
                         )
                     ; commentaire action en cours
                     (:action-sa-warn appli list-msg-crea)
                     )
              ; sinon
              (if bvisu
                  (setq titre-fen (catenate #M#:txt-const:visu-de nom-inst))
                  (if bcreation
                      (setq titre-fen (catenate #M#:txt-const:creer nom-inst))
                      (setq titre-fen (catenate #M#:txt-const:edition-de nom-inst))
                      ))
              (:action-sa-warn appli list-msg-clic)
              )
          
          (if (>= (read-from-string aidaversion) 1.5403)
              (progn
                     (send 'new-image
                           appli
                           (header
                                   (rectangle 0 0 32 32) ; logo 
                                   edl-titre
                                   (view) ; pas de bouton
                                   (view) ; pas de commande 
                                   image-ed
                                   t
                                   t
                                   )
                           )
                     (#:ivan:ed-util:titre-fenetre appli #M#:txt-const:editeur-inst)
                     (send 'set-line edl-titre titre-fen)
                     )
              ; SINON
              (send 'new-image
                    appli
                    image-ed
                    )
              (#:ivan:ed-util:titre-fenetre appli titre-fen)
              )                                        
          ; dimensionnement de l'application
          (send 'fit-to-contents appli)
          
          ;  l'application est activee
          (grab-application
                  (send 'translate
                        appli
                        #:ivan:ed-global:%x #:ivan:ed-global:%y
                        ))
          ; renvoyer l'instance
          (#:ivan:ed-util:symbole (send 'get-line (component 'edl-instance appli)))
          )
    )

;;;#########################################################################################
;;;
;;;                                    CONSTRUCTION DES CHAMPS 
;;;
;;;#########################################################################################

;;;*****************************************************************************************
;;; construction de l'image de visualisation du commentaire de l'instance.
;;; suivant la taille du commentaire, l'image sera affichee ou non dans un scroller.

(de :const-ch-scroll-com (appli schema-clas)
    (lets
          (  
           ; commentaire de la classe (liste de chaines) 
           (list-comm (#:ivan:ed-util:val-fac-he schema-clas 'lui-meme '$com))
           (list-com
                   (mapcar '(lambda (chaine)
                                    (let ( (chaine-font (font #:ivan:ed-global:fstd chaine)))
                                         chaine-font
                                         ))
                           list-comm ))
           ; colonne des lignes
           (col-com (apply 'column list-com))
           
           ; largeur max d'1 ligne en nombre de caracteres (attention #wc = 9)
           (larg (#:ivan:ed-util:larg-max list-comm))
           
           ; hauteur du commentaire (nombre de lignes)
           (haut (ifn (null list-com)
                      (length list-com)))
           
           ; hauteur de l'image commentaire si scroller
           (h-scroll (+ {scroller}:scrollbarheight
                        (#:ivan:ed-util:hc :%h-max)
                        2))
           ; largeur de l'image commentaire si scroller
           (w-scroll (+ {scroller}:scrollbarwidth
                        (#:ivan:ed-util:wc :%l-ei)
                        2))
           
           ; img du commentaire     
           (img-com 
                   (cond
                         ; pas de commentaire = 1 ligne vide
                         ((null list-com)
                          (rectangle 0 0 w-scroll 0)
                          )
                         
                         ; le commentaire tient en largeur et hauteur : boite rectangulaire
                         ((and (<= larg :%l-ei) (<= haut :%h-max))
                          (view
                                (rectangle 0 0 w-scroll (#:ivan:ed-util:hc haut))
                                col-com
                                )
                          )
                         
                         ; le commentaire est trop large et trop haut : scroller   
                         ((and (> larg :%l-ei) (> haut :%h-max))
                          (scroller 0 0
                                  w-scroll h-scroll
                                  col-com
                                  t t
                                  )
                          )
                         
                         ; le commentaire est trop large : scroller horizontal                   
                         ((> larg :%l-ei)
                          (horizontalscroller 0 0
                                  w-scroll h-scroll                               
                                  col-com
                                  )
                          )
                         
                         ; le commentaire est trop haut : scroller vertical              
                         (t         
                            (verticalscroller 0 0
                                    w-scroll h-scroll
                                    col-com
                                    )
                            )
                         ))
           )
          (add-component appli 'img-com img-com)
          
          ; image finale
          (if (null list-com)
              (rectangle 0 0 (+ 12 w-scroll) 0)
              ; sinon il faut sauter une ligne
              (column
                      (#:ivan:ed-util:clusterimage img-com #M#:txt-const:comment)
                      ""))
          )
    )
;;;*****************************************************************************************
;;; construction de la liste des attributs qui ont ete inferes
;;; cette fonction effectue, en fonction du booleen bins, soit l'insertion de nom-attr
;;; soit sa suppression de la liste des attributs inferes.

(de :const-ch-list-attr-inf (appli nom-attr bins)
    (lets
          (
           (list
                 (if bins
                     ; cas de l'insertion 
                     (if (member nom-attr ({in-edinst}:ch-list-attr-inf appli))
                         ; l'attribut appartient deja a la liste
                         ({in-edinst}:ch-list-attr-inf appli)
                         ; sinon inserer l'attribut dans la liste
                         (if nom-attr
                             ; inserer le nom de l'attribut dans la liste
                             (reverse (cons nom-attr
                                            (reverse ({in-edinst}:ch-list-attr-inf appli))))
                             ; pas de modif de la liste
                             ({in-edinst}:ch-list-attr-inf appli)
                             )
                         )
                     ; sinon : nom-attr a-t-il ete modifie par l'editeur d'attributs ?
                     (if (#:ivan:ed-util:val-attr-sym nom-attr nom-inst-trav)
                         ; il existe une valeur sans inference  donc l'attribut n'est plus
                         ; a inferer : il est supprime de la liste.
                         (#:ivan:ed-util:diff-liste
                                 ({in-edinst}:ch-list-attr-inf appli)
                                 (list nom-attr))
                         ; sinon l'attribut doit toujours etre infere.
                         ({in-edinst}:ch-list-attr-inf appli)
                         )
                     )
                 )
           )
          list
          )
    )
;;;*****************************************************************************************
;;; construction du champ titre. la classe de l'instance est affichee dans un cadre

(de :const-ch-titre (appli nom-classe nom-inst)
    (lets
          (  (titre-attr (catenate #M#:txt-const:classe-de-i (string nom-inst)))            
           (img (centeredview 
                        (rectangle 0 0 (send 'width (component 'img-com appli)) 0)
                        (font #:ivan:ed-global:fstdb
                              (string nom-classe))
                        ))
           )
          ; image du champ 
          (#:ivan:ed-util:clusterimage img titre-attr)
          )
    )
;;;*****************************************************************************************
;;; construction du champ editeur de ligne pour saisir nom-inst dans le cas d'une creation

(de :const-ch-edl-inst (appli nom-inst nom-classe bcreation bappel)
    ;(PRINT "ei>> :const-ch-edl-inst")
    (lets
          (
           ; bouton d'effacement de l'editeur de chaine
           (bout-effacer (standardbutton #:ivan:ed-global:txt-effacer))
           ; valeur initiale de l'editeur de ligne
           (val-initiale (if nom-inst
                             ; chaine du nom de l'instance
                             (string nom-inst)
                             ; chaine vide
                             '""
                             ))
           ; largeur de l'editeur de ligne calculee dynamiquement
           (larg
                 (- (send 'width (component 'img-com appli))
                    (send 'width (#:ivan:ed-util:centeredimage #:ivan:ed-global:txt-instance))
                    7
                    #wc
                    (send 'width bout-effacer)
                    ) )
           ; editeur de ligne pour saisir le nom de l'instance
           (edl-instance (lineedit 0 0 larg val-initiale))
           
           (img-edl (row
                         (#:ivan:ed-util:centeredimage #:ivan:ed-global:txt-instance)
                         (#:ivan:ed-util:boxedimage edl-instance)
                         " "
                         bout-effacer
                         ))
           
           )
          
          (add-component appli 'edl-instance edl-instance)
          ({application}:userdata edl-instance (list appli bcreation nom-inst bappel))
          (set-action edl-instance ':action-edl-instance)
          
          (add-component appli 'bout-effacer bout-effacer)
          (set-action bout-effacer ':action-bout-effacer)
          
          (if (and (null nom-inst) (null bappel))
              ; afficher l'editeur de ligne
              (#:ivan:ed-util:clusterimage img-edl #M#:txt-const:zone-edi)
              ; sinon
              (rectangle 0 0 (send 'width ({in-edinst}:ch-scroll-com appli)) 0))
          )
    )
;;;*****************************************************************************************
;;; construction du selectionneur d'appli affichant l'aide textuelle de l'action en cours
;;; les messages seront transmis au composant sa-warn par la fonction  :action-sa-warn

(de :const-ch-sa-action (appli)
    ;(PRINT "ei>> :const-ch-sa-action")
    (let
         (
          (sa-warn (selectorapplication 0 0
                           (send 'width (component 'img-com appli))
                           (#:ivan:ed-util:hc 3)
                           nil))
          )
         
         (add-component appli 'sa-warn sa-warn)
         ; image finale centree
         (#:ivan:ed-util:clusterimage
                 sa-warn
                 "Message"
                 )
         )
    )
;;;*****************************************************************************************
;;; construction du scroller d'attributs

(de :const-ch-scroll-attr 
    (appli nom-inst-trav schema-clas bcreation classes-attr sauf-classes-attr)
    ;(PRINT "ei>> :const-ch-scroll-attr")
    (lets
          (  
           ; liste des images de chaque attribut
           (li (:const-img-attr
                       nom-inst-trav schema-clas bcreation classes-attr sauf-classes-attr))
           (col (apply 'column li))
           ; selectionneur d'image associe
           (sa-attr 
                   (selectorapplication 0 0
                           (send 'width col) (send 'height col)
                           col ))
           (scroll-attr
                   (scroller 0 0
                           (send 'width (component 'img-com appli)) ; largeur
                           (- (#:ivan:ed-util:hc :%h-ei) 
                              (send 'height ({in-edinst}:ch-edl-inst appli))) ; hauteur
                           sa-attr
                           t t))
           )
          (add-component appli 'sa-attr sa-attr)
          (add-component appli 'scroll-attr scroll-attr)
          
          ; image renvoyee  
          (#:ivan:ed-util:clusterimage scroll-attr #M#:txt-const:sel-att)
          )
    )
;;;*****************************************************************************************
;;; construit la liste des images
;;; si une liste des classes d'attribut est donnee , la fonction renvoie une liste
;;; contenant lles images de chacune des classes (fonction const-clas), sinon elle 
;;; renvoie une liste contenant l'image de chaque attribut (const-attr)

(de :const-img-attr (nom-inst-trav schema-clas bcreation classes-attr sauf-classes-attr)
    ;(PRINT "ei>> :const-img-attr")
    (ifn classes-attr
         (:const-attr nom-inst-trav schema-clas bcreation classes-attr sauf-classes-attr)
         ; sinon
         (:const-clas nom-inst-trav schema-clas bcreation classes-attr sauf-classes-attr)
         )
    )
;;;*****************************************************************************************
;;; construction sequentielle de l'image de chacune des classes d'attribut presentes dans
;;; classes-attr. il s'agit de l'image de chacun des attributs de la classe centree dans
;;; un cadre avec le nom de la classe d'attribut en en-tete.

(de :const-clas (nom-inst-trav schema-clas bcreation classes-attr sauf-classes-attr) 
    ;(PRINT "ei>> :const-clas")
    (lets
          (
           ; liste des images des classes d'attribut
           list-img-classe-attr
           )
          ; boucle principale
          (while classes-attr
                 (lets
                       (
                        ; nom de la classe 
                        (nom-classe-attr (car classes-attr))
                        ; liste des images pour les attributs de cette classe
                        (col  (apply 'column 
                                     (:const-attr
                                             nom-inst-trav
                                             schema-clas
                                             bcreation
                                             (list (#:ivan:ed-util:symbole nom-classe-attr))
                                             nil) ))
                        (vue-clas-attr 
                                (view
                                      (rectangle 0 0
                                              (- (#:ivan:ed-util:wc :%l-ei) 14) ; largeur
                                              (send 'height col)                ; hauteur
                                              )  
                                      col ))
                        ; image finale de la classe (decalage a droite et saut d'1 ligne)
                        (img-classe-attr
                                (column
                                        (row
                                             (rectangle 0 0 7 0)
                                             (#:ivan:ed-util:clusterimage vue-clas-attr nom-classe-attr) )
                                        "" ))
                        ) 
                       ; inserer de l'image dans lla liste des images
                       (newr list-img-classe-attr img-classe-attr)
                       ; classe d'attribut suivante
                       (nextl classes-attr)
                       )
                 )
          ; renvoyer la liste des images
          list-img-classe-attr
          )
    )
;;;*****************************************************************************************
;;; construction de la liste des images de chaque attribut.
;;; une image est la rangee (row img-pg "=" img-pd), representation
;;; de (attribut = valeur) avec :
;;;   - img-pg = image de la partie gauche.
;;;              il s'agit du nom de l'attribut.
;;;              un clic sur la pg declenche l'appel a l'editeur d'attributs.
;;;   - img-pd = image de la partie droite.
;;;              il s'agit de la valeur de l'attribut.
;;;              1er  cas : pas de valeur ; un ? est affiche. le clic declenche
;;;                         l'inference de la valeur.
;;;                         si echec, le ? est reaffiche ;
;;;                         sinon, la valeur inferee est affichee entre 2 caracteres *.
;;;              2eme cas : la valeur est de type simple. un message est affiche.
;;;              3eme cas : la valeur est une instance shirka.
;;;                        un clic declenche l'appel a l'editeur d'instances sur 
;;;                        cet objet.

(de :const-attr (nom-inst-trav schema-clas bcreation classes-attr sauf-classes-attr)
    ;(PRINT "ei>> :const-attr")
    (lets 
          (
           ; liste des attributs
           (list-attr
                   (#:shirka:noyau:l-nom-att-he-class schema-clas
                           classes-attr
                           sauf-classes-attr))
           ; liste des attributs sous forme de chaine
           (list-attr-ch
                   (mapcar '#:ivan:ed-util:nom-ch 
                           (if #:ivan:ed-global:attralpha
                               (sortl list-attr)
                               list-attr)))
	   ; liste correspondante qui recupere les couples $un/$liste-de <type> pour chaque attribut
	   (l-nature (mapcar '#:ivan:ed-util:nature
			(makelist (length list-attr-ch) schema-clas)
			(if #:ivan:ed-global:attralpha (sortl list-attr) list-attr)))
	   ; liste correspondante qui indique pour chaque attribut son type
           (l-type (mapcar 'cadr l-nature))
           ; liste des attributs a inferer
           (list-attr-inf ({in-edinst}:ch-list-attr-inf appli))
           ; largeur max d'1 nom d'attribut
           (larg-pg (+ 3 (#:ivan:ed-util:larg-max list-attr-ch)))
           ; schema de l'instance
           (schema-inst (#:ivan:ed-util:schema nom-inst-trav))
           ; liste d'images constituees en parcourant la liste des attributs
           list-img
           )
          
          ; boucle de parcours de la liste des attributs
          (while list-attr-ch
                 (lets 
                       (  ; attribut courant
                          (nom-attr (car list-attr-ch))
			  ; type de 'attribut
			  (tp-attr (car l-type))
                          ; l'attribut doit-il etre infere ?
                          (binf (member nom-attr list-attr-inf))
                          ; y-a-t-il une valeur par defaut ?
                          ; l'attribut est-il etre infere ?
                          (list-defaut (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$defaut))
                          (list-valeur (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$valeur))
                          (infer-proc  (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$sib-exec))
                          (infer-filt  (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$sib-filtre))
                          (battr-inf (or list-defaut list-valeur infer-proc infer-filt))
                          
                          ; liste des valeurs de l'attribut sous forme de symbole
                          (list-val-sy
                                  (if binf
                                      ; alors inference
                                      (#:ivan:ed-util:val-attr-inf-sym nom-attr nom-inst-trav)
                                      ; sinon
                                      (#:ivan:ed-util:val-attr-sym nom-attr nom-inst-trav)
                                      ))
                          ; s'il n'y a pas de valeur et qu'il y a une facette $valeur
                          (list-val-sym
                                  (if (null list-val-sy)
                                      list-valeur
                                      list-val-sy))
                          ; liste des valeurs de l'attribut sous forme de chaine
                          (list-val-ch
                                  (if (listp list-val-sym)
                                      (mapcar 'string (#:ivan:ed-util:list-plate list-val-sym))
                                      (string list-val-sym)))
                          ; liste des valeurs de l'attribut pour affichage (chaine de caracteres)
                          (list-val-attr
                                  (if (listp list-val-ch)
                                      list-val-ch
                                      (list list-val-ch)))
                          ; type shirka de l'attribut
                          (type-attr
                                  (car (#:ivan:ed-util:type-simple?
                                               (cadr (#:ivan:ed-util:nature
                                                             (#:ivan:ed-util:classe-de nom-inst-trav)
                                                             nom-attr)))))
                          ; image de la partie gauche
                          (img-pg (stringmenu (list nom-attr)))
                          ; image de la partie droite
                          (img-pd 
                                  (if (null list-val-sym)
                                      ; alors [afficher un ? pour declencher l'inference]
                                      (if battr-inf
                                          (stringselector '("?"))
                                          (stringselector '(""))
                                          )
                                      ; sinon il existe des valeurs. elles seront affichees via la fonction
                                      ; :val-inf? qui verifie si l'attribut est infere. Si c'est le cas, les
                                      ; valeurs sont encadrees par le caractere *.
                                      (if (equal (string type-attr) "chaine")
                                          ; alors [afficher les chaines entre guillemets]
                                          (stringselector (:val-inf? (:list-ch list-val-attr) binf))
                                          ; creation ou type non chaine
                                          (stringselector (:val-inf? list-val-attr binf)))))
                          )
                       ; la largeur de la partie gauche est redefinie (cadrage du signe =)
                       (send 'w img-pg (#:ivan:ed-util:wc larg-pg))
                       
                       ; action de la partie gauche
                       (set-action img-pg (cond ((member 'entite-scarp
						     (#:shirka:noyau:ancetres schema-clas))
                                                 ':action-img-scarp
                                                )
						((equal tp-attr 'objet)
						 ':action-img-tp-objet
						)
						(t ':action-img-pg)))
                       
                       ; action de la partie droite
                       (if (null list-val-sym)
                         ; pas de valeur connue : action = declencher l'inference
                         (if battr-inf
                             (set-action img-pd ':action-img-pd-inf)
                             nil)
			 ; sinon
			 (if (equal tp-attr 'objet)
			   (set-action img-pd ':action-img-tp-objet)
                           ; sinon 
                           (if (null type-attr)
                               ; c'est un type complexe     : action = editer l'instance
                               (set-action img-pd ':action-img-pd-tc)
                               ; sinon c'est un type simple : action = editer l'attribut
                               (set-action img-pd ':action-img-pd-ts)
                               )))
                       ; transferer le nom de l'attribut pour action de la partie droite 
                       ({application}:userdata img-pd nom-attr)                    
                       ; insertion de l'image dans la liste des images list-img
                       
                       (newr list-img (row img-pg " = " img-pd))
                       ; attribut suivant
                       (nextl list-attr-ch)
		       (nextl l-type)
                       ) ; lets
                 ) ; while
          ; la liste des images est renvoyee
          list-img
          ) ; let
    )
;;;*****************************************************************************************
;;; si binf est vrai, la fonction renvoie une liste dont les elements sont encadres
;;; par des *, sinon elle renvoie list-val.

(de :val-inf? (list-val binf)
    (if binf
        (:cat list-val)
        list-val
        )
    )
;;;*****************************************************************************************

(de :cat (l)
    (if (null l) 
        nil
        (reverse
                (cons 
                      (catenate "* " (car l) " *") 
                      (:cat (cdr l))
                      )))
    )
;;;*****************************************************************************************
;;; ajoute des guillemets aux chaines de caracteres de la liste list-val

(de :list-ch (list-val)
    (let
         ( (gui '"""") )
         
         (if (null list-val)
             nil
             (cons
                   (catenate (catenate gui (car list-val)) gui)
                   (:list-ch (cdr list-val))) ))
    )
;;;*****************************************************************************************
;;; construction de la ligne de validation

(de :const-ch-ligne-valid 
    (appli nom-classe nom-inst-trav bcreation attr-mere inst-mere bappel)
    ;(PRINT "ei>> :const-ch-ligne-valid")
    (lets
          (
           (bout-valider   (standardbutton #:ivan:ed-global:txt-valider))
           (bout-annuler   (standardbutton #:ivan:ed-global:txt-annuler))
           (bout-daccord   (standardbutton #:ivan:ed-global:txt-daccord))
           (bout-supprimer (standardbutton (foreground (make-named-color "red")
                                                   #:ivan:ed-global:txt-supprimer)))
           (ligne
                  (#:ivan:ed-util:centeredimage
                          (cond
                                ( (equal bappel 'modsup)
                                 (row bout-valider "   " bout-annuler "   " bout-supprimer))
                                ( (equal bappel 'visu)
                                 (row bout-daccord))
                                ( (equal bappel 'visusup)
                                 (row bout-daccord "   " bout-supprimer))
                                ( t
                                   (row bout-valider "   " bout-annuler))
                                )))
           )
          
          (send 'w ligne (- (send 'width ({in-edinst}:ch-scroll-com appli)) 14))
          
          ; bouton valider
          ({application}:userdata bout-valider
                  (list nom-classe bcreation nom-inst-trav attr-mere inst-mere))
          (set-action bout-valider ':action-bout-valider)
          
          ; bouton annuler
          ({application}:userdata bout-annuler nom-inst-trav)
          (set-action bout-annuler ':action-bout-annuler)
          
          ; bouton d'accord
          ({application}:userdata bout-daccord nom-inst-trav)
          (set-action bout-daccord ':action-bout-daccord)
          
          ; bouton supprimer
          ({application}:userdata bout-supprimer nom-inst-trav)
          (set-action bout-supprimer ':action-bout-supprimer)
          
          ; image
          (#:ivan:ed-util:clusterimage ligne #M#:txt-const:comande)
          )
    )

;;;#########################################################################################
;;;                                   
;;;                                    ACTIONS DES CHAMPS 
;;;
;;;#########################################################################################

;;;*****************************************************************************************
;;; action du bouton standard de validation

(de :action-bout-valider (b)
    (lets
          ( 
           (list-param       ({application}:userdata b))
           (nom-classe       (nth 0 list-param))
           (bcreation        (nth 1 list-param))
           (nom-inst-trav    (nth 2 list-param))
           (attr-mere        (nth 3 list-param))
           (inst-mere        (nth 4 list-param))
           ; recuperation du nom de l'instance dans l'editeur de ligne
           (nom-inst-edl-ch  (send 'get-line (component 'edl-instance b)))
           (nom-inst-edl-sym (#:ivan:ed-util:symbole nom-inst-edl-ch))
           )
          
          (when (activatedp b)
                ; je declenche l'action de l'editeur de ligne 
                ; si vrai, alors je peux declencher l'action du bouton valider
                (when (:action-edl-instance (component 'edl-instance b))
                      ; la valeur est-elle admissible pour l'instance mere ?
                      (ifn (or  (null inst-mere)
                               (#:shirka:moteur:admissible?
                                       nom-inst-trav
                                       (#:ivan:ed-util:ch-as-sym attr-mere)
                                       (#:shirka:noyau:schema (#:ivan:ed-util:classe-de inst-mere))
                                       (#:shirka:noyau:schema inst-mere)
                                       ))
                           ; ALORS elle n'est pas admissible
                           (:action-sa-warn b
                                   (list
                                         ""
                                         (font #:ivan:ed-global:fstdb #M#:txt-const:inst-non-adm)))
                           ; SINON
                           ; [la valeur est admissible] ou [il n'y a pas d'instance mere]
                           ; on peut creer l'instance
                           (if bcreation
                               (#:ivan:ed-util:cr-inst nom-classe nom-inst-edl-sym))
                           
                           ; recopier les modifs vers l'instance d'origine
                           (#:ivan:ed-util:copie-inst
                                   (#:ivan:ed-util:schema nom-inst-trav)
                                   (#:ivan:ed-util:schema nom-inst-edl-sym))
                           
                           ;  suppression de l'instance de travail
                           (#:ivan:ed-util:sup-inst (#:ivan:ed-util:schema nom-inst-trav) nil)
                           
                           ; supprimer l'instance de la liste des instances en modification
                           (setq #:ivan:ed-global:%list-nom-inst
                                 (delete nom-inst-edl-sym #:ivan:ed-global:%list-nom-inst))
                           
                           ; fermer la fenetre
                           (remove-application (component 'top b))
                           
(when #:trace:ed
                           (PRINT "SORTIE EI ======================================")
)
                           ) ; endifn admissible?
                      ) ;when
                ) ; end when
          )
    )
;;;*****************************************************************************************
;;; action du bouton standard d'annulation

(de :action-bout-annuler (b)
    (lets
          (
           (nom-inst-trav   ({application}:userdata b))
           ; recuperation du nom de l'instance dans l'editeur de ligne
           (nom-inst-edl-ch  (send 'get-line (component 'edl-instance b)))
           (nom-inst-edl-sym (#:ivan:ed-util:symbole nom-inst-edl-ch))
           )
          (when (activatedp b)
                
                ; supprimer l'instance de la liste des instances en modification
                (setq #:ivan:ed-global:%list-nom-inst
                      (delete nom-inst-edl-sym #:ivan:ed-global:%list-nom-inst))
                
                ; supprimer l'instance de travail
                (#:ivan:ed-util:sup-inst (#:ivan:ed-util:schema nom-inst-trav) nil)
                
                ; mettre a vide le nom de l'instance pour pouvoir renvoyer nil
                ; deproteger l'editeur ds le cas de la creation
                ({medite}:readonly-flag (component 'edl-instance b) nil)
                ; vider l'editeur
                (send 'set-line (component 'edl-instance b) '"")
                
                ; fermer la fenetre
                (remove-application (component 'top b))
                
(when #:trace:ed
                (PRINT "SORTIE EI ======================================")
)
                )
          )
    )
;;;*****************************************************************************************
;;; action du bouton standard d'accord

(de :action-bout-daccord (b)
    (lets
          (
           (nom-inst-trav   ({application}:userdata b))
           ; recuperation du nom de l'instance dans l'editeur de ligne
           (nom-inst-edl-ch  (send 'get-line (component 'edl-instance b)))
           (nom-inst-edl-sym (#:ivan:ed-util:symbole nom-inst-edl-ch))
           )
          (when (activatedp b)
                
                ; supprimer l'instance de la liste des instances en modification
                (setq #:ivan:ed-global:%list-nom-inst
                      (delete nom-inst-edl-sym #:ivan:ed-global:%list-nom-inst))
(when #:trace:ed
                (PRINT "     #:ivan:ed-global:%list-nom-inst :" #:ivan:ed-global:%list-nom-inst) 
)
                ; supprimer l'instance de travail
                (#:ivan:ed-util:sup-inst (#:ivan:ed-util:schema nom-inst-trav) nil)
                
                ; fermer la fenetre
                (remove-application (component 'top b))
                
(when #:trace:ed
                (PRINT "SORTIE EI ======================================")
)
                )
          )
    )
;;;*****************************************************************************************
;;; action du bouton de suppression de l'instance

(de :action-bout-supprimer (b)
    (lets
          (
           (nom-inst-trav   ({application}:userdata b))
           ; recuperation du nom de l'instance dans l'editeur de ligne
           (nom-inst-edl-ch  (send 'get-line (component 'edl-instance b)))
           (nom-inst-edl-sym (#:ivan:ed-util:symbole nom-inst-edl-ch))
           )
          
          (when (activatedp b)
                ; supprimer l'instance de la liste des instances en modification
                (setq #:ivan:ed-global:%list-nom-inst
                      (delete nom-inst-edl-sym #:ivan:ed-global:%list-nom-inst))
                ; supprimer l'instance de travail
                (#:ivan:ed-util:sup-inst (#:ivan:ed-util:schema nom-inst-trav) nil)
                ; supprimer l'instance
                (#:ivan:ed-util:sup-inst (#:ivan:ed-util:schema nom-inst-edl-sym) nil)
                ; fermer la fenetre
                (remove-application (component 'top b))
                
(when #:trace:ed
                (PRINT "SORTIE EI ======================================")
)
                )
          )
    )
;;;*****************************************************************************************
;;; action du bouton standard d'effacement de la zone d'edition

(de :action-bout-effacer (b)
    (let
         ()
         (when (activatedp b)
               ; vider l'editeur de ligne de son contenu     
               (send 'set-line (component 'edl-instance b) '"")
               ; message
               (:envoie-msg b 
                       (list ""
                             (font #:ivan:ed-global:fstd #M#:txt-const:donner-nom-inst) ))
               ))
    )
;;;*****************************************************************************************
;;; action de l'image partie droite. Cas valeur a inferer lors du clic sur "?" de l'image

(de :action-img-pd-inf (img-pd)
    (lets
          (
           ; nom de l'attribut          
           (nom-attr      ({application}:userdata img-pd))
           ; chaine selectionnee
           (val-sel    (send 'get-selected-string img-pd))
           (pere       (component 'top img-pd))
           (list-param ({application}:userdata pere))
           (nom-classe        (nth 0 list-param))
           (bcreation         (nth 1 list-param))
           (nom-inst-trav     (nth 2 list-param))
           (inst-trav         (nth 3 list-param))
           (classes-attr      (nth 4 list-param))
           (sauf-classes-attr (nth 5 list-param))
           (nom-inst          (nth 6 list-param))
           (bvisu             (nth 7 list-param))
           ; vecteur du schema de classe
           (schema-clas   (#:ivan:ed-util:schema nom-classe))
           
           (list-msg-echec
                   (list
                         (row (font #:ivan:ed-global:fstd #M#:txt-const:val-inst)
                              (font #:ivan:ed-global:fstdb nom-attr) )
                         (font #:ivan:ed-global:fstd #M#:txt-const:pas-inf) ))
           (list-msg-succes
                   (list 
                         (row (font #:ivan:ed-global:fstd #M#:txt-const:clic-att)
                              (font #:ivan:ed-global:fstdb nom-attr))
                         (font #:ivan:ed-global:fstd #M#:txt-const:pour-mod-val-inf) ))
           )
          (ifn val-sel
               ; alors (cas de la reinitialisation)
               't
               ; sinon
(when #:trace:ed
               (PRINT "     ACTION PARTIE DROITE INFERENCE : " val-sel)
)
               ; reinitialiser le selectionneur
               (send 'set-selected-string img-pd "")
               
               (when (:action-edl-instance (component 'edl-instance img-pd))
                     ; inferer l'attribut
                     (let 
                          (
                           (valeurs (#:shirka:moteur:valeur?
                                            (#:ivan:ed-util:symbole nom-attr) nom-inst-trav))
                           )
                          (if valeurs
                              ; l'inference a reussie : reaffichage
                              (progn
                                     ; inserer l'attribut dans la liste des attributs inferes
                                     ({in-edinst}:ch-list-attr-inf
                                             (component 'top img-pd)
                                             (:const-ch-list-attr-inf
                                                     (component 'top img-pd)
                                                     nom-attr
                                                     t))
                                     ; envoyer la nouvelle image du selectionneur
                                     (:nouv-img-scroll-attr
                                             (component 'top img-pd) 
                                             nom-inst-trav
                                             schema-clas
                                             bcreation
                                             classes-attr
                                             sauf-classes-attr
                                             t)
                                     ; message de succes
                                     (unless bvisu (:action-sa-warn img-pd list-msg-succes))
                                     )
                              ; sinon message d'echec
                              (:action-sa-warn img-pd list-msg-echec)
                              )
                          )
                     )
               )
          ) 
    )
;;;*****************************************************************************************
;;; action de l'image partie droite. cas de l'attribut de type simple

(de :action-img-pd-ts (img-pd)
    (lets
          (
           (nom-attr      ({application}:userdata img-pd))
           ; chaine selectionnee
           (val-sel (:enleve-stars (send 'get-selected-string img-pd)))
           (pere              (component 'top img-pd))
           (list-param        ({application}:userdata pere))
           (nom-classe        (nth 0 list-param))
           (bcreation         (nth 1 list-param))
           (nom-inst-trav     (nth 2 list-param))
           (inst-trav         (nth 3 list-param)) 
           (classes-attr      (nth 4 list-param))
           (sauf-classes-attr (nth 5 list-param))
           (nom-inst          (nth 6 list-param))
           (bvisu             (nth 7 list-param))
           )
          (ifn val-sel
               ; alors (cas de la reinitialisation)
               't
               ; sinon
(when #:trace:ed
               (PRINT "     ACTION PARTIE DROITE TS : " val-sel)
)
               ; reinitialiser le selectionneur
               (send 'set-selected-string img-pd "")
               (when (:action-edl-instance (component 'edl-instance img-pd))
                     (if bvisu
                         (:action-sa-warn img-pd
                                 (list
                                       ""
                                       (font #:ivan:ed-global:fstd #M#:txt-const:mode-visu)))
                         ; on peut cliquer sur le nom de l'attribut
                         (:action-sa-warn img-pd
                                 (list 
                                       (row (font #:ivan:ed-global:fstd #M#:txt-const:pour-mod-val)
                                            (font #:ivan:ed-global:fstdb val-sel)
                                            (font #:ivan:ed-global:fstd ",") )
                                       (row (font #:ivan:ed-global:fstd #M#:txt-const:clic-att)
                                            (font #:ivan:ed-global:fstdb nom-attr)
                                            (font #:ivan:ed-global:fstd ".") ) ))
                         )
                     )
               )
          ) 
    )
;;;*****************************************************************************************
;;; action de l'image partie droite. cas de l'attribut de type complexe

(de :action-img-pd-tc (img-pd)
    (lets
          (
           (nom-attr      ({application}:userdata img-pd) )
           ; valeur selectionnee (c'est le nom de l'attribut)
           (val-sel (:enleve-stars (send 'get-selected-string img-pd)))
           (pere              (component 'top img-pd))
           (list-param        ({application}:userdata pere))
           (nom-classe        (nth 0 list-param))
           (bcreation         (nth 1 list-param))
           (nom-inst-trav     (nth 2 list-param))
           (inst-trav         (nth 3 list-param)) 
           (classes-attr      (nth 4 list-param))
           (sauf-classes-attr (nth 5 list-param))
           (nom-inst          (nth 6 list-param))
           (bvisu             (nth 7 list-param))
           (bappel            (nth 8 list-param))
           ; vecteur du schema de classe
           (schema-clas       (#:ivan:ed-util:classe-de nom-inst-trav) )
           (list-nature       (#:ivan:ed-util:nature 
                                      schema-clas
                                      (#:ivan:ed-util:ch-as-sym nom-attr)))
           (type-attr         (cadr list-nature))
           )
          
          (ifn val-sel
               ; alors
               't
               ; sinon
(when #:trace:ed
               (PRINT "     ACTION PARTIE DROITE TC : " val-sel)
)
               ; reinitialiser le selectionneur
               (send 'set-selected-string img-pd "")
               
               (when (:action-edl-instance (component 'edl-instance img-pd))
                     
                     ; le nom dans l'editeur de ligne est correcte
                     (if (member (#:ivan:ed-util:symbole val-sel) #:ivan:ed-global:%list-nom-inst)
                         ; erreur
                         (:envoie-msg img-pd
                                 (list
                                       (font #:ivan:ed-global:fstd #M#:txt-const:ivalid-act)
                                       (font #:ivan:ed-global:fstd #M#:txt-const:deja-editee) ))
                         ; sinon
                         (lets
                               (
                                (val-sel-sym (#:ivan:ed-util:ch-as-sym val-sel))
                                
                                ; classe de la valeur selectionnee : 
                                ; (ce n'est pas la classe de l'attribut)
                                (classe-val-sel
                                        (#:ivan:ed-util:nom-sch-sym 
                                                (#:ivan:ed-util:classe-de val-sel-sym)) )
                                (meme-classe
                                        (equal (#:ivan:ed-util:schema classe-val-sel) schema-clas))
                                (pour-classes (if meme-classe classes-attr nil))
                                (sauf-classes (if meme-classe sauf-classes-attr nil))
                                )
                               ; lancer l'editeur d'instance
                               (:edit classe-val-sel
                                      val-sel-sym
                                      pour-classes
                                      sauf-classes
                                      nom-attr
                                      nom-inst-trav
                                      bappel)
                               ; message en retour
                               (:action-sa-warn2 img-pd
                                       (list
                                             (row (font #:ivan:ed-global:fstd
							#M#:txt-const:viens-visu)
                                                  (font #:ivan:ed-global:fstdb val-sel) )
                                             (row (font #:ivan:ed-global:fstd #M#:txt-const:de-att)
                                                  (font #:ivan:ed-global:fstdb nom-attr)
                                                  (font #:ivan:ed-global:fstd ".") )
                                             (font #:ivan:ed-global:fstd #M#:txt-const:sel-att-com))
                                       ))
                         ) ; lets
                     ) ; endif member
               ) ; when
          ) ; ifn val-sel
    )

(de :enleve-stars (str)
        (if (equal (substring str 0 1) "*")
                (substring str 2 (- (slength str) 4))
                str
        )
)

;;;*****************************************************************************************
;;; action de l'image partie gauche / droite si le type de l'attribut est objet

(de :action-img-tp-objet (img-p)
	(:envoie-msg img-p (list ""
			(font #:ivan:ed-global:fstd #M#:txt-const:imp-type-obj)))
)

(de :action-img-scarp (img-p)
        (:envoie-msg img-p (list ""
                        (font #:ivan:ed-global:fstd #M#:txt-const:imp-scarp)))
)

;;;*****************************************************************************************
;;; action de l'image partie gauche

(de :action-img-pg (img-pg)
    (lets
          (
           (nom-edl-inst-sym
                   (#:ivan:ed-util:symbole (send 'get-line (component 'edl-instance img-pg))))  
           (val-sel    (send 'get-selected-string img-pg))
           
           (pere              (component 'top img-pg))
           (list-param        ({application}:userdata pere))
           (nom-classe        (nth 0 list-param))
           (bcreation         (nth 1 list-param))
           (nom-inst-trav     (nth 2 list-param))
           (inst-trav         (nth 3 list-param))
           (classes-attr      (nth 4 list-param))
           (sauf-classes-attr (nth 5 list-param))
           (nom-inst          (nth 6 list-param))
           (bvisu             (nth 7 list-param))
           ; vecteur du schema de classe
           (schema-clas       (#:ivan:ed-util:schema nom-classe))
           )
          (ifn val-sel
               ; alors
               't
               ; sinon
(when #:trace:ed
               (PRINT "     ACTION PARTIE GAUCHE : attribut > " val-sel " <")
)
               ; reinitialiser le selectionneur
               (send 'set-selected-string img-pg "")
               ; tester si on est en visualisation uniquement ?
               (if bvisu
                   ; erreur
                   (:envoie-msg img-pg
                           (list ""
                                 (font #:ivan:ed-global:fstd #M#:txt-const:imp-visu) ))
                   ; sinon on peut appeler l'editeur d'attributs
                   (when (:action-edl-instance (component 'edl-instance img-pg))
                         ; lancer l'editeur d'attribut
                         (#:ivan:ed-edattr:edit
                                 nom-edl-inst-sym
                                 (#:ivan:ed-util:ch-as-sym val-sel)
                                 (#:ivan:ed-util:ch-as-sym nom-inst-trav) )
                         
                         
                         ; faut-il supprimer l'attribut de la liste des attributs inferes ?
                         ({in-edinst}:ch-list-attr-inf     (component 'top img-pg)
                                 (:const-ch-list-attr-inf
                                         (component 'top img-pg)
                                         val-sel
                                         nil))
                         ; envoyer la nouvelle image du selectionneur
                         (:nouv-img-scroll-attr  
                                 (component 'top img-pg) 
                                 nom-inst-trav
                                 schema-clas
                                 bcreation
                                 classes-attr
                                 sauf-classes-attr
                                 t)
                         ; message en retour
                         (:action-sa-warn2 img-pg
                                 (list
                                       (row (font #:ivan:ed-global:fstd #M#:txt-const:viens-edit)
                                            (font #:ivan:ed-global:fstdb val-sel) )
                                       ""
                                       (font #:ivan:ed-global:fstd #M#:txt-const:sel-att-com)
                                       ))
                         ) ; when
                   ) ; if bvisu
               ) ; ifn val-sel 
          ) 
    )
;;;*****************************************************************************************
;;; Renvoie la nouvelle image du selectionneur d'attributs apres les fonctions
;;; :action-img-pd-inf et :action-img-pg

(de :nouv-img-scroll-attr
    (appli nom-inst-trav schema-clas bcreation classes-attr sauf-classes-attr binf)
    (lets
          (
           (li (:const-img-attr
                       nom-inst-trav schema-clas bcreation classes-attr sauf-classes-attr))
           (col (apply 'column li))
           ; selectionneur d'image associe
           (sa-attr 
                   (selectorapplication 0 0
                           (send 'width col) (send 'height col)
                           col ))
           (scroll-attr
                   (scroller 0 0
                           (- (send 'width ({in-edinst}:ch-scroll-com appli)) 14)
                           (- (#:ivan:ed-util:hc :%h-ei)
                              (send 'height ({in-edinst}:ch-edl-inst appli))) ; hauteur
                           
                           sa-attr
                           t t)) 
           )
          (if binf
              (send 'set-image (component 'scroll-attr appli) scroll-attr)
              (send 'new-image (component 'scroll-attr appli) scroll-attr)
              )
          )
    )
;;;*****************************************************************************************
;;; envoie du message list-msg a au composant sa-warn de l'application appli

(de :action-sa-warn (appli list-msg)
    (let
         (
          (col-msg (apply 'column list-msg)) 
          )
         (if (member 'sa-warn (all-components appli))
             (send 'set-image (component 'sa-warn appli) col-msg) 
             (prompt-warn col-msg)
             )
         )
    )
;;;*****************************************************************************************
;;; envoie le message list-msg en declenchant 2 beeps

(de :envoie-msg (comp list-msg)
    (tybeep)
    (tybeep)
    (:action-sa-warn comp list-msg) 
    )
;;;*****************************************************************************************
;;; envoi du message list-msg au composant sa-warn de l'application appli. 
;;; message par (send 'new-image...) plutot que (send 'set-image...)
;;; pour permettre un raffraichissement visuel plus agreable
;;; (cas ou la fenetre de l'editeur d'instances avait ete masquee; il faut la
;;; reafficher avec le message d'action)

(de :action-sa-warn2 (appli list-msg)
    (let
         (
          (col-msg (apply 'column list-msg)) 
          )
         (if (member 'sa-warn (all-components appli))
             (send 'new-image (component 'sa-warn appli) col-msg)
             (prompt-warn col-msg)
             )
         )
    )
;;;*****************************************************************************************
;;; action de l'editeur de ligne (pour saisir le nom de l'instance en creation)
;;; le bouton est aussi actif pour verifier si le nom saisi est admissible

(de :action-edl-instance (edl)
    (lets
          (
           (list-param  ({application}:userdata edl))
           (appli       (nth 0 list-param))
           (bcreation   (nth 1 list-param))
           (nom-inst    (nth 2 list-param))
           (bvisu       (nth 3 list-param))
           (nom-edl-ch  (send 'get-line edl))
           (nom-edl-sym (#:ivan:ed-util:symbole nom-edl-ch))
           bok
           )
          (setq bok
                (ifn nom-edl-sym
                     (progn
(when #:trace:ed
                            (PRINT "cas : pas de valeur")
)
                            (unless bvisu 
                                    (:envoie-msg edl
                                            (list
                                                  ""
                                                  (font #:ivan:ed-global:fstdb
							#M#:txt-const:donner-nom)))
                                    (current-keyboard-focus-window ({application}:window edl) )
                                    )
                            nil
                            )
                     ; sinon
                     (if (and bcreation (member nom-edl-sym (#:ivan:ed-util:l-inst-t-sym 'objet)))
                         ; le nom existe deja !
                         (let
                              (
                               (classe-nom-edl 
                                       (#:shirka:noyau:nom-sch (#:ivan:ed-util:classe-de nom-edl-sym))) 
                               ) 
(when #:trace:ed
                              (PRINT "     cas : l'instance existe deja")
)
                              (:envoie-msg edl
                                      (list
                                            (row (font #:ivan:ed-global:fstd #M#:txt-const:l-inst)
                                                 (font #:ivan:ed-global:fstdb nom-edl-ch) )
                                            (row (font #:ivan:ed-global:fstd #M#:txt-const:classe)
                                                 (font #:ivan:ed-global:fstdb
                                                       (string classe-nom-edl) )
                                                 (font #:ivan:ed-global:fstd ")" ) )
                                            (font #:ivan:ed-global:fstd #M#:txt-const:existe-deja)))
                              nil
                              )
                         ; sinon
                         (if (numberp (#:ivan:ed-util:ch-as-sym nom-edl-ch))
                             (progn
                                    (:envoie-msg edl
                                            (list
                                                  (font #:ivan:ed-global:fstd
							#M#:txt-const:nom-inval)
                                                  ""
                                                  (font #:ivan:ed-global:fstd #M#:txt-const:no-num)))
                                    nil
                                    )
                             ; sinon
                             (unless nom-inst
                                     (authorize-application (component 'scroll-attr edl))
                                     ; envoyer le message d'action
                                     (:action-sa-warn edl
                                             (list
                                                   ""
                                                   ))
                                     ; insertion de l'instance
                                     (setq #:ivan:ed-global:%list-nom-inst 
                                           (newr #:ivan:ed-global:%list-nom-inst nom-edl-sym))
                                     
                                     ; mettre a jour le titre de la fenetre pour le cas de la creation
                                     (if (>= (read-from-string aidaversion) 1.5403)
                                         (send 'set-line (component 'edl-titre appli)
                                               (catenate #M#:txt-const:creer nom-edl-ch) )
                                         ; SINON
                                         
                                         (#:ivan:ed-util:titre-fenetre
                                                 (component 'top appli)
                                                 (catenate #M#:txt-const:creer nom-edl-ch))
                                         )
                                     ; proteger l'editeur de ligne
                                     ({medite}:readonly-flag edl 't)
                                     (send 'inhibit-application (component 'bout-effacer edl) )
                                     )
                             ; vrai
                             t
                             )
                         )
		    )
                )
          ; renvoyer la valeur
          bok
          )
    )
