;;; repertoire: ~/LeShirka/LeIvan/CodeIvan/Editeur/
;
;     +----------------------------------------+-----------------------------------------+
;     | fichier : ivan-ed-editer.ll            | package : #:ivan:ed-editer              |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface de Visualisation et d'Aide a la Navigation                 |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Sylvain Grivaud          1991                             |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     | descriptif du package :                                                          |
;     |                                                                                  |
;     |  Ce fichier contient 4 fonctions :                                               |
;     |   > :ei (nom-classe nom-inst pour-classes sauf-classes bappel)                   |
;     |      = appel de l'editeur d'instances pour l'instance nom-inst de la classe      |
;     |        nom-classe. pour-classes est la liste des classes d'attribut a considerer |
;     |        et sauf-classes, la liste des classes d'attribut a ne pas considerer.     |
;     |        Ces 2 arguments sont exclusifs.                                           |
;     |        En fonction de la valeur de bappel, la ligne de validation de l'editeur   |
;     |        sera differente :                                                         |
;     |     ------------------------------+----------------+--------------+------------  |
;     |     !   boutons de validation     !      cas       !   instance   !   bappel  !  |
;     | ----+-----------------------------+----------------+--------------+------------  |
;     | ! 1 ! VALIDER  ANNULER            !  creation      !n'existe pas  !     nil   !  |
;     | ----+-----------------------------+----------------+--------------+------------  |
;     | ! 2 ! VALIDER  ANNULER            !  modification  ! existe       !     nil   !  |
;     | ----+-----------------------------+----------------+--------------+------------  |
;     | ! 3 ! VALIDER  ANNULER  SUPPRIMER !  modification  ! existe       !  'modsup  !  |
;     | !   !                             !  suppression   !              !           !  |
;     | ----+-----------------------------+----------------+--------------+------------  |
;     | ! 4 !          D'ACCORD           !  visualisation ! existe       !  'visu    !  |
;     | ----+-----------------------------+----------------+--------------+------------  |
;     | ! 5 !      D'ACCORD SUPPRIMER     !  visualisation ! existe       !  'visusup !  |
;     | !   !                             !  suppression   !              !           !  |
;     | ----------------------------------+----------------+--------------+------------  |
;     |                                                                                  |
;     |   > :ea (nom-inst nom-attr)                                                      |
;     |      = appel a l'editeur d'attributs pour l'attribut nom-attr de l'instance      |
;     |        nom-inst.                                                                 |
;     |                                                                                  |
;     |   > :attribut-shirka (vf)                                                        |
;     |      = positionne la variable globale #:ivan:ed-global:battribut-shirka a        |
;     |        vrai ou faux.                                                             |
;     |        par defaut, le booleen est faux (pas des visualisations des attributs     |
;     |   > :meta-attribut (vf)                                                          |
;     |      = positionne la variable globale #:ivan:ed-global:meta-attribut a           |
;     |        vrai ou faux.                                                             |
;     |        par defaut, le booleen est faux (pas des visualisations des attributs)    |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+

;;;*****************************************************************************************

(defvar #:sys-package:colon '#:ivan:ed-editer)

;;;*****************************************************************************************
;;; appel a l'editeur d'instance pour l'instance nom-inst de la classe nom-classe
;;; bappel est vrai = visualisation uniquement, il ne sera pas possible de saisir.
;;; les classes d'attributs a considerer ou a exclure sont respectivement contenues
;;; dans les listes pour-classes et sauf-classes.

(de :ei (nom-classe 
                nom-inst 
                pour-classes 
                sauf-classes 
                bappel)
    (let
         (
          (cur-font (current-font))
          editeur
          )
         
         (current-font #:ivan:font:f9x15b)
         (setq editeur (#:ivan:ed-edinst:edit 
                               nom-classe
                               nom-inst
                               (mapcar 'string pour-classes)
                               (mapcar 'string sauf-classes) 
                               nil nil bappel))
         (current-font cur-font)
         editeur
         )
    )
;;;*****************************************************************************************
;;; appel a l'editeur d'attribut pour l'attribut nom-attr de l'instance nom-inst

(de :ea (nom-inst
                nom-attr)
    (lets
          ( 
           (cur-font (current-font)) 
           (nom-classe  (#:ivan:ed-util:nom-sch-sym (#:ivan:ed-util:classe-de nom-inst)))
           ; vecteur du schema de classe
           (schema-clas   (#:ivan:ed-util:schema (#:ivan:ed-util:symbole nom-classe)) )
           
           ; booleen de creation 
           (bcreation (or
                          (null nom-inst)
                          (not
                               (member (#:ivan:ed-util:ch-as-sym nom-inst)
                                       (#:ivan:ed-util:l-inst-t-sym schema-clas)))))
           
           ; creation instance de travail
           (nom-inst-trav (#:ivan:ed-util:ch-as-sym (catenate "instrav-" (gensym))))
           
           ; inst. temporaire
           (inst-trav     (#:ivan:ed-util:cr-inst nom-classe nom-inst-trav)) 
           )
          
          (current-font #:ivan:font:f9x15b)
          
          ; initialisation de l'instance de travail dans le cas de la modification
          (unless bcreation                        
                  (#:ivan:ed-util:copie-inst (#:ivan:ed-util:schema nom-inst) inst-trav))
          
          ; insertion du nom de l'instance de travail
          (setq #:ivan:ed-global:%list-inst-trav
                (newr #:ivan:ed-global:%list-inst-trav (string nom-inst-trav)))
          
          ; editeur d'attribut
          (#:ivan:ed-edattr:edit nom-inst nom-attr nom-inst-trav) 
          
          ; recopier les modifs vers l'instance d'origine
          (#:ivan:ed-util:copie-inst (#:ivan:ed-util:schema nom-inst-trav)
                  (#:ivan:ed-util:schema nom-inst))
          
          ;  suppression de l'instance de travail
          (#:ivan:ed-util:sup-inst (#:ivan:ed-util:schema nom-inst-trav) nil)
          
          (current-font cur-font)
          ) 
    )
;;;*****************************************************************************************
;;; positionne la variable globale battribut-shirka a vrai / faux

(de :attribut-shirka (vf)
    (setq #:ivan:ed-global:%battribut-shirka  vf)
    (if vf
(when #:trace:ed
        (print "     Classe d'attribut attribut-shirka non visualisee !")
)
(when #:trace:ed
        (print "     Classe d'attribut attribut-shirka visualisee !")
)
    ))
;;;*****************************************************************************************
;;; positionne la variable globale battribut-shirka a vrai / faux

(de :meta-attribut (vf)
    (setq #:ivan:ed-global:%bmeta-attribut  vf)
    (if vf
(when #:trace:ed
        (print "     Classe d'attribut meta-attribut   non visualisee !")
)
(when #:trace:ed
        (print "     Classe d'attribut meta-attribut   visualisee !")
)
    ))
;;;*****************************************************************************************
