;;; Repertoire: ~/LeShirka/LeIvan/CodeIvan/Editeur/
;
;     +----------------------------------------+-----------------------------------------+
;     | fichier : ivan-ed-util.ll              | package : #:ivan:ed-util                |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface de Visualisation et d'Aide a la Navigation                 |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Sylvain Grivaud          1991                             |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     | descriptif du package :                                                          |
;     |                                                                                  |
;     |  Ce fichier contient toutes les fonctions utilitaires utilisees pour la constru- |
;     |  de l'editeur d'instances et de l'editeur d'attributs.                           |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+

;;;*****************************************************************************************

(defvar #:sys-package:colon '#:ivan:ed-util)

;;;*****************************************************************************************
;;; valeur de l'attribut nom-attr de l'instance nom-inst avec inference . symbole

(de :val-attr-inf-sym (nom-attr nom-inst)
    (let
         (
          (valeur 
                  (#:shirka:moteur:valeur? (:symbole nom-attr) (:symbole nom-inst) ))
          )
         ; je transforme le resultat en 1 liste
         (unless (listp valeur)
                 (setq valeur (list valeur))
                 )
         ; verifier s'il y a des vecteurs : dans ce cas, j'extraie le nom du schema
         (mapcar '(lambda (valeur)
                          (let
                               (
                                (val (if (vectorp valeur)
                                         (:nom-sch-sym valeur)
                                         valeur))
                                )
                               val
                               ))
                 valeur
                 )
         )
    )
;;;*****************************************************************************************
;;; valeur de l'attribut nom-attr de l'instance nom-inst avec inference . chaine

(de :val-attr-inf-ch (nom-attr nom-inst)
    (let
         ( (l-val (:val-attr-inf-sym nom-attr nom-inst)) )
         (if (consp l-val)
             (mapcar 'string l-val)
             (string l-val) ))
    )
;;;*****************************************************************************************
;;; valeur de l'attribut nom-attr de l'instance nom-inst SANS inference . symbole

(de :val-attr-sym (nom-attr nom-inst)
    (lets
          (
           (schema-inst   (:schema nom-inst))
           (classe-inst   (:classe-de nom-inst))
           (schema-classe (:schema classe-inst))
           (position      (:pos-nom-att schema-classe nom-attr))
           list
           )
          (ifn position 
(when #:trace:ed
               (PRINT "attr " nom-attr " inst " nom-inst " pas de position ")
)
               (setq list (:val-inst-sym schema-inst position) )
               )
          list
          )
    )
;;;*****************************************************************************************
;;; valeur de l'attribut nom-attr de l'instance nom-inst SANS inference . chaine

(de :val-attr-ch (nom-attr nom-inst)
    (let
         ( (l-val (:val-attr-sym nom-attr nom-inst)) )
         
(when #:trace:ed
         (PRINT "l-val " l-val)
)
         (if (consp l-val)
             (mapcar 'string l-val)
             (string l-val) ))
    )
;;;*****************************************************************************************
;;; image img centree dans une boite rectangulaire avec contour et sans effet de bord

(de :boxedimage (img)
    (boxedimage 0 0
            (+ 7 (send 'width img)) (+ 7 (send 'height img))
            img
            )             
    )
;;;*****************************************************************************************
;;; image img centree dans une boite rectangulaire sans contour et sans effet de bord

(de :centeredimage (img)
    (centeredimage 0 0
            (+ 7 (send 'width img)) (+ 7 (send 'height img))
            img
            )             
    )
;;;****************************************************************************************
;;; cadrer verticalement img2 dans img1

(de :centeredview-h (img1 img2)
    (centeredview
            (rectangle 0 0 
                    0 (send 'height img1))
            img2
            )
    )
;;;*****************************************************************************************
;;; cadrer honrizontalement img2 dans img1

(de :centeredview-w (img1 img2)
    (centeredview
            (rectangle 0 0
                    (send 'width img1) 0)
            img2
            )
    )
;;;*****************************************************************************************
   
(de :clusterimage (img titre)
    
    (if (< (read-from-string aidaversion) 1.5401)
        ;-------------------------------------------------------------------------
        (let
             (
              (ti  (font #:ivan:ed-global:ftitre titre) )
              )
             
             (:boxedimage
                     (column 
                             ti
                             (rectangle 0 0 0 5)
                             img
                             )
                     )
             )
        ;-------------------------------------------------------------------------
        ; sinon
        ;-------------------------------------------------------------------------
        (lets
              (
               (ti (font #:ivan:ed-global:ftitre titre))
               
               (image
                      (centeredimage
                              0 0
                              (+ 7 (send 'width img)) (+ 7 (send 'height img)) 
                              img
                              )
                      )
               (clust-ti (clusterimage ti img nil nil))
               )
              clust-ti
              )
        ;-------------------------------------------------------------------------
        ) ; if
    )
;;;*****************************************************************************************
;;; cadrer le titre de la fenetre

(de :titre-fenetre (appli titre)
    (send 'set-title appli (format () "~48@A" titre) )
    )
;;;*****************************************************************************************
;;; reinitialiser le scroller scroll

(de :positionne (scroll ins-en-fin)
    (let
         ()
         (send 'reinitialise scroll)
         (if ins-en-fin
             (send 'ensure-visible scroll
                   0 (send 'height ({scroller}:scrollee scroll)))
             ; sinon
             (send 'ensure-visible scroll 0 0)
             ))
    )
;;;*****************************************************************************************
;;;  renvoyer la largeur en pixel en fonction de la largeur du caractere courant

(de :wc (largeur)
    (* largeur #wc)
    )
;;;*****************************************************************************************
;;;  renvoyer la hauteur en pixel en fonction de la hauteur du caractere courant

(de :hc (hauteur)
    (* hauteur #hc)
    )
;;;*****************************************************************************************
;;; renvoyer la largeur de la chaine de largeur max d'une liste de chaines

(de :larg-max (l)
    (if (null l)
        0
        (apply 'max (mapcar 'plength l))
        )
    )
;;;*****************************************************************************************
;;; Il s'agit de l'insertion en fin selon la syntaxe Shirka. (#:shirka:moteur:aj-val ...)
;;; Case a cocher est  VRAI   =
;;;              insertion en en-tete donc valeur pour shirka : nil
;;;                    FAUSSE =  (valeur par defaut)
;;;              insertion en fin     donc valeur pour shirka : t

(de :ins-en-fin? (cb)
    ; vrai = insertion en fin
    ; faux = insertion en tete
    (not (send 'checkedp cb))
    )
;;;*****************************************************************************************
;;; difference de listes : on n'utilise pas le module <sets> pour etre compatible en 1.54

(de :diff-liste (l1 l2)
    (let
         ( (l (if (listp l1) l1 (list l1))) )
         (cond
               ((null l) nil)
               ((member (car l) l2) (:diff-liste (cdr l) l2))
               (t (cons (car l) (:diff-liste (cdr l) l2))))
         )
    )
;;;*****************************************************************************************
;;; supprime l'element de rang n de la liste l
;;; soit la liste (a b c),
;;; (supp-nl 1 l) -> (b c); (supp-nl 2 l) -> (a c); (supp-nl 3 l) -> (a b)

(de :supp-nl (n l)
    (let
         (
          (len (length l))
          (r   (- n 1))
          )
         (if (and (<= 1 n) (<= n len))             
             (append (firstn r l) (lastn (- len r 1) l))
(when #:trace:ed
             (print "impossible supprimer element " n " de la liste " l)
             (print "le traitement s'arrete sur une division par 0")
)
             (/ 1 0) ; pour faire tomber en erreur !
             )
         )
    )
;;;*****************************************************************************************
;;; remplacer l'element de rang n de la liste l par la valeur v

(de :remp-nlv (n l v)
    (let
         (
          (len (length l))
          (r   (- n 1))
          )
         (if (and (<= 1 n) (<= n len))
             (append (firstn r l) (list v) (lastn (- len r 1) l))
(when #:trace:ed
             (print "impossible remplacer element " n " de la liste " l)
             (print "le traitement s'arrete sur une division par 0")
)
             (/ 1 0) ; pour faire tomber en erreur !
             )
         )
    )
;;;*****************************************************************************************
;;; renvoyer la liste plate contenant les atomes de liste
;;; (ne prend qu'une occurence de chaque atome)

(de :list-plate (liste)
    (let
         (
          (l (:list-plate2 liste))
          li
          )
         (mapc '(lambda (atom)
                        (unless (member atom li)       
                                (setq li (cons atom li))
                                )
                        )
               l
               )
         (reverse li)
         )
    )
;;;*****************************************************************************************
;;; renvoyer la liste plate des atomes de liste

(de :list-plate2 (liste)
    (cond
          ( (null liste)  nil )
          ( (atom liste) (cons liste nil) )
          ( t           (append
                                (:list-plate2 (car liste) )
                                (:list-plate2 (cdr liste) ) 
                                )
             )
          )
    )
;;;*****************************************************************************************
;;; version de F. DORKELD

(de :list-plate3 (liste)
    (if (consp liste)
        ; alors 
        (append (:list-plate3 (car liste))
                (:list-plate3 (cdr liste)) )
        ; sinon
        (if liste (list liste)) )
    )
;;;*****************************************************************************************
;;; renvoyer la liste list triee par ordre alphabetique

(de :list-alpha (list)
    (sortl
           (mapcar 'string list))
    )
;;;*****************************************************************************************
;;; trier la liste list pour quelle soit ordonnee dans l'arbre
;;; (1 2 3 4) renvoie (4 1 3 2)

(de :tri-arbre (list)
    (cond ( (null list) nil)                                      
          ( t (cons (car (reverse list)) (:tri-arbre (cdr (reverse list)))) ) 
          )     
    )
;;;*****************************************************************************************
;;; ecriture dans le fichier nom-fich de la liste l

(de :ecrire (nom-fich l)
    (with ((outchan (openo nom-fich)))
          (mapcar 'print l)
          )
    )
;;;*****************************************************************************************
;;; lecture du fichier nom-fichier  renvoie une liste

(de :lire (nom-fich)
    (let
         (
          (liste nil)
          )
         (ifn (probefile nom-fich)
              ; alors
(when #:trace:ed
              (print "Erreur: le fichier " nom-fich " n'existe pas.")
)
              ; sinon
(when #:trace:ed
              (print "Chargement de " nom-fich)
)
              (with ((inchan (openi nom-fich)))
                    (untilexit eof (newr liste (read))))
              liste)
         )
    )

;;;##########################################################################################
;;;##########################################################################################
;;;  
;;;  Fonctions de manipulation de SHIRKA qui sont utilisees par le systeme ELSA (L. Buisson)
;;;  Les fonctions acceptent comme type d'entree des symboles ou des chaines
;;;  Ce type leur est indifferent.
;;;  Elles rendent soit un type 'chaine' soit un type 'symbole' (ou 'numerique') selon
;;;  qu'elles sont postfixees par "ch" ou "sym"
;;;
;;;##########################################################################################
;;;##########################################################################################

;;;*****************************************************************************************
(de :aj-valeur (inst nom-att valeur b)
    ;       Ajout d'une valeur
    (#:shirka:moteur:aj-valeur
            (:schema inst)
            (:symbole nom-att)
            valeur
            b)
    )
;;;*****************************************************************************************
(de :ch-as-sym (ch)
    ;       Transforme la chaine ch en symbole
    (implode (explode ch))
    )
;;;*****************************************************************************************
(de :objet-simple (valeur type-attr)
    ;    
    (when (> (slength valeur) 0)
          (if (eq type-attr 'chaine)
              valeur
              (implode (explode valeur))))
    )
;;;*****************************************************************************************
(de :classe-de (inst)
    ;       Classe d'une instance
    (#:shirka:noyau:classe-de (:symbole inst))
    )
;;;*****************************************************************************************
(de :copie-inst (inst1 inst2)
    ;       Fonction de copie d'une instance dans une autre
    (for (i 3 1 (sub1 (vlength inst1)))
         (vset inst2 i (vref inst1 i)))
    )
;;;*****************************************************************************************
(de :cr-inst (class inst)
    ;       Cette fonction cree une instance de la classe class de nom inst.
    ;       Aucun attribut instanciable n'est fourni.
    (#:shirka:moteur:creer-instance class () (:symbole inst))
    )
;;;*****************************************************************************************
(de :creer-instance (nom-classe valeurs inst)
    ;       Creation d'instance avec attributs instanciables
    (#:shirka:moteur:creer-instance nom-classe valeurs ( :symbole inst))
    )
;;;*****************************************************************************************
(de :l-inst-ch (class)
    ;       Cette fonction rend la ou les instances propre d'une classe sous 
    ;       forme de chaines.
    (let
         ( (l-inst (:l-inst-sym class)) )
         (if (consp l-inst)
             (mapcar 'string l-inst)
             (string l-inst) ))
    )
;;;*****************************************************************************************
(de :l-inst-sym (class)
    ;       Cette fonction rend la ou les instances propre d'une classe sous 
    ;       forme de symboles.
    (#:shirka:noyau:l-inst (#:shirka:noyau:schema (:symbole class)))
    )
;;;*****************************************************************************************
(de :l-inst-t-sym (class)
    ;       Cette fonction rend toutes les instances d'une classe sous forme de
    ;       symboles.
    (#:shirka:noyau:l-inst-t (#:shirka:noyau:schema (:symbole class)))
    )
;;;*****************************************************************************************
(de :l-inst-t-ch (class)
    ;       Cette fonction rend toutes les instances d'une classe sous forme
    ;       de chaines de caracteres.
    (let ((l-inst (:l-inst-t-sym class)))
         (if (consp l-inst)
             (mapcar 'string l-inst)
             (string l-inst)))
    )
;;;*****************************************************************************************
(de :nature (schema nom-att)
    ;       Nature et type d'un attribut
    (#:shirka:noyau:nature schema (:symbole nom-att))
    )
;;;*****************************************************************************************
(de :nom-ch (schema)
    ;       Donne le nom d'un schema sous forme de chaine,
    ;       sauf s'il correspond a un type  simple
    (string (if (vectorp schema )
                (:nom-sch-sym schema)
                schema))
    )
;;;*****************************************************************************************
(de :nom-sch-sym (schema)
    ;       Nom d'un schema, retourne comme symbole
    ;       L'argument peut etre un nom
    (if (vectorp schema)
        (#:shirka:noyau:nom-sch schema)
        (:symbole schema)
        )
    )
;;;*****************************************************************************************
(de :nom-sch-ch (schema)
    ;       Nom d'un schema, retourne comme chaine
    ;       L'argument peut etre un nom
    (string (:nom-sch-sym schema))
    )
;;;*****************************************************************************************
(de :pos-nom-att (schema nom-att)
    ;       Position d'un attribut
    (#:shirka:noyau:pos-nom-att schema (:symbole nom-att))
    )
;;;*****************************************************************************************
(de :schema (inst)
    ;      Cette fonction permet de consulter la structure d'une instance inst.
    ;       L'argument peut etre un schema
    (ifn (vectorp inst)
         (#:shirka:noyau:schema (:symbole inst))
         inst)
    )
;;;*****************************************************************************************
(de :sup-inst (inst b)
    ;       Cette fonction supprime l'instance de nom inst
    (#:shirka:moteur:sup-inst (#:shirka:noyau:schema (:symbole inst)) b)
    )
;;;*****************************************************************************************
(de :sup-valeur (inst nom-att anc-val)
    ;       Suppression d'une valeur dans un attribut
    (#:shirka:moteur:sup-valeur
            (:schema inst)
            (:symbole nom-att)
            anc-val)
    )
;;;*****************************************************************************************
(de :sup-toutes-valeurs (inst nom-att)
    ;       Suppression de toutes les valeurs d'un attribut 
    (let 
         ( (valeurs (:valeur-sym (:symbole nom-att) (:nom-sch-sym inst))) )
         (mapc
               '#:shirka:moteur:sup-valeur
               (cirlist (:schema inst))
               (cirlist (:symbole nom-att))
               valeurs))
    )
;;;*****************************************************************************************
(de :symbole (var)
    ;       Cette fonction force le type de var pour etre un symbole.
    (if (stringp var)
        (symbol () var)
        var)
    )
;;;*****************************************************************************************
(de :type-simple? (type)
    ;       Test si un type donne est simple
    (#:shirka:noyau:type-simple? (:symbole type))
    )
;;;*****************************************************************************************
(de :valeur-sym (nom-att inst)
    ;       Cette fonction rend la ou les valeurs de l'attribut nom-att de 
    ;       l'instance inst sous forme de symbole.
    (#:shirka:moteur:valeur? (:symbole nom-att) (:symbole inst))
    )
;;;*****************************************************************************************
(de :valeur-ch (nom-att inst)
    ;       Cette fonction rend la ou les valeurs de l'attribut nom-att de 
    ;       l'instance inst sous la forme de chaines de caracteres.
    ;       Si la liste est vide, elle rend "".
    (let ((l-val (:valeur-sym nom-att inst)))
         (if (consp l-val)
             (mapcar 'string l-val)
             (string l-val)))
    )
;;;*****************************************************************************************
(de :val-fac-he (schema nom-att facette)
    ;       Valeur d'une facette
    (#:shirka:noyau:val-fac-he schema (:symbole nom-att) facette)
    )
;;;*****************************************************************************************
(de :val-inst-ch (instance position)
    ;       Valeur d'un attribut connu par sa position. chaine de caracteres
    (string (#:shirka:noyau:val-inst instance position))
    )
;;;*****************************************************************************************
(de :val-inst-sym (instance position)
    ;       Valeur d'un attribut connu par sa position. symbole
    (#:shirka:noyau:val-inst instance position)
    )
;;;*****************************************************************************************
