;;; Repertoire :  ~/LeShirka/LeIvan/CodeIvan/Hypertexte/
;
;     +----------------------------------------+-----------------------------------------+
;     | fichier :  ivan-hy-interf.ll           | package : #:ivan:hy-interf              |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                                       IVAN                                       |
;     |             Interface de Visualisation et d'Aide a la Navigation                 |
;     |                dans une base de connaissances a objets Shirka                    |
;     |                        Sylvain Grivaud          1991                             |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;

;;;*****************************************************************************************

(defvar #:sys-package:colon '#:ivan:hy-interf)

;;;*****************************************************************************************
;;; creation de l'hypertexte

(de :hy-creation ()
    (lets
          ((user  (hyperuser "Utilisateur"))
           (tv    (hypertravelagency user ))
           (web   (hyperweb)               )
           )
          ; definition de l'editeur
          (:hy-define-editor tv)
          
          ; lien entre le reseau et l'agence
          (hy-connect tv web)
          
          ; renvoyer l'agence de voyage
          tv)
    )
;;;*****************************************************************************************
;;; construction du reseau

(de :hy-build-web (web repertoire)
    (let
         (
          (reptext      (catenate repertoire "-hytext/"))
          (repimag      (catenate repertoire "-hyimag/"))
          l-textfiles    ; liste des fichiers textes
          l-textdoors    ; liste des portes du texte courant
          name-file      ; nom du fichier contenant le texte
          tfile          ; fichier contenant le texte
          ifile          ; fichier contenant l'icone
          door           ; porte multiple
          l-imagfiles    ; liste des fichiers images
          )
         
         (setq
               l-textfiles (cdr (#:systeme:unix:catalogue (pathname reptext)))
               l-imagfiles (cdr (#:systeme:unix:catalogue (pathname repimag)))
               )
(when #:trace:hy
         (PRINT "repertoire: " repertoire "textes: " l-textfiles)
)
(when #:trace:hy
         (PRINT "images: " l-imagfiles)
)
         ; => il faut maintenant creer les concepts
         (mapc '(lambda (name-file)
                        (setq
                              ; creer la liste des mots-cles du texte courant name-file :
                              ; c'est la liste des portes - le nom de ce fichier
                              l-textdoors (set-difference l-textfiles (list name-file))
                              ; la porte multiple de cette classe
                              door
                              (multipledoor
                                      (catenate "->" name-file) ; nom de la porte
                                      l-textdoors ; liste des portes 
                                      ())
                              ; nom du fichier contenant le texte
                              tfile     (catenate reptext name-file)      ; le nom du fichier
                              ifile     (catenate repimag name-file ".i") ; l'icone 
                              )
(when #:trace:hy
                        (PRINT "tfile " tfile)
                        (PRINT "ifile " ifile)
)
                        ; ajouter le concept
                        (hy-web-add-concept
                                web 
                                (hyperpage
                                        name-file
                                        tfile
                                        ""  ; pas de fichier des portes
                                        (if (member (catenate name-file ".i") l-imagfiles)
                                            ifile   ; il existe une icone pour ce fichier texte
                                            ; hy-------------------------------------------------------------
					    ; pas de fichier image
					    (if #:scarp:hytxt:charger?
                                            	(catenate #:scarp:home-dir "interface/ic-blanc.i")
						(catenate #:ivan:*home* "CodeIvan/Icone/ic-blanc.i")
					    )
                                            ; ---------------------------------------------------------------
                                            )
                                        ""  ; pas de fichier des portes pour l'image
                                        )
                                )
                        ; ajouter les portes dans le texte
                        (hy-web-add-door 
                                (hy-web-get-concept web name-file)
                                'partie-text
                                door) 
                        )
               ; il faut creer les concepts pour les textes qui existent
               l-textfiles
               )
         'done 
         )
    )
;;;*****************************************************************************************
;;; la page -----> on l'edite avec un editeur de texte principalement

(de :hy-define-editor (tv)
    ;
    ; ------------------------------------------------------------
    ;               definition du scenario de l'editeur
    ; ------------------------------------------------------------
    (hy-define-editor 
            tv 
            '{hyperpage}
            t 
            'makepageeditor
            '(hy-refresh         ; reafficher l'application
                     hy-show     ; la rajouter
                     )
            '())
    ; ------------------------------------------------------------
    ;               definition du scenario des composants
    ; ------------------------------------------------------------
    
    ; afficher le texte de la page dans le texteditor
    (hy-define-component 
            tv 
            '{hyperpage} 
            'text 
            '(hy-fill-one hy-fill-doors-one hy-refresh-one hy-hilite-door-one) 
            '(hy-reset-one)
            'maj-te-text
            '(partie-text))
    
    ;;; AJOUT SCROLLER
    ; redimensionner le scroller
    (hy-define-component
            tv
            '{hyperpage}
            'scroll-te
            '(hy-reset-one)
            '()
            ()
            ())
    
    ; afficher l'image dans le texteditor : application
    (hy-define-component 
            tv 
            '{hyperpage} 
            'imag 
            '(hy-fill-one hy-refresh-one) 
            '()
            'maj-ap-imag
            '())
    
    ; afficher le nom de la page dans le stringedit
    (hy-define-component
            tv
            '{hyperpage}
            'page
            '(hy-fill-one hy-refresh-one)
            '()
            'maj-se-titre
            ())
    
    ; changer le texte du bouton 'bout-classe
    (hy-define-component
            tv
            '{hyperpage}
            'bout-classe
            '(hy-fill-one hy-refresh-one)
            '()
            'maj-bout-classe 
            ())
    
    ; afficher l'historique des noeuds dans le stringmenu
    (hy-define-component
            tv
            '{hyperpage}
            'sm-histo 
            '(hy-fill-one)
            '()
            'maj-sm-histo 
            ())
    
    ; redimensionner le scroller de l'historique
    (hy-define-component
            tv
            '{hyperpage}
            'scroll-histo 
            '(hy-reset-one)
            '()
            ()
            ())
    
    )
;;;*****************************************************************************************
;;; editeur de page

(de makepageeditor ()
    (lets 
          (
           (te-text       (textedit 0 0 #90wc #60hc))
           ;;; AJOUT D'UN SCROLLER POUR LE TEXTE DE DOCUMENTATION
           (scroll-te     (scroller 0 0 #47wc #10hc te-text t t))
           (ap-imag       (application '{application} 
                                  0 0
                                  #45wc #10hc
                                  (view)))
           ;-hy---------------------------------------------------------------
           ; modif pour rendre le module hypertexte homogene avec IVAN et IACE
           (bout-quitter  (standardbutton #M#:txt-const:revenir))
           ;------------------------------------------------------------------
           (se-titre  (stringedit 0 0 #40wc ""))
           ; boutons de commande
           (sm-histo      (stringmenu nil))
           (scroll-histo  (scroller 0 0
                                  ; #20wc #13hc
                                  #20wc #15hc
                                  sm-histo 
                                  t t))
           (bout-classe   (standardbutton
                                  (rectangle 0 0 
                                          (send 'w scroll-histo) #3hc)))
           
           ; cluster IVAN
           (clu-ivan
                   (#:ivan:ed-util:clusterimage
                           bout-classe
                           ;-hy---------------------------------------------
                           ; modif pour rend le module compatible avec IACE
                           ;                  "IVAN"
                           "Action"
                           ;------------------------------------------------
                           ))
           
           ; titre de la fenetre
           (titre-fenetre
                   (row
                        (centeredview
                                (rectangle 0 0 0 (- (send 'height (icon 0 0 ic-httx)) 4))
				#M#:txt-const:doc-sur
                                )
			(centeredview
				(boxedimage 0 0 
					(+ 2 (send 'width se-titre))
					(- (send 'height (icon 0 0 ic-httx)) 4)
					se-titre)
			)
                        ))
           
           ; cluster de l'historique
           (clu-histo
                   (#:ivan:ed-util:clusterimage
                           scroll-histo 
			   #M#:txt-const:history))
           
           ;          ; cluster du texte
           ;         (clu-texte
           ;            (#:ivan:ed-util:clusterimage
           ;               te-text
           ;               "Texte"))
           
           ; cluster du texte
           (clu-texte
                   (#:ivan:ed-util:clusterimage scroll-te #M#:txt-const:txt))
           
           ; cluster de l'icone
           (clu-imag
                   (#:ivan:ed-util:clusterimage
                           ap-imag
                           ""))
           
           ; les commandes
           (col-commandes
                   (column
                           clu-histo
                           clu-ivan
                           ))
           
           (appli  (application 
                           '{application} 
                           ;                       290 510 0 0
                           80 310 0 0
                           (header
                                   (icon 0 0 ic-httx)
                                   titre-fenetre
                                   (centeredview
                                     	(rectangle 0 0 0
					   (- (send 'height (icon 0 0 ic-httx)) 4)
					)
                                     	bout-quitter)
                                   (view)
                                   (row
                                        col-commandes                             
                                        clu-texte
                                        clu-imag)
                                   t
                                   () 
                                   )
                           ))
           )
          ;-----------------------------------------------------------------------------
          ; definition des caracteristiques du composant textedit :
          ; mise en valeur des portes, curseur invisible, separateur, ...
          ;
          ; les styles sont utilises par les methodes croisees hy-add-door, hy-show-door
          (te-set-selection-style    te-text 'te-hidden-style)
          
          (te-define-style te-text 
                  'hy-multiple-style
                  'foreground (make-named-color "yellow")
                  'background (make-named-color "blue")) 
          
          (te-hide-cursor            te-text)
          (te-readonly               te-text t)
          ; action quand on selectionne une porte avec le bouton gauche de la souris
          (te-set-local-binding      te-text 'down0 ':hy-go-through) 
          
          (te-delimiter te-text 45 nil) ; le caractere "-" n'est pas un separateur
          ;-----------------------------------------------------------------------------
          ; definition des composants nommes
          
          (add-component appli 'top             appli)
          ;-hy--------------------------------------------------------------------------
          (add-component appli 'bout-quitter    bout-quitter)
          
          ; composants ayant un scenario : voir la fonction :hy-define-editor
          (add-component appli 'text            te-text)
          ;;; AJOUT SCROLLER
          (add-component appli 'scroll-te       scroll-te)
          (add-component appli 'imag            ap-imag)
          
          (add-component appli 'page            se-titre)
          (add-component appli 'sm-histo        sm-histo )
          (add-component appli 'scroll-histo    scroll-histo)
          (add-component appli 'bout-classe     bout-classe)
          ;-----------------------------------------------------------------------------
          ; definition des actions des composants nommes
          
          (set-action bout-quitter ':action-bout-quitter)
          (set-action bout-classe  ':action-bout-classe)
          (set-action sm-histo     ':action-sm-histo )
          ;-----------------------------------------------------------------------------
          ; l'editeur est pret : il faut le renvoyer
          (send 'fit-to-contents appli)
          
          (send 'set-title appli #M#:txt-const:aide)
          appli
          )
    )

;;;#########################################################################################
;;;#########################################################################################
;;;                                   
;;;                            ACTIONS DES COMPOSANTS DE L'EDITEUR
;;;                                   
;;;#########################################################################################
;;;#########################################################################################

;;;*****************************************************************************************
;;; revenir a IVAN = fermer la fenetre

(de :action-bout-quitter (b)
    ; quitter
    (remove-application (component 'top  b))
    ;-hy----------------------------------------
    ; organisation du retour vers les interfaces
  (if #:scarp:hytxt:appele?
    (setq #:scarp:hytxt:appele? ())
    (ifn var-hyper
	 (pop-application #:ivan:appli)
         (setq var-hyper nil)
	 (pop-application #:iace:appli)
    )
  )
    ;-------------------------------------------
    (setq
          #:ivan:hy-global:list-histo nil  ; reinitialiser la liste de l'historique
          )     
    )
;;;*****************************************************************************************
;;; la porte nom-porte est-elle une classe ? si ce n'est pas le cas, il faut inhiber
;;; le bouton de retour a IVAN bouton (composant bout-classe de l'editeur)

(de :existe-classe (nom-porte bouton)
 (if #:scarp:hytxt:appele?
    (lets ((nom-porte-sym (#:si:symbole nom-porte))
           (classe? (when (getprop nom-porte-sym 'schema) (#:si:classe? nom-porte-sym)))
           (tache? (when classe? 
                        (or (member 'tache (#:si:ancetres nom-porte-sym))
                            (member 'procedure (#:si:ancetres nom-porte-sym)))))

	  )
	  (ifn tache? 	(inhibit-application bouton)
			(authorize-application bouton)
			({application}:userdata bouton nom-porte-sym)
	  )
    )
    ;else
    (lets
          (
           (nom-classe-sym (#:ivan:ed-util:symbole nom-porte))
           ;-hy-----------------------------------------
           ; modif pour inserer l'hypertexte dans iace
           ; retrouver le noeud
           (noeud-classe (if (getprop nom-classe-sym 'noeud)
                             (getprop nom-classe-sym 'noeud)
                             (getprop nom-classe-sym 'noeud-iace)
                             )
                   )
           ; nom de la classe selectionnee dans IVAN
           (nom-classe (if (catcherror () (component 'edl-classe #:ivan:appli))
                           (send 'get-line (component 'edl-classe #:ivan:appli))
                           (send 'get-line (component 'edl-classe #:iace:appli))
                           )
                   )
           ; vecteur des concepts
           (vect-classes (if (catcherror () (send 'ch-vect-classes #:ivan:appli))
                             (send 'ch-vect-classes #:ivan:appli)
                             (send 'ch-vect-classes #:iace:appli)
                             )
                   )
           ;--------------------------------------------
           ; indice de recherche
           (indmax (vlength vect-classes) ) 
           (ind 0)
           ; booleen de recherche du concept auquel appartient la classe
           trouve
           bnoeud
           )
          (if noeud-classe
              (setq bnoeud t)
              (setq bnoeud ())
              )
          (ifn bnoeud
               ; ALORS
               ; le noeud n'existe pas dans l'arbre charge : 
               ; existe-t-il dans un autre concept ?
               ;-hy-----------------------------------------
               ; modif pour inserer l'hypertexte dans iace.
               (let ()
                    (while (and (< ind indmax) (not trouve) )
                           (let ((test-appli (if (catcherror () (send 'ch-concept #:ivan:appli))
                                                 (equal (caadr (vref vect-classes ind)) (send 'ch-concept #:ivan:appli))
                                                 (equal (caadr (vref vect-classes ind)) (send 'ch-concept #:iace:appli))
                                                 )
                                         )
                                 )
                                (if test-appli
                                    ;---------------------------------------------
                                    ; alors
                                    (setq ind (+ 1 ind))
                                    ; sinon
                                    (setq trouve
                                          (member nom-classe-sym (cadr (vref vect-classes ind) )))
                                    (unless trouve
                                            (setq ind (+ 1 ind))
                                            )
                                    )
                                )
                           )
                    ; inhiber le bouton si non trouve
                    (ifn trouve
                         (inhibit-application bouton)
                         (authorize-application bouton)
                         )
                    )
               ; SINON
               (authorize-application bouton) ; le bouton n'est inhibe que si ce n'est pas une classe
               )
          ; ind contient le rang du concept
          ({application}:userdata bouton (list bnoeud ind nom-classe-sym))
          )
    ))
;;;*****************************************************************************************
;;; action du bouton bout-classe :
;;;  1. fermer la fenetre de l'hypertexte
;;;  2. selectionner la classe
;;;     il faut changer de concept si necessaire

(de :action-bout-classe (b)
 (if #:scarp:hytxt:appele?
    (let  ((nom-classe-sym ({application}:userdata b))
	   w h
	  )
                ;---------------------------------------------------------
                ; fermer la fenetre Hypertexte
                (:action-bout-quitter (component 'bout-quitter b))
		(if #:scarp:board-expert? 
		    	(progn
				(when #:scarp:if:sel:appli
					(setq w (send 'w #:scarp:if:sel:appli))
					(setq h (send 'h #:scarp:if:sel:appli))
					; necessaire pour afficher le selecteur sans qu'il soit apercu
					; ce qui est necessaire pour afficher correctement le graph de decomp
					(send 'grow #:scarp:if:sel:appli 0 0)
					(add-application #:scarp:if:sel:appli)
					(#:scarp:if:sel:change-select (string nom-classe-sym))
					(remove-application #:scarp:if:sel:appli)
					(send 'grow #:scarp:if:sel:appli w h)
				)
				(#:scarp:if:bd:resolution ())
			)
			(#:scarp:if:bd:act-sel nom-classe-sym 'selection-cl)
			(#:scarp:if:bd:act-hier nom-classe-sym)
		)
    )
    (lets
          (
           (list-param ({application}:userdata b))
           ; retrouver le noeud
           (noeud-classe   (nth 0 list-param))  
           (ind            (nth 1 list-param))
           (nom-classe-sym (nth 2 list-param))
           concept-retour
           (bclasse t)
           )
          (unless noeud-classe
                  ;-hy-----------------------------------------
                  ; modif pour inserer l'hypertexte dans iace.
                  (setq concept-retour (if (catcherror () (send 'ch-list-concepts #:ivan:appli))
                                           (nth ind (send 'ch-list-concepts #:ivan:appli))
                                           (nth ind (send 'ch-list-concepts #:iace:appli))
                                           )
                        )
                  ;--------------------------------------------
                  (if
                      (confirm
                              (column
                                      (font #:ivan:in-global:fstdb #M#:txt-const:attention)
                                      (row
                                           (font #:ivan:in-global:fstdb #M#:txt-const:pour-sel-cl)
                                           (font #:ivan:in-global:fstdb (string nom-classe-sym))
                                           (font #:ivan:in-global:fstd ",")
					)
                                      (row
                                           (font #:ivan:in-global:fstd  #M#:txt-const:change-cpt)
                                           (font #:ivan:in-global:fstdb (string concept-retour))
                                           (font #:ivan:in-global:fstd" !")
					)
                                      )
                              )
                      ; ALORS declencher l'action du bouton de menu
                      ;-hy------------------------------------------------------------------------------------------------------------
                      ; changement de concept; dans le cas IACE il y a ajout de concept si l'instance existe pour le concept precedant
                      ; sinon, on ne fait rien
                      (ifn var-hyper
                           (#:ivan:in-interf:action-menu-concept
                                   #:ivan:appli
                                   ind 
                                   t)
                           (#:iace:fc-interf:action-menu-concept
                                   #:iace:appli
                                   ind 
                                   t)
                           )
                      ;----------------------------------------------------------------------------------------------------------------
                      ; SINON ne pas changer de classe
                      (setq bclasse nil)
                      )
                  )
          (when bclasse
                ; repositionner
                ;-hy-----------------------------------------------------
                ; retour vers IACE ou vers IVAN il faut changer de classe
                (ifn var-hyper
                     (#:ivan:in-interf:reinit-browser
                             #:ivan:appli 
                             (getprop nom-classe-sym 'noeud) ; il faut retrouver le noeud ds le concept
                             nom-classe-sym)
                     (#:iace:fc-interf:reinit-browser
                             #:iace:appli 
                             (getprop nom-classe-sym 'noeud-iace) ; il faut retrouver le noeud ds le concept
                             nom-classe-sym)
                     )
                ;---------------------------------------------------------
                ; fermer la fenetre Hypertexte
                (:action-bout-quitter (component 'bout-quitter b))
                )
          )
    ))
;;;*****************************************************************************************
;;; action du selectionneur de chaine de caracteres
;;; quand on selectionne une chaine, on se repositionne sur cette porte

(de :action-sm-histo  (sm)
    (let 
         (
          ; l'agence de voyage 
          (tv    (hy-travel (component 'top  sm)))
          ; la chaine selectionnee dans le selectionneur de chaine
          (porte (send 'get-selected-string sm))
          ; nom de la classe
          (nomclasse (send 'get-line (component 'page sm)))
          )
         (when porte
               (if (equal porte nomclasse)
                   ; delectionner selectionneur
                   (send 'set-selected-string sm "")
                   ; sinon
                   ; se repositionner sur cette porte
                   (hy-move-to tv porte)
                   )
               )
         )
    )
;;;*****************************************************************************************
;;; selection d'une porte avec la souris = deplacements au travers du lien pointe
;;;  Si on a une porte normale,  on cherche le lien associe,
;;;  Si on a une porte multiple, on positionne sa destination et 
;;;                              on se deplace vers cette destination
;;; NB : en fait, IVAN n'a que des portes multiples

(de :hy-go-through (te)
    ; positionner le curseur a la position pointee par la souris
    (te-mouse-set-cursor-position te)
    
    (let 
         (
          (editor (component 'top te))
          )
         (tag done 
              (te-map-selections-at
                      ; textedit
                      te 
                      ; fonction
                      (lambda (sel)
                              (let 
                                   (
                                    (door (hy-editor-get-door editor 'text sel))
                                    (string ())
                                    )
                                   (when door
                                         (when (multipledoorp door)
                                               (te-map-zones-under 
                                                       te 
                                                       sel 
                                                       (lambda (s)
                                                               (setq string s)
                                                               )
                                                       )
                                               (hy-web-door-destination door string)
                                               )
                                         (hy-editor-move-through-to-destination
                                                 editor 
                                                 'text 
                                                 'contents 
                                                 sel)
                                         (exit done)
                                         )
                                   )
                              )
                      )
              )
         )
    )
