;;; Repertoire: ~/LeShirka/LeIvan/CodeIvan/Hypertexte/
;
;     +----------------------------------------+-----------------------------------------+
;     | fichier :  ivan-in-struct.ll           |                                         |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface de Visualisation et d'Aide a la Navigation                 |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Sylvain Grivaud          1991                             |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     | descriptif :                                                                     |
;     |                                                                                  |
;     | Ce fichier contient la definition des structures AIDA utilisees pour la cons-    |
;     | truction du module hypertexte                                                    |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+


;;;*****************************************************************************************
;;;    structure d'une page : je definie la classe {hyperpage} comme etant une sous-classe
;;;                           de {hyperconcept}
;;;    une page possede - un nom : ce sera le nom de la porte selectionnee :
;;;                                - la classe SHIRKA
;;;                                - un mot cle
;;;                     - un fichier texte d'explication qui sera affiche a l'ecran
;;;                     - un fichier des portes

;;;---------------------------------------------------------------------------------------
;;;                             definition de la classe AIDA
;;;---------------------------------------------------------------------------------------
(eval-when (load eval compile)
        (defabbrev hyperpage {hyperconcept}:hyperpage))

(eval-when (load eval compile)
        (defstruct 
                {hyperpage}
                )
        )

;;;---------------------------------------------------------------------------------------
;;;                             definitions des fonctions 
;;;---------------------------------------------------------------------------------------

(setq #:sys-package:colon '{hyperpage})

;;;---------------------------------------------------------------------------------------
;;; creation du concept AIDA

(de hyperpage (name textfile textdfile imagfile imagdfile)
    (:initialise-structure (new '{hyperpage})
            name         ; nom 
            textfile     ; nom du fichier texte
            textdfile    ; nom du fichier des portes du texte
            imagfile     ; nom du fichier contenant l'icone
            imagdfile    ; nom du fichier des portes de l'icone
            )
    )
;;;---------------------------------------------------------------------------------------
;;;  une page est un concept constitue
;;;   - d'une information textuelle :
;;;     = le texte est une PARTIE (selon la terminologie AIDA)
;;;   - d'une image qui est une icone AIDA

(de :initialise-structure (page name textfile textdfile imagfile imagdfile)
    ; creation de la classe
    (send-super '{hyperpage} 'initialise-structure page name)
    
    ; creation de la partie texte : elle s'appelle 'partie-text 
    ; et fait reference au composant nomme 'text de l'editeur de pages
    (hy-web-add-part page 'partie-text (hyperpart 'text    textfile textdfile))
    
    ; creation de la partie image : elle s'appelle 'partie-imag 
    ; et fait reference au composant nomme 'imag de l'editeur de pages
    (hy-web-add-part page 'partie-imag (hyperpart 'bitmap imagfile imagdfile))
    
    ; je renvoie la classe
    page
    )
;;;---------------------------------------------------------------------------------------

(de :prin (page)
    (prin "#<page " (hy-web-element-name page) ">")
    )
;;;---------------------------------------------------------------------------------------
;;;           definition des fonctions utilisees par les scenarios des composants
;;;---------------------------------------------------------------------------------------
;;; composant 'text = recuperer le fichier texte et l'afficher dans le textedit

(de :maj-te-text (page)
    (hy-web-contentsfile (hy-web-get-part page 'partie-text))
    )
;;;---------------------------------------------------------------------------------------
;;; composant 'imag = recuperer le fichier icone et l'afficher

(de :maj-ap-imag (page)
    (lets 
          (
           (partie-image (hy-web-get-part page 'partie-imag))
           ;; soit on recupere le bitmap precedemment charge,
           ;; soit on charge le bitmap
           (icone (or (hy-web-contents partie-image)
                      (libloadicon
                              (hy-web-contentsfile (hy-web-get-part page 'partie-imag)))
                      ))
           )
          ;;; On met dans la partie-image le bitmap trouve
          ;;; (pour permettre de le recuperer apres sans
          ;;; avoir a le recharger)
          ;(if (hy-web-contents partie-image) (print "icone") (print "pas icone"))
          (hy-web-contents partie-image icone)
          (icon 
                0 0 
                ;;; MODIFICATION : on suppose que les icones ont la bonne taille!!
                ;;;         (send 'grow
                icone
                ;;;               #45wc (send 'h icone) )
                ))
    )
;;;---------------------------------------------------------------------------------------
;;; composant 'imag = recuperer le fichier icone et l'afficher

;(de :maj-ap-imag (page)
;   (let 
;      (
;         (icone (libloadicon
;                   (hy-web-contentsfile (hy-web-get-part page 'partie-imag))))
;      )
;      (icon 
;         0 0 
;         (send 'grow
;               icone
;               #25wc (send 'h icone) )
;      ))
;)
;;;---------------------------------------------------------------------------------------
;;; composant 'se-titre = recuperer le nom de la page et l'afficher dans le stringedit

(de :maj-se-titre (page)
    (hy-web-element-name page)
    )
;;;---------------------------------------------------------------------------------------
;;; composant 'sm-histo = recuperer le nom de la page et l'ajouter a la liste des classes
;;; (les portes) qui ont ete consultees; renvoyer la liste pour reaffichage

(de :maj-sm-histo (page)
    (let
         (
          (nom-porte (hy-web-element-name page))
          )
         ; j'insere le nom de la porte dans la liste
         (setq #:ivan:hy-global:list-histo (cons nom-porte #:ivan:hy-global:list-histo))
         ; je renvoie la liste
         #:ivan:hy-global:list-histo)
    )
;;;---------------------------------------------------------------------------------------
;;; composant 'bout-classe = recuperer le nom de la page et changer l'image du bouton

(de :maj-bout-classe (page)
    (lets
          (
           (nom-porte (hy-web-element-name page))
	   (nom-porte-sym (symbol () nom-porte))
	   (classe? (when (getprop nom-porte-sym 'schema)
			(#:shirka:noyau:val-inst (#:shirka:noyau:schema nom-porte-sym) %est-un)))
	   (tache? (when (and #:scarp:hytxt:appele? classe?) 
			(or (member 'tache (#:si:ancetres nom-porte-sym))
			    (member 'procedure (#:si:ancetres nom-porte-sym)))))
	   (txt (if #:scarp:hytxt:appele?
			(#:scarp:if:texte (if tache? 	#M#:txt-const:selectionner
							#M#:txt-const:imp-selectionner))
			(font #:ivan:in-global:fstd (if classe? #M#:txt-const:aller
								#M#:txt-const:imp-aller))))
           (nouv-button  (buttonbox
                                 0 0
                                 #20wc #3hc
                                 (column
                                         (centeredview
                                                 (rectangle 0 0 #20wc 0)
						 txt
					 )
                                         (centeredview
                                                 (rectangle 0 0 #20wc 0)
						 (if #:scarp:hytxt:appele?
							(#:scarp:if:texte nom-porte)
							(font #:ivan:in-global:fstd nom-porte)
						 )
                                         )
                                 )
			 )
                   )
           bout-classe
           )
          ; ici se trouve le seul moyen trouve pour pouvoir recuperer
          ; la taille du bouton bout-classe dans l'editeur de page
          ; ca marche car je n'ai qu'un seul editeur (reutilise
          ; pour toutes les pages) 
          (hy-map-aidaeditors
                  #:ivan:hy-global:tv
                  (type-of page)
                  (lambda (ed)
                          (setq
                                bout-classe (component 'bout-classe ed)
                                )
                          )
                  )
          ; je teste si la porte est une classe pour activer le bouton
          (#:ivan:hy-interf:existe-classe nom-porte bout-classe)
          ; je dimensionne l'image du nouvon bouton a la taille du composant 
          ; bout-classe de l'editeur
          (send 'grow nouv-button
                (send 'width  bout-classe)
                (send 'height bout-classe))
          ; je renvoie l'image qui sera affichee par la methode hy-fill-one/hy-refresh-one
          nouv-button
          )
    )
;;;---------------------------------------------------------------------------------------
