;;; Repertoire: ~/LeShirka/LeIvan/CodeIvan/Interface/
;
;     +----------------------------------------+-----------------------------------------+
;     | fichier :  ivan-in-infatt.ll           | package : #:ivan:in-infatt              |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface de visualisation et d'Aide a la Navigation                 |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Sylvain Grivaud          1991                             |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     | descriptif du package :                                                          |
;     |                                                                                  |
;     | La fonction principale :edit cree une instance de la structure {in-infatt} pour  |
;     | fournir des informations sur les facettes de l'attribut.                         |
;     | Dans le cas d'un attribut complexe, le nom de la classe utilisee pour definir le |
;     | type de l'attribut est un bouton actif si la classe existe dans la representation|
;     | graphique (liste list-classes). Cliquer sur ce bouton permettra de repositionner |
;     | l'arbre sur cette classe.                                                        |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+


;;;*****************************************************************************************

(defvar #:sys-package:colon '#:ivan:in-infatt)

;;;*****************************************************************************************

;;; structure pour fournir les informations :

(defabbrev in-infatt {application}:in-infatt)    
(defstruct {in-infatt}
        ch-titre         
        ch-com           
        ch-dom          
        ch-card          
        ch-infer         
        ch-valeur       
        ch-contraintes    
        
        ch-classe
        )
;;;*****************************************************************************************

(defvar {scroller}:scrollbarwidth  20)     
(defvar {scroller}:scrollbarheight 20)

(defvar :%l-ea 45)  ; Largeur editeur attribut
(defvar :%h-max 4)  ; Hauteur max pour affichage du commentaire

;;;*****************************************************************************************
;;; fonction creant une instance de la structure {in-infatt} pour permettre l'edition
;;; de l'attribut nom-attr

(de :edit (nom-classe nom-attr vect-classes)
    
    (lets
          (   ; application generale
             (appli (omakeq {in-infatt} x 0 y 0 w 0 h 0) )
             ; vecteur du schema de classe
             (schema-clas   (#:ivan:ed-util:schema nom-classe))
             ; facettes  $domaine , $sauf, $intervalle
             (list-dom    (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$domaine))
             (list-sauf   (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$sauf))
             (list-inter  (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$intervalle))
             (card-min    (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$cardmin))
             (card-max    (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$cardmax))
             (list-defaut (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$defaut))
             (list-valeur (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$valeur))
             (infer-proc  (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$sib-exec))
             (infer-filt  (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$sib-filtre))
             ; nature shirka de l'attribut
             (list-nature (#:ivan:ed-util:nature schema-clas nom-attr))
             ; l'attribut est-il une liste ?
             (bliste (eq (car list-nature) '$liste-de))
             ; type shirka de l'attribut : 
             ; c'est un reel, un entier, une chaine, un symbole ou si c'est un objet 
             ; complexe sa classe. 
             (type-attr-temp (cadr list-nature))
             (bspec-loc (equal (ascii (sref (string type-attr-temp) 0)) '%))
             (type-attr
                     (if bspec-loc
                         ; le type de l'attribut commence par un %. C'est le cas
                         ; de la specialisation locale. Je cherche la classe mere.
                         (car (#:shirka:noyau:peres (#:shirka:noyau:schema type-attr-temp)))
                         ; sinon c'est le type simple ou la classe
                         type-attr-temp))
             ; est-ce un type simple ?
             (btype-simple (not (null (#:ivan:ed-util:type-simple? type-attr))))
             ; est-ce une chaine ?
             (bchaine (eq type-attr 'chaine))
             
             (bout-daccord (standardbutton "Retour IVAN"))
             
             (edl-titre (lineedit 0 0 #30wc ""))
             image-ed
             
             )
          (unless (listp list-dom)    (setq list-dom    (list list-dom))    )
          (unless (listp list-sauf)   (setq list-sauf   (list list-sauf))   )
          (unless (listp list-inter)  (setq list-inter  (list list-inter))  )
          (unless (listp list-valeur) (setq list-valeur (list list-valeur)) )
          (unless (listp list-defaut) (setq list-defaut (list list-defaut)) )
          
          ;(PRINT "     type-attr  = " type-attr) 
          ;(PRINT "     list-sauf  = " list-sauf)
          ;(PRINT "     list-inter = " list-inter)
          ;(PRINT "     card-max   = " card-max)
          ;(PRINT "     card-min   = " card-min)
          ;(PRINT "     list-valeur   = " list-valeur " " )
          ;(PRINT "     list-defaut   = " list-defaut " " )
          
          ; composant top pour reperer le pere de plus haut niveau
          ({application}:components appli (acons 'top appli ()))
          
          ; construction des champs de l'application
          ({in-infatt}:ch-com          appli  (:const-ch-com    appli
                                                      nom-attr
                                                      schema-clas))
          ({in-infatt}:ch-titre        appli  (:const-ch-titre  appli 
                                                      nom-attr
                                                      bliste
                                                      type-attr
                                                      bspec-loc
                                                      btype-simple
                                                      vect-classes))
          
          
          ({in-infatt}:ch-dom        appli  (:const-ch-dom  appli
                                                    bliste
                                                    btype-simple
                                                    type-attr 
                                                    list-dom))
          
          ({in-infatt}:ch-card         appli  (:const-ch-card   appli
                                                      card-min
                                                      card-max))
          
          ({in-infatt}:ch-infer        appli  (:const-ch-infer  appli
                                                      infer-proc
                                                      infer-filt))
          
          ({in-infatt}:ch-valeur        appli  (:const-ch-valeur appli
                                                       list-valeur
                                                       list-defaut))
          
          ({in-infatt}:ch-contraintes  appli  (:const-ch-contraintes  appli
                                                      list-sauf
                                                      list-inter))
          
          ({in-infatt}:ch-classe       appli nil)
          
          ; assemblage pour constituer l'image finale
          (setq image-ed
                (#:ivan:ed-util:centeredimage
                        (column      
                                ({in-infatt}:ch-titre        appli)
                                ({in-infatt}:ch-com          appli)
                                ({in-infatt}:ch-valeur       appli)
                                ({in-infatt}:ch-card         appli)
                                ({in-infatt}:ch-infer        appli)
                                ({in-infatt}:ch-dom          appli)
                                ({in-infatt}:ch-contraintes  appli)
                                )
                        )
                )
          
          (send 'new-image
                appli
                (header
                        (rectangle 0 0 32 32) ; logo 
                        edl-titre
                        bout-daccord ; pas de bouton
                        (view) ; pas de commande 
                        image-ed
                        t
                        t
                        )
                )
          (#:ivan:ed-util:titre-fenetre appli (catenate "Informations sur un attribut"))
          (send 'set-line edl-titre           (catenate "Attribut " nom-attr))
          
          (set-action bout-daccord ':action-bout-daccord)
          
          ; dimensionnement de l'application
          (send 'fit-to-contents appli)
          ;  l'application est activee
          (grab-application
                  (send 'translate
                        appli
                        #:ivan:ed-global:%x #40hd
                        ))
          ; valeur en retour
          ({in-infatt}:ch-classe appli)
          
          )
    )

;;;#########################################################################################
;;;
;;;                                    CONSTRUCTION DES CHAMPS 
;;;
;;;#########################################################################################

;;;*****************************************************************************************
;;; construction de l'image de visualisation du commentaire de l'attribut
;;; en fonction de la taille du commentaire, l'image sera affichee ou non 
;;; dans un scroller.

(de :const-ch-com (appli nom-attr schema-clas)
    
    ;(PRINT ".. :const-ch-com")
    
    (lets
          (  
           ; commentaire de la classe (liste de chaines) 
           (list-comm (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$com)) 
           (list-com
                   (mapcar '(lambda (chaine)
                                    (let ( (chaine-font (font #:ivan:ed-global:fstd chaine)))
                                         chaine-font
                                         ))
                           list-comm))
           ; colonne des lignes
           (col-com (apply 'column list-com))
           
           ; largeur max d'1 ligne en nombre de caracteres (attention #wc = 9)
           (larg (#:ivan:ed-util:larg-max list-comm))
           
           ; hauteur du commentaire (nombre de lignes)
           (haut (ifn (null list-com)
                      (length list-com)))
           
           ; hauteur de l'image commentaire si scroller
           (h-scroll (+ {scroller}:scrollbarheight
                        (#:ivan:ed-util:hc :%h-max)
                        2))
           ; largeur de l'image commentaire si scroller
           (w-scroll (+ {scroller}:scrollbarwidth
                        (#:ivan:ed-util:wc :%l-ea)
                        2))
           
           ; img du commentaire     
           (img-com 
                   (cond
                         ; pas de commentaire = 1 ligne vide
                         ((null list-com)
                          (rectangle 0 0 w-scroll 0)
                          )
                         
                         ; le commentaire tient en largeur et hauteur : boite rectangulaire
                         ((and (<= larg :%l-ea) (<= haut :%h-max))
                          (view
                                (rectangle 0 0 w-scroll (#:ivan:ed-util:hc haut))
                                col-com
                                )
                          )
                         
                         ; le commentaire est trop large et trop haut : scroller   
                         ((and (> larg :%l-ea) (> haut :%h-max))
                          (scroller 0 0
                                  w-scroll h-scroll
                                  col-com
                                  t t
                                  )
                          )
                         
                         ; le commentaire est trop large : scroller horizontal                   
                         ((> larg :%l-ea)
                          (horizontalscroller 0 0
                                  w-scroll h-scroll                               
                                  col-com
                                  )
                          )
                         
                         ; le commentaire est trop haut : scroller vertical              
                         (t         
                            (verticalscroller 0 0
                                    w-scroll h-scroll
                                    col-com
                                    )
                            )
                         ))
           )
          (add-component appli 'img-com img-com)
          
          ; image finale
          (if (null list-com)
              (rectangle 0 0 (+ 12 w-scroll) 0)
              ; sinon
              (#:ivan:ed-util:clusterimage img-com "Commentaire")
              )
          )
    )

;;;*****************************************************************************************
;;; construction du champ titre. Le type de l'attribut est affiche dans un cadre

(de :const-ch-titre (appli nom-attr bliste type-attr bspec-loc btype-simple vect-classes)
    
    ;(PRINT ".. :const-ch-titre")
    (lets
          (
           (indmax (vlength vect-classes))
           
           (liste-de (if bliste 
                         (font #:ivan:ed-global:fstd "liste de ")
                         " " ))
           (spec-loc (if bspec-loc
                         (font #:ivan:ed-global:fstd "(spcialisation locale)" )
                         "" ))
           (titre-attr (catenate "Type de l'attribut " (string nom-attr))) 
           
           (bout-classe (if btype-simple
                            (font #:ivan:ed-global:fstdb
                                  (string type-attr))
                            ; on doit pouvoir cliquer sur la classe
                            (standardbutton (font #:ivan:ed-global:fstd (string type-attr)))
                            ))
           
           (img (centeredview 
                        (rectangle 0 0 (send 'width (component 'img-com appli)) 0)
                        (row
                             (centeredview
                                     (rectangle 0 0 0 (send 'height bout-classe))
                                     liste-de
                                     )
                             bout-classe
                             " "
                             (centeredview
                                     (rectangle 0 0 0 (send 'height bout-classe))
                                     spec-loc
                                     )
                             )
                        ))
           
           trouve
           (ind 0)
           )
          
          (unless btype-simple
                  
                  ; l'attribut est un objet complexe :
                  ; type-attr est le nom de la classe dont il faut retrouver le concept
                  ; pour savoir si le bouton doit etre active et pouvoir charger le 
                  ; concept en cas d'activation du bouton
                  (let
                       ()
                       (while (and (< ind indmax) (not trouve) )
                              (setq trouve 
                                    (member type-attr (cadr (vref vect-classes ind) )))
                              (unless trouve
                                      (setq ind (+ 1 ind)) )
                              )
                       ; ind contient le rang du concept
                       (ifn trouve
                            ; le bouton n'est pas active
                            (inhibit-application bout-classe)
                            ; sinon
                            ({application}:userdata bout-classe (list ind type-attr))
                            (set-action bout-classe ':action-bout-classe)
                            )
                       )
                  ) ; unless
          
          ; image du champ 
          (#:ivan:ed-util:clusterimage img titre-attr)
          )
    )

;;;*****************************************************************************************
;;; construction du scroller des valeurs de la facette $domaine

(de :const-ch-dom (appli bliste btype-simple type-attr list-dom)
    
    ;(PRINT ".. :const-ch-dom")
    (lets
          (
           (sm-dom     (stringmenu (mapcar 'string list-dom)))
           
           (larg (- (div (send 'width ({in-infatt}:ch-com appli)) 2) 12))
           
           ; scroller
           (scroll-dom (scroller 0 0
                               larg  #10hc
                               sm-dom
                               t t))
           ; titre de l'image
           (titre-img "Domaine")
           
           )
          
          (add-component appli 'scroll-dom scroll-dom)
          (add-component appli 'sm-dom sm-dom)
          
          (set-action sm-dom ':action-reinit-sm)
          
          (if (null list-dom)
              ; ce n'est pas un objet et il n'y a pas de domaine
              (rectangle 0 0 0 0)
              ; afficher le selectionneur
              (#:ivan:ed-util:clusterimage scroll-dom titre-img)
              )
          )
    )

;;;*****************************************************************************************
;;; construction du champ des facettes de cardinalite $cardmin et $cardmax 

(de :const-ch-card (appli card-min card-max)
    
    ;(PRINT ".. :const-ch-card")
    (let
         (
          (img-card 
                  (column
                          (if card-min
                              (row
                                   (font #:ivan:ed-global:fstd 
                                         "     minimum : ")
                                   (font #:ivan:ed-global:fstdb
                                         (string card-min))) 
                              (rectangle 0 0 0 0) 
                              )
                          (if card-max
                              (row
                                   (font #:ivan:ed-global:fstd 
                                         "     maximum : ")
                                   (font #:ivan:ed-global:fstdb
                                         (string card-max))) 
                              (rectangle 0 0 0 0) 
                              )
                          )
                  )
          
          )
         (send 'resize img-card (send 'width (component 'img-com appli)) nil)
         
         ; image du champ 
         (if (or card-min card-max)      
             (#:ivan:ed-util:clusterimage img-card "Cardinalit")
             (rectangle 0 0 0 0)
             )
         )
    )

;;;*****************************************************************************************
;;; facettes d'inference $sib-exec et $sib-filtre

(de :const-ch-infer (appli infer-proc infer-filt)
    
    ;(PRINT ".. :const-ch-infer")
    (lets
          (
           (ti-infer-proc (font #:ivan:ed-global:fstdb
                                "par attachement procdural"))
           
           (ti-infer-filt (font #:ivan:ed-global:fstdb
                                "par filtrage"))
           
           (img-infer
                   (centeredview
                           (rectangle 0 0 
                                   (send 'width (component 'img-com appli)) 0)
                           (column
                                   (if infer-proc
                                       ti-infer-proc
                                       (if infer-filt
                                           ti-infer-filt
                                           ""))
                                   )
                           )
                   )
           
           )
          
          ;(send 'resize img-infer (send 'width (component 'img-com appli)) nil)
          
          ; image du champ 
          (if (or infer-proc infer-filt)      
              (#:ivan:ed-util:clusterimage img-infer "Infrable")
              (rectangle 0 0 0 0)
              )
          )
    )

;;;*****************************************************************************************
;;; facettes de valeur $valeur et $defaut

(de :const-ch-valeur (appli list-valeur list-defaut)
    
    (lets
          (
           ; CAS DE $valeur
           (sm-valeur    (stringmenu (mapcar 'string list-valeur)))
           (larg-valeur  (div (- (send 'width (component 'img-com appli)) 24)
                              2))
           (scroll-valeur  (scroller 0 0
                                   larg-valeur #5hc
                                   sm-valeur
                                   t t))
           ; image de la facette $valeur
           (img-valeur   (if (null list-valeur)
                             (rectangle 0 0 (+ larg-valeur 12) 0)
                             (#:ivan:ed-util:clusterimage 
                                     scroll-valeur 
                                     "Valeur(s)") ))
           ; CAS DE $defaut
           (sm-defaut    (stringmenu (mapcar 'string list-defaut)))
           (larg-defaut  (- (send 'width ({in-infatt}:ch-com appli))
                            (send 'width img-valeur)
                            12))
           (scroll-defaut  (scroller 0 0
                                   larg-defaut #5hc
                                   sm-defaut
                                   t t))
           ; image de la facette $defaut
           (img-defaut   (if (null list-defaut)
                             (rectangle 0 0 (+ larg-defaut 12) 0)
                             (#:ivan:ed-util:clusterimage 
                                     scroll-defaut 
                                     "Valeur(s) par dfaut") ))
           ; image finale
           (img 
                (if (and (null list-valeur) (null list-defaut) )
                    (rectangle 0 0 0 0)
                    ; sinon
                    (row img-valeur img-defaut)
                    ))
           )
          
          (set-action sm-valeur ':action-reinit-sm)
          (set-action sm-defaut ':action-reinit-sm)
          
          ; image renvoyee
          img
          )
    )

;;;*****************************************************************************************
;;; construction du champ des contraintes a partir des facettes $sauf et $intervalle

(de :const-ch-contraintes (appli list-sauf list-inter)
    
    ;(PRINT ".. :const-ch-contraintes")
    (lets
          (
           ; CAS DE $intervalle
           (list-binf    (:list-bornes list-inter 3) )
           (list-bsup    (:list-bornes list-inter 4) )
           (sm-binf      (stringmenu (mapcar 'string list-binf)))
           (sm-bsup      (stringmenu (mapcar 'string list-bsup)))
           (tit-binf     (font #:ivan:ed-global:fstd "Borne inf.") )
           (tit-bsup     (font #:ivan:ed-global:fstd "Borne sup.") )
           (titre        (row
                              (rectangle 0 0 {scroller}:scrollbarwidth 0)
                              tit-binf
                              "  " 
                              tit-bsup) )
           (scroll-inter (scroller 0 0
                                 (send 'width titre) #5hc
                                 (row sm-binf "  " sm-bsup)
                                 t t))
           ; image de la facette $intervalle
           (img-inter (if (null list-inter)
                          (rectangle 0 0 (send 'width titre) 0)
                          (#:ivan:ed-util:clusterimage
                                  (column
                                          titre
                                          scroll-inter)
                                  "Intervalle") ))
           
           ; CAS DE $sauf
           (sm-sauf      (stringmenu (mapcar 'string list-sauf)))
           (larg-sauf    (- (send 'width ({in-infatt}:ch-com appli))
                            (send 'width img-inter)
                            12))
           (scroll-sauf  (scroller 0 0
                                 larg-sauf #5hc
                                 sm-sauf
                                 t t))
           ; image de la facette $sauf
           (img-sauf     (if (null list-sauf)
                             (rectangle 0 0 (+ larg-sauf 12) 0)
                             (#:ivan:ed-util:clusterimage 
                                     scroll-sauf 
                                     "Valeurs interdites") ))
           
           ; image finale
           (img 
                (if (and (null list-sauf) (null list-inter) )
                    (rectangle 0 0 0 0)
                    ; sinon
                    (row img-sauf img-inter)
                    ))
           )
          
          (send 'w sm-binf (send 'width tit-binf))
          (send 'w sm-bsup (send 'width tit-bsup))
          
          (when (or list-sauf list-inter)
                (send 'resize img 
                      nil
                      (max (send 'height img-sauf) (send 'height img-inter))
                      ))
          
          (set-action sm-sauf ':action-reinit-sm)
          (set-action sm-binf ':action-reinit-sm)
          (set-action sm-bsup ':action-reinit-sm)
          
          
          ; image renvoyee
          img
          )
    )

;;;*****************************************************************************************
;;; construit la liste des bornes min ou max en fonction la variable pos

(de :list-bornes (int pos)
    
    (let
         (
          (list nil)
          )
         (cond
               ((null int) nil) 
               ((atom int) (newr list (#:ivan:ed-util:val-inst-ch int pos)))
               (t (append (:list-bornes (car int) pos)
                          (:list-bornes (cdr int) pos) ))))
    )

;;;#########################################################################################
;;;                                   
;;;                                    ACTIONS DES CHAMPS 
;;;
;;;#########################################################################################

;;;*****************************************************************************************
;;; action du bouton standard d'accord

(de :action-bout-daccord (b)
    
    ;(PRINT " :action-bout-daccord")
    (lets
          (
           )
          (when (activatedp b)
                
                ; fermer la fenetre
                (remove-application (component 'top b))
                
                ; renvoyer nil
                nil
                )
          )
    )

;;;*****************************************************************************************
;;; action du bouton classe  (cas d'un objet de type complexe)

(de :action-bout-classe (b)
    
    ;(PRINT ".. :action-bout-classe")
    (lets
          (
           (list-param ({application}:userdata b))
           (iconcept   (nth 0 list-param) )
           (classe     (nth 1 list-param) )
           )
          
          (when (activatedp b)
                ; fermer la fenetre
                (remove-application (component 'top b))
                
                ; champ renvoye
                ({in-infatt}:ch-classe appli (list iconcept classe) )
                )
          )
    )

;;;*****************************************************************************************
;;; reinitialise le scroller sm pour desselectionner la valeur qui a ete cliquee

(de :action-reinit-sm (sm)
    
    (let
         ( 
          (valeur-sym (#:ivan:ed-util:symbole (send 'get-selected-string sm)))
          )
         (ifn valeur-sym
              ; alors rien (cas de la reinitialisation du sm)
              't
              ; sinon
              ; reinitialiser le selecteur
              (send 'set-selected-string sm "")
              )
         )
    )
;;;*****************************************************************************************
