;;; Repertoire: ~/LeShirka/LeIvan/CodeIvan/Interface/
;
;     +----------------------------------------+-----------------------------------------+
;     | fichier :  ivan-in-interf+hy.ll        | package : #:ivan:in-interf              |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface de Visualisation et d'Aide a la Navigation                 |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Sylvain Grivaud          1991                             |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     |                                                                                  |
;     | Fonctions du package utilisant l'Hypertexte AIDA                                 |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+

;;;*****************************************************************************************

(defvar #:sys-package:colon '#:ivan:in-interf)

;;;*****************************************************************************************
;;; fonction principale de construction de l'interface

(de :interface ()
    (let
         (
          ; application generale
          (appli (omakeq {in-interf} x 0 y 0 w 0 h 0) )
          
          (bout-ivan     (standardbutton #M#:txt-const:infos))
          (bout-option   (standardbutton #M#:txt-const:options))
          (bout-quitter  (standardbutton #M#:txt-const:fin))
          image-appli
          )
         
         ; action du bouton quitter
         (set-action bout-quitter ':action-bout-quitter)
         
         ; action du bouton ivan
         (set-action bout-ivan ':action-bout-ivan)
         
         ; action du bouton options 
         (set-action bout-option ':action-bout-option)
         
         ; composant top pour reperer le pere de plus haut niveau
         ({application}:components appli (acons 'top appli ()))
         (add-component appli  'bout-quitter bout-quitter)
         ;
         ; champs dynamiques
         ;
         ; liste des concepts
         ({in-interf}:ch-list-concepts   appli
                 (:const-ch-list-concepts    appli))
         ; vecteur des listes des classes 
         ({in-interf}:ch-vect-classes    appli
                 (:const-ch-vect-classes     appli))
         ; il n'y a pas encore de concept choisi
         ({in-interf}:ch-concept         appli "")
         ; donc pas de liste des classes du concept
         ({in-interf}:ch-list-hierarchie  appli nil)
         ; donc pas de liste des classes
         ({in-interf}:ch-list-classes    appli  nil)
         ;
         ; champs graphiques
         ;
         ({in-interf}:ch-commandes    appli 
                 (:const-ch-commandes appli))
         ; a ce niveau, il n'y aura pas d'arbre car aucun concept de choisi
         ({in-interf}:ch-ed-arbre     appli 
                 (:const-ch-ed-arbre  appli))
         ({in-interf}:ch-classes      appli 
                 (:const-ch-classes   appli))
         ({in-interf}:ch-scrollers    appli 
                 (:const-ch-scrollers appli))
         ({in-interf}:ch-edl-concept  appli 
                 (:const-ch-edl-concept appli))
         ({in-interf}:ch-edl-classe    appli 
                 (:const-ch-edl-classe appli))
         ({in-interf}:ch-sa-action     appli 
                 (:const-ch-sa-action  appli))
         ;
         ; creation de l'image de l'application
         ;
         (setq image-appli
               (column
                       (row
                            (column
                                    ({in-interf}:ch-commandes appli)
                                    ""
                                    )
                            ({in-interf}:ch-ed-arbre appli)
                            )
                       (row
                            ({in-interf}:ch-edl-concept appli) 
                            ({in-interf}:ch-sa-action appli) 
                            )
                       (row
                            (column
                                    ({in-interf}:ch-edl-classe appli)
                                    ({in-interf}:ch-classes appli)
                                    )
                            ({in-interf}:ch-scrollers appli)
                            )
                       )
               )
         
         (send 'grow image-appli #:ivan:in-global:larg-appli #:ivan:in-global:haut-appli)
         
         ; assemblage des differents champs pour constituer l'image finale
         (send 'set-image
               appli
               (header
                       logo
                           (row "    " bout-ivan "    " bout-option)
                       bout-quitter
                       (view)
                       image-appli      
                       t
                       nil
                       )
               )
         
         (send 'fit-to-contents appli)
         (send 'set-title appli "IVAN")
         appli
         )
    )

;;;*****************************************************************************************
;;; action de l'editeur de ligne : met a jour les differents scrollers

(de :action-edl-classe (edl)
    (lets
          (
           (nom-classe-ch  (send 'get-line edl))
           (nom-classe-sym (#:ivan:ed-util:symbole nom-classe-ch))
           list-surclasse
           list-sousclasse
           list-inst
           list-instance
           list-attribut
           list-classattr
           schema-clas
           )
          (ifn nom-classe-sym
               (inhibit-application (component 'bout-repos edl))
               ; on ne peut pas creer d'instance
               (inhibit-application (component 'bout-nouvinst edl))
               ; on ne peut pas faire de selection
               (inhibit-application (component 'bout-interro edl))
               ; on ne peut pas appeler le systeme de documentation
               (inhibit-application (component 'bout-hyper edl))
               ; sinon il faut mettre a jour le browser
               (setq
                     schema-clas     (#:shirka:noyau:schema      nom-classe-sym)
                     list-surclasse  (#:shirka:noyau:peres       schema-clas)
                     list-sousclasse (:l-spec-vraies             schema-clas)
                     list-inst       (#:ivan:ed-util:l-inst-ch   schema-clas)
                     list-instance   (if (listp list-inst) 
                                         list-inst
                                         (list list-inst) )
                     list-attribut   (:list-attribut             edl)
                     list-classattr  (:list-classattr            schema-clas)
                     )
               (authorize-application (component 'bout-nouvinst edl))  
               (authorize-application (component 'bout-repos    edl))
               (authorize-application (component 'bout-interro  edl))  
               (when (hy-web-get-concept #:ivan:hy-global:web nom-classe-ch) 
                     (authorize-application (component 'bout-hyper edl))  
                     )
               )
          (:reinit-sm
                  (component 'scroll-surclasse edl) 
                  (component 'sm-surclasse     edl) 
                  (sortl list-surclasse))
          
          (:reinit-sm
                  (component 'scroll-sousclasse edl)
                  (component 'sm-sousclasse     edl)
                  (sortl list-sousclasse))
          
          (:reinit-sm
                  (component 'scroll-attribut   edl)
                  (component 'sm-attribut       edl)
                  list-attribut)
          
          (:reinit-sm
                  (component 'scroll-classattr  edl)
                  (component 'sm-classattr      edl)
                  list-classattr)
          (:action-sm-classattr (component 'sm-classattr edl))
          
          ; il faut pouvoir choisir les classes d'attribut
          
          (send 'selection-limit 
                (component 'sm-classattr edl)
                (length list-classattr))
          
          (:reinit-sm
                  (component 'scroll-instance   edl)
                  (component 'sm-instance       edl)
                  (sortl list-instance))
          
          (unless nom-classe-sym
                  (inhibit-application (component 'bout-nouvinst edl))
                  (inhibit-application (component 'bout-interro  edl))
                  (inhibit-application (component 'bout-hyper    edl))
                  )
          
          )
    )

;;;*****************************************************************************************
;;; action du bouton quitter = fermer l'application apres une confirmation

(de :action-bout-quitter (b)
    (when (confirm #M#:txt-const:sur-de-quitter)
          (when (componentp 'requete b)      
                ; fermer si elle existe la fenetre des requetes
                (remove-application (component 'requete b) ) 
                ; detruire le composant de IVAN
                (remove-component (component 'top b) 'requete)
                )
          ; fermer si elle existe la fenetre de classification
          ; mais ne pas detruire le composant
          (when (componentp 'iace b)      
                (remove-application (component 'iace b) ) )
          ; fermer Ivan
          (remove-application (send 'father b))
          )
    )

;;;*****************************************************************************************
;;; action du bouton charger = ouvrir une fenetre pour saisir le nom de la base a charger
;;; (code identique dans "fc-interf" de iace)

(de :action-bout-charger (b)
    (lets ( i
           (edl-chemin (lineedit 0 0 (+ #5wc (send 'width #:shirka:*bases*)) #:shirka:*bases*))
           (img-chemin (#:ivan:ed-util:boxedimage (row (font #:ivan:in-global:fstd 
							#M#:txt-const:chemin)
                                                       (font #:ivan:in-global:fstdb edl-chemin)
                                                       )))
           (list-files (cdr (#:systeme:unix:catalogue (pathname #:shirka:*bases*))))
           (nom-base (ask-strings (column (font #:ivan:in-global:fstdb  #M#:txt-const:charger)
                                          ""
                                          img-chemin
                                          ""
                                          (row (font #:ivan:in-global:fstd
							#M#:txt-const:pour-charger)
                                               (font #:ivan:in-global:fstd
							#M#:txt-const:faire)
                                               )
                                          )
                             list-files nil 1 (length list-files) nil
                   ))
           (chem (send 'get-line edl-chemin))
           )
          (when nom-base
                (mapc (lambda (nom-f)
                              (lets ((nom-fichier (catenate chem nom-f))
                                     (nbel (length ({in-interf}:ch-list-concepts (component 'top b))))
                                     )
                                    (:inhiber-com b)
                                    (ifn (:charger-fic nom-fichier b)
                                         (progn (:action-sa-warn b nil t
                                                        (list (row (font #:ivan:in-global:fstdb 
									#M#:txt-const:echec)
                                                                   (font #:ivan:in-global:fstd  
									#M#:txt-const:le-fic)
                                                                   (font #:ivan:in-global:fstdb
									nom-fichier)
                                                                   (font #:ivan:in-global:fstd
									#M#:txt-const:pas-ouv))))
                                                (:autoriser-com (component 'top b))
                                                ) 
                                         ; sinon: mettre a jour liste des concepts/classes
                                         (:maj-concept-charger nbel b)
                                         ; meme chose du cote de iace
                                         (when (and (componentp 'iace b) (activatedp (component 'iace b)))
                                             (#:iace:fc-interf:maj-concept-charger 
                                                      nbel 
                                                      (component 'bouts-charger (component 'iace b)) )
                                             )
;hy--------------------------------------------------------------------------
; mettre a jour le reseau hypertexte dans le cas ou il n'y a que iace actif.
(#:ivan:hy-interf:hy-build-web #:ivan:hy-global:web nom-fichier)
;----------------------------------------------------------------------------
                                         (:autoriser-com (component 'top b)) 
                                         (:action-sa-warn
                                                 b nil t
                                                 (list (row (font #:ivan:in-global:fstd
									#M#:txt-const:sel-cp))))
                                         )
                                    )
                              )
                      nom-base 
                      )
                )
          )
    )
