;;; Repertoire: ~/LeShirka/LeIvan/CodeIvan/Interface/
;
;     +----------------------------------------+-----------------------------------------+
;     | fichier :  ivan-in-interf-hy.ll        | package : #:ivan:in-interf              |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface de Visualisation et d'Aide a la Navigation                 |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Sylvain Grivaud          1991                             |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     |                                                                                  |
;     | Fonctions du package n'utilisant pas l'Hypertexte AIDA                           |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+

;;;*****************************************************************************************

(defvar #:sys-package:colon '#:ivan:in-interf)

;;;*****************************************************************************************
;;; fonction principale de construction de l'interface

(de :interface ()
    (let
         (
          ; application generale
          (appli (omakeq {in-interf} x 0 y 0 w 0 h 0) )
          
          (bout-quitter  (standardbutton " Quitter ") )
          (bout-ivan     (standardbutton
                                 (font #:ivan:in-global:fstd "A propos d'IVAN") ))
          (bout-option   (standardbutton
                                 (font #:ivan:in-global:fstd "Options") ))
          
          image-appli
          )
         
         ; action du bouton quitter
         (set-action bout-quitter ':action-bout-quitter)
         
         ; action du bouton ivan
         (set-action bout-ivan ':action-bout-ivan)
         
         ; action du bouton options 
         (set-action bout-option ':action-bout-option)
         
         ; composant top pour reperer le pere de plus haut niveau
         ({application}:components appli (acons 'top appli ()))
         (add-component appli  'bout-quitter bout-quitter)
         ;
         ; champs dynamiques
         ;
         ; liste des concepts
         ({in-interf}:ch-list-concepts   appli
                 (:const-ch-list-concepts    appli))
         ; vecteur des listes des classes 
         ({in-interf}:ch-vect-classes    appli
                 (:const-ch-vect-classes     appli))
         ; il n'y a pas encore de concept choisi
         ({in-interf}:ch-concept         appli "")
         ; donc pas de liste des classes du concept
         ({in-interf}:ch-list-hierarchie  appli nil)
         ; donc pas de liste des classes
         ({in-interf}:ch-list-classes    appli  nil)
         ;
         ; champs graphiques
         ;
         ({in-interf}:ch-commandes      appli 
                 (:const-ch-commandes appli))
         ; a ce niveau, il n'y aura pas d'arbre car aucun concept de choisi
         ({in-interf}:ch-ed-arbre       appli 
                 (:const-ch-ed-arbre  appli))
         ({in-interf}:ch-classes       appli 
                 (:const-ch-classes appli))
         ({in-interf}:ch-scrollers      appli 
                 (:const-ch-scrollers appli))
         ({in-interf}:ch-edl-concept     appli 
                 (:const-ch-edl-concept appli))
         ({in-interf}:ch-edl-classe     appli 
                 (:const-ch-edl-classe appli))
         ({in-interf}:ch-sa-action      appli 
                 (:const-ch-sa-action  appli))
         ;
         ; creation de l'image de l'application
         ;
         (setq image-appli
               (column
                       (row
                            (column
                                    ({in-interf}:ch-commandes appli)
                                    ""
                                    )
                            ({in-interf}:ch-ed-arbre appli)
                            )
                       (row
                            ({in-interf}:ch-edl-concept appli) 
                            ({in-interf}:ch-sa-action appli) 
                            )
                       (row
                            (column
                                    ({in-interf}:ch-edl-classe appli)
                                    ({in-interf}:ch-classes appli)
                                    )
                            ({in-interf}:ch-scrollers appli)
                            )
                       )
               )
         
         (send 'grow image-appli #:ivan:in-global:larg-appli #:ivan:in-global:haut-appli)
         
         ; assemblage des differents champs pour constituer l'image finale
         (send 'set-image
               appli
               (header
                       logo
                       (row "   " bout-ivan "    " bout-option)
                       bout-quitter
                       (view)
                       image-appli      
                       t
                       nil
                       
                       )
               )
         
         ;;;;;       ; creation de l'hypertexte (pas dans cette version)
         ;;;;;      (setq
         ;;;;;         #:ivan:hy-global:tv  (#:ivan:hy-interf:hy-creation)
         ;;;;;         #:ivan:hy-global:web (hy-connectedp #:ivan:hy-global:tv)
         ;;;;;      )
         
         (send 'fit-to-contents appli)
         (send 'set-title appli "IVAN")
         appli
         )
    )

;;;*****************************************************************************************
;;; action de l'editeur de ligne : met a jour les differents scrollers

(de :action-edl-classe (edl)
    (lets
          (
           (nom-classe-ch  (send 'get-line edl))
           (nom-classe-sym (#:ivan:ed-util:symbole nom-classe-ch))
           list-surclasse
           list-sousclasse
           list-inst
           list-instance
           list-attribut
           list-classattr
           schema-clas
           )
          (ifn nom-classe-sym
               (inhibit-application (component 'bout-repos edl))
               ; on ne peut pas creer d'instance
               (inhibit-application (component 'bout-nouvinst edl))
               ; on ne peut pas faire de selection
               (inhibit-application (component 'bout-interro edl))
               ; on ne peut pas appeler le systeme de documentation
               (inhibit-application (component 'bout-hyper edl))
               ; sinon il faut mettre a jour le browser
               (setq
                     schema-clas     (#:shirka:noyau:schema      nom-classe-sym)
                     list-surclasse  (#:shirka:noyau:peres       schema-clas)
                     list-sousclasse (:l-spec-vraies             schema-clas)
                     list-inst       (#:ivan:ed-util:l-inst-ch   schema-clas)
                     list-instance   (if (listp list-inst) 
                                         list-inst
                                         (list list-inst) )
                     list-attribut   (:list-attribut             edl)
                     list-classattr  (:list-classattr            schema-clas)
                     )
               (authorize-application (component 'bout-nouvinst edl))  
               (authorize-application (component 'bout-repos   edl))
               (authorize-application (component 'bout-interro edl))  
               
               ;;;;;         (when (hy-web-get-concept #:ivan:hy-global:web nom-classe-ch) 
               ;;;;;            (authorize-application (component 'bout-hyper   edl))  
               ;;;;;         )
               )
          (:reinit-sm
                  (component 'scroll-surclasse  edl) 
                  (component 'sm-surclasse      edl) 
                  (sortl list-surclasse))
          
          (:reinit-sm
                  (component 'scroll-sousclasse edl)
                  (component 'sm-sousclasse     edl)
                  (sortl list-sousclasse))
          
          (:reinit-sm
                  (component 'scroll-attribut   edl)
                  (component 'sm-attribut       edl)
                  list-attribut)
          
          (:reinit-sm
                  (component 'scroll-classattr   edl)
                  (component 'sm-classattr       edl)
                  list-classattr)
          (:action-sm-classattr (component 'sm-classattr edl))
          
          ; il faut pouvoir choisir les classes d'attribut
          
          (send 'selection-limit 
                (component 'sm-classattr edl)
                (length list-classattr))
          
          (:reinit-sm
                  (component 'scroll-instance   edl)
                  (component 'sm-instance       edl)
                  (sortl list-instance))
          
          (unless nom-classe-sym
                  (inhibit-application (component 'bout-nouvinst edl))
                  (inhibit-application (component 'bout-interro  edl))
                  (inhibit-application (component 'bout-hyper    edl))
                  )
          
          )
    )

;;;*****************************************************************************************
;;; action du bouton quitter = fermer l'application apres une confirmation

(de :action-bout-quitter (b)
    (when (confirm "Etes-vous sur de vouloir quitter ? ")
          
          (when (componentp 'requete b)      
                ; je ferme si elle existe la fenetre des requetes
                (remove-application (component 'requete b) ) 
                ; je detruis le composant de IVAN
                (remove-component (component 'top b) 'requete)
                )
          ; je ferme si elle existe la fenetre de classification
          ; mais je ne detruis pas le composant
          (when (componentp 'iace b)      
                (remove-application (component 'iace b) ) )
          ; je ferme Ivan
          (remove-application (send 'father b))
          ;;;;       ; detruire l'hypertexte
          ;;;;      (hy-disconnect #:ivan:hy-global:tv)
          )
    )

;;;*****************************************************************************************
;;; action du bouton charger = ouvrir une fenetre pour saisir le nom de la base a charger

(de :action-bout-charger (b)
    (lets
          (
           i
           (edl-chemin (lineedit 0 0
                               (+ #5wc (send 'width #:shirka:*bases*))
                               #:shirka:*bases*) )
           
           (img-chemin
                   (#:ivan:ed-util:boxedimage
                           (row
                                (font #:ivan:in-global:fstd
                                      "Chemin d'acces : ")
                                edl-chemin
                                )))
           (list-files (cdr (#:systeme:unix:catalogue (pathname #:shirka:*bases*))))
           
           (nom-base
                   (ask-strings
                           (column
                                   (font #:ivan:in-global:fstdb
                                         "Charger une base Shirka")
                                   ""
                                   img-chemin
                                   ""
                                   (row
                                        (font #:ivan:in-global:fstd
                                              "Pour charger le(s) fichier(s)," )
                                        (font #:ivan:in-global:fstd
                                              " utilisez le bouton droit de la souris :") )
                                   )
                           list-files
                           nil
                           1
                           (length list-files)
                           nil
                           )
                   )
           
           (chem  (send 'get-line edl-chemin) )
           )
          (when nom-base
                (mapc 
                      (lambda (nom-f)
                              (lets
                                    (
                                     (nom-fichier (catenate chem nom-f))
                                     (nbel (length ({in-interf}:ch-list-concepts (component 'top b)))) 
                                     )
                                    ; inhiber toutes les commandes
                                    (:inhiber-com b)
                                    ; charger la base
                                    (ifn (:charger-fic nom-fichier b)
                                         ; ALORS 
                                         (progn
                                                ; message d'erreur
                                                (:action-sa-warn
                                                        b nil nil
                                                        (list
                                                              (row
                                                                   (font #:ivan:in-global:fstdb "ECHEC ! ")
                                                                   (font #:ivan:in-global:fstd  "Le fichier ")
                                                                   (font #:ivan:in-global:fstdb nom-fichier)
                                                                   (font #:ivan:in-global:fstd  " n'a pas ete ouvert."))))
                                                ; reinitialiser les boutons de commande
                                                (:autoriser-com (component 'top b))
                                                ) 
                                         ; SINON
                                         ; mettre a jour la liste des concepts et la liste des classes
                                         (:maj-concept-charger nbel b)
                                         ; meme chose du cote de iace (20.09.91)
                                         (when (and (componentp 'iace b) (activatedp (component 'iace b)) )
                                               (#:iace:fc-interf:maj-concept-charger
                                                       nbel
                                                       (component 'bouts-charger (component 'iace b) ))
                                               )
                                         
                                         ;;;;                      ; mettre a jour le reseau hypertexte
                                         ;;;;                     (#:ivan:hy-interf:hy-build-web #:ivan:hy-global:web nom-fichier)
                                         
                                         ; reinitialiser les boutons de commande
                                         (:autoriser-com (component 'top b)) 
                                         ; message
                                         (:action-sa-warn
                                                 b nil t
                                                 (list
                                                       (font #:ivan:in-global:fstd
                                                             "Voila, c'est fait. Vous pouvez choisir un concept.")))
                                         )
                                    ) ; lets
                              ) ; lambda
                      nom-base
                      ) ; mapc
                )
          )
    )
