;;; Repertoire: ~/LeShirka/LeIvan/CodeIvan/Interface/
;;;
;     +----------------------------------------+-----------------------------------------+
;     | fichier :  ivan-in-interf.ll           | package : #:ivan:in-interf              |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface de Visualisation et d'Aide a la Navigation                 |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Sylvain Grivaud          1991                             |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     | descriptif du package :                                                          |
;     |                                                                                  |
;     | La fonction principale :interface cree une instance de la structure {in-interf}. |
;     | Cette structure contient les champs de representation de l'interface de naviga-  |
;     | consultation.                                                                    |
;     | L'interface est une fenetre composee de 5 parties fonctionnelles :               |
;     |                                                                                  |
;     |                                                                                  |
;     |        |-------------------------------------------------+-----------|           |
;     |        |           LIGNE D'ENTETE                        | 'quitter' |           |
;     |        |---------------+---------------------------------+-----------|           |
;     |        |   COMMANDES   |               REPRESENTATION GRAPHIQUE      |           |
;     |        |---------------+---------------------------------------------|           |
;     |        |                   BROWSER DES CLASSES                       |           |
;     |        |-------------------------------------------------------------|           |
;     |                                                                                  |
;     | Le fichier est en 4 parties :                                                    |
;     |    1. Fonction principale                                                        |
;     |    2. Fonctions de construction des champs                                       |
;     |    3. Fonctions d'action des composants                                          |
;     |    4. Fonctions diverses (utilitaires)                                           |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;

;;;*****************************************************************************************

(defvar #:sys-package:colon '#:ivan:in-interf)

;;;*****************************************************************************************

;;;(de :interface ()
;;;
;;;    fonction principale de construction de l'interface.
;;;
;;;    si       Hypertexte => voir "ivan-in-interf+hy.ll"
;;;    si pas d'Hypertexte => voir "ivan-in-interf-hy.ll"


;;;#########################################################################################
;;;#########################################################################################
;;;
;;;                                   CONSTRUCTION DES CHAMPS 
;;;
;;;#########################################################################################
;;;#########################################################################################

;;;*****************************************************************************************
;;; Fonction d'extraction des concepts d'une base Shirka.
;;; Il s'agit de la liste des sous-classes directes de "objet" 
;;; sans les classes internes a Shirka (objet-simple, schema, ...)

(de :const-ch-list-concepts (appli)
    (lets
          (  
           (schema-objet (#:shirka:noyau:schema 'objet))
           (list-concepts
                   (#:ivan:ed-util:diff-liste 
                           ; (vref sous-classes-objet 5) 
                           (:l-spec-vraies schema-objet)
                           '(objet-simple schema attribut facette intervalle attach-proc erreur)))
           
           )
          (when #:ivan:in-global:bobjet
                (newr list-concepts 'objet) )
          (sortl list-concepts)
          )
    )

;;;*****************************************************************************************
;;; fonction qui construit pour chaque concept la hierarchie de ses classes et la
;;; liste plate de ses classes.
;;; Chaque element du vecteur est une liste de 2 listes.
;;; Cette fonction est declenchee lors du chargement d'un fichier.

(de :const-ch-vect-classes (appli)
    (lets
          (
           (list-concepts ({in-interf}:ch-list-concepts appli))
           (vect (makevector (length list-concepts) () ))
           list-h
           list-c
           (rg 0)
           )
          
          (while list-concepts 
                 (lets
                       (
                        (concept (car list-concepts))
                        ; liste de la hierarchie du concept
                        (list-h  (:list-hierarchie concept))
                        
                        ; liste des classes
                        (list-c  (#:ivan:ed-util:list-plate list-h))
                        )
                       (vset vect rg (list list-h list-c))
                       (setq rg (+ rg 1))
                       (nextl list-concepts)
                       )
                 )
          vect
          )
    )

;;;*****************************************************************************************
;;;    Fonction de construction du champ des commandes = colonne des boutons repartie en 3
;;;    groupes : 
;;;      1. Fonction sur une base Shirka
;;;      2. Fonctions sur l'arbre
;;;      3. Aide

(de :const-ch-commandes (appli)
    (lets
          (
           ; definition des titres
           (titre-clu1   #M#:txt-const:gest-bases)
           (titre-clu2   #M#:txt-const:visu)
           (titre-clu3   #M#:txt-const:appel-hyp)
           (titre-clu4   #M#:txt-const:iace)
           (titre-clu5   #M#:txt-const:requ)
           
           ; cas du bouton de choix d'un concept
           
           (menu-concept (create-menu #M#:txt-const:choisir))
           
           (bout-concept  (menubutton "Concepts"
                                  0 0
                                  0 0
                                  menu-concept) ) 
           
           ; definition des boutons de commandes
           (bout-chemin       (standardbutton #M#:txt-const:rep))
           (bout-verifier     (standardbutton #M#:txt-const:verif))
           (bout-charger      (standardbutton #M#:txt-const:charg))
           (bout-sauver       (standardbutton #M#:txt-const:sauv))
           (bout-interro      (standardbutton #M#:txt-const:interro))
           (bout-prettytree   (standardbutton #M#:txt-const:redessin))
           (bout-standardtree (standardbutton #M#:txt-const:taille-norm))
           (bout-reduct1      (standardbutton #M#:txt-const:red-1))
           (bout-reduct2      (standardbutton #M#:txt-const:red-2))
           (bout-minitree     (standardbutton #M#:txt-const:vue-gen))
           (bout-repos        (standardbutton #M#:txt-const:repos))
           (bout-aller        (standardbutton #M#:txt-const:select))
           (bout-hyper        (standardbutton #M#:txt-const:docu))
           (bout-classer      (standardbutton #M#:txt-const:classif))
           
           ; definition des cluster
           (clu1
                 (#:ivan:ed-util:clusterimage 
                         (column
                                 bout-chemin
                                 bout-verifier
                                 bout-charger
                                 bout-sauver
                                 bout-concept
                                 )
                         titre-clu1
                         ))
           
           (clu2
                 (#:ivan:ed-util:clusterimage 
                         (column
                                 bout-prettytree 
                                 bout-standardtree
                                 bout-reduct1
                                 bout-reduct2
                                 bout-minitree
                                 bout-aller
                                 bout-repos
                                 )
                         titre-clu2
                         ))
           
           (clu3
                 (#:ivan:ed-util:clusterimage 
                         (column
                                 bout-hyper
                                 )
                         titre-clu3
                         ))
           
           (clu4
                 (#:ivan:ed-util:clusterimage 
                         (column
                                 bout-classer
                                 )
                         titre-clu4
                         ))
           
           (clu5
                 (#:ivan:ed-util:clusterimage
                         (column
                                 bout-interro
                                 )
                         titre-clu5
                         ))
           )
          
          (add-component appli 'menu-concept  menu-concept)
          (add-component appli 'bout-concept  bout-concept)
          (add-component appli 'bout-verifier bout-verifier)
          (add-component appli 'bout-charger  bout-charger)
          (add-component appli 'bout-chemin   bout-chemin)
          (add-component appli 'bout-sauver   bout-sauver)
          
          (add-component appli 'bout-prettytree   bout-prettytree)
          (add-component appli 'bout-standardtree bout-standardtree)
          (add-component appli 'bout-reduct1      bout-reduct1)
          (add-component appli 'bout-reduct2      bout-reduct2)
          (add-component appli 'bout-minitree     bout-minitree)
          (add-component appli 'bout-aller        bout-aller)
          (add-component appli 'bout-repos        bout-repos)
          
          (add-component appli 'bout-interro      bout-interro)
          (add-component appli 'bout-hyper        bout-hyper)
          (add-component appli 'bout-classer      bout-classer)

          (set-action bout-chemin        ':action-bout-chemin)          
          (set-action bout-verifier      ':action-bout-verifier)
          (set-action bout-charger       ':action-bout-charger)
          (set-action bout-sauver        ':action-bout-sauver)
          
          (set-action bout-prettytree    ':action-bout-prettytree)
          (set-action bout-standardtree  ':action-bout-standardtree)
          (set-action bout-reduct1       ':action-bout-reduct1)
          (set-action bout-reduct2       ':action-bout-reduct2)
          (set-action bout-minitree      ':action-bout-minitree) ; minitree
          (set-action bout-aller         ':action-bout-aller)
          (set-action bout-repos         ':action-bout-repos)
          
          ; module des requetes
          (set-action bout-interro       ':action-bout-interro)
          ; hypertexte
          (set-action bout-hyper         ':action-bout-hyper)
          
          ; interface de classification
          (set-action bout-classer       ':action-bout-classer)
          
          (:creer-menu appli)
          (:autoriser-com appli) 
          
          (column
                  clu1 ; bases
                  clu2 ; arbres
                  clu5 ; requetes
                  clu3 ; hypertexte
                  clu4)
          
          )
    )

;;;*****************************************************************************************
;;; construction de la representation graphique des classes

(de :const-ch-ed-arbre (appli)
    (lets
          (
           (titre-clu   #M#:txt-const:arbre-spec)
           (ed-arbre    (view))
           (scroll-arbre 
                   (scroller 0 0
                           #80wd #50hd
                           ed-arbre
                           t t) )
           
           (clu
                (#:ivan:ed-util:clusterimage scroll-arbre titre-clu)
                )
           )
          (add-component appli 'ed-arbre ed-arbre) 
          (add-component appli 'scroll-arbre scroll-arbre)
          clu
          )
    )

;;;*****************************************************************************************
;;; construction des scrollers = sur-classes et sous-classes

(de :const-ch-classes (appli)
    (lets
          (
           
           (larg (div (- #:ivan:in-global:larg-appli 48) 4))
           ; definition des titres
           (titre-surclasse  #M#:txt-const:sur-cl)
           (titre-sousclasse #M#:txt-const:sous-cl)
           
           ; definition des selectionneurs de chaine
           (sm-surclasse  (stringmenu nil) )
           (sm-sousclasse (stringmenu nil) )
           
           
           ; definition des scrollers
           (scroll-surclasse
                   (scroller
                           0 0
                           larg #4hc
                           sm-surclasse
                           t t) )
           
           (scroll-sousclasse
                   (scroller
                           0 0
                           larg #14hc
                           sm-sousclasse
                           t t) )
           
           
           ; definition des cluster
           (clu-surclasse
                   (#:ivan:ed-util:clusterimage 
                           scroll-surclasse
                           titre-surclasse
                           ))
           
           (clu-sousclasse
                   (#:ivan:ed-util:clusterimage 
                           scroll-sousclasse
                           titre-sousclasse
                           ))
           
           )
          
          (add-component appli 'sm-surclasse  sm-surclasse)
          (add-component appli 'sm-sousclasse sm-sousclasse)
          
          
          (add-component appli 'scroll-surclasse  scroll-surclasse)
          (add-component appli 'scroll-sousclasse scroll-sousclasse)
          
          (set-action sm-surclasse  ':action-sm-surclasse)
          (set-action sm-sousclasse ':action-sm-sousclasse)
          
          (column
                  clu-surclasse
                  clu-sousclasse
                  )
          )
    )

;;;*****************************************************************************************
;;; construction des scrollers = attributs, classes d'attributs et instances

(de :const-ch-scrollers (appli)
    (lets
          (
           
           (larg (div (- #:ivan:in-global:larg-appli 48) 4))
           ; definition des titres
           (titre-attribut   #M#:txt-const:atts)
           (titre-classattr  #M#:txt-const:cl-atts)
           (titre-attrvisu   #M#:txt-const:atts-visu)
           (titre-instance   #M#:txt-const:insts)
           
           ; definition des selectionneurs de chaine
           (sm-attribut   (stringmenu nil) )
           
           (sm-classattr  (stringmenu nil) )
           (sm-attrvisu   (stringmenu nil) )
           
           (sm-instance   (stringmenu nil) )
           
           (cb-titre
                   (font #:ivan:in-global:fstd #M#:txt-const:propres))
           (cb-locaux     (checkbox 0 0
                                  (- larg #2wc) (+ 4 #hc)
                                  cb-titre))
           
           (bout-nouvinst
                   (standardbutton (font #:ivan:in-global:fstd #M#:txt-const:creer-inst)))
           
           ; definition des scrollers
           
           (scroll-instance
                   (scroller
                           0 0
                           larg #19hc
                           sm-instance
                           t t) )
           
           (scroll-attribut
                   (scroller
                           0 0
                           larg #17hc
                           sm-attribut
                           t t) )         
           
           (scroll-classattr
                   (scroller
                           0 0
                           larg #7hc
                           sm-classattr
                           t t) )              
           
           (scroll-attrvisu
                   (scroller
                           0 0
                           (- larg #wc) #7hc
                           sm-attrvisu
                           t t) ) 
           
           ; definition des cluster
           
           (clu-attribut
                   (#:ivan:ed-util:clusterimage
                           (column
                                   cb-locaux
                                   scroll-attribut
                                   )
                           titre-attribut
                           ))
           
           (clu-attrvisu
                   (#:ivan:ed-util:clusterimage
                           scroll-attrvisu  
                           titre-attrvisu
                           ))
           
           (clu-classattr
                   (#:ivan:ed-util:clusterimage
                           (column
                                   (centeredview
                                           (rectangle 0 0 0 (send 'height cb-locaux))
                                           (font #:ivan:in-global:fmicro #M#:txt-const:clic-droit))
                                   scroll-classattr
                                   (centeredview 
                                           (rectangle 0 0 larg 0)
                                           (icon 0 0 ic-arrow-down)
                                           )
                                   (centeredview 
                                           (rectangle 0 0 larg 0)
                                           clu-attrvisu
                                           )
                                   )
                           titre-classattr
                           ))
           
           (clu-instance
                   (#:ivan:ed-util:clusterimage 
                           (column
                                   (row "      " bout-nouvinst " ")
                                   scroll-instance
                                   )
                           titre-instance
                           ))
           )
          
          (add-component appli 'sm-instance       sm-instance)
          (add-component appli 'sm-attribut       sm-attribut)
          (add-component appli 'sm-classattr      sm-classattr)
          (add-component appli 'sm-attrvisu       sm-attrvisu)
          
          (add-component appli 'scroll-instance   scroll-instance)
          (add-component appli 'scroll-attribut   scroll-attribut)
          (add-component appli 'scroll-classattr  scroll-classattr)
          (add-component appli 'scroll-attrvisu   scroll-attrvisu)
          
          (add-component appli 'cb-locaux cb-locaux)
          (add-component appli 'bout-nouvinst bout-nouvinst)
          
          (set-action cb-locaux ':action-cb-locaux)
          (set-action bout-nouvinst ':action-bout-nouvinst)
          
          (send 'checked cb-locaux nil)
          
          (set-action sm-attribut  ':action-sm-attribut)
          (set-action sm-classattr ':action-sm-classattr)
          (set-action sm-attrvisu  ':action-sm-attrvisu)
          (set-action sm-instance  ':action-sm-instance)
          
          (inhibit-application bout-nouvinst) 
          
          (row
               clu-attribut
               clu-classattr
               clu-instance
               )
          )
    )

;;;*****************************************************************************************
;;; construction de l'editeur de ligne contenant le nom du concept selectionne

(de :const-ch-edl-concept (appli)
    (let
         (
          (edl-concept (lineedit 0 0 (send 'width (component 'scroll-surclasse appli)) ""))
          )
         (add-component appli 'edl-concept edl-concept)
         ; image renvoyee
         (#:ivan:ed-util:clusterimage edl-concept #M#:txt-const:cpt-sel)
    ))

;;;*****************************************************************************************
;;; construction de l'editeur de ligne contenant le nom de la classe courante

(de :const-ch-edl-classe (appli)
    (let
         (
          (edl-classe (lineedit 0 0 (send 'width (component 'scroll-surclasse appli)) ""))
          )
         (add-component appli 'edl-classe edl-classe)
         (set-action edl-classe ':action-edl-classe)
         ; image renvoyee
         (#:ivan:ed-util:clusterimage edl-classe #M#:txt-const:cl-sel)
    ))

;;;*****************************************************************************************
;;; construction du champ d'affichage de message d'information/d'alerte

(de :const-ch-sa-action (appli)
    (let
         (
          (sa-warn (selectorapplication
                           0 0
                           (* 3 (send 'width (component 'scroll-surclasse appli)))
                           (send 'height (component 'edl-classe appli))
                           nil))
          )
         (add-component appli 'sa-warn sa-warn)
         ; image renvoyee
         (#:ivan:ed-util:clusterimage sa-warn #M#:txt-const:message)
    ))

;;;#########################################################################################
;;;#########################################################################################
;;;
;;;                             FONCTIONS DE CONSTRUCTION INTERMEDIAIRES
;;;
;;;#########################################################################################
;;;#########################################################################################

;;;*****************************************************************************************
;;; Creation du menu deroulant associe au bouton de choix d'un concept

(de :creer-menu (appli)
    (let
         (
          (list-concept ({in-interf}:ch-list-concepts appli) )
          (pos 0)
          (menu (component 'menu-concept appli))
          
          )
         (while list-concept
                (let
                     (
                      (concept (car list-concept))
                      )
                     (menu-insert-item
                             menu
                             0
                             pos
                             (format () "~18@A" (string concept)) ; cadrage sur 18 caracteres 
                             1
                             `(lambda (a)
                                      (:action-menu-concept (quote ,appli) (quote ,pos) t)
                                      a)
                             )
                     (setq pos (+ pos 1))
                     (nextl list-concept)
                     )
                )
         )
    )

;;;*****************************************************************************************
;;; Fonction de construction de l'arbre a partir de la liste des classes du concept

(de :creer-arbre (appli)
    (let
         (  
          (list-hierar ({in-interf}:ch-list-hierarchie appli))
          (list-classe ({in-interf}:ch-list-classes    appli))
          (cur-font    (current-font))
          )
         ; recherche du nom de classe la + large pour cadrage
         (current-font #:ivan:in-global:fmicro)
         (setq #:ivan:in-global:larg 
               (#:ivan:ed-util:wc (#:ivan:ed-util:larg-max list-classe)))
         (current-font cur-font)
         
         (let
              (
               (ed-arbre (treeeditor 0 0 0 0 nil))
               noeud-pere
               )
              (if (null list-hierar)
                  ed-arbre
                  ; sinon
                  (setq noeud-pere (omakeq {in-noeud}
                                           nom (string (car list-classe))
                                           titre (string (car list-classe))
                                           ))
                  (send 'set-current ed-arbre nil)
                  (send 'insert ed-arbre noeud-pere)
                  (putprop (car list-hierar) noeud-pere 'noeud)
                  ; appel fonction de creation
                  (:creer-noeud ed-arbre list-hierar)
                  )
              )
         )
    )

;;;*****************************************************************************************
;;; Fonction recursive de creation des noeuds (instance de la classe {in-noeud})
;;; representants les classes

(de :creer-noeud (ed-arbre list-classe)
    (let
         (
          (pere (car list-classe))
          (list-fils (cdr list-classe))
          )
         (while list-fils
                (let
                     (
                      (car-list-fils (car list-fils) )
                      )
                     
                     (if (atom car-list-fils)
                         ; alors il faut inserer la classe
                         (:inserer-fils ed-arbre pere car-list-fils)
                         ; sinon le car est une liste
                         ; je cree le fils
                         (:inserer-fils ed-arbre pere (car car-list-fils))
                         (if (< (length car-list-fils) #:ivan:in-global:limite)
                             ; alors appel recursif pour construire le sous-arbre du fils
                             (:creer-noeud ed-arbre car-list-fils)
                             ; sinon il y a trop de fils : il faut creer un noeud qui contient
                             ; comme propriete tous les noms des sous-classes
                             (:inserer-fils2 ed-arbre (car car-list-fils) (cdr car-list-fils))
                             )
                         ; je reinitialise le pere
                         (send 'set-current ed-arbre pere)
                         )
                     ; classe suivante
                     (nextl list-fils)
                     )
                )
         ed-arbre
         )
    )

;;;*****************************************************************************************
(de :inserer-fils (ed-arbre pere fils) 
    (lets
          (            
           (noeud-fils (getprop fils 'noeud))
           (noeud-pere (getprop pere 'noeud))
           nouv-noeud
           )
          (if noeud-fils
              ; le noeud fils existe deja cas de l'heritage multiple
              ; il faut rajouter sa nouvelle descendance
              (progn
                     (send 'set-current ed-arbre noeud-pere)
                     (send 'link ed-arbre noeud-pere noeud-fils)
                     (send 'set-current ed-arbre noeud-fils)
                     )
              ; sinon
              ; le noeud fils n'existe pas : il faut le creer puis le
              ; rattacher a son pere
              (setq nouv-noeud (omakeq {in-noeud}
                                       nom (string fils)
                                       titre (string fils) ))
              (send 'set-current ed-arbre noeud-pere)
              (send 'insert ed-arbre nouv-noeud)
              (putprop fils nouv-noeud 'noeud)
              )
          )
    )

;;;*****************************************************************************************
;;; fils est une liste contenant plus de 10 elements; le pere de cette liste a deja
;;; ete dessine. Il faut construire le noeud regroupant l'ensemble des sous-classes.
;;; Le champ nom du noeud est initialise a vide pour reinitialiser le browser.

(de :inserer-fils2 (ed-arbre pere list-fils)
    (lets
          (
           (titre-noeud (length list-fils))
           (nouv-noeud  (omakeq {in-noeud2}
                                nom "" 
                                titre (catenate " " (string titre-noeud) " classes")
                                ))
           (noeud-pere (getprop pere 'noeud))
           )
          (send 'set-current ed-arbre noeud-pere)
          (send 'insert ed-arbre nouv-noeud)
          ; le noeud a l'ecran represente toutes les classes,
          ; je dois rattacher la propriete nom-de-classe => nouv-noeud
          ; pour retrouver a partir du nom de la classse le noeud.
          
          (mapc '(lambda (classe)
                         (putprop classe nouv-noeud 'noeud)
                         )
                (#:ivan:ed-util:list-plate list-fils)
                )
          )
    )

;;;#########################################################################################
;;;#########################################################################################
;;;                                   
;;;                            ACTIONS DES CHAMPS ET DES COMPOSANTS
;;;                                   
;;;#########################################################################################
;;;#########################################################################################

;;;*****************************************************************************************
;;; action du menu deroulant : on a choisi un concept
;;; il faut charger sa representation graphique

(de :action-menu-concept (appli pos bfois)
    (let
          (
           (concept      (nth pos ({in-interf}:ch-list-concepts appli)))
           (list-classes (vref ({in-interf}:ch-vect-classes appli) pos))
           )

	  (:actualiser-avec appli concept list-classes)

          ; mettre a jour le concept selectionne dans l'interface de classification
          (when (and (componentp 'iace appli) (activatedp (component 'iace appli)) bfois)
                ; le concept est dessine
                (#:iace:fc-interf:action-menu-concept #:iace:appli pos nil)
                )

))

;;;*****************************************************************************************
;;; chargement de la representation graphique d'un concept ou d'une sous-hierarchie
;;;	(le dernier cas se presente pour la visualisation de la hierarchie pour les noeuds
;;;	 ensemblistes)

(de :actualiser-avec (appli concept list-classes)

          (:action-sa-warn
                  appli t t
                  (list
                        (row
                             (font #:ivan:in-global:fstd
				(if list-classes #M#:txt-const:pat-cpt #M#:txt-const:pat-cl))
                             (font #:ivan:in-global:fstdb (string concept))
                             (font #:ivan:in-global:fstd "...")
                             )))
 
          (unless list-classes 
		(setq list-classes (:list-hierarchie concept))
		(setq list-classes (list list-classes (#:ivan:ed-util:list-plate list-classes)))
	  )
         
          (:inhiber-com appli)
          
          ; il faut supprimer la propriete des noeuds avant de charger le nouveau concept 
          (:demarquer-ed-arbre appli)
          
          ; les variables sont reinitialisees pour la construction de nouv-ed-arbre
          (setq #:ivan:in-global:bminitree nil) 
          (setq #:ivan:in-global:breduct1  nil)
          (setq #:ivan:in-global:breduct2  nil)
          
          ; je reinitialise l'editeur de ligne
          (:reinit-edl-classe (component 'edl-classe appli) "")
          
          ; concept courant
          ({in-interf}:ch-concept appli concept)
          
          ; liste de la hierarchie du concept
          ({in-interf}:ch-list-hierarchie appli (car  list-classes))
          ; liste des classes
          ({in-interf}:ch-list-classes    appli (cadr list-classes))
          
          ; j'affiche le nom du concept
          (:reinit-edl-concept (component 'edl-concept appli) concept)
          
          ; il faut supprimer la propriete des noeuds de l'arbre du nouveau concept
          (:demarquer-ed-arbre appli)
          (lets
                (
                 (scroll-arbre    (component 'scroll-arbre appli) )
                 (nouv-ed-arbre   (:creer-arbre appli)) 
                 nouv-scroll-arbre
                 inside         
                 )
                
                (send 'pretty nouv-ed-arbre)
                (setq inside         (send 'image nouv-ed-arbre))
                (send 'grow nouv-ed-arbre
                      (+ (send 'x inside) (* 2 (send 'width  inside)))
                      (+ {scroller}:scrollbarheight (send 'height inside)))
                
                (setq nouv-scroll-arbre
                      (scroller 0 0
                              (send 'width scroll-arbre)
                              (send 'height scroll-arbre)
                              nouv-ed-arbre
                              t t) )
                
                
                (set-component appli 'ed-arbre nouv-ed-arbre)
                
                (set-action nouv-ed-arbre ':action-ed-arbre)
                
                (send 'set-image scroll-arbre nouv-scroll-arbre)
                (set-component    appli 'scroll-arbre nouv-scroll-arbre)
                
                (send 'set-current nouv-ed-arbre nil)
                
                (:action-bout-prettytree (component 'bout-prettytree appli))
                
                (:autoriser-com appli)

		(send 'set-current (component 'ed-arbre appli) (getprop concept 'noeud))
                
                (:action-sa-warn
                        appli nil t
                        (list
                              (row
                                   (font #:ivan:in-global:fstd #M#:txt-const:sel-cl))))
                )
    )

;;;*****************************************************************************************
;;; action de l'editeur d'arbre = on a clique sur un noeud
;;; transmet le nom du noeud a l'editeur de ligne

(de :action-ed-arbre (ea)
    (lets
          (
           (noeud-courant (send 'get-current ea))
           (nom-noeud     (when noeud-courant (string (send 'nom noeud-courant))))
           )
          ; reinitialiser l'editeur de ligne
          (when nom-noeud (:reinit-edl-classe (component 'edl-classe ea) nom-noeud))
          
          (if (typep noeud-courant '{node}:in-noeud2)
              ; le noeud selectionne est un noeud "ensembliste"
              (:action-sa-warn
                      ea nil nil
                      (list
                            (row
                                 (font #:ivan:in-global:fstdb #M#:txt-const:attention)
                                 (font #:ivan:in-global:fstd #M#:txt-const:noeud-ens) )))
              ; effacer la zone de message
              (:action-sa-warn ea nil nil nil)
              )
          )
    )

;;;*****************************************************************************************
;;; action de l'editeur de ligne : met a jour les differents scrollers
;;;(de :action-edl-classe (edl)
;;;
;;;    si       Hypertexte => voir "ivan-in-interf+hy.ll"
;;;    si pas d'Hypertexte => voir "ivan-in-interf-hy.ll"

;;;*****************************************************************************************
;;; action du menu de chaines de caracteres : scroller des SUR-CLASSES
;;; il faut repositionner l'arbre sur la nouvelle classe. Un test particulier est
;;; effectue pour savoir si l'on peut se repositionner sur la classe 'objet

(de :action-sm-surclasse (sm)
    (let
         (
          (classe-sel-sym   (#:ivan:ed-util:symbole (send 'get-selected-string sm)))
          noeud-classe
          )
         (when classe-sel-sym
	       ; chercher le noeud
	       (setq noeud-classe (getprop classe-sel-sym 'noeud))
               (cond
                   ((and (equal classe-sel-sym 'objet)
                        (null noeud-classe)
                        (not #:ivan:in-global:bobjet) ) ; la surclasse est objet
                    (:action-sa-warn
                                  sm nil t
                                  (list
                                        (font #:ivan:in-global:fstd #M#:txt-const:obj)))
                    (send 'set-selected-string sm "")
                   )
		   ((null noeud-classe)	; la classe selectionnee fait partie d'un
					; noeud ensembliste sous sa surclasse
		    (:actualiser-avec #:ivan:appli classe-sel-sym ())
		    (:reinit-browser sm (getprop classe-sel-sym 'noeud) classe-sel-sym)
		   )
                   ; sinon, on doit se repositionner
		   (t
                    ; reinitialiser
                    (:reinit-browser sm noeud-classe classe-sel-sym)
                   ))
               )
         )
    )

;;;*****************************************************************************************
;;; action du menu de chaines de caracteres : scroller des SOUS-CLASSES

(de :action-sm-sousclasse (sm)
    (let
         (
          (classe-sel-sym   (#:ivan:ed-util:symbole (send 'get-selected-string sm)))
          noeud-classe
          )
         (when classe-sel-sym
               ; chercher le noeud
               (setq noeud-classe (getprop classe-sel-sym 'noeud))
               ; reinitialiser
	       (ifn (typep noeud-classe '{node}:in-noeud2)
		(:reinit-browser sm noeud-classe classe-sel-sym)
	       	(:actualiser-avec #:ivan:appli classe-sel-sym ())
               ))
         )
    )

;;;*****************************************************************************************
;;; action du menu de chaines de caracteres : scroller des INSTANCES
;;; il faut appeler l'editeur d'instances

(de :action-sm-instance (sm)
    (let
         (
          ; nom de l'instance
          (valeur (send 'get-selected-string sm))
          ; je recupere la liste des classes d'attributs selectionnes
          (pour-classes (send 'get-selected-strings
                              (component 'sm-classattr sm)))
          nom-inst
          nom-classe-ch
          nom-classe-sym
          )
         (ifn valeur
              ; alors rien : cas de la reinitialisation du sm
              't
              ; sinon
              (setq
                    
                    nom-inst (#:ivan:ed-util:symbole valeur) 
                    ; je recupere le nom de la classe
                    nom-classe-ch (send 'get-line 
                                        (component 'edl-classe sm))
                    nom-classe-sym (#:ivan:ed-util:symbole nom-classe-ch)
                    
                    )
              ; appeler l'editeur d'instance
              (#:ivan:ed-editer:ei
                      nom-classe-sym
                      nom-inst
                      pour-classes 
                      nil
                      'modsup ; bappel est en mode modification/suppression
                      )
              ; reinitialiser le selecteur = reafficher la liste des instances car
              ; l'instance a peut etre ete supprime ou on a pu creer des instances
              ; a partir de l'editeur d'attribut, declencher l'action de l'ed. de ligne
              (:reinit-edl-classe (component 'edl-classe sm) nom-classe-ch)
              )
         )
    )

;;;*****************************************************************************************
;;; action du menu de chaines de caracteres : scroller des ATTRIBUTS
;;; ouvre une fenetre d'information sur cet attribut.
;;; La fonction (#:ivan:in-infatt:edit renvoie nil (pas de repositionnement) ou la
;;; liste (concept classe) dans le cas d'un objet complexe. Il faut alors changer
;;; si c'est necessaire le concept, puis repositionner l'arbre sur la classe

(de :action-sm-attribut (sm)
    (lets
          (
           (nom-attr      (send 'get-selected-string sm))
           (nom-attr-sym  (#:ivan:ed-util:symbole nom-attr))
           (nom-classe    (send 'get-line (component 'edl-classe sm)))
           list-retour
           pos-retour
           classe-retour
           concept-retour
           (bclasse t)   ; booleen de positionnement de la classe 
           )
          (ifn nom-attr-sym
               ; alors rien (cas de la reinitialisation du sm)
               't
               ; sinon
               ; appeler la fonction d'information sur l'attribut
               (setq list-retour
                     (#:ivan:in-infatt:edit 
                             nom-classe 
                             nom-attr
                             ({in-interf}:ch-vect-classes (component 'top sm))
                             )
                     )
               
               ; reinitialiser le selecteur
               (send 'set-selected-string sm "")
               ; repositionner l'arbre ?
               (when list-retour
                     (setq
                           pos-retour     (car  list-retour) 
                           concept-retour (nth  pos-retour ({in-interf}:ch-list-concepts
                                                                   (component 'top sm) ) ) 
                           classe-retour  (cadr list-retour) 
                           )
                     
                     ; changer de concept ?
                     (when (not (equal concept-retour 
                                       ({in-interf}:ch-concept (component 'top sm)) ))
                           (if
                               (confirm
                                       (column 
                                               (font #:ivan:in-global:fstdb 
							#M#:txt-const:attention)
                                               (row 
                                                    (font #:ivan:in-global:fstd
							#M#:txt-const:pour-sel-cl)
                                                    (font #:ivan:in-global:fstdb
                                                          (string classe-retour))
                                                    (font #:ivan:in-global:fstd ",") )
                                               (row 
                                                    (font #:ivan:in-global:fstd
							#M#:txt-const:change-cpt)
                                                    (font #:ivan:in-global:fstdb
                                                          (string concept-retour) )
                                                    (font #:ivan:in-global:fstd
                                                          " !") )
                                               )
                                       )
                               
                               ; ALORS je declenche l'action du bouton de menu
                               (:action-menu-concept 
                                       (component 'top sm)
                                       pos-retour
                                       t)
                               ; SINON je ne dois pas changer de classe
                               (setq bclasse nil)
                               )
                           )
                     (when bclasse
                           ; repositionner
                           (:reinit-browser sm (getprop classe-retour 'noeud) classe-retour)
                           )
                     )
               )
          )
    )

;;;*****************************************************************************************
;;; reinitialiser le scroller des attributs visualises

(de :action-sm-classattr (sm)
    (lets
          (
           (pour-classes (send 'get-selected-strings sm) )
           
           (nom-classe-sym
                   (#:ivan:ed-util:symbole 
                           (send 'get-line (component 'edl-classe sm))))
           
           (sauf-cl      (if #:ivan:ed-global:%battribut-shirka
                             (cons 'attribut-shirka nil))   )
           (sauf-classes (if #:ivan:ed-global:%bmeta-attribut
                             (cons 'meta-attribut sauf-cl)) )
           list-attr 
           )
          (when nom-classe-sym
                ; s'il y a une classe, la liste des attributs doit etre la meme
                ; que dans l'editeur d'instances : si pas de classes d'attributs
                ; alors tri alphabetique, sinon tri de chacune des listes des
                ; attributs des classes d'attributs
                (setq
                      ; fonction Shirka : (:... classe pour-classes sauf-classes)
                      list-attr
                      (if pour-classes
                          (apply 'append
                                 (mapcar (lambda (classattr)
                                                 (let
                                                      ( (li (#:shirka:noyau:l-nom-att-he-class
                                                                    (#:shirka:noyau:schema nom-classe-sym)
                                                                    (list (#:ivan:ed-util:symbole classattr))
                                                                    nil))
                                                       )
                                                      (if #:ivan:ed-global:attralpha
                                                          (sortl li)
                                                          li)
                                                      )
                                                 )
                                         pour-classes
                                         )
                                 )
                          ; sinon
                          (if #:ivan:ed-global:attralpha
                              (sortl
                                     (#:shirka:noyau:l-nom-att-he-class
                                             (#:shirka:noyau:schema nom-classe-sym)
                                             nil
                                             sauf-classes
                                             ) )
                              (#:shirka:noyau:l-nom-att-he-class
                                      (#:shirka:noyau:schema nom-classe-sym)
                                      nil
                                      sauf-classes
                                      )
                              )
                          )
                      )
                )
          ; reinitialiser le scroller
          (:reinit-sm
                  (component 'scroll-attrvisu sm)
                  (component 'sm-attrvisu sm)
                  list-attr)
          )
    )

;;;*****************************************************************************************
;;; reinitialiser le scroller des attributs visualises

(de :action-sm-attrvisu (sm)
    (lets
          (
           ; nom de l'attribut
           (valeur (send 'get-selected-string sm))
           )
          (when valeur
                (send 'set-selected-string sm "")
                )
          )
    )

;;;*****************************************************************************************
;;; bouton interrogation

(de :action-bout-interro (b)
    (if (componentp 'requete b)
        ; il faut mettre la fenetre des requetes a l'avant plan
        (pop-application (component 'requete b))
        ; sinon il faut creer la fenetre pour la premiere fois
        (#:ivan:re-interf:interface (component 'top b))
        )
    )

;;;*****************************************************************************************
;;; bouton redessiner

(de :action-bout-prettytree (b)
    (let
         (
          (ed-arbre   (component 'ed-arbre b))
          )
         (send 'pretty ed-arbre)
         )
    )

;;;*****************************************************************************************
;;; bouton taille standard

(de :action-bout-standardtree (b)
    (unless (inhibitedp b)
            ; les variables sont initialisees pour la construction de nouv-ed-arbre
            (setq #:ivan:in-global:bminitree nil) 
            (setq #:ivan:in-global:breduct1  nil) 
            (setq #:ivan:in-global:breduct2  nil)
            
            (:redessiner-arbre b)
            )
    )

;;;*****************************************************************************************
;;; bouton taille reduite = construction des noeuds avec un texte sans les cadres

(de :action-bout-reduct1 (b)
    (unless (inhibitedp b)
            ; le bouton taille reduite est activable
            (setq #:ivan:in-global:bminitree nil)
            (setq #:ivan:in-global:breduct1  t)
            (setq #:ivan:in-global:breduct2  nil)
            
            (:redessiner-arbre b)
            )
    )

;;;*****************************************************************************************
;;; bouton taille reduite = construction des noeuds dans un petit rectangle

(de :action-bout-reduct2 (b)
    (unless (inhibitedp b)
            ; le bouton taille reduite est activable
            (setq #:ivan:in-global:bminitree nil)
            (setq #:ivan:in-global:breduct1  nil)
            (setq #:ivan:in-global:breduct2  t)
            ;
            (:redessiner-arbre b)
            )
    )

;;;*****************************************************************************************
;;; bouton minitree = c'est une bascule

(de :action-bout-minitree (b)
    (lets
          (
           (ed-arbre   (component 'ed-arbre b))
           )
          
          (:inhiber-com b)
          
          (if #:ivan:in-global:bminitree
              ; bminitree est deja vrai = reinitialiser l'arbre
              (progn
                     (send 'standardtree ed-arbre)
                     (setq #:ivan:in-global:bminitree nil)
                     (:autoriser-com b)
                     )
              
              ; bminitree est faux = faire un minitree de l'arbre 
              
              (send 'minitree ed-arbre)
              (setq #:ivan:in-global:bminitree t)
              (authorize-application (component 'bout-minitree  b))
              (authorize-application (component 'bout-quitter   b))
              )
          
          )
    )

;;;*****************************************************************************************
;;; repositionne l'arbre sur la classe selectionnee

(de :action-bout-aller (b)
    (lets
          (
           (list-classes ({in-interf}:ch-list-classes (component 'top b)))
           (nbclasses    (length list-classes))
           (classe-sel-ch (ask-string
                                  (column
                                          (font #:ivan:in-global:fstdb #M#:txt-const:sel-cla)
                                          ""
                                          (row
                                               (font #:ivan:in-global:fstd #M#:txt-const:arb)
                                               (font #:ivan:in-global:fstdb (string nbclasses))
                                               (font #:ivan:in-global:fstd " classes.") )
                                          ""
                                          (font #:ivan:in-global:fstd #M#:txt-const:sel-cl-rep)
                                          )
                                  (#:ivan:ed-util:list-alpha list-classes)
                                  nil
                                  ))
           (classe-sel-sym (#:ivan:ed-util:symbole classe-sel-ch))
           
           )
          (when classe-sel-sym
                ; reinitialiser
                (:reinit-browser b  (getprop classe-sel-sym 'noeud) classe-sel-sym)
                )
          )
    )

;;;*****************************************************************************************
;;; repositionne l'arbre sur le noeud courant

(de :action-bout-repos (b)
 (let ((noeud-courant (send 'get-current (component 'ed-arbre b))))
  (when noeud-courant
    (lets
          (
           (image-noeud   (send 'editordata noeud-courant))
           (x             (send 'x image-noeud))
           (y             (:calc-y noeud-courant b))
           (w             (send 'width  image-noeud))
           (h             (send 'height image-noeud))
           )
          
          (send 'ensure-visible (component 'scroll-arbre b) x y)
          ; le point de coordonnee inferieur droit doit aussi etre visible
          (send 'ensure-visible (component 'scroll-arbre b) (+ x w) (+ y h) )
          
          (send 'pretty (component 'ed-arbre b))
          )
    )))

;;;*****************************************************************************************
;;; bouton d'appel a l'hypertexte 

(de :action-bout-hyper (b)
    (let
         (
          (nom-classe-ch  (send 'get-line (component 'edl-classe b)))
          )
         ; deplacement sur la page
         (hy-move-to #:ivan:hy-global:tv nom-classe-ch)
         )
    )

;;;*****************************************************************************************
;;; action de la case a cocher du scroller des attributs
;;; si elle est vrai, il ne faut visualiser que les attributs propres de la classe

(de :action-cb-locaux (cb)
    (lets
          (
           (nom-classe-ch  (send 'get-line (component 'edl-classe cb)))
           (nom-classe-sym (#:ivan:ed-util:symbole nom-classe-ch))
           (list-attribut (if nom-classe-sym
                              (:list-attribut cb)
                              nil))
           )
          (:reinit-sm
                  (component 'scroll-attribut cb)
                  (component 'sm-attribut     cb)
                  list-attribut)
          )
    )

;;;*****************************************************************************************
;;; creation d'une nouvelle instance ; l'action est similaire a l'action du sm des
;;; sm des instances

(de :action-bout-nouvinst (b)
    (lets
          (
           ; je recupere la liste des classes d'attributs selectionnes
           (pour-classes (send 'get-selected-strings
                               (component 'sm-classattr b)))
           ; je recupere le nom de la classe
           (nom-classe-ch (send 'get-line 
                                (component 'edl-classe b)))
           (nom-classe-sym (#:ivan:ed-util:symbole nom-classe-ch))
           schema-clas
           inst
           )
          (when nom-classe-sym
                
                (setq
                      schema-clas (#:ivan:ed-util:schema nom-classe-sym)
                      ; appeler l'editeur d'instance
                      inst        (#:ivan:ed-editer:ei
                                          nom-classe-sym
                                          nil
                                          pour-classes
                                          nil
                                          nil ; bappel est a nil /cas de la creation/
                                          ) 
                      )
                ; reinitialiser le selecteur = reafficher la liste des instances
                ; je declenche l'action de l'editeur de ligne
                (when inst
                      (:reinit-edl-classe (component 'edl-classe b) nom-classe-ch)
                      )
                )
          )
    )

;;;*****************************************************************************************
;;; action du bouton classer = interface de classification

(de :action-bout-classer (b)
    (when (and (componentp 'iace b) (activatedp (component 'iace b)) ) 
          ; il faut mettre la fenetre de classif a l'avant plan
          (pop-application (component 'iace b))
          )
    )

;;;*****************************************************************************************

;;;(de :action-bout-quitter (b)
;;;     action du bouton quitter = fermer l'application apres une confirmation
;;;
;;;    si       Hypertexte => voir ivan-in-interf+hy.ll
;;;    si pas d'Hypertexte => voir ivan-in-interf-hy.ll

;;;*****************************************************************************************
;;; action du bouton option

(de :action-bout-option (b)
    (lets
          (
           (bout-daccord (standardbutton #M#:txt-const:retour-ivan))
           (cb-attralpha (checkbox 0 0 #50wc #hc
                                 (font #:ivan:in-global:fstd #M#:txt-const:ordre-alpha)))
           (cb-bobjet    (checkbox 0 0 #50wc #hc
                                 (font #:ivan:in-global:fstd #M#:txt-const:visu-objet)))
           (option
                   (header
                           (icon 0 0 ic-artemis)
			   #M#:txt-const:opts
                           bout-daccord
                           (view)
                           (column 
                                   ""
                                   cb-attralpha
                                   ""
                                   cb-bobjet
                                   ""
                                   )
                           t
                           t))
           )
          (set-action bout-daccord ':action-bout-daccord)
          (set-action cb-attralpha ':action-cb-attralpha)
          (set-action cb-bobjet    ':action-cb-bobjet)
          
          ; la case a cocher doit avoir la valeur du booleen
          (send 'checked cb-attralpha #:ivan:ed-global:attralpha)
          
          ; la case a cocher doit avoir la valeur du booleen
          (send 'checked cb-bobjet #:ivan:in-global:bobjet)
          
          (grab-application (add-application option))
          )
    )

;;;*****************************************************************************************
;;; action du bouton d'accord = fermer l'application

(de :action-bout-daccord (b)
    (remove-application (send 'father b) )
    )

;;;*****************************************************************************************
;;; action de la case a cocher : attribut par ordre alphabetique
;;; ATTENTION : entorse au mode de programmation en utilisant
;;;             directement la variable #:ivan:appli

(de :action-cb-attralpha (cb)
    (setq #:ivan:ed-global:attralpha (send 'checkedp cb))
    ; je reinitialise le navigateur
    (:reinit-edl-classe (component 'edl-classe #:ivan:appli) 
            (send 'get-line (component 'edl-classe  #:ivan:appli)))
    
    )

;;;*****************************************************************************************
;;; action de la case a cocher : visualisation a partir de la classe 'objet

(de :action-cb-bobjet (cb)
    (setq #:ivan:in-global:bobjet (send 'checkedp cb))
    )

;;;*****************************************************************************************
;;; action du bouton ivan = afficher une fenetre d'information

(de :action-bout-ivan (b)
    (let
         (
          (list-doc 
                  (list
                        (font #:ivan:in-global:fstdb #M#:txt-const:expl-ivan)
                        ""
                        (row    
                             "      "
                             (font #:ivan:in-global:fstdb "I")
                             (font #:ivan:in-global:fstd  "nterface de") )
                        (row    
                             "      "
                             (font #:ivan:in-global:fstdb "V")
                             (font #:ivan:in-global:fstd  "isualisation et d'") )
                        (row    
                             "      "
                             (font #:ivan:in-global:fstdb "A")
                             (font #:ivan:in-global:fstd  "ide a la") )
                        (row    
                             "      "
                             (font #:ivan:in-global:fstdb "N")
                             (font #:ivan:in-global:fstd  "avigation") )
                        ""
                        (row    
                             "(C) Projet Sherpa"
                             (font #:ivan:in-global:fmicro "(*)")
                             "-INRIA, 1991" )
                        ""
                        (row    
                             (font #:ivan:in-global:fstd #M#:txt-const:vers-util-1)
                             (font #:ivan:in-global:fstdb (string #:ivan:in-global:version))
                             (font #:ivan:in-global:fstd #M#:txt-const:vers-util-2)
                             )
                        ""
                        (font #:ivan:in-global:fstdb #M#:txt-const:auteur)
                        ""
                        (font #:ivan:in-global:fstd #M#:txt-const:toute-dem-1)
                        (font #:ivan:in-global:fstd #M#:txt-const:toute-dem-2)
                        (font #:ivan:in-global:fstd #M#:txt-const:toute-dem-3)
                        ""
                        (font #:ivan:in-global:fstd "Contact :")
                        ""
                        (font #:ivan:in-global:fstdb "Francois RECHENMANN")
                        (font #:ivan:in-global:fstd "INRIA Rhone-Alpes")
                        (font #:ivan:in-global:fstd
                              "     LIFIA / IMAG, 46 avenue Flix Viallet")
                        (font #:ivan:in-global:fstd
                              "     38031 GRENOBLE Cedex 1")
                        (font #:ivan:in-global:fstdb
                              "     Tel    : (33) 76 57 45 63")
                        (font #:ivan:in-global:fstdb
                              "     e_mail : Francois.Rechenmann@imag.fr")
                        (font #:ivan:in-global:fstdb
                              "-----------------------------------------------------")
                        (font #:ivan:in-global:fmicro
                              "(*) Accord INRIA, CNRS, UJF (Grenoble 1), INPG")
                        ))
          )
         (:affichage-doc list-doc)
         )
    )

;;;*****************************************************************************************
;;; boucle jusqu'a ce que l'on clique sur le bouton 'annuler'
;;; permet de modifier le repertoire courant (variable #:shirka:*bases*)

(de :action-bout-chemin (b)
    (lets
          (
           (edl-chemin (lineedit 0 0 (+ #5wc (send 'width #:shirka:*bases*))
                               #:shirka:*bases*))
           (img-chemin
                   (#:ivan:ed-util:boxedimage 
                           (row 
                                (font #:ivan:in-global:fstd #M#:txt-const:chemin)
                                edl-chemin
                                )))
           (list-dir 
                   (car (#:systeme:unix:catalogue (pathname #:shirka:*bases*))))
           (nom-chemin t)
           )
          (set-action edl-chemin 
                  (lambda (edl) 
                          (setq #:shirka:*bases* (send 'get-line edl))
                          ))
          (while nom-chemin
                 (setq edl-chemin (lineedit 0 0 (+ #5wc (send 'width #:shirka:*bases*)) #:shirka:*bases*) 
                       img-chemin (#:ivan:ed-util:boxedimage (row (font #:ivan:in-global:fstd 
									 #M#:txt-const:chemin)
                                                                  edl-chemin
                                                                  )
                                          )
                       list-dir 
                       (car (#:systeme:unix:catalogue (pathname #:shirka:*bases*)))
                       nom-chemin (ask-string (column (font #:ivan:in-global:fstdb 
							#M#:txt-const:changer-rep)
                                                      ""
                                                      img-chemin
                                                      ""
                                                      (font #:ivan:in-global:fstd
							#M#:txt-const:pos-rep)
                                                      )
                                          list-dir
                                          t
                                          )
                       )  
                 (cond ((null nom-chemin) nil)
                       ((equal nom-chemin "..") (#:systeme:unix:aller-dans nom-chemin)
                        (if (equal (plength (current-directory)) 2)
                            (setq #:shirka:*bases* "/")
                            (setq #:shirka:*bases* (current-directory))
                            )
                        )
                       ( t (catenate #:shirka:*bases* nom-chemin "/")
                          (#:systeme:unix:aller-dans (pathname (catenate #:shirka:*bases* nom-chemin "/")))
                          (setq #:shirka:*bases* (current-directory))
                          )
                       )
                 )
          )
    )

;;;*****************************************************************************************
;;; action du bouton verifier = ouvrir une fenetre pour saisir le nom de la base a verifier
;;; appelle le selectionneur de fichiers dans la nouvelle version aida (1.65)

(de :action-bout-verifier (b)
    (lets
          (
           (path-base (ask-file (pathname #:shirka:*bases*)))
           (nom (pathname-name path-base))
           (nbel (length ({in-interf}:ch-list-concepts (component 'top b))))
           )
          (when nom
                ; inhiber toutes les commandes
                (:inhiber-com b)
                ; verifier la base choisie
(when #:trace:if
                (print) (print "Verifier la base: " path-base)
)
                (:verifier-fic path-base nom b) 
                ; messages de fin d'analyse
                (if (> #:shirka:verif:nb-errors 0)
                    (progn
                           ; message d'erreur
                           (:action-sa-warn
                                   b nil t
                                   (list
                                         (row
                                              (font #:ivan:in-global:fstdb #M#:txt-const:erreurs)
                                              (font #:ivan:in-global:fstd
                                                (catenate (string #:shirka:verif:nb-errors) "."))
                                              (font #:ivan:in-global:fstd #M#:txt-const:le-fic)
                                              (font #:ivan:in-global:fstdb #M#:txt-const:nom-fic)
                                              (font #:ivan:in-global:fstd #M#:txt-const:lig-err))))
                           )
                    ; sinon message de fin 
                    (:action-sa-warn
                            b nil t
                            (list
                                  (row
                                       (font #:ivan:in-global:fstd #M#:txt-const:correct)
                                       (font #:ivan:in-global:fstdb #:shirka:*verif*)
                                       (font #:ivan:in-global:fstdb nom))))
                    ) ; fi
                ) ; end-when
          ; reinitialiser les boutons de commande
          (:autoriser-com (component 'top b))
          (if nom ()
              ; erreur de manip en selection <nom-base>
              (:action-sa-warn
                      b nil t
                      (list
                            (row
                                 (font #:ivan:in-global:fstdb #M#:txt-const:pas-de-base)
                                 (font #:ivan:in-global:fstdb path-base)))))
          )
    )

;;;****************************************************************************************
;;; appel du verificateur syntaxique d'une base
;;; equivaut a la commande "Verifier" dans le mode commande clavier de Shirka

(de :verifier-fic (path-fichier base b)
    (let ((base-source path-fichier)
          (old-margin (rmargin))
          )
         ;
         (protect
                 (tag compile
                      (#:shirka:verif:init-globales)
                      (#:shirka:verif:select-format)
                      ;
                      ; *** le source existe par defaut (interf IVAN):
                      (defvar #:shirka:verif:titre-liste base-source)
                      (defvar #:shirka:verif:ftabul (concat #:shirka:*verif* base ".sh"))
                      ;
                      ; Prelude
                      ; -------
                      ; positionner marge droite format "a l'italienne"
                      (rmargin #:shirka:verif:nbcars-eff)
                      ;
                      (inchan  (openi base-source))
                      (outchan (setq #:shirka:verif:canal-t (openo #:shirka:verif:ftabul)))
                      (outchan (setq #:shirka:verif:canal-l (openo #:shirka:verif:fliste)))
                      ;
                      ; message
                      (:action-sa-warn
                              b t t
                              (list
                                    (row
                                         (font #:ivan:in-global:fstd #M#:txt-const:pat-anal)
                                         (font #:ivan:in-global:fstdb path-fichier))))
                      ;----------------------------
                      (#:shirka:verif:analyse-synt )
                      ;----------------------------
                      )
                 ; Postlude
                 ; --------
                 (close)
                 ; restaurer:
                 (rmargin old-margin)
                 (output ())
                 )
         )
    )

;;;*****************************************************************************************
;;;(de :action-bout-charger (b)
;;;    action du bouton charger = ouvrir une fenetre pour saisir le nom de la base a charger
;;;
;;;    si       Hypertexte => voir "ivan-in-interf+hy.ll"
;;;    si pas d'Hypertexte => voir "ivan-in-interf-hy.ll"

(de :maj-concept-charger (nbel b)
    (let
         (
          i
          )
         ; supprimer les concepts
         (for (i 0 1 (- nbel 1) 'ok)
              (menu-delete-item (component 'menu-concept b) 0 0))
         ; mettre a jour la liste des concepts
         ({in-interf}:ch-list-concepts
                 (component 'top b) 
                 (:const-ch-list-concepts (component 'top b) ))
         ; mettre a jour le vecteur des classes
         ({in-interf}:ch-vect-classes
                 (component 'top b) 
                 (:const-ch-vect-classes (component 'top b) ))
         ; reinitialiser la liste des concepts dans le bouton concept
         (:creer-menu (component 'top b))
         
         )
    )

;;;*****************************************************************************************
;;; action du bouton sauver = ouvrir une fenetre pour saisir concept + classes a sauver

(de :action-bout-sauver (b)
    (lets
          (
           (sm-concepts
                   (stringmenu 
                           (mapcar 'string ({in-interf}:ch-list-concepts (component 'top b)) )))
           (sm-classes  (stringmenu nil) )
           (scroll-concepts 
                   (scroller
                           0 0
                           #20wc #10hc
                           sm-concepts
                           t t) )
           (scroll-classes 
                   (scroller
                           0 0
                           #20wc #10hc
                           sm-classes
                           t t) )
           (clu-concepts
                   (#:ivan:ed-util:clusterimage scroll-concepts "Concept(s)") )
           (clu-classes
                   (#:ivan:ed-util:clusterimage scroll-classes #M#:txt-const:classes))
           
           (titre-toutes 
                   (font #:ivan:in-global:fstd #M#:txt-const:toutes-cl))
           
           (bout-toutes (standardbutton
                                (centeredview
                                        (rectangle 0 0
                                                (send 'width clu-classes) 0)
                                        titre-toutes)))
           
           (edl-chemin (lineedit 0 0
                               (+ #5wc (send 'width #:shirka:*bases*))
                               #:shirka:*bases*) )
           
           (img-concepts
                   (column
                           clu-concepts
                           (rectangle 0 0 0 (send 'height bout-toutes) )))
           
           (img-classes
                   (column
                           clu-classes
                           bout-toutes
                           ))
           
           
           (fleche-droite
                   (centeredview
                           (rectangle 0 0
                                   (send 'width (icon 0 0 ic-arrow-right))
                                   (send 'height img-classes) ) 
                           (icon 0 0 ic-arrow-right)) )
           
           
           (fleche-bas
                   (centeredview
                           (rectangle 0 0
                                   (send 'width img-classes) 0)
                           (icon 0 0 ic-arrow-down) ))
           
           (img-scrollers
                   (row
                        img-concepts
                        fleche-droite
                        (column
                                img-classes
                                fleche-bas)
                        ))
           
           (img-chemin
                   (#:ivan:ed-util:boxedimage
                           (view
                                 (rectangle 0 0
                                         (send 'width img-scrollers) 0)
                                 (row
                                      (font #:ivan:in-global:fstd #M#:txt-const:chemin)
                                      edl-chemin)
                                 )))
           
           nom-base
           nom-fichier
           (bsauver t)
           bfini
           )
          
          (inhibit-application bout-toutes)
          
          (add-component (component 'top b) 'bout-toutes bout-toutes)
          (add-component (component 'top b) 'sm-concepts sm-concepts)
          (add-component (component 'top b) 'scroll-classes scroll-classes)
          (add-component (component 'top b) 'sm-classes  sm-classes)
          
          
          ({application}:userdata
                  sm-concepts
                  (list
                        ({in-interf}:ch-vect-classes (component 'top b)) 
                        (component 'scroll-classes b)
                        (component 'sm-classes b)
                        (component 'bout-toutes b)
                        )
                  )
          ({application}:userdata
                  bout-toutes
                  (component 'sm-classes b)
                  )
          
          (set-action sm-concepts ':action-sm-concepts)
          (set-action bout-toutes  ':action-bout-toutes)
          
          (:inhiber-com b)
          ;------------------------------------------------------------------------
          (while (not bfini)
                 
                 (setq nom-base
                       (ask-string
                               (column                 
                                       (font #:ivan:in-global:fstdb #M#:txt-const:sauve-insts)
                                       ""
                                       img-scrollers
                                       img-chemin
                                       (centeredview
                                               (rectangle 0 0 (send 'width img-chemin) 0)
                                               (icon 0 0 ic-arrow-down))
                                       (font #:ivan:in-global:fstd  #M#:txt-const:sauve-fic)
                                       )
                               (cdr (#:systeme:unix:catalogue (pathname #:shirka:*bases*)))
                               t
                               )
                       )
                 (ifn nom-base
                      
                      ; l'utilisateur a clique sur annuler
                      (setq
                            bfini   t
                            bsauver nil )
                      
                      ; sinon
                      
                      (setq 
                            ; construction du nom du fichier
                            nom-fichier (catenate (send 'get-line edl-chemin) nom-base)  
                            )
                      
                      (when (probefile nom-fichier)
                            (setq bsauver
                                  (confirm
                                          (column 
                                                  (font #:ivan:in-global:fstdb
							#M#:txt-const:attention)
                                                  (row 
                                                       (font #:ivan:in-global:fstd
							(catenate " " #M#:txt-const:le-fic))
                                                       (font #:ivan:in-global:fstdb
                                                             (string nom-base))
                                                       (font #:ivan:in-global:fstd
							#M#:txt-const:ecrase))
                                                  ""
                                                  (row 
                                                       (font #:ivan:in-global:fstd
							#M#:txt-const:clic-on)
                                                       (font #:ivan:in-global:fstdb
							#M#:txt-const:d-accord)
                                                       (font #:ivan:in-global:fstd
							#M#:txt-const:pour-sauver))
                                                  (row 
                                                       (font #:ivan:in-global:fstdb
							#M#:txt-const:annuler)
                                                       (font #:ivan:in-global:fstd
							#M#:txt-const:pour-aband))
                                                  )))
                            ) ; endwhen
                      (when bsauver
                            (if (null (send 'get-selected-strings sm-classes))
                                ; il n'y a pas de classes selectionnees = boucler
                                (:action-sa-warn
                                        b t t
                                        (list
                                              (row
                                                   (font #:ivan:in-global:fstdb
							#M#:txt-const:sauver-imp)
                                                   (font #:ivan:in-global:fstd
							#M#:txt-const:sel-cl-ou-clic)
                                                   (font #:ivan:in-global:fstdb
							#M#:txt-const:annuler)
                                                   (font #:ivan:in-global:fstd
							#M#:txt-const:pour-aband)
                                                   )))
                                ; sinon
                                (setq bfini t) ; sortir de la boucle
                                ; bsauver est deja vrai
                                )
                            ) ; when bsauver
                      ) ; ifn nom-base
                 
                 ) ; endwhile
          ;------------------------------------------------------------------------
          (ifn  bsauver
               (:action-sa-warn
                       b nil nil nil)
               ; sinon 
               (:sauver-fic
                       nom-fichier
                       b
                       (mapcar '#:ivan:ed-util:symbole
                               (send 'get-selected-strings sm-classes) )
                       )
               
               )
          
          (:autoriser-com b) 
          )
    )

;;;*****************************************************************************************
;;; action du sm de choix d'un concept lors d'une sauvegarde

(de :action-sm-concepts (sm)
    (lets
          (
           (concept (send 'get-selected-string sm))
           (pos     (send 'get-selected-rank   sm))
           (list-param     ({application}:userdata sm))
           (vect           (nth 0 list-param) )
           (scroll-classes (nth 1 list-param) )
           (sm-classes     (nth 2 list-param) )
           (bout-toutes    (nth 3 list-param) )
           list-classes 
           )
          (ifn pos
               (inhibit-application bout-toutes)
               ; sinon
               ; liste des classes du concepts
               (setq list-classes  (vref vect pos))
               ; autoriser le bouton de selection
               (authorize-application bout-toutes)
               )
          
          
          ; afficher la liste
          (send 'set-strings 
                sm-classes
                (mapcar 'string (cadr list-classes))
                )
          
          ; toutes les classes peuvent etre choisies
          (send 'selection-limit
                sm-classes
                (length (cadr list-classes))
                )
          
          (send 'reinitialise scroll-classes)
          
          )
    )

;;;*****************************************************************************************
;;; bouton de selection de toutes les classes pour une sauvegarde

(de :action-bout-toutes (b)
    (let
         (
          (sm-classes ({application}:userdata b))
          (list-classes (send 'get-strings sm-classes))
          )
         ; tout selectionner
         (send 'set-selected-strings sm-classes list-classes)
         )
    )
    
;;;#########################################################################################
;;;#########################################################################################
;;;                                  
;;;                                      FONCTIONS INTERNES
;;;
;;;#########################################################################################
;;;#########################################################################################

;;;*****************************************************************************************
;;; renvoie l'ordonnee de l'image du noeud selectionne

(de :calc-y (noeud-classe c)
    (let
         (
          (y (send 'y (send 'editordata noeud-classe)))
          )
         (when (<= (- (send 'height (component 'scroll-arbre c)) y)
                   {scroller}:scrollbarheight )
               (setq y (+ y {scroller}:scrollbarheight))
               )
         y
         )
    )

;;;*****************************************************************************************
;;; liste de toutes les specialisations directes de "schema"
;;; sans les filtres ou attachements proceduraux
;;; ni les specialisations locales

(de :l-spec-vraies (schema)
    (prog1 
           (:l-spec-vraies1 schema)
           (#:shirka:noyau:demarquer-fils schema)
           )
    )

;;;*****************************************************************************************

(de :l-spec-vraies1 (schema)
    (mapcan '(lambda (spec)
                     (unless (or (getprop spec 'est-un-filtre)
                                 (getprop spec 'spec-loc)
                                 (getprop spec 'traite)
                                 )
                             (putprop spec t 'traite)
                             ; construire la liste resultat
                             (list spec)
                             )
                     )
            (#:shirka:noyau:l-spec schema)
            )
    )

;;;*****************************************************************************************
;;; Fonction qui renvoie la hierarchie des sous-classes de "classe-mere".
;;; La convention suivante est adoptee :
;;; La liste (p f1 (f2 pf1 (pf2 apf1 apf2) pf3) (f3 pf4)) est la representation de :
;;;                               p
;;;                               |
;;;               |---------------|---------------|
;;;               f1              f2              f3
;;;                               |               |
;;;                    |----------|----------|    |
;;;                    pf1        pf2        pf3  pf4
;;;                               |
;;;                         |-----|-----|
;;;                         apf1        apf2

(de :list-hierarchie (classe-mere)
    (let
         (
          ; liste des sous-classe directes
          (list-classe-mere 
                  (sortl (:l-spec-vraies (#:shirka:noyau:schema classe-mere))))
          ; liste resultats
          (list-resultat    (list classe-mere))
          )
         (while list-classe-mere
                ; parcours des sous-classes
                (lets
                      (
                       (sous-classe (car list-classe-mere))
                       (list-sous-classe-trav (:list-hierarchie sous-classe))
                       (list-sous-classe (if (= (length list-sous-classe-trav) 1)
                                             list-sous-classe-trav
                                             (list list-sous-classe-trav) ))
                       )
                      ; inserer le resultat
                      (setq list-resultat (append list-resultat list-sous-classe))
                      (nextl list-classe-mere)
                      )
                )
         list-resultat
         )
    )

;;;*****************************************************************************************
;;; renvoie la liste des attributs de la classe selectionnee
;;; si la case a cocher est vrai, seuls les attributs locaux a la classe sont affiches

(de :list-attribut (c)
    (lets
          (
           (etat (send 'checkedp (component 'cb-locaux c)))
           
           (nom-classe-ch  (send 'get-line (component 'edl-classe c)))
           (nom-classe-sym (#:ivan:ed-util:symbole nom-classe-ch))
           list-attribut
           schema-clas
           l-attr
           )
          (when nom-classe-sym
                (setq
                      schema-clas     (#:shirka:noyau:schema            nom-classe-sym)
                      list-attribut   (if etat
                                          ; attributs propres de la classe
                                          (#:shirka:noyau:l-nom-att schema-clas)
                                          ; sinon tous
                                          (#:shirka:noyau:l-nom-att-he-class schema-clas nil nil))
                      ))
          (when (member 'util-par list-attribut)
                (setq list-attribut (#:ivan:ed-util:diff-liste
                                            list-attribut
                                            '(util-par))
                      l-attr (cons 'util-par l-attr)
                      ))
          (when (member 'nom-inst list-attribut)
                (setq list-attribut (#:ivan:ed-util:diff-liste
                                            list-attribut
                                            '(nom-inst))
                      l-attr (cons 'nom-inst l-attr)
                      ))
          (when (member 'est-un list-attribut)
                (setq list-attribut (#:ivan:ed-util:diff-liste
                                            list-attribut
                                            '(est-un))
                      l-attr (cons 'est-un l-attr)
                      ))
          ; les attributs sont-ils par ordre alphabetique ?
          (when #:ivan:ed-global:attralpha
                (setq list-attribut (sortl list-attribut)) )
          
          ; je renvoie la liste finale
          (append list-attribut l-attr)
          )
    )

;;;*****************************************************************************************
;;; liste des classes d'attributs definies pour les attributs de la classe selectionnee

(de :list-classattr (nom-classe)
    (let 
         (
          (schema (#:shirka:noyau:schema nom-classe)) 
          list
          l-attr                                           
          )                                                                                  
         (mapc
               '(lambda (att)
                        (let
                             (
                              (classe-att (#:shirka:noyau:classe-att-he schema att)) 
                              )                                                                    
                             (ifn (member classe-att list)
                                  (setq list (cons classe-att list))
                                  )
                             )                                                                      
                        )
               (#:shirka:noyau:l-nom-att-he schema)
               )
         (when (member 'attribut list)
               (setq list (#:ivan:ed-util:diff-liste
                                  list
                                  '(attribut))
                     l-attr (cons 'attribut l-attr)
                     ))
         (when (member 'attribut-shirka-est-un list)
               (setq list (#:ivan:ed-util:diff-liste
                                  list
                                  '(attribut-shirka-est-un))
                     l-attr (cons 'attribut-shirka-est-un l-attr)
                     ))
         (when (member 'attribut-shirka-sauf-est-un list)
               (setq list (#:ivan:ed-util:diff-liste
                                  list
                                  '(attribut-shirka-sauf-est-un))
                     l-attr (cons 'attribut-shirka-sauf-est-un l-attr)
                     ))
         ; je renvoie la liste finale
         (append 
                 (sortl list)
                 (reverse l-attr)  )
         )     
    )

;;;*****************************************************************************************
;;; affichage dans une fenetre de la documentation contenue dans la liste <list>

(de :affichage-doc (list)
    (lets
          (
           (bout-daccord  (standardbutton #M#:txt-const:retour-ivan))
           (doc
                (header
                        (icon 0 0 ic-artemis)
                        #M#:txt-const:doc
                        bout-daccord
                        (view)
                        (apply 'column list)
                        t
                        t))
           )
          (set-action bout-daccord ':action-bout-daccord)
          
          (add-application doc)
          )
    )

;;;*****************************************************************************************
;;; Lors du clic sur les boutons de changement de taille de la representation, cette 
;;; fonction est declenchee pour representer l'arbre sous sa nouvelle forme.

(de :redessiner-arbre (b)
    ; inhiber les boutons de commande
    (:inhiber-com b)
    ; message
    (:action-sa-warn
            b t nil
            (list
                  (row
                       (font #:ivan:in-global:fstd #M#:txt-const:pat-mod-gr))))
    ; changer la forme des noeuds = c'est fait par la methode 'title (cf la structure
    ; des noeuds /fichier ivan-in-struct.ll/
    (send 'mapnodes (component 'ed-arbre b)
          (lambda (node)
                  (send 'set-image 
                        (send 'editordata node)
                        (send 'title node)
                        )
                  )
          )
    (send 'standardtree (component 'ed-arbre b))
    
    ; message de fin
    (:action-sa-warn
            b nil nil
            (list
                  (row
                       (font #:ivan:in-global:fstd #M#:txt-const:sel-cl))))
    ; autoriser les commandes
    (:autoriser-com b)
    )

;;;*****************************************************************************************
;;; chargement de schemas inscrits dans un fichier
;;; repris de FR-fcts-commandes.ll

(de :charger-fic (nom-fichier b)
    (if (probefile nom-fichier)
        (progn
               ; message
               (:action-sa-warn
                       b t t
                       (list
                             (row
                                  (font #:ivan:in-global:fstd #M#:txt-const:pat-charg-fic)
                                  (font #:ivan:in-global:fstdb nom-fichier))))
               (with ((inchan (openi nom-fichier))
                      (typech '|{| 'cmsymb)
                      (typech '|}| 'cmsymb)
                      (typech '|&| 'cmsymb)
                      (typech '|[| 'cmsymb)
                      (typech '|]| 'cmsymb)
                      (typech '|;| 'cmsymb))
                     
                     (:impr-res
                             (tag erreur-descr
                                  (untilexit eof
                                          (let ((caract (read)) )
                                               (setq %deja-compile ())
                                               (if (consp caract) ;lecture d'une fonction Lisp
                                                   (eval caract)
                                                   ; on evalue le schema shirka:
                                                   (setq un-schema (#:shirka:top:trad-sch caract () ))
                                                   (if un-schema
                                                       ; le schema existe (ce n'est pas du commentaire)
                                                       (def-sh un-schema)
                                                       )
                                                   )
                                               )
                                          )
                                  nom-fichier
                                  )
                             )
                     )
               )
        ; sinon
(when #:trace:if
        (print "Erreur : la base " nom-fichier " n'existe pas.")
)
        (#:shirka:top:impr-res '())
        nil
        )
    )

;;;*****************************************************************************************

(de :impr-res (valeur)
    (let ( (bok t))
         (prin "-> ")
         (ifn valeur
              (progn
                     (prin "chec")
                     (setq bok nil)
                     )
              ; sinon
              (if (consp valeur)
                  ; alors
                  (mapc '(lambda (val)
                                 (prin val " ")
                                 )
                        valeur)
                  ; sinon
                  (if (vectorp valeur)
                      (prin (#:shirka:noyau:nom-sch valeur))
                      (prin valeur)
                      )
                  )
              )
         (terpri)
         bok
         )
    )

;;;*****************************************************************************************
;;; sauvegarde des schemas d'instances dans un fichier
;;; repris de FR-fct-commandes.ll

(de :sauver-fic (nom-fichier b l-nom-classe)
(when #:trace:if
    (print) (print "Sauver-instances")
    (print nom-fichier)
    (print l-nom-classe)
)
    (:action-sa-warn
            b t t
            (list
                  (row
                       (font #:ivan:in-global:fstd #M#:txt-const:pat-sauve)
                       (font #:ivan:in-global:fstdb nom-fichier))))

    (with ((outchan (openo nom-fichier)))
          (protect 
                  (progn 
                         (mapc '(lambda (nom-classe)
                                        (if (getprop nom-classe 'schema)
                                            (mapc '(lambda (nom-sch)
                                                           (#:shirka:vi:visualiser
                                                                   (#:shirka:noyau:schema nom-sch)
                                                                   nil nil
                                                                   0 nil)
                                                           (terpri)
                                                           )
                                                  (#:shirka:noyau:l-inst 
                                                          (#:shirka:noyau:schema nom-classe))
                                                  )
                                            )
                                        )
                               l-nom-classe
                               )
                         (terpri)
                         )
                  (close (outchan))
                  )
          ) ; with
    (:action-sa-warn
            b nil t
            (list
                  (row
                       (font #:ivan:in-global:fstd #M#:txt-const:fait))))
    )

;;;*****************************************************************************************
;;; initialiser l'etat de Ivan quand des bases on ete chargees en Shirka (sans passer par IVAN)
;;  (active en particulier le bouton qui permet de choisir un concept)

(de :initialise-boutons ()
    (let ((menu-concept (create-menu #M#:txt-const:l-cpts))
	  (old-menu-concept (component 'menu-concept #:ivan:appli))
	 )
         ; mettre a jour la liste des concepts
         ({in-interf}:ch-list-concepts
                 #:ivan:appli
                 (:const-ch-list-concepts #:ivan:appli))
         ; mettre a jour le vecteur des classes
         ({in-interf}:ch-vect-classes
                 #:ivan:appli
                 (:const-ch-vect-classes #:ivan:appli))
         ; reinitialiser la liste des concepts dans le bouton concept
	 (kill-menu old-menu-concept)
	 (set-component #:ivan:appli 'menu-concept menu-concept)
	 (send 'menu (component 'bout-concept #:ivan:appli) menu-concept)
         (:creer-menu #:ivan:appli)
         (:autoriser-com #:ivan:appli)
))

;;;*****************************************************************************************
;;; autoriser les boutons de commandes

(de :autoriser-com (a)
    (let
         (
          (appli (component 'top a))
          )
         (if ({in-interf}:ch-list-concepts appli)
             ; il y a des concepts : les commandes doivent etre autorisees
             (if (#:ivan:ed-util:symbole ({in-interf}:ch-concept appli))
                 ; il y a un concept de charge
                 (progn
                        (authorize-application (component 'bout-quitter      appli))
                        (authorize-application (component 'bout-chemin       appli))
                        (authorize-application (component 'bout-charger      appli))
                        (authorize-application (component 'bout-verifier     appli))
                        (authorize-application (component 'bout-sauver       appli))
                        (authorize-application (component 'bout-concept      appli))
                        (when (#:ivan:ed-util:symbole
                                      (send 'get-line (component 'edl-classe appli)))
                              (authorize-application (component 'bout-interro      appli))
                              )
                        (authorize-application (component 'bout-prettytree   appli))
                        (if #:ivan:in-global:bminitree
                            (authorize-application (component 'bout-standardtree appli))        
                            (inhibit-application   (component 'bout-standardtree appli)) )
                        (if #:ivan:in-global:breduct1      
                            (authorize-application (component 'bout-standardtree appli))        
                            (authorize-application (component 'bout-reduct1      appli)) )
                        
                        (if #:ivan:in-global:breduct2       
                            (authorize-application (component 'bout-standardtree appli))        
                            (authorize-application (component 'bout-reduct2      appli)) )
                        
                        (authorize-application (component 'bout-minitree     appli))
                        (authorize-application (component 'bout-aller        appli))
                        (when (#:ivan:ed-util:symbole
                                      (send 'get-line (component 'edl-classe appli)))
                              (authorize-application (component 'bout-repos appli)) )
                        (when (componentp 'iace a)
                              (authorize-application (component 'bout-classer appli))
                              )
                        )
                 ; il n'y a pas de concept charge 
                 (authorize-application (component 'bout-quitter      appli))
                 (authorize-application (component 'bout-chemin       appli))
                 (authorize-application (component 'bout-charger      appli))
                 (authorize-application (component 'bout-verifier     appli))
                 (inhibit-application   (component 'bout-sauver       appli))
                 (authorize-application (component 'bout-concept      appli))
                 (inhibit-application   (component 'bout-interro      appli))
                 (inhibit-application   (component 'bout-prettytree   appli))
                 (inhibit-application   (component 'bout-standardtree appli))        
                 (inhibit-application   (component 'bout-reduct1      appli))
                 (inhibit-application   (component 'bout-reduct2      appli))
                 (inhibit-application   (component 'bout-minitree     appli))
                 (inhibit-application   (component 'bout-aller        appli))
                 (inhibit-application   (component 'bout-repos        appli))
                 (inhibit-application   (component 'bout-hyper        appli))
                 (inhibit-application   (component 'bout-classer      appli))
                 )
             ; sinon la seule commande permise est charger/verifier une base
             (authorize-application (component 'bout-quitter    appli))
             (authorize-application (component 'bout-chemin     appli))
             (authorize-application (component 'bout-verifier   appli))
             (authorize-application (component 'bout-charger    appli))
             (inhibit-application (component 'bout-sauver       appli))
             (inhibit-application (component 'bout-concept      appli))
             (inhibit-application (component 'bout-interro      appli))
             (inhibit-application (component 'bout-prettytree   appli))
             (inhibit-application (component 'bout-standardtree appli))        
             (inhibit-application (component 'bout-reduct1      appli))
             (inhibit-application (component 'bout-reduct2      appli))
             (inhibit-application (component 'bout-minitree     appli))
             (inhibit-application (component 'bout-aller        appli))
             (inhibit-application (component 'bout-repos        appli))
             (inhibit-application (component 'bout-hyper        appli))
             (inhibit-application (component 'bout-classer      appli))
             ) 
         )
    )

;;;****************************************************************************************
;;; inhiber les boutons de commande

(de :inhiber-com (appli)
    ; inhiber les boutons de commande
    (inhibit-application (component 'bout-quitter      appli))
    (inhibit-application (component 'bout-chemin       appli))
    (inhibit-application (component 'bout-charger      appli))
    (inhibit-application (component 'bout-verifier     appli))
    (inhibit-application (component 'bout-sauver       appli))
    (inhibit-application (component 'bout-concept      appli))
    (inhibit-application (component 'bout-interro      appli))
    (inhibit-application (component 'bout-prettytree   appli))
    (inhibit-application (component 'bout-standardtree appli))
    (inhibit-application (component 'bout-reduct1      appli))
    (inhibit-application (component 'bout-reduct2      appli))
    (inhibit-application (component 'bout-minitree     appli))
    (inhibit-application (component 'bout-aller        appli))
    (inhibit-application (component 'bout-repos        appli))
    (inhibit-application (component 'bout-hyper        appli))
    (inhibit-application (component 'bout-classer      appli))
    )

;;;*****************************************************************************************
;;; envoi du message list-msg au composant sa-warn de l'application appli

(de :action-sa-warn (appli bcouleur bbeep list-msg)
    (lets
          (
           (col-msg (apply 'column list-msg))
           (message (if bcouleur
                        (background wheat col-msg)
                        col-msg))
           )
          (send 'set-image (component 'sa-warn appli) message)
          
          (when bbeep (tybeep))
          )
    )

;;;*****************************************************************************************
;;; reinitialise les stringmenu (menu de chaines de caracteres) des scrollers

(de :reinit-sm (scroll sm list)
    ; afficher la liste
    (send 'set-strings sm (mapcar 'string list))
    ; repositionner le scroller en tete
    (#:ivan:ed-util:positionne scroll nil)
    
    )

;;;*****************************************************************************************
;;;  reinitialise l'editeur de ligne (contenant le nom du concept)

(de :reinit-edl-concept (edl nom-concept)
    ; deproteger
    ({medite}:readonly-flag edl nil)
    ; nom du concept
    (send 'set-line edl (string nom-concept))
    ; proteger     
    ({medite}:readonly-flag edl t) 
    )

;;;*****************************************************************************************
;;; reinitialise l'editeur de ligne (contenant le nom de la classe)

(de :reinit-edl-classe (edl msg)
    ; deproteger
    ({medite}:readonly-flag edl nil)
    ; nom de la classe ou rien
    (send 'set-line edl msg)
    ; proteger     
    ({medite}:readonly-flag edl t) 
    ; declencher l'action de l'editeur de ligne
    (:action-edl-classe edl)
    )

;;;*****************************************************************************************
;;; reinitialise le browser dans le cas ou l'on est parti d'une classe 
;;;    . action-sm-surclasse
;;;    . action-sm-sousclasse
;;;    . action-bout-aller
;;; il faut repositionner l'arbre et declencher a nouveau l'action de l'editeur de 
;;; ligne dans le cas ou le noeud trouve est ensembliste.

(de :reinit-browser (comp noeud classe)
    ; positionner le noeud ce qui declenche l'action de l'editeur d'arbre
    (send 'set-current  (component 'ed-arbre comp) noeud)
    ; par l'action du bouton bout-repos, je repositionne l'arbre
    (:action-bout-repos (component 'bout-repos comp))
    
    (when (typep noeud '{node}:in-noeud2)
          ; je declenche l'action de l'editeur de ligne dans
          ; le cas d'un noeud "ensembliste" avec le nom de la classe
          (:reinit-edl-classe (component 'edl-classe comp) (string classe))
          )
    )

;;;*****************************************************************************************
;;; supprime la propriete 'noeud des noms de classe lors du changement de concept

(de :demarquer-ed-arbre (appli)
    (mapc
          (lambda (classe)
                  (when (getprop classe 'noeud)
                        (remprop classe 'noeud)
                        )
                  )
          ({in-interf}:ch-list-classes appli)
          )
    )

;;;*****************************************************************************************
