;;; Repertoire: ~/LeShirka/LeIvan/CodeIvan/Interface/
;;;
;;;     +----------------------------------------+-----------------------------------------+
;;;     | fichier :  ivan-in-struct.ll           |                                         |
;;;     +----------------------------------------+-----------------------------------------+
;;;     |                                                                                  |
;;;     |                     Projet de memoire CNAM en informatique                       |
;;;     |             Interface de Visualisation et d'Aide a la Navigation                 |
;;;     |                dans une base de connaissances a objets Shirka.                   |
;;;     |                        Sylvain Grivaud          1991                             |
;;;     |                                                                                  |
;;;     +----------------------------------------------------------------------------------+
;;;     | descriptif :                                                                     |
;;;     |                                                                                  |
;;;     | Ce fichier contient la definition des structures AIDA utilisees pour la cons-    |
;;;     | de l'interface.                                                                  |
;;;     |                                                                                  |
;;;     +----------------------------------------------------------------------------------+

;;;*****************************************************************************************
;;;
;;;    structure pour l'interface IVAN
;;;

(defabbrev in-interf {application}:in-interf)

(defstruct {in-interf}
        ch-list-concepts     ; liste des concepts
        ch-list-hierarchie   ; liste de la hierarchie formee par le concept courant
        ch-list-classes      ; liste des classes du concept courant
        ch-vect-classes      ; vecteur des listes des hierarchies de chaque concept
        ch-concept           ; concept selectionne
        ch-ed-arbre          ; arbre de la representation graphique
        ch-commandes         ; boutons de commande
        ch-classes           ; bloc des sur-classes / sous-classes
        ch-scrollers         ; bloc des differents scrollers du navigateur
        ch-edl-concept       ; editeur de ligne du concept selectionne
        ch-edl-classe        ; editeur de ligne de la classe selectionnee
        ch-sa-action         ; selecteur d'application pour zone de message
        )

;;;*****************************************************************************************
;;;    structure pour dessiner l'arbre des classes : cas general
;;;    une classe correspond a un noeud et reciproquement
;;;    l'image du noeud est fonction des booleens #:ivan:in-global:breduct1
;;;                                               #:ivan:in-global:breduct2

(defabbrev in-noeud {node}:in-noeud)
(defstruct {in-noeud}
        nom
        titre
        )

(de {in-noeud}:title (noeud)
    
    (cond
          
          (#:ivan:in-global:breduct1
                  (font #:ivan:in-global:freduct1
                        ({in-noeud}:titre noeud)
                        )
                  )
          
          (#:ivan:in-global:breduct2
                  (boxedimage 0 0 9 9 "")
                  )
          
          
          (t         
             (boxedimage
                     0 0
                     (+ 6 #:ivan:in-global:larg) (+ 4 #hc)
                     ;            #:ivan:in-global:larg (+ 4 #hc)
                     (font #:ivan:in-global:fmicro 
                           ({in-noeud}:titre noeud)
                           )
                     )
             )
          )
    
    )

;;;*****************************************************************************************
;;;    structure pour dessiner l'arbre des classes : cas du noeud ensembliste
;;;    c'est un noeud qui est la representation de plusieurs classes

(defabbrev in-noeud2 {node}:in-noeud2)
(defstruct {in-noeud2}
        nom
        titre
        )

;;;*****************************************************************************************

(de {in-noeud2}:title (noeud)
    (let
         (
          (img (row
                    (icon 0 0 ic-peigne)
                    (font #:ivan:in-global:fmicro 
                          ({in-noeud2}:titre noeud) )))
          )
         (cond
               
               (#:ivan:in-global:breduct1
                       (font #:ivan:in-global:freduct1
                             ({in-noeud2}:titre noeud)
                             )
                       )
               
               (#:ivan:in-global:breduct2
                       (boxedimage 0 0 15 9 "") ; la largeur est 15 pour ce type
                       )
               
               
               (t
                  (boxedimage
                          0 0
                          (+ 6 #:ivan:in-global:larg) (+ 4 #hc)
                          ; (send 'width img)) (+ 6 #hc)
                          ;               #:ivan:in-global:larg (+ 4 #hc)
                          img
                          )
                  )
               )
         )
    )

;;;*****************************************************************************************
