;;; Repertoire: ~/LeShirka/LeIvan/CodeIvan/Requete/
;
;     +----------------------------------------+-----------------------------------------+
;     | fichier :  ivan-re-edattr.ll           | package : #:ivan:re-edattr              |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface de Visualisation et d'Aide a la Navigation                 |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Sylvain Grivaud          1991                             |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     | descriptif du package :                                                          |
;     |                                                                                  |
;     | La fonction :edit permet l'edition d'un attribut d'une instance Shirka.          |
;     | Une instance de la structure AIDA {in-edattr} composee de 8 champs est creee.    |
;     | La taille de la fenetre finale est fixe mais les composants sont crees dynami-   |
;     | quement en fonction de l'attribut.                                               |
;     |                                                                                  |
;     | en entree :                                                                      |
;     |   - le nom de la classe                                                          |
;     |   - le nom de l'attribut                                                         |
;     |                                                                                  |
;     | en sortie :                                                                      |
;     |   - la valeur de l'attribut                                                      |
;     +----------------------------------------------------------------------------------+

;;;*****************************************************************************************

(defvar #:sys-package:colon '#:ivan:re-edattr)

;;;*****************************************************************************************

(defvar {scroller}:scrollbarwidth  20)     
(defvar {scroller}:scrollbarheight 20)

(defvar :%l-ea 45)  ; Largeur editeur attribut

(defvar :%h-max 4)  ; Hauteur max pour affichage du commentaire

;;;*****************************************************************************************
;;; appel a l'editeur d'attribut pour l'attribut nom-attr de la classe nom-classe-ch
;;; le comparateur est comp.

(de :ea (nom-classe-ch nom-attr valeur-prec comp)
    (lets
          ( 
           (nom-classe  (#:ivan:ed-util:symbole nom-classe-ch))
           ; vecteur du schema de classe
           (schema-clas   (#:ivan:ed-util:schema (#:ivan:ed-util:symbole nom-classe)) )
           
           ; creation instance de travail
           (nom-inst-trav (#:ivan:ed-util:ch-as-sym (catenate "instrav-" (gensym))))
           
           ; inst. temporaire
           (inst-trav     (#:ivan:ed-util:cr-inst nom-classe nom-inst-trav)) 
           
           ; valeur de l'attribut
           val
           )
          
          ;
          ; inserer le nom de l'instance de travail
          ;
          (setq #:ivan:ed-global:%list-inst-trav
                (newr #:ivan:ed-global:%list-inst-trav (string nom-inst-trav)))
          ;
          ; lancer l'editeur d'attribut specifique au module des requetes
          ;
          (#:ivan:re-edattr:edit nom-attr valeur-prec nom-inst-trav comp) 
          
          ;
          ; extraire la valeur de l'attribut de l'instance
          ;
          (setq val (#:ivan:ed-util:val-attr-sym nom-attr nom-inst-trav))
          
          ;       
          ;  supprimer l'instance de travail
          ;
          (#:ivan:ed-util:sup-inst (#:ivan:ed-util:schema nom-inst-trav) nil)
          
          ;
          ; renvoyer la valeur
          ;
          
          val
          ) 
    )

;;;*****************************************************************************************
;;; fonction creant une instance de la structure {in-edattr} pour permettre l'edition
;;; de l'attribut nom-attr de l'instance nom-inst-trav.

(de :edit (nom-attr valeur-prec nom-inst-trav comp)
    (lets
          (   ; application generale
             (appli (omakeq {in-edattr} x 0 y 0 w 0 h 0) )
             
             ; vecteur du schema de classe
             (schema-clas   (#:ivan:ed-util:classe-de nom-inst-trav))
             
             ; facettes  $domaine , $sauf, $intervalle , $valeur, $card
             (list-dom    (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$domaine))
             (list-sauf   (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$sauf))
             (list-inter  (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$intervalle))
             (list-valeur (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$valeur))
             (card-min    (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$cardmin))
             (card-max    (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$cardmax))
             (bvaleur     (not (null list-valeur)))
             
             ; nature shirka de l'attribut
             (list-nature (#:ivan:ed-util:nature schema-clas nom-attr))
             ; l'attribut est-il une liste ?
             (bliste         (eq (car list-nature) '$liste-de))
             
             (blistecomp (or (eq (car list-nature) '$liste-de)
                             (eq comp              #M#:txt-const:text-appart)) )
             
             ; type shirka de l'attribut : 
             ; reel, entier, chaine, symbole ou si c'est un objet complexe sa classe
             (type-attr-temp (cadr list-nature))
             (bspec-loc (equal (ascii (sref (string type-attr-temp) 0)) '%))
             (type-attr
                     (if bspec-loc
                         ; le type de l'attribut commence par un %. C'est le cas
                         ; de la specialisation locale; chercher la classe mere.
                         (car (#:shirka:noyau:peres (#:shirka:noyau:schema type-attr-temp)))
                         ; sinon c'est le type simple ou la classe
                         type-attr-temp))
             ; est-ce un type simple ?
             (btype-simple (not (null (#:ivan:ed-util:type-simple? type-attr))))
             ; est-ce une chaine ?
             (bchaine (eq type-attr 'chaine))
             
             (edl-titre (lineedit 0 0 #40wc ""))
             image-ed
             
             )
          (unless (listp list-dom)    (setq list-dom    (list list-dom))    )
          (unless (listp list-sauf)   (setq list-sauf   (list list-sauf))   )
          (unless (listp list-inter)  (setq list-inter  (list list-inter))  )
          
(when #:trace:re
          (PRINT "ENTREE EA module des requetes....................")
)
          ;-----------------------------------------------------------------------------------
          ;
          ; initialiser l'instance avec ses valeurs precedentes
          ;
          (if blistecomp
              (if bliste
                  (progn
                         (#:ivan:ed-util:sup-toutes-valeurs nom-inst-trav nom-attr)
                         (mapc '#:ivan:ed-util:aj-valeur 
                               (cirlist (#:ivan:ed-util:schema nom-inst-trav))
                               (cirlist nom-attr)
                               valeur-prec
                               (cirlist 't) )
                         
                         )
                  ; sinon c'est le cas du comparateur appartient sur un type non-liste
                  (#:ivan:ed-util:sup-valeur
                          (#:ivan:ed-util:schema nom-inst-trav) 
                          nom-attr
                          nil)
                  (#:ivan:ed-util:aj-valeur
                          (#:ivan:ed-util:schema nom-inst-trav)
                          nom-attr
                          valeur-prec
                          nil)
                  )
              ; sinon
              (if 
                  (let
                       ( (val (string valeur-prec)) )
                       (#:ivan:ed-util:sup-valeur
                               (#:ivan:ed-util:schema nom-inst-trav) 
                               nom-attr
                               nil)
                       (#:ivan:ed-util:aj-valeur
                               (#:ivan:ed-util:schema nom-inst-trav)
                               nom-attr
                               (#:ivan:ed-util:objet-simple val type-attr) 
                               nil)
                       )
                  ; alors
(when #:trace:re
                  (print "     initialisation ancienne valeur effectuee")
)
                  ; sinon
(when #:trace:re
                  (print "     probleme ancienne valeur >" 
                         (#:ivan:ed-util:objet-simple valeur-prec type-attr) "<")
)
                  )
              )
          ;-----------------------------------------------------------------------------------
(when #:trace:re
          (PRINT "     nom-inst-trav " nom-inst-trav " " (type-of nom-inst-trav))
          (PRINT "     nom-attr      " nom-attr    " " (type-of nom-attr))
          (PRINT "     valeur-prec   " valeur-prec)
          (PRINT "     type-attr     " type-attr) 
)
          
          ; composant top pour reperer le pere de plus haut niveau
          ({application}:components appli (acons 'top appli ()))
          
          ; construction des champs de l'application
          ({in-edattr}:ch-scroll-com   appli  (:const-ch-scroll-com   appli
                                                      nom-attr
                                                      schema-clas))
          ({in-edattr}:ch-titre        appli  (:const-ch-titre        appli 
                                                      nom-attr
                                                      bliste
                                                      type-attr
                                                      bspec-loc))
          ({in-edattr}:ch-sa-action    appli  (:const-ch-sa-action    appli))
          ({in-edattr}:ch-edc-val      appli  (:const-ch-edc-val      appli 
                                                      nom-inst-trav
                                                      nom-attr
                                                      bliste
                                                      blistecomp
                                                      btype-simple
                                                      type-attr
                                                      valeur-prec
                                                      bvaleur))
          ({in-edattr}:ch-ligne-valid  appli  (:const-ch-ligne-valid  appli))
          ({in-edattr}:ch-scroll-val   appli  (:const-ch-scroll-val   appli
                                                      nom-inst-trav
                                                      nom-attr
                                                      schema-clas
                                                      bliste
                                                      blistecomp
                                                      type-attr
                                                      card-min
                                                      card-max
                                                      list-valeur)) 
          ({in-edattr}:ch-scroll-choix appli  (:const-ch-scroll-choix appli
                                                      bliste
                                                      blistecomp
                                                      btype-simple
                                                      type-attr
                                                      list-dom))
          ({in-edattr}:ch-contraintes  appli  (:const-ch-contraintes  appli
                                                      list-sauf
                                                      list-inter))
          ; assemblage pour constituer l'image finale
          (setq image-ed
                (column
                        (#:ivan:ed-util:centeredimage
                                (view
                                      (filledbox 0 0
                                              (send 'width ({in-edattr}:ch-sa-action  appli)) 619 0)
                                      (column      
                                              ({in-edattr}:ch-titre appli)
                                              "" 
                                              ({in-edattr}:ch-scroll-com appli)
                                              ({in-edattr}:ch-sa-action  appli)
                                              ""  
                                              (row
                                                   ({in-edattr}:ch-scroll-val   appli)
                                                   ({in-edattr}:ch-scroll-choix appli) )
                                              "" 
                                              ({in-edattr}:ch-edc-val     appli)
                                              ""
                                              ({in-edattr}:ch-contraintes appli)
                                              ""
                                              )
                                      )
                                )
                        ({in-edattr}:ch-ligne-valid appli)
                        )
                )
          (send 'new-image
                appli
                (header
                        (rectangle 0 0 32 32) ; logo 
                        edl-titre
                        (view) ; pas de bouton
                        (view) ; pas de commande 
                        image-ed
                        t
                        t
                        )
                )
          (#:ivan:ed-util:titre-fenetre appli #M#:ed-crit)
          (send 'set-line edl-titre (catenate #M#:crit-sur nom-attr))
          
          ; dimensionnement de l'application
          (send 'fit-to-contents appli)
          
          ;  l'application est activee
          (grab-application
                  (send 'translate
                        appli
                        #45wd #:ivan:ed-global:%y
                        ))
          ; pas de trace a l'ecran
          't
          )
    )

;;;#########################################################################################
;;;
;;;                                    CONSTRUCTION DES CHAMPS 
;;;
;;;#########################################################################################

;;;*****************************************************************************************
;;; construction de l'image de visualisation du commentaire de l'attribut
;;; suivant la taille du commentaire, l'image s'affiche ou non dans un scroller

(de :const-ch-scroll-com (appli nom-attr schema-clas)
    
    (lets
          (  
           ; commentaire de la classe (liste de chaines) 
           (list-comm (#:ivan:ed-util:val-fac-he schema-clas nom-attr '$com)) 
           (list-com
                   (mapcar '(lambda (chaine)
                                    (let ( (chaine-font (font #:ivan:ed-global:fstd chaine)))
                                         chaine-font
                                         ))
                           list-comm))
           ; colonne des lignes
           (col-com (apply 'column list-com))
           
           ; largeur max d'1 ligne en nombre de caracteres (attention #wc = 9)
           (larg (#:ivan:ed-util:larg-max list-comm))
           
           ; hauteur du commentaire (nombre de lignes)
           (haut (ifn (null list-com)
                      (length list-com)))
           
           ; hauteur de l'image commentaire si scroller
           (h-scroll (+ {scroller}:scrollbarheight
                        (#:ivan:ed-util:hc :%h-max)
                        2))
           ; largeur de l'image commentaire si scroller
           (w-scroll (+ {scroller}:scrollbarwidth
                        (#:ivan:ed-util:wc :%l-ea)
                        2))
           
           ; img du commentaire     
           (img-com 
                   (cond
                         ; pas de commentaire = 1 ligne vide
                         ((null list-com)
                          (rectangle 0 0 w-scroll 0)
                          )
                         
                         ; le commentaire tient en largeur et hauteur : boite rectangulaire
                         ((and (<= larg :%l-ea) (<= haut :%h-max))
                          (view
                                (rectangle 0 0 w-scroll (#:ivan:ed-util:hc haut))
                                col-com
                                )
                          )
                         
                         ; le commentaire est trop large et trop haut : scroller   
                         ((and (> larg :%l-ea) (> haut :%h-max))
                          (scroller 0 0
                                  w-scroll h-scroll
                                  col-com
                                  t t
                                  )
                          )
                         
                         ; le commentaire est trop large : scroller horizontal                   
                         ((> larg :%l-ea)
                          (horizontalscroller 0 0
                                  w-scroll h-scroll                               
                                  col-com
                                  )
                          )
                         
                         ; le commentaire est trop haut : scroller vertical              
                         (t         
                            (verticalscroller 0 0
                                    w-scroll h-scroll
                                    col-com
                                    )
                            )
                         ))
           )
          (add-component appli 'img-com img-com)
          
          ; image finale
          (if (null list-com)
              (rectangle 0 0 (+ 12 w-scroll) 0)
              ; sinon il faut sauter une ligne
              (column
                      (#:ivan:ed-util:clusterimage img-com #M#:txt-const:comment)
                      ""))
          )
    )
;;;*****************************************************************************************
;;; construction du champ titre. Le type de l'attribut est affiche dans un cadre

(de :const-ch-titre (appli nom-attr bliste type-attr bspec-loc)
    (lets
          (
           (liste-de (if bliste 
                         (font #:ivan:ed-global:fstd #M#:txt-const:liste-de)
                         " " ))
           (spec-loc (if bspec-loc
                         (font #:ivan:ed-global:fstd #M#:txt-const:spec-loc)
                         "" ))
           (titre-attr (catenate #M#:txt-const:att-type (string nom-attr)))    
           (img (centeredview 
                        (rectangle 0 0 (send 'width (component 'img-com appli)) 0)
                        (row
                             liste-de
                             (font #:ivan:ed-global:fstdb (string type-attr))
                             " "
                             spec-loc)
                        ))
           
           )
          ; image du champ 
          (#:ivan:ed-util:clusterimage img titre-attr)
          )
    )
;;;*****************************************************************************************
;;; construction d'un editeur de chaine pour saisir la valeur de l'attribut
;;; NB : la structure stringedit permet d'associer des filtres de saisie, de validation
;;;      et un message d'erreur en fonction du type de l'attribut

(de :const-ch-edc-val 
    (appli nom-inst-trav nom-attr bliste blistecomp btype-simple type-attr 
           valeur-prec bvaleur)
    (lets
          (
           ; bouton d'effacement de l'editeur de chaine
           (bout-effacer (standardbutton #:ivan:ed-global:txt-effacer))
           ; valeur initiale de l'editeur de chaine
           (val-initiale (#:ivan:ed-util:nom-ch
                                 (if (or blistecomp
                                         (null valeur-prec))
                                     '"" 
                                     valeur-prec)) )
           (titre-edc (row
                           (font #:ivan:ed-global:fstdb
                                 (string nom-attr))
                           (font #:ivan:ed-global:fstdb
                                 " ") ))
           ; largeur de l'editeur de chaine calculee dynamiquement
           (larg 
                 (- (send 'width (component 'img-com appli))
                    (send 'width (#:ivan:ed-util:centeredimage titre-edc))
                    7
                    #wc
                    (send 'width bout-effacer)
                    ))
           
           ; editeur de chaine pour saisir la valeur de l'attribut
           (edc-val (stringedit 0 0 larg val-initiale))
           
           (img-edc (row
                         (#:ivan:ed-util:centeredimage titre-edc)
                         (#:ivan:ed-util:boxedimage edc-val)
                         " "
                         bout-effacer))
           )
          
          ; fonctions de validation et filtre de saisie en fonction du type
          (cond
                ( (eq type-attr 'entier)
                 (send 'filter-func edc-val ':filtre-edc-entier)
                 (send 'validate-func edc-val ':valid-edc-entier) )
                ( (eq type-attr 'reel)
                 (send 'filter-func edc-val ':filtre-edc-reel)
                 (send 'validate-func edc-val ':valid-edc-reel) )
                ( t
                   (send 'filter-func edc-val ':filtre-edc)
                   (send 'validate-func edc-val ':valid-edc) )
                )
          ; fonction d'erreur de l'editeur de chaine
          (send 'error-func edc-val ':erreur-edc-val)
          
          (add-component appli 'edc-val edc-val)
          ({application}:userdata edc-val
                  (list bliste btype-simple type-attr nom-inst-trav nom-attr
                        blistecomp))
          (set-action edc-val ':action-edc-val)
          
          ({application}:userdata bout-effacer (list nom-inst-trav nom-attr blistecomp))
          (set-action bout-effacer ':action-bout-effacer)
          
          (when bvaleur (inhibit-application bout-effacer))
          
          ; afficher l'editeur de chaine
          (#:ivan:ed-util:clusterimage img-edc #M#:txt-const:zone-edi)
          )
    )
;;;-----------------------------------------------------------------------------------------
;;;
;;;                                 filtres de l'editeur de chaine
;;;-----------------------------------------------------------------------------------------

(de :filtre-edc-entier (edc char)
    ; ENTIER : char doit etre un chiffre, le signe - ou le signe +
    (let 
         ( (bok   (or
                      (digitp char)
                      (equal (cascii "-") char)
                      (equal (cascii "+") char)   
                      ))
          )
         (:filtre-msg bok)
         )
    )
;;;-----------------------------------------------------------------------------------------

(de :filtre-edc-reel (edc char)
    ; REEL : idem entier plus le point decimal
    (let 
         ( (bok   (or
                      (digitp char)
                      (equal (cascii "-") char)
                      (equal (cascii "+") char)   
                      (equal (cascii ".") char)
                      ))
          )
         (:filtre-msg bok)
         )
    )
;;;-----------------------------------------------------------------------------------------

(de :filtre-edc (edc char)
    ; CAS GENERAL : idem REEL plus les caracteres alphabetiques
    (let 
         ( (bok   (or
                      (letterp char)
                      (digitp char)
                      (equal (cascii "-") char)
                      (equal (cascii "+") char)   
                      (equal (cascii ".") char)
                      ;;;                  (equal (cascii " ") char)
                      (equal (cascii ",") char) 
                      (equal (cascii "(") char)   
                      (equal (cascii ")") char)
                      (equal (cascii "[") char)
                      (equal (cascii "]") char) 
                      ))
          )
         (:filtre-msg bok)
         )
    )
;;;-----------------------------------------------------------------------------------------
;;; envoie d'un message d'erreur si la saisie est incorrecte

(de :filtre-msg (bok)
    (if bok
        (:action-sa-warn (component 'sa-warn edc) '(""))
        (:action-sa-warn (component 'sa-warn edc) '("" #M#:txt-const:caractere-invalide))
        (tybeep)
        )
    bok
    )
;;;-----------------------------------------------------------------------------------------
;;;
;;;                        fonctions de validation de l'editeur de chaine
;;;-----------------------------------------------------------------------------------------

(de :valid-edc-entier (edc)
    ; ENTIER
    (lets
          (
           (rep (send 'get-line edc))
           (val (when (> (slen rep) 0)
                      (read-from-string rep)
                      ))
           )
          (fixp val)
          )
    )
;;;-----------------------------------------------------------------------------------------

(de :valid-edc-reel (edc)
    ; REEL
    (lets
          (
           (rep (send 'get-line edc))
           (val (when (> (slen rep) 0)
                      (read-from-string rep)
                      ))
           )
          (floatp val)
          )
    )
;;;-----------------------------------------------------------------------------------------

(de :valid-edc (edc)
    ; CAS GENERAL : on a soit une chaine, soit un symbole
    (lets
          (
           (rep (send 'get-line edc))
           (val (when (> (slen rep) 0)
                      (read-from-string rep)
                      ))
           )
          (or (symbolp val) (stringp val))
          )
    )
;;;-----------------------------------------------------------------------------------------
;;;                           fonction d'erreur de l'editeur de chaine
;;;-----------------------------------------------------------------------------------------

(de :erreur-edc-val (edc)
    (:action-sa-warn
            (component 'sa-warn edc)
            (list
                  (font #:ivan:ed-global:fstdb #M#:txt-const:valid-impossible)
                  (font #:ivan:ed-global:fstd #M#:txt-const:incorrect-type) ))
    )
;;;*****************************************************************************************
;;; construction d'un selecteur d'application pour afficher l'aide textuelle
;;; sur l'action en cours. 
;;; les messages seront transmis au composant sa-warn par la fonction :action-sa-warn.

(de :const-ch-sa-action (appli)
    (let
         (
          (sa-warn (selectorapplication 0 0
                           (send 'width (component 'img-com appli))
                           (#:ivan:ed-util:hc 3)
                           nil))
          )
         
         (add-component appli 'sa-warn sa-warn)
         ; image finale centree
         (#:ivan:ed-util:clusterimage sa-warn #M#:txt-const:message)
    ))
;;;*****************************************************************************************
;;; construction du scroller de valeurs de l'attribut.
;;; l'image ne sera activee que dans le cas d'une liste.

(de :const-ch-scroll-val 
    (appli nom-inst-trav nom-attr schema-clas bliste blistecomp type-attr card-min 
           card-max list-valeur)
    (lets
          ( 
           ; liste des valeurs sous forme de symbole sans faire appel au moteur d'inference
           (list-val-sy (#:ivan:ed-util:val-attr-sym nom-attr nom-inst-trav) )
           (list-val-sym
                   (if (null list-val-sy)
                       list-valeur
                       list-val-sy))
           ; liste des valeurs de l'attribut sous forme de chaine
           (list-val-ch
                   (if (listp list-val-sym)
                       (mapcar 'string list-val-sym)
                       (list (string list-val-sym))))
           ; menu de chaine de caracteres des valeurs
           (sm-val (stringmenu list-val-ch))
           ; bouton radio pour insertion en tete / en fin de liste
           (cb-entete (checkbox 0 0 #20wc #hc #M#:txt-const:ins-tete))
           
           ; largeur du scroller est fixe
           (larg (- (div (send 'width ({in-edattr}:ch-scroll-com appli)) 2) 12))
           
           ; scroller de valeur
           (scroll-val (scroller 0 0
                               larg #10hc 
                               sm-val
                               t t))
           ; titre de l'image
           (titre-img (if blistecomp #M#:txt-const:valeurs #M#:txt-const:valeur))
           ; image du selecteur de valeurs
           (img-sel-val 
                   (if blistecomp
                       (column
                               scroll-val
                               (if (or card-min card-max)
                                   ""
                                   (rectangle 0 0 0 0) )
                               (if card-min
                                   (row #M#:txt-const:card-min (string card-min)) 
                                   (rectangle 0 0 0 0) 
                                   )
                               (if card-max
                                   (row #M#:txt-const:card-max (string card-max)) 
                                   (rectangle 0 0 0 0) 
                                   )
                               ""
                               cb-entete)
                       ; sinon on ne peut pas inserer donc on n'affiche simplement le scroller
                       scroll-val
                       ))
           )
          
          (add-component appli 'scroll-val scroll-val)
          (add-component appli 'sm-val sm-val)
          (add-component appli 'img-sel-val img-sel-val)
          (add-component appli 'cb-entete cb-entete)
          
          ({application}:userdata sm-val (list nom-inst-trav nom-attr type-attr bliste))
          (set-action sm-val ':action-sm-val)
          
          (set-action cb-entete ':action-cb-entete)
          
          ; image renvoyee  
          (if blistecomp
              (#:ivan:ed-util:clusterimage img-sel-val titre-img)
              (rectangle 0 0 (+ 12 larg) 0)
              )
          
          )
    )
;;;*****************************************************************************************
;;; construction du scroller des valeurs possibles ie soit les instances, soit le domaine

(de :const-ch-scroll-choix (appli bliste blistecomp btype-simple type-attr list-dom)
    (lets
          (
           ; liste des valeurs a choisir sous forme de chaine
           (list-val-pos
                   (if (null list-dom)
                       ; construire la liste des instances dans le cas d'un objet
                       (#:ivan:ed-util:diff-liste (#:ivan:ed-util:l-inst-t-ch type-attr)
                               #:ivan:ed-global:%list-inst-trav)
                       ; sinon les valeurs du domaine sont affichees
                       list-dom))
           (img-sel-val  (component 'img-sel-val appli))
           
           (sm-choix     (stringmenu (mapcar 'string list-val-pos)))
           
           (larg (- (div (send 'width ({in-edattr}:ch-scroll-com appli)) 2) 12))
           
           ; scroller
           (scroll-choix (scroller 0 0
                                 larg #10hc
                                 sm-choix
                                 t t))
           ; titre de l'image
           (titre-img 
                   (if (null list-dom)
                       (row #M#:txt-const:inst-de (string type-attr))
                       ; sinon 
                       #M#:txt-const:domaine))
           
           ; modif du 4.10.91
           (bout-choisir (standardbutton (font #:ivan:ed-global:fmicro #M#:txt-const:choise)))
           (bout-reinit  (standardbutton (font #:ivan:ed-global:fmicro #M#:txt-const:reinit)))
           
           (img-sel-inst
                   (column
                           scroll-choix
                           (rectangle 0 0 larg (max 9 (- (send 'height img-sel-val)
                                                         #10hc
                                                         (send 'height bout-choisir))))
                           (centeredview
                                   (rectangle 0 0 larg 0)
                                   (row bout-choisir " " bout-reinit)
                                   )
                           )
                   
                   ) 
           
           )
          
          (add-component appli 'scroll-choix scroll-choix)
          (add-component appli 'sm-choix sm-choix)
          (add-component appli 'bout-choisir bout-choisir)
          (add-component appli 'bout-reinit  bout-reinit)
          
          ({application}:userdata sm-choix (list bliste blistecomp))
          (set-action sm-choix ':action-sm-choix)
          
          
          ({application}:userdata bout-reinit (list list-dom type-attr))
          (set-action bout-reinit ':action-bout-reinit)
          (inhibit-application bout-reinit) 
          
          (set-action bout-choisir ':action-bout-choisir)
          
          (if (and btype-simple (null list-dom))
              ; ce n'est pas un objet et il n'y a pas de domaine
              (rectangle 0 0 0 0)
              ; afficher le selectionneur
              (#:ivan:ed-util:clusterimage img-sel-inst titre-img)
              )
          )
    )
;;;*****************************************************************************************
;;; construction de la ligne de validation avec 2 boutons

(de :const-ch-ligne-valid
    (appli)
    (lets
          (
           (bout-valider (standardbutton #:ivan:ed-global:txt-valider))
           
           (ligne (centeredview
                          (rectangle 0 0 (send 'width (component 'img-com appli)) 0)
                          bout-valider))        
           )
          
          ; bouton valider
          (add-component appli 'bout-valider bout-valider)
          ({application}:userdata bout-valider) 
          (set-action bout-valider ':action-bout-valider)
          
          ; image
          (#:ivan:ed-util:clusterimage ligne #M#:txt-const:commande)
          )
    )
;;;*****************************************************************************************
;;; construction du champ des contraintes a partir des facettes $sauf et $intervalle

(de :const-ch-contraintes (appli list-sauf list-inter)
    (lets
          (
           ; CAS DE $intervalle
           (list-binf    (:list-bornes list-inter 3) )
           (list-bsup    (:list-bornes list-inter 4) )
           (sm-binf      (stringmenu (mapcar 'string list-binf)))
           (sm-bsup      (stringmenu (mapcar 'string list-bsup)))
           (tit-binf     (font #:ivan:ed-global:fstd #M#:txt-const:borne-inf) )
           (tit-bsup     (font #:ivan:ed-global:fstd #M#:txt-const:borne-sup) )
           (titre        (row
                              (rectangle 0 0 {scroller}:scrollbarwidth 0)
                              tit-binf
                              "  " 
                              tit-bsup) )
           (scroll-inter (scroller 0 0
                                 (send 'width titre) #5hc
                                 (row sm-binf "  " sm-bsup)
                                 t t))
           ; image de la facette $intervalle
           (img-inter (if (null list-inter)
                          (rectangle 0 0 (send 'width titre) 0)
                          (#:ivan:ed-util:clusterimage
                                  (column
                                          titre
                                          scroll-inter)
                                  #M#:txt-const:intervalle) ))
           
           ; CAS DE $sauf
           (sm-sauf      (stringmenu (mapcar 'string list-sauf)))
           (larg-sauf    (- (send 'width ({in-edattr}:ch-scroll-com appli))
                            (send 'width img-inter)
                            12))
           (scroll-sauf  (scroller 0 0
                                 larg-sauf #5hc
                                 sm-sauf
                                 t t))
           ; image de la facette $sauf
           (img-sauf     (if (null list-sauf)
                             (rectangle 0 0 (+ larg-sauf 12) 0)
                             (#:ivan:ed-util:clusterimage
				     scroll-sauf
                                     #M#:txt-const:vals-interdites) ))
           
           ; image finale
           (img 
                (if (and (null list-sauf) (null list-inter) )
                    (rectangle 0 0 0 0)
                    ; sinon
                    (row img-sauf img-inter)
                    ))
           )
          
          (send 'w sm-binf (send 'width tit-binf))
          (send 'w sm-bsup (send 'width tit-bsup))
          
          (when (or list-sauf list-inter)
                (send 'resize img 
                      nil
                      (max (send 'height img-sauf) (send 'height img-inter))
                      ))
          
          (set-action sm-sauf ':action-reinit-sm)
          (set-action sm-binf ':action-reinit-sm)
          (set-action sm-bsup ':action-reinit-sm)
          
          ; image renvoyee
          img
          )
    )
;;;*****************************************************************************************
;;; construit la liste des bornes min ou max en fonction la variable pos

(de :list-bornes (int pos)
    (let
         (
          (list nil)
          )
         (cond
               ((null int) nil) 
               ((atom int) (newr list (#:ivan:ed-util:val-inst-ch int pos)))
               (t (append (:list-bornes (car int) pos)
                          (:list-bornes (cdr int) pos) ))))
    )

;;;#########################################################################################
;;;                                   
;;;                                    ACTIONS DES CHAMPS 
;;;
;;;#########################################################################################

;;;*****************************************************************************************
;;; action du bouton standard de validation
;;; l'editeur de chaine est declenchee pour ajouter directement son contenu a l'attribut.
;;; si l'ajout a ete possible, le booleen 'bok' prend la valeur vrai et la fenetre est 
;;; fermee sinon, un message d'action a ete envoye et la fenetre ne doit pas etre fermee

(de :action-bout-valider (b)
    (let
         (
          (bok (:action-edc-val (component 'edc-val b)) )
          )
         ;  la valeur est admissible et a ete rajoutee : on ferme la fenetre
         (when bok
               (remove-application (component 'top b))
(when #:trace:re
               (PRINT "SORTIE  EA module des requetes....................")
)
               )
         )
    )
;;;*****************************************************************************************
;;; action du bouton standard d'effacement de la zone d'edition

(de :action-bout-effacer (b)
    (lets
          (
           (list-param    ({application}:userdata b))
           (nom-inst-trav (nth 0 list-param))
           (nom-attr      (nth 1 list-param))
           (bliste        (nth 2 list-param))
           )
          (when (activatedp b)
                ; vider l'editeur de ligne de son contenu     
                (send 'set-line (component 'edc-val b) '"") 
                ; si l'attribut n'est pas une liste, supprimer sa valeur
                (unless bliste
                        (if (#:ivan:ed-util:sup-valeur (#:ivan:ed-util:schema nom-inst-trav)
                                    nom-attr
                                    nil)
(when #:trace:re
                            (print "     la valeur de l'attribut " nom-attr " a ete supprimee !")
                            (print "     probleme pour supprimer la valeur de l'attribut " nom-attr)  
)
                            )  
                        )
                ; raz de la zone de message
                (:action-sa-warn b
                        (list ""
                              (font #:ivan:ed-global:fstd #M#:txt-const:clic-val)))
                )
          )
    )
;;;*****************************************************************************************

(de :action-edc-val (edc)
    (lets
          (
           (list-param    ({application}:userdata edc))
           (bliste           (nth 0 list-param))
           (btype-simple     (nth 1 list-param))
           (type-attr        (nth 2 list-param))
           (nom-edc-sym-trav (nth 3 list-param))
           (nom-attr         (nth 4 list-param))
           (blistecomp       (nth 5 list-param))
           
           (pere          (component 'top edc))
           (sm-val        (component 'sm-val edc))
           (scroll-val    (component 'scroll-val edc))
           (cb-entete     (component 'cb-entete edc))
           
           (nom-edc-ch    (send 'get-line edc))
           (nom-edc-sym   (#:ivan:ed-util:symbole nom-edc-ch))
           (nom-classe    (#:ivan:ed-util:symbole type-attr))
           valret
           )
          
          (:action-sa-warn edc '(""))
          
          (if (<= (slength nom-edc-ch) 0)
              ; alors
              t
              ; sinon
              (:bajout-val bliste
                      nom-inst-trav
                      nom-attr
                      nom-edc-ch
                      sm-val 
                      type-attr
                      cb-entete
                      pere
                      blistecomp
                      )
              ) ; endif <=
          )
    )
;;;-----------------------------------------------------------------------------------------

(de :bajout-val (bliste nom-inst-trav nom-attr valeur sm-val type-attr cb-entete pere
                        blistecomp)
    
(when #:trace:re
    (PRINT ":bajout-val : " (#:shirka:noyau:schema nom-inst-trav))
)
    
    (if blistecomp
        ; alors : cas de l'attribut multi-value
        (:ajoute-val-sm
                nom-inst-trav
                nom-attr
                valeur
                sm-val
                type-attr
                (#:ivan:ed-util:ins-en-fin? cb-entete) ; vrai = ins en fin  faux = ins en tete
                bliste
                )
        ; sinon : cas de la valeur simple
        (:ajoute-val
                nom-inst-trav
                nom-attr
                valeur
                pere
                type-attr
                )
        ) ; endif blistecomp
    )
;;;*****************************************************************************************
;;; cas de l'attribut multi-value
;;; ajoute la chaine valeur a la liste des valeurs du selecteur de chaines sm-val
;;; en ins-en-fin de la liste pour l'attribut nom-attr de l'instance nom-inst-trav;
;;; si l' ajout est possible, la fonction prend la valeur vrai (t) , faux (nil) sinon

(de :ajoute-val-sm (nom-inst-trav nom-attr valeur sm-val type-attr ins-en-fin bliste)
    (lets
          (
           (old (send 'get-strings sm-val))
           (new (if ins-en-fin
                    (reverse (cons valeur (reverse old)))
                    (cons valeur old)))
           (nouv-list
                   (mapcar '#:ivan:ed-util:objet-simple
                           new
                           (cirlist type-attr)
                           ) )
           baj-val
           )
          (setq baj-val
                (if bliste
                    (#:shirka:moteur:aj-valeur
                            (#:ivan:ed-util:schema nom-inst-trav)
                            (#:ivan:ed-util:symbole nom-attr)
                            (#:ivan:ed-util:objet-simple valeur type-attr)
                            ins-en-fin
                            )
                    ; l'attribut n'est pas une liste mais on utilise le comparateur apparait
                    (let
                         ()
                         (#:ivan:ed-util:sup-valeur (#:ivan:ed-util:schema nom-inst-trav)
                                 nom-attr
                                 nil)
                         (#:ivan:ed-util:aj-valeur
                                 (#:ivan:ed-util:schema nom-inst-trav)
                                 nom-attr
                                 nouv-list
                                 nil)
                         )
                    )
                )
          (if baj-val    
              ; alors la valeur est admissible et a ete rajoutee
              (progn
                     ; ajouter la valeur dans la liste des valeurs
                     (send 'set-strings sm-val new)         
                     ; reinitialiser le scroller au debut en cas d'insertion en-tete
                     (#:ivan:ed-util:positionne (component 'scroll-val sm-val) ins-en-fin)
                     ; raz de l'editeur de ligne
                     (send 'set-line (component 'edc-val sm-val) "")
                     ; raz de la zone de message
                     (:action-sa-warn sm-val
                             (list "" (font #:ivan:ed-global:fstd #M#:txt-const:clic-val)))
                     ; la modif a ete possible : renvoyer vrai (bouton valider)
                     t
                     )
              ; sinon la valeur est non admissible
              ; il faut remettre les valeurs precedentes dans l'instance
              (unless bliste
                      (#:ivan:ed-util:aj-valeur
                              (#:ivan:ed-util:schema nom-inst-trav)
                              nom-attr
                              (mapcar '#:ivan:ed-util:objet-simple
                                      old
                                      (cirlist type-attr)
                                      )                   
                              nil)
                      )
              ; message d'erreur
              (:envoie-msg sm-val
                      (list
                            (font #:ivan:ed-global:fstdb #M#:txt-const:valeur-non-admiss)
                            (font #:ivan:ed-global:fstd #M#:txt-const:changer-val)
                            (font #:ivan:ed-global:fstd #M#:txt-const:effacer-avant) ))
              ; la modif n'a pas ete possible : renvoyer faux (bouton valider)
              nil
              )
          )
    )
;;;*****************************************************************************************
;;; cas de l'attribut mono-value
;;; d'abord supprimer l'ancienne valeur avant de pouvoir rajouter la nouvelle valeur
;;; si l' ajout est possible, la fonction prend la valeur vrai (t) , faux (nil) sinon

(de :ajoute-val (nom-inst-trav nom-attr valeur appli type-attr)
    (let
         ()
         (if 
             (let
                  ()
                  (#:ivan:ed-util:sup-valeur (#:ivan:ed-util:schema nom-inst-trav) nom-attr nil)
                  (#:ivan:ed-util:aj-valeur  (#:ivan:ed-util:schema nom-inst-trav)
                          nom-attr
                          (#:ivan:ed-util:objet-simple valeur type-attr)
                          nil)
                  )
             ; alors
             (progn
                    ; raz de la zone de message
                    (:action-sa-warn appli
                            (list ""
                                  (font #:ivan:ed-global:fstd #M#:txt-const:clic-val)))
                    
                    ; la modif a ete possible : renvoyer vrai (bouton valider)
                    t
                    )
             ; sinon la valeur est non admissible
             (:envoie-msg appli
                     (list
                           (font #:ivan:ed-global:fstdb #M#:txt-const:valeur-non-admiss)
                           (font #:ivan:ed-global:fstd #M#:txt-const:changer-val)
                           (font #:ivan:ed-global:fstd #M#:txt-const:effacer-avant) ))
             ; la modif n'a pas ete possible : renvoyer faux (bouton valider)
             nil
             )
         )
    )
;;;*****************************************************************************************
;;; envoi du message list-msg au composant sa-warn de l'application appli

(de :action-sa-warn (appli list-msg)
    (let
         (
          (col-msg (apply 'column list-msg)) 
          )
         (send 'set-image (component 'sa-warn appli) col-msg)
         )
    )
;;;*****************************************************************************************
;;; envoyer le message list-msg en declenchant 2 beeps 

(de :envoie-msg (comp list-msg)
    (tybeep)
    (tybeep)
    (:action-sa-warn comp list-msg) 
    )
;;;*****************************************************************************************
;;; action du selecteur de menus sm (scroller de valeurs)
;;; envoie la chaine selectionnee (valeur) vers l'editeur de chaine
;;; et supprime la valeur de la liste des valeurs

(de :action-sm-val (sm)
    (lets
          ( 
           (valeur        (send 'get-selected-string sm))
           (rang          (if valeur (+ (send 'get-selected-rank sm) 1)))
           (list-param    ({application}:userdata sm))
           (nom-inst-trav (nth 0 list-param))
           (nom-attr      (nth 1 list-param))
           (type-attr     (nth 2 list-param))
           (bliste        (nth 3 list-param))
           bsup-val
           nouv-list
           )
          (ifn valeur
               ; alors rien
               't
               ; sinon
               (setq
                     ; supprimer l'element selectionne
                     nouv-list (#:ivan:ed-util:supp-nl rang (send 'get-strings sm))
                     ; supprimer la valeur selectionnee de la liste des valeurs de l'instance
                     bsup-val
                     (if bliste
                         (#:shirka:moteur:sup-valeur
                                 (#:ivan:ed-util:schema nom-inst-trav)
                                 (#:ivan:ed-util:symbole nom-attr)
                                 (#:ivan:ed-util:objet-simple valeur type-attr)
                                 )
                         ; sinon c'est le cas de l'acces du comparateur appartient
                         ; je supprime la valeur de l'instance
                         (#:ivan:ed-util:sup-valeur (#:ivan:ed-util:schema nom-inst-trav) nom-attr nil)
                         (#:ivan:ed-util:aj-valeur
                                 (#:ivan:ed-util:schema nom-inst-trav)
                                 nom-attr
                                 (mapcar '#:ivan:ed-util:objet-simple
                                         nouv-list
                                         (cirlist type-attr)
                                         )                   
                                 nil)
                         )
                     )
               (ifn bsup-val
                    ; alors [ca ne devrait pas arriver !]
                    (:action-sa-warn sm 
                            (list
                                  (font #:ivan:ed-global:fstd #M#:txt-const:supp-imposs)
                                  ""
                                  (font #:ivan:ed-global:fstd #M#:txt-const:cry-for-FR)))
                    ; sinon
                    ; supprimer la valeur de la liste du selectionneur
                    (send 'set-strings sm nouv-list)
                    ; envoyer la valeur a l'editeur de chaine
                    (send 'set-line (component 'edc-val sm) valeur)
                    )
               )
          )
    )
;;;*****************************************************************************************
;;; action de la case a cocher dans le cas d'une liste de valeurs

(de :action-cb-entete (cb)
    (let
         (
          (choix (send 'checkedp cb))       
          )
         (if choix
             (:action-sa-warn cb
                     (list
                           (font #:ivan:ed-global:fstd #M#:txt-const:ins-tete)))
             (:action-sa-warn cb 
                     (list
                           (font #:ivan:ed-global:fstd #M#:txt-const:ins-fin)))
             ))
    )
;;;****************************************************************************************
;;; action du selecteur de menus sm (scroller de choix des valeurs possibles)
;;; envoie la chaine selectionnee vers l'editeur de ligne

(de :action-sm-choix (sm)
    (lets
          ( 
           (valeur (send 'get-selected-string sm))
           (valeur-sym (#:ivan:ed-util:symbole valeur))
           (list-param ({application}:userdata sm))
           (bliste     (nth 0 list-param))
           (blistecomp (nth 1 list-param))
           (list-val (send 'get-strings (component 'sm-val sm)))
           )
          (ifn valeur-sym
               ; alors rien (cas de la reinitialisation du sm)
               't
               ; sinon
               ; reinitialiser le selecteur
               (send 'set-selected-string sm "")
               ; envoyer la valeur a l'editeur de ligne
               (send 'set-line (component 'edc-val sm) valeur)
               ; message d'action
               (if blistecomp
                   (:action-sa-warn sm
                           (list
                                 (font #:ivan:ed-global:fstd #M#:txt-const:faire-rc)
                                 (font #:ivan:ed-global:fstd #M#:txt-const:pour-aj-val)))
                   ; sinon
                   (:action-sa-warn sm 
                           (list 
                                 (font #:ivan:ed-global:fstd #M#:txt-const:faire-rc)
                                 (font #:ivan:ed-global:fstd #M#:txt-const:ou-valid)))
                   )
               )
          )
    )
;;;*****************************************************************************************
;;; bouton reinit = recuperer la liste de la classe et la renvoyer dans le selectionneur

(de :action-bout-reinit (b)
    (lets
          (
           (list-param ({application}:userdata b))
           (list-dom   (nth 0 list-param))
           (type-attr  (nth 1 list-param))
           
           ; liste des valeurs a choisir sous forme de chaine
           (list-val-pos
                   (if (null list-dom)
                       ; construire la liste des instances dans le cas d'un objet
                       (#:ivan:ed-util:diff-liste (#:ivan:ed-util:l-inst-t-ch type-attr)
                               #:ivan:ed-global:%list-inst-trav)
                       ; sinon les valeurs du domaine sont affichees
                       list-dom))
           )
          (send 'set-strings (component 'sm-choix b) list-val-pos)
          (send 'set-line (component 'edc-val b) "")
          (inhibit-application b)
          )
    )
;;;*****************************************************************************************
;;; action du bouton choisir = ouvrir une fenetre pour saisir le nom du fichier a charger

(de :action-bout-choisir (b)
    (lets
          (
           (edl-chemin (lineedit 0 0
                               (+ #5wc (send 'width #:shirka:*bases*))
                               #:shirka:*bases*) )
           
           (img-chemin
                   (#:ivan:ed-util:boxedimage
                           (row
                                (font #:ivan:in-global:fstd #M#:txt-const:chemin)
                                edl-chemin
                                )))
           (list-files (cdr (#:systeme:unix:catalogue (pathname #:shirka:*bases*))))
           
           (nom-base
                   (ask-string
                           (column                 
                                   (font #:ivan:in-global:fstdb #M#:txt-const:liste-inst)
                                   ""
                                   img-chemin
                                   ""
                                   (font #:ivan:in-global:fstd #M#:txt-const:charg-fic)
                                   )
                           list-files
                           nil
                           )
                   )
           (nom-fichier (catenate (send 'get-line edl-chemin) nom-base))
           )
          (when nom-base
                
                ; charger le fichier
                (setq 
                      #:ivan:re-global:list-inst (mapcar 'string (#:ivan:ed-util:lire nom-fichier))
                      )
                ; initialiser le selectionneur
                (send 'set-strings (component 'sm-choix b) #:ivan:re-global:list-inst)
                (send 'set-line (component 'edc-val b) "")
                (autorise-application (component 'bout-reinit b))
                )
          )
    )
;;;*****************************************************************************************
;;; reinitialise le scroller sm pour de-selectionner la valeur qui a ete cliquee
;;; sert pour les sm des facettes $sauf et $intervalle (contraintes)

(de :action-reinit-sm (sm)
    (let
         ( 
          (valeur (send 'get-selected-string sm))
          )
         (ifn valeur
              ; alors rien (cas de la reinitialisation du sm)
              't
              ; sinon
              ; reinitialiser le selecteur
              (send 'set-selected-string sm "")
              )
         )
    )
;;;*****************************************************************************************
