;;; Repertoire: ~/LeShirka/LeIvan/CodeIvan/Requete/
;
;     +----------------------------------------+-----------------------------------------+
;     | fichier :  ivan-re-interf.ll           | package : #:ivan:re-interf              |
;     +----------------------------------------+-----------------------------------------+
;     |                                                                                  |
;     |                     Projet de memoire CNAM en informatique                       |
;     |             Interface de Visualisation et d'Aide a la Navigation                 |
;     |                dans une base de connaissances a objets Shirka.                   |
;     |                        Sylvain Grivaud          1991                             |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+
;     | descriptif du package :                                                          |
;     |                                                                                  |
;     | Le fichier est en 4 parties :                                                    |
;     |    1. Fonction principale                                                        |
;     |    2. Fonctions de construction des champs                                       |
;     |    3. Fonctions d'action des composants                                          |
;     |    4. Fonctions diverses (utilitaires)                                           |
;     |                                                                                  |
;     +----------------------------------------------------------------------------------+

;;;*****************************************************************************************

(defvar #:sys-package:colon '#:ivan:re-interf)

;;;*****************************************************************************************
(defvar {scroller}:scrollbarwidth  20)     
(defvar {scroller}:scrollbarheight 20)

(defvar :%l-ei 45)   ; Largeur editeur instance en pixel (45 caracteres X 9)
(defvar :%h-max 3)   ; Hauteur max pour affichage du commentaire (en nombre de ligne)
(defvar :lmaxcar 60)
(defvar :lmaxpix (+ (#:ivan:ed-util:wc :lmaxcar) 20 12))

;;;*****************************************************************************************
;;; fonction de construction d'une requete principale
;;; l'argument appli contient la definition de l'interface d'appel IVAN

(de :interface (appli)
    (lets
          (
           ; application generale
           (requete (omakeq {in-requet} x 0 y 0 w 0 h 0) )
           (bout-quitter  (standardbutton #M#:txt-const:fin) )
           (bout-masquer  (standardbutton #M#:txt-const:retour-ivan) )
           image-requete
           )
          
          ; action du bouton quitter
          ({application}:userdata bout-quitter appli)
          (set-action bout-quitter ':action-bout-quitter)
          
          ({application}:userdata bout-masquer appli)
          (set-action bout-masquer ':action-bout-masquer)
          
          ; composant top pour reperer le pere de plus haut niveau
          ({application}:components requete (acons 'top requete ()))
          
          (add-component requete  'bout-quitter bout-quitter)
          (add-component requete  'bout-masquer bout-masquer)
          ;
          ; champs graphiques
          ;
          ({in-requet}:ch-message         requete 
                  (:const-ch-message      requete))
          ({in-requet}:ch-commentaire     requete 
                  (:const-ch-commentaire  requete))
          ({in-requet}:ch-classe          requete 
                  (:const-ch-classe       requete))
          ({in-requet}:ch-attributs       requete 
                  (:const-ch-attributs    requete))
          ({in-requet}:ch-type            requete 
                  (:const-ch-type         requete))
          ({in-requet}:ch-comparateurs    requete 
                  (:const-ch-comparateurs requete))
          ({in-requet}:ch-connecteurs     requete 
                  (:const-ch-connecteurs  requete))
          ({in-requet}:ch-requete         requete 
                  (:const-ch-requete      requete))
          ({in-requet}:ch-instances       requete 
                  (:const-ch-instances    requete))
          ({in-requet}:ch-commandes       requete 
                  (:const-ch-commandes requete))
          ;
          ; creation de l'image de l'application
          ;
          (setq image-requete
                (column
                        ({in-requet}:ch-message requete)
                        (#:ivan:ed-util:clusterimage
                                (row
                                     ({in-requet}:ch-classe      requete)
                                     ({in-requet}:ch-commentaire requete)
                                     )
                                #M#:txt-const:requete-su
                                )
                        (#:ivan:ed-util:clusterimage
                                (column                              
                                        (row
                                             ({in-requet}:ch-attributs requete)
                                             (column 
                                                     ({in-requet}:ch-type requete)
                                                     (row
                                                          ({in-requet}:ch-comparateurs requete)
                                                          ({in-requet}:ch-connecteurs requete)       
                                                          )
                                                     )
                                             )
                                        (rectangle 0 0 0 (div #hc 2))
                                        ({in-requet}:ch-requete        requete)
                                        )
				#M#:txt-const:const-req
                                )
                        (centeredview
                                (rectangle 0 0
                                        (send 'width ({in-requet}:ch-instances requete)) 0)
                                (icon 0 0 ic-arrow-down)
                                )
                        ({in-requet}:ch-instances requete)
                        )
                )
          
          ; assemblage des differents champs pour constituer l'image finale
          (send 'set-image
                requete
                (header
                        (icon 0 0 ic-requete)
			#M#:txt-const:interro-base
                        ; boutons masquer et quitter:
                        (row bout-masquer "  " bout-quitter)
                        (column
                                ({in-requet}:ch-commandes requete)
                                "")
                        image-requete      
                        t
                        nil
                        )
                )
          
          (send 'fit-to-contents requete)
          (send 'set-title requete #M#:txt-const:req)
          
          ; j'active la fenetre
          (add-application requete)
          
          ; reinitialiser les variables globales
          (:reinit-var-global (send 'get-line (component 'edl-classe appli)))
          
          ; classe courante
          (:reinit-edl-classe (component 'edl-classe requete)
                  (send 'get-line (component 'edl-classe appli)))
          
          ; reinitialiser la liste des instances de la classe
          (#:ivan:in-interf:reinit-sm
                  (component 'scroll-instances requete)  
                  (component 'sm-instances     requete)
                  (:list-instances (send 'get-line (component 'edl-classe requete))) 
                  )
          (:reinit-card
                  (component 'card-instances requete)
                  (send 'get-strings (component 'sm-instances requete))
                  )
          
          (:autorise-sel requete)
          
          (add-component appli 'requete requete)
          (add-component requete 'ivan appli)
          
          'Requete
          )
    )

;;;#########################################################################################
;;;#########################################################################################
;;;
;;;                                    CONSTRUCTION DES CHAMPS 
;;;
;;;#########################################################################################
;;;#########################################################################################

;;;*****************************************************************************************
;;;  construction du champ d'affichage de message d'information/d'alerte

(de :const-ch-message (requete)
    (let
         (
          (sa-warn (selectorapplication
                           0 0
                           (#:ivan:ed-util:wc :lmaxcar)
                           #hc
                           nil))
          )
         (add-component requete 'sa-warn sa-warn)
         ; image renvoyee
         (#:ivan:ed-util:clusterimage sa-warn #M#:txt-const:message)
         )
    )

;;;*****************************************************************************************
;;; construction de l'image de visualisation du commentaire de la classe
;;; dans ce programme, il s'agit d'un selectionneur d'images (selectorapplication)
;;; dont l'image sera modifiee pour chaque nouvelle classe
;;; (fonction :nouv-img-commentaire)

(de :const-ch-commentaire (requete)
    (lets
          (
           ; hauteur de l'image commentaire si scroller
           (h-scroll (+ {scroller}:scrollbarheight
                        (#:ivan:ed-util:hc :%h-max)
                        2))
           ; largeur de l'image commentaire si scroller
           (w-scroll (+ {scroller}:scrollbarwidth
                        (#:ivan:ed-util:wc :%l-ei)
                        2))
           ; image pour avoir la taille
           (img (#:ivan:ed-util:clusterimage
                        (rectangle 0 0 w-scroll h-scroll) #M#:txt-const:tp))
           ; selecteur qui contiendra l'image du commentaire   
           (sa-com (selectorapplication
                           0 0
                           (send 'width img) (send 'height img)
                           nil))
           (img-com (rectangle 0 0 w-scroll h-scroll))
           )
          (add-component requete 'sa-com sa-com)
          (add-component requete 'img-com img-com)
          ; image renvoyee
          sa-com
          )
    )
;;;*****************************************************************************************
;;; construction de l'editeur de ligne contenant le nom de la classe

(de :const-ch-classe (requete)
    (let
         (
          (edl-classe (lineedit 0 0 #15wc ""))
          )
         (add-component requete 'edl-classe edl-classe)
         (set-action edl-classe ':action-edl-classe)
         ; image renvoyee
         (centeredview
                 (rectangle 0 0 0 (send 'height (component 'img-com requete)))
                 edl-classe)
         )
    )
;;;*****************************************************************************************
;;; construction du scroller contenant la liste des attributs de la classe

(de :const-ch-attributs (requete)
    (lets
          (
           (sm-attributs   (stringmenu nil))
           
           ; definition du scroller
           (scroll-attributs
                   (scroller
                           0 0
                           #20wc #10hc
                           sm-attributs
                           t t) )
           
           ; definition image
           (clu-attributs
                   (#:ivan:ed-util:clusterimage scroll-attributs #M#:txt-const:atts))
           (img-attributs (button clu-attributs))
           )
          
          (add-component requete 'sm-attributs     sm-attributs)
          (add-component requete 'scroll-attributs scroll-attributs)
          (add-component requete 'img-attributs img-attributs)
          (set-action sm-attributs ':action-sm-attributs)
          
          ; image renvoyee
          img-attributs
          )
    )
;;;*****************************************************************************************
;;; construction du champ contenant l'image de l'attribut :
;;; elle pourra contenir un bouton pointant sur la classe de definition de
;;; l'attribut (cas des objets complexes).
;;; l'image est modifiee par la fonction :nouv-img-type

(de :const-ch-type (requete)
    (lets
          (
           (sa-type (selectorapplication
                            0 0
                            #20wc
                            #2hc
                            ""))
           ; definition image
           (clu-type 
                   (#:ivan:ed-util:clusterimage sa-type #M#:txt-const:tp))
           (img-type (button clu-type))
           )
          
          (add-component requete 'sa-type  sa-type)
          (add-component requete 'img-type img-type)
          
          ; image renvoyee
          img-type
          )
    )
;;;*****************************************************************************************
;;; construction du cluster des comparateurs : ce sont les operations de recherche
;;; autorisees
;;; en fonction du type de l'attribut, certains boutons seront inhibes.   

(de :const-ch-comparateurs (requete)
    (lets
          (
           (bout-egal    (button #M#:txt-const:text-egal))
           (bout-diff    (button #M#:txt-const:text-diff))
           (bout-sup     (button #M#:txt-const:text-sup))
           (bout-inf     (button #M#:txt-const:text-inf))
           (bout-supegal (button #M#:txt-const:text-supegal))
           (bout-infegal (button #M#:txt-const:text-infegal))
           (bout-cont    (button #M#:txt-const:text-cont))
           (bout-ncont   (button #M#:txt-const:text-ncont))
           (bout-deb     (button #M#:txt-const:text-deb))
           (bout-fin     (button #M#:txt-const:text-fin))
           (bout-incl    (button #M#:txt-const:text-incl))
           (bout-appart  (button #M#:txt-const:text-appart))
           
           (img-comp
                   (column
                           bout-egal
                           bout-diff
                           bout-sup 
                           bout-inf
                           bout-supegal 
                           bout-infegal
                           bout-cont
                           bout-ncont 
                           bout-deb
                           bout-fin 
                           bout-incl
                           bout-appart))
           
           ; definition image
           (clu-comparateurs
                   (#:ivan:ed-util:clusterimage
                           (centeredview
                                   (rectangle 0 0 #20wc (send 'height img-comp))
                                   img-comp
                                   )
                           #M#:txt-const:comps
                           )
                   )
           (img-comparateurs (button clu-comparateurs))
           )
          (add-component requete 'img-comp img-comp) 
          (add-component requete 'img-comparateurs img-comparateurs) 
          (add-component requete 'bout-egal     bout-egal)
          (add-component requete 'bout-diff     bout-diff)
          (add-component requete 'bout-sup      bout-sup)
          (add-component requete 'bout-inf      bout-inf)
          (add-component requete 'bout-supegal  bout-supegal)
          (add-component requete 'bout-infegal  bout-infegal)
          (add-component requete 'bout-cont     bout-cont)
          (add-component requete 'bout-ncont    bout-ncont)
          (add-component requete 'bout-deb      bout-deb)
          (add-component requete 'bout-fin      bout-fin)
          (add-component requete 'bout-incl     bout-incl)
          (add-component requete 'bout-appart   bout-appart)
          
          (set-action bout-egal     ':action-comp)
          (set-action bout-diff     ':action-comp) 
          (set-action bout-sup      ':action-comp)
          (set-action bout-inf      ':action-comp)
          (set-action bout-supegal  ':action-comp)
          (set-action bout-infegal  ':action-comp)
          (set-action bout-cont     ':action-comp)
          (set-action bout-ncont    ':action-comp)
          (set-action bout-deb      ':action-comp)
          (set-action bout-fin      ':action-comp)
          (set-action bout-incl     ':action-comp)
          (set-action bout-appart   ':action-comp)
          
          ; image du champ 
          img-comparateurs
          )
    )
;;;*****************************************************************************************
;;; construction du cluster des connecteurs logiques

(de :const-ch-connecteurs (requete)
    (lets
          (
           (conn-et   (button #M#:txt-const:et))
           (conn-ou   (button #M#:txt-const:ou))
           (conn-sauf (button #M#:txt-const:sauf))
           (img-conn
                   (centeredview
                           (rectangle 0 0
                                   #10wc (send 'height (component 'scroll-attributs requete)))
                           (column
                                   conn-et
                                   "" ""
                                   conn-ou
                                   "" ""
                                   conn-sauf)))
           
           ; definition image
           (clu-connecteurs
                   (#:ivan:ed-util:clusterimage img-conn #M#:txt-const:conns))
           (img-connecteurs (button clu-connecteurs))
           )
          (add-component requete 'img-conn  img-conn)
          (add-component requete 'img-connecteurs  img-connecteurs)
          (add-component requete 'conn-et   conn-et)
          (add-component requete 'conn-ou   conn-ou)
          (add-component requete 'conn-sauf conn-sauf)     
          
          (set-action conn-et   ':action-conn)     
          (set-action conn-ou   ':action-conn)
          (set-action conn-sauf ':action-conn)
          ; image renvoyee
          img-connecteurs
          )
    )
;;;*****************************************************************************************
;;; construction du champ d'affichage de la requete en construction/a executer
;;; l'image est modifiee a la construction de requete par la fonction :nouv-img-requete

(de :const-ch-requete (requete)
    (let
         (
          (scroll-requete (scroller
                                  0 0
                                  (#:ivan:ed-util:wc :lmaxcar)
                                  #10hc
                                  (view)
                                  t t))
          )
         (add-component requete 'scroll-requete scroll-requete)
         
         ; image renvoyee
         (#:ivan:ed-util:clusterimage scroll-requete #M#:txt-const:req)
         )
    )
;;;*****************************************************************************************
;;; construction du bloc de resultat d'une requete. Il contient 3 zones :
;;;    - le scroller des instances selectionnees
;;;    - une case a cocher
;;;    - un bouton de sauvegarde

(de :const-ch-instances (requete)
    (lets
          (  
           ;
           ; cluster des instances de depart
           ;
           (sm-instances   (stringmenu nil) )
           (card-instances (lineedit 0 0 #3wc ""))
           (card-selection (lineedit 0 0 #3wc "")) 
           
           ; definition du scroller
           (scroll-instances
                   (scroller
                           0 0
                           #30wc #7hc
                           sm-instances
                           t t) )
           
           ; definition du cluster
           (clu-instances
                   (#:ivan:ed-util:clusterimage
                           (column scroll-instances card-instances)
			   #M#:txt-const:sel-parmi
                           ))
           
           ;
           ; cluster des instances selectionnees
           ;
           (sm-selection  (stringmenu nil) )
           
           ; definition du scroller
           (scroll-selection
                   (scroller
                           0 0
                           #30wc #7hc
                           sm-selection
                           t t) )
           
           ; definition du cluster
           (clu-selection
                   (#:ivan:ed-util:clusterimage 
                           (column scroll-selection card-selection)
			   #M#:txt-const:inst-sel
                           ))
           
           )
          
          (add-component requete 'scroll-instances scroll-instances)
          (add-component requete 'sm-instances     sm-instances)
          (add-component requete 'scroll-selection scroll-selection)
          (add-component requete 'sm-selection     sm-selection)
          (add-component requete 'card-instances   card-instances)
          (add-component requete 'card-selection   card-selection)
          
          (set-action sm-instances ':action-sm-instances)
          (set-action sm-selection ':action-sm-selection)
          
          (#:ivan:ed-util:clusterimage
                  (row
                       clu-instances
                       (centeredview
                               (rectangle 0 0 0 (send 'height clu-selection))
                               (icon 0 0 ic-arrow-right)
                               )
                       clu-selection
                       )
		  #M#:txt-const:res-req
                  )
          )
    )
;;;*****************************************************************************************

(de :const-ch-commandes (requete)
    (lets
          (
           larg-max
           (bout-repertoire       (standardbutton #M#:txt-const:r-rep))
           (bout-initialiser      (standardbutton #M#:txt-const:r-reinit))
           (bout-majclasse        (standardbutton 
                                          (column
                                                  (font #:ivan:in-global:fstdb
                                                        #M#:txt-const:init-to-1)
                                                  (font #:ivan:in-global:fstdb
                                                        #M#:txt-const:init-to-2) )))
           
           (bout-vider            (standardbutton #M#:txt-const:r-vider))
           (bout-charger          (standardbutton #M#:txt-const:charg))
           (bout-sauver           (standardbutton #M#:txt-const:sauv))
           (bout-executer         (standardbutton (column "" #M#:txt-const:r-cherche "")))
           (bout-conserver        (standardbutton #M#:txt-const:r-cons))
           (bout-chargersel       (standardbutton #M#:txt-const:charg))
           (bout-sauversel        (standardbutton #M#:txt-const:sauv))
           
           (clu-syst
                   (#:ivan:ed-util:clusterimage  bout-repertoire #M#:txt-const:sys))
           
           (clu-const
                   (#:ivan:ed-util:clusterimage
                           (column
                                   bout-initialiser
                                   bout-majclasse
                                   )
			   #M#:txt-const:clas))
           
           (clu-req
                   (#:ivan:ed-util:clusterimage
                           (column
                                   bout-vider
                                   ""
                                   bout-charger
                                   bout-sauver
                                   ""
                                   bout-executer
                                   )
                           #M#:txt-const:req))
           
           (clu-sel
                   (#:ivan:ed-util:clusterimage
                           (column
                                   bout-conserver
                                   ""
                                   bout-chargersel
                                   bout-sauversel
                                   )
                           #M#:txt-const:selec))
           
           (larg-max (max (send 'width clu-syst)
                          (send 'width clu-const)
                          (send 'width clu-req)
                          (send 'width clu-sel))) 
           )
          (send 'resize clu-syst  larg-max ())
          (send 'resize clu-const larg-max ())
          (send 'resize clu-req   larg-max ())
          (send 'resize clu-sel   larg-max ())
          ;
          ; les composants nommes
          ;
          (add-component requete 'bout-repertoire          bout-repertoire)
          (add-component requete 'bout-initialiser         bout-initialiser)
          (add-component requete 'bout-majclasse           bout-majclasse)
          
          (add-component requete 'bout-vider               bout-vider)
          (add-component requete 'bout-charger             bout-charger)
          (add-component requete 'bout-sauver              bout-sauver)
          (add-component requete 'bout-executer            bout-executer)
          
          (add-component requete 'bout-chargersel          bout-chargersel)
          (add-component requete 'bout-sauversel           bout-sauversel)
          (add-component requete 'bout-conserver           bout-conserver)
          ;
          ; les actions des composants
          ;
          (set-action bout-repertoire      ':action-bout-repertoire)
          (set-action bout-initialiser     ':action-bout-initialiser)
          (set-action bout-majclasse       ':action-bout-majclasse)
          
          (set-action bout-vider           ':action-bout-vider)
          (set-action bout-charger         ':action-bout-charger)
          (set-action bout-sauver          ':action-bout-sauver)
          (set-action bout-executer        ':action-bout-executer)
          
          (set-action bout-conserver       ':action-bout-conserver)
          (set-action bout-chargersel      ':action-bout-chargersel)
          (set-action bout-sauversel       ':action-bout-sauversel)
          ;
          ; image finale
          ;
          (column
                  clu-syst
                  (rectangle 0 0 0 10)
                  (row
                       clu-const
                       (#:ivan:ed-util:centeredview-h
                               clu-const
                               (icon 0 0 ic-arrow-right)) )
                  (rectangle 0 0 0 303)
                  (row
                       clu-req
                       (#:ivan:ed-util:centeredview-h
                               clu-req
                               (icon 0 0 ic-arrow-right)) )
                  (rectangle 0 0 0 30)
                  (row
                       clu-sel
                       (#:ivan:ed-util:centeredview-h
                               clu-sel
                               (icon 0 0 ic-arrow-right)) )
                  )
          )
    )

;;;*****************************************************************************************
;;;                                  
;;;                            ACTIONS DES CHAMPS ET DES COMPOSANTS
;;;
;;;*****************************************************************************************

;;;*****************************************************************************************
;;;                     ACTION DES COMPOSANTS DU CONSTRUCTEUR DE REQUETE
;;;*****************************************************************************************
;;; action de l'editeur de ligne contenant la classe

(de :action-edl-classe (edl)
    (lets
          (
           (nom-classe-ch  (send 'get-line edl))
           (nom-classe-sym (#:ivan:ed-util:symbole nom-classe-ch))
           )
          ;
          ; mettre a jour le commentaire de la classe
          ;
          (:nouv-img-commentaire edl nom-classe-sym)
          ;
          ; mettre a jour la liste des attributs
          ;
          (#:ivan:in-interf:reinit-sm
                  (component 'scroll-attributs edl)
                  (component 'sm-attributs edl)
                  (:list-attributs nom-classe-sym)
                  )
          ;
          ; raz du type de l'attribut
          ;
          (send 'set-image (component 'sa-type edl) (view))
          ;
          ; raz du type de l'attribut (les comparateurs sont actives)
          ;
          (:nouv-img-type edl nil nil nil nil nil)
          
          ;
          ; declencher la mise a jour de l'image de la requete
          ;            
          (:nouv-img-requete edl)
          )
    )
;;;*****************************************************************************************
;;; action du selectionneur d'attributs

(de :action-sm-attributs (sm)
    (lets
          (
           (nom-attr-ch    (send 'get-selected-string sm))
           (nom-attr-sym   (#:ivan:ed-util:symbole nom-attr-ch))
           (nom-classe-ch  (send 'get-line (component 'edl-classe sm)))
           (nom-classe-sym (#:ivan:ed-util:symbole nom-classe-ch))
           img-type 
           schema-clas   
           list-nature
           bliste
           type-attr-temp
           bspec-loc
           type-attr
           btype-simple
           bchaine 
           liste-de
           spec-loc
           list-reverse
           list-attr
           )
          (when nom-attr-sym
                (setq
                      ; vecteur du schema de classe
                      schema-clas   (#:ivan:ed-util:schema nom-classe-sym)
                      ; nature shirka de l'attribut
                      list-nature (#:ivan:ed-util:nature schema-clas nom-attr-sym)
                      ; l'attribut est-il une liste ?
                      bliste (eq (car list-nature) '$liste-de)
                      ; type shirka de l'attribut : 
                      ; c'est un reel, un entier, une chaine, un symbole ou si c'est un objet 
                      ; complexe sa classe. 
                      type-attr-temp (cadr list-nature)
                      bspec-loc (equal (ascii (sref (string type-attr-temp) 0)) '%)
                      type-attr
                      (if bspec-loc
                          ; le type de l'attribut commence par un %. C'est le cas                
                          ; de la specialisation locale. Je cherche la classe mere.
                          (car (#:shirka:noyau:peres (#:shirka:noyau:schema type-attr-temp)))
                          ; sinon c'est le type simple ou la classe
                          type-attr-temp)
                      ; est-ce un type simple ?
                      btype-simple (not (null (#:ivan:ed-util:type-simple? type-attr)))
                      ; est-ce une chaine ?
                      bchaine (eq type-attr 'chaine)
                      
                      liste-de (if bliste 
                                   (font #:ivan:ed-global:fstd #M#:txt-const:liste-de)
                                   " " )
                      spec-loc (if bspec-loc
                                   (font #:ivan:ed-global:fstd #M#:txt-const:spec-loc)
                                   "" )
                      ) 
                ; 
                ; reinitialiser le selectionneur
                ;
                (send 'set-selected-string sm "")
                ;
                ; verifier le top
                ;
                (ifn (or  (equal #:ivan:re-global:top 0)
                         (equal #:ivan:re-global:top 2) )
                     ; alors erreur
                     (:envoie-msg sm
                             (list
                                   (font #:ivan:in-global:fstdb #M#:txt-const:impos-sel)))
                     ; sinon
                     ;
                     ; mettre a jour le nom de l'attribut selectionne
                     ;
                     (setq #:ivan:re-global:nom-attr nom-attr-sym)
                     ;
                     ; declencher la mise a jour de l'image du type de l'attribut
                     ; :img-type construit l'image et inhibe les comparateurs
                     (:nouv-img-type sm
                             nom-attr-sym
                             type-attr
                             btype-simple
                             liste-de 
                             spec-loc)
                     ;
                     ; inserer le nom de l'attribut dans la liste des attributs
                     ;
                     (if (equal #:ivan:re-global:top 0)
                         ; il s'agit d'un nouveau critere = inserer l'attribut
                         (newr #:ivan:re-global:list-attr (vector
                                                                  nom-attr-ch 
                                                                  nom-classe-ch
                                                                  type-attr))
                         ; le top = 2 , il s'agit d'un critere complexe = rajouter l'attribut
                         ; au dernier element de la liste
                         (setq
                               list-reverse (reverse #:ivan:re-global:list-attr)
                               list-attr    (car list-reverse)
                               ; la liste des attributs
                               #:ivan:re-global:list-attr
                               (reverse 
                                       (rplaca 
                                               list-reverse
                                               (append1 list-attr (vector
                                                                          nom-attr-ch
                                                                          nom-classe-ch
                                                                          type-attr))
                                               ))
                               )
                         )
                     ; mettre a jour les variables globales :
                     (setq
                           ; la requete
                           #:ivan:re-global:list-requete
                           (list
                                 #:ivan:re-global:nom-classe
                                 #:ivan:re-global:list-attr
                                 #:ivan:re-global:list-comp
                                 #:ivan:re-global:list-val
                                 #:ivan:re-global:list-conn
                                 )
                           ; le nom de la classe pour l'editeur d'attributs
                           #:ivan:re-global:classe-ea  nom-classe-ch
                           
                           ; incrementer le top
                           #:ivan:re-global:top (+ 1 #:ivan:re-global:top)
                           )
                     ;
                     ; declencher la mise a jour de l'image de la requete
                     ;            
                     (:nouv-img-requete sm)
                     
                     ; inhiber les boutons comparateur
                     (:inhibit-comp sm bliste btype-simple type-attr)
                     )
                )
          )   
    )
;;;*****************************************************************************************
;;; action des boutons 'comparateur'

(de :action-comp (b)
    (let
         (
          (sauvtop #:ivan:re-global:top)
          )
         ;
         ; verifier le top
         ;
         (ifn (or (equal #:ivan:re-global:top 1)
                  (equal #:ivan:re-global:top 3) )
              ; alors erreur
              (:envoie-msg b
                      (list
                            (font #:ivan:in-global:fstdb #M#:txt-const:impos-comp)))
              ; sinon
              ;
              ; inserer le nom du comparateur dans la liste des comparateurs
              ;
              (newr #:ivan:re-global:list-comp (send 'image b))
              
              ; mettre a jour la requete
              (setq #:ivan:re-global:list-requete
                    (list
                          #:ivan:re-global:nom-classe
                          #:ivan:re-global:list-attr
                          #:ivan:re-global:list-comp
                          #:ivan:re-global:list-val
                          #:ivan:re-global:list-conn
                          ))               
              ;
              ; incrementer le top
              ;
              (setq #:ivan:re-global:top 4)
              ;
              ; declencher la mise a jour de l'image de la requete
              ;            
              (:nouv-img-requete b)
              ;
              ; declencher l'appel a l'editeur d'attributs
              ;
              (setq 
                    val-attr (#:ivan:re-edattr:ea 
                                     #:ivan:re-global:classe-ea
                                     #:ivan:re-global:nom-attr
                                     nil                          ; pas de valeur precedente
                                     (send 'image b)              ; le comparateur
                                     )
                    )
              ;
              ; inserer la valeur dans la liste
              ;
              (newr #:ivan:re-global:list-val val-attr)
              
              ; mettre a jour la requete
              (setq #:ivan:re-global:list-requete
                    (list
                          #:ivan:re-global:nom-classe
                          #:ivan:re-global:list-attr
                          #:ivan:re-global:list-comp
                          #:ivan:re-global:list-val
                          #:ivan:re-global:list-conn
                          ))
              ;
              ; incrementer le top
              ;
              (setq #:ivan:re-global:top 5)
              ; 
              ; reinitialiser la classe si necessaire
              ;
              (when (equal sauvtop 3)
                    (:reinit-edl-classe
                            (component 'edl-classe b)
                            #:ivan:re-global:nom-classe
                            ))                 
              ;
              ; declencher la mise a jour de l'image de la requete
              ;            
              (:nouv-img-requete b)               
              )
         )
    )
;;;*****************************************************************************************
;;; action des boutons 'connecteur'

(de :action-conn (b)
    ;
    ; verifier le top
    ;
    (ifn (equal #:ivan:re-global:top 5)
         ; alors erreur
         (:envoie-msg b
                 (list
                       (font #:ivan:in-global:fstdb #M#:txt-const:impos-con)))
         ; sinon
         ;
         ; inserer le nom du connecteur dans la liste des connecteurs
         ;
         (newr #:ivan:re-global:list-conn (send 'image b))
         
         ; mettre a jour les variables globales
         
         (setq 
               ; la requete
               #:ivan:re-global:list-requete
               (list
                     #:ivan:re-global:nom-classe
                     #:ivan:re-global:list-attr
                     #:ivan:re-global:list-comp
                     #:ivan:re-global:list-val
                     #:ivan:re-global:list-conn
                     ) 
               ; le top
               #:ivan:re-global:top 0
               )
         ;
         ; declencher la mise a jour de l'image de la requete
         ;            
         (:nouv-img-requete b)
         )
    )
;;;*****************************************************************************************
;;; action du selectionneur des instances

(de :action-sm-instances (sm)
    (:action-sm-selection sm)
    )
;;;*****************************************************************************************
;;; action du selectionneur de la selection trouvee
;;; appel a l'editeur d'instances sur cette instance

(de :action-sm-selection (sm)
    (let
         (
          ; nom de l'instance
          (valeur (send 'get-selected-string sm))
          nom-inst
          nom-classe-sym
          )
         (ifn valeur
              ; alors rien : cas de la reinitialisation du sm
              't
              ; sinon
              (send 'set-selected-string sm "")
              
              (setq
                    nom-inst (#:ivan:ed-util:symbole valeur) 
                    ; je recupere le nom de la classe
                    nom-classe-sym (#:shirka:noyau:nom-sch 
                                           (#:shirka:noyau:classe-de nom-inst))
                    )
              ; appeler l'editeur d'instance
              (#:ivan:ed-editer:ei
                      nom-classe-sym
                      nom-inst
                      nil 
                      nil
                      'visu ; bappel est en mode visualisation
                      )
              )
         )
    ) 
;;;*****************************************************************************************
;;; action du bouton classe de l'attribut

(de :action-bout-classe (b)
    (lets
          (
           (nom-classe ({application}:userdata b))
           (list-attr  #:ivan:re-global:list-attr)
           (attr       (nth (- (length list-attr) 1) list-attr)) ; dernier elt
           (nouv-attr  (if (listp attr)
                           attr
                           (list attr)
                           ))
           (nouv-list  (reverse (rplaca (reverse list-attr) nouv-attr)))
           )
          ;
          ; modifier les variables globales
          ;
          (setq
                ; la liste des attributs
                #:ivan:re-global:list-attr nouv-list
                ; la requete
                #:ivan:re-global:list-requete
                (list
                      #:ivan:re-global:nom-classe
                      #:ivan:re-global:list-attr
                      #:ivan:re-global:list-comp
                      #:ivan:re-global:list-val
                      #:ivan:re-global:list-conn
                      ) 
                )
          ;
          ; reinitialiser le top
          ;
          (setq #:ivan:re-global:top 2)
          
          ;
          ; reinitialiser la classe 
          ;
          (:reinit-edl-classe
                  (component 'edl-classe b)
                  nom-classe
                  )     
          )
    )
;;;*****************************************************************************************
;;;                     ACTION DES BOUTONS 'quitter' ET 'masquer'
;;;*****************************************************************************************

;;;*****************************************************************************************
;;; action du bouton masquer

(de :action-bout-masquer (b)
    (let
         (
          (appli ({application}:userdata b))
          )
         (move-behind-application b appli)
         
         (pop-application appli)
         )
    )
;;;*****************************************************************************************
;;; action du bouton quitter = fermer l'application apres une confirmation

(de :action-bout-quitter (b)
    (let
         (
          (appli ({application}:userdata b))
          )
         (when (componentp 'requete appli)
               (remove-component appli 'requete))
         ; reinitialiser les variables globales
         
         (:reinit-var-global "")
         
         (remove-application (send 'father b))
         )
    )
;;;*****************************************************************************************
;;;                             ACTION DES BOUTONS DE COMMANDE
;;;*****************************************************************************************

;;;-----------------------------------------------------------------------------------------
;;;
;;;                                     cluster SYSTEME
;;;
;;;-----------------------------------------------------------------------------------------

;;;*****************************************************************************************
;;; boucle jusqu'a ce que l'on clique sur le bouton 'annuler'
;;; permet de modifier le repertoire courant (variable #:shirka:*bases*)

(de :action-bout-repertoire (b)
    (lets
         (
          (edl-chemin
                   (lineedit 0 0
                           (+ #5wc (send 'width #:shirka:*bases*))
                           #:shirka:*bases*))
          (img-chemin
                   (#:ivan:ed-util:boxedimage
                           (row
                                (font #:ivan:in-global:fstd #M#:txt-const:chemin)
                                edl-chemin
                                )))
          (list-dir
                  (car (#:systeme:unix:catalogue (pathname #:shirka:*bases*))))
          (nom-chemin t)
          )
         
         (set-action edl-chemin
                 (lambda (edl) 
                         (setq #:shirka:*bases* (send 'get-line edl))
                         ))
         
         (while nom-chemin
                (setq
                      edl-chemin (lineedit 0 0
                                         (+ #5wc (send 'width #:shirka:*bases*))
                                         #:shirka:*bases*) 
                      img-chemin
                      (#:ivan:ed-util:boxedimage
                              (row
                                   (font #:ivan:in-global:fstd #M#:txt-const:chemin)
                                   edl-chemin
                                   ))
                      list-dir
                              (car (#:systeme:unix:catalogue (pathname #:shirka:*bases*)))
                      nom-chemin
                      (ask-string
                              (column                 
                                      (font #:ivan:in-global:fstdb #M#:txt-const:changer-rep)
                                      ""
                                      img-chemin
                                      ""
                                      (font #:ivan:in-global:fstd #M#:txt-const:pos-rep)
                                      )
                              list-dir
                              t
                              )
                      )  
                
                (cond
                      ( (null nom-chemin) (when #:trace:re (print "Exit rpertoire")))
                      
                      ( (equal nom-chemin "..")
                       
                       (#:systeme:unix:aller-dans nom-chemin)
                       
                       (if (equal (plength (current-directory)) 2) ; pour eviter la bogue
                           (setq #:shirka:*bases* "/")
                           (setq #:shirka:*bases* (current-directory))
                           )
                       )
                      ( t
                         (#:systeme:unix:aller-dans 
                                 (pathname (catenate #:shirka:*bases* nom-chemin "/")))
                         ;
                         (setq #:shirka:*bases* (current-directory))
                         )
                      )
                )
         )
    )
;;;-----------------------------------------------------------------------------------------
;;;
;;;                                     cluster CONSTRUCTEUR
;;;
;;;-----------------------------------------------------------------------------------------

;;;*****************************************************************************************

(de :action-bout-initialiser (b)
    (when (confirm
                  (column
                          (font #:ivan:in-global:fstd #M#:txt-const:reinit-fen)
                          ""
                          (font #:ivan:in-global:fstdb #M#:txt-const:attention)
                          
                          (font #:ivan:in-global:fstd #M#:txt-const:req-perdue)
                          ""
                          ""
                          (row 
                               (font #:ivan:in-global:fstd #M#:txt-const:clic-on)
                               (font #:ivan:in-global:fstdb #M#:txt-const:d-accord)
                               (font #:ivan:in-global:fstd #M#:txt-const:pour-init) )
                          (row
                               (font #:ivan:in-global:fstd #M#:txt-const:ou-sur)
                               (font #:ivan:in-global:fstdb #M#:txt-const:annuler)
                               (font #:ivan:in-global:fstd #M#:txt-const:pour-aband) )
                          ))
          
          ; reinitialiser les variables globales
          (:reinit-var-global (send 'get-line (component 'edl-classe
                                                      (component 'ivan b))))
          
          ; classe courante
          (:reinit-edl-classe (component 'edl-classe b)
                  (send 'get-line (component 'edl-classe
                                          (component 'ivan b) )))
          
          ; reinitialiser la liste des instances de la classe
          (#:ivan:in-interf:reinit-sm
                  (component 'scroll-instances b)  
                  (component 'sm-instances     b)
                  (:list-instances (send 'get-line (component 'edl-classe b)))
                  )
          (:reinit-card
                  (component 'card-instances b)
                  (send 'get-strings (component 'sm-instances b))
                  )
          
          ; reinitialiser la selection
          (#:ivan:in-interf:reinit-sm
                  (component 'scroll-selection b)  
                  (component 'sm-selection     b)
                  nil
                  )
          (:reinit-card 
                  (component 'card-selection b)
                  (send 'get-strings (component 'sm-selection b))
                  )
          
          (:autorise-sel b)
          )
    )
;;;*****************************************************************************************

(de :action-bout-majclasse (b)
    ; reinitialiser les variables globales
    (:reinit-var-global (send 'get-line (component 'edl-classe
                                                (component 'ivan b))))
    
    ; classe courante
    (:reinit-edl-classe (component 'edl-classe b)
            (send 'get-line (component 'edl-classe
                                    (component 'ivan b) )))
    
    ; reinitialiser la selection
    (#:ivan:in-interf:reinit-sm
            (component 'scroll-selection b)  
            (component 'sm-selection     b)
            nil
            )
    (:reinit-card 
            (component 'card-selection b) 
            nil
            )
    
    (:autorise-sel b)
    )
;;;-----------------------------------------------------------------------------------------
;;;
;;;                                     cluster REQUETE
;;;
;;;-----------------------------------------------------------------------------------------

;;;*****************************************************************************************
;;; action du bouton vider = reinitialise la requete

(de :action-bout-vider (b)
    ;
    ; reinitialiser les variables globales
    ;
    (:reinit-var-global (send 'get-line (component 'edl-classe b)))
    ;
    ; raz du type de l'attribut
    ;
    (send 'set-image (component 'sa-type b) (view))
    ;
    ; declencher la mise a jour de l'image de la requete
    ;            
    (:nouv-img-requete b)
    (:autorise-sel b)
    )
;;;*****************************************************************************************
;;; action du bouton charger = ouvrir une fenetre pour saisir le nom du fichier a charger

(de :action-bout-charger (b)
    (lets
          (
           (edl-chemin (lineedit 0 0
                               (+ #5wc (send 'width #:shirka:*bases*))
                               #:shirka:*bases*) )
           (img-chemin
                   (#:ivan:ed-util:boxedimage
                           (row
                                (font #:ivan:in-global:fstd #M#:txt-const:chemin)
                                edl-chemin
                                )))
           (list-files (cdr (#:systeme:unix:catalogue (pathname #:shirka:*bases*))))
           
           (nom-base
                   (ask-string
                           (column                 
                                   (font #:ivan:in-global:fstdb #M#:txt-const:charge-req)
                                   ""
                                   img-chemin
                                   ""
                                   (font #:ivan:in-global:fstd #M#:txt-const:charg-fic)
                                   )
                           list-files
                           nil
                           )
                   )
           (nom-fichier (catenate (send 'get-line edl-chemin) nom-base))
           )
          (when nom-base
                
                ; charger le fichier
                (setq 
                      #:ivan:re-global:list-requete (#:ivan:ed-util:lire nom-fichier)
                      #:ivan:re-global:nom-classe (nth 0 #:ivan:re-global:list-requete)
                      #:ivan:re-global:classe-ea  (nth 0 #:ivan:re-global:list-requete)
                      #:ivan:re-global:nom-attr   ""
                      #:ivan:re-global:list-attr  (nth 1 #:ivan:re-global:list-requete)
                      #:ivan:re-global:list-comp  (nth 2 #:ivan:re-global:list-requete)
                      #:ivan:re-global:list-val   (nth 3 #:ivan:re-global:list-requete)
                      #:ivan:re-global:list-conn  (nth 4 #:ivan:re-global:list-requete)
                      #:ivan:re-global:top 5
                      )
                ; classe courante
                (:reinit-edl-classe (component 'edl-classe b)
                        #:ivan:re-global:nom-classe)
                
                ; reinitialiser la liste des instances de la classe
                (#:ivan:in-interf:reinit-sm
                        (component 'scroll-instances b)  
                        (component 'sm-instances     b)
                        (:list-instances (send 'get-line (component 'edl-classe b))) 
                        )
                (:reinit-card
                        (component 'card-instances b)
                        (send 'get-strings (component 'sm-instances b))
                        )
                (:autorise-sel b)
                )
          )
    )
;;;*****************************************************************************************

(de :action-bout-sauver (b)
    (:sauver-liste b 
            #:ivan:re-global:list-requete #M#:txt-const:sauv-req)
    )
;;;*****************************************************************************************
;;; lance l'evaluation de la requete

(de :action-bout-executer (b)
    (lets
          (
           (nom-classe-ch (send 'get-line (component 'edl-classe b)))
           ; liste en entree (c'est l'intersection des instances a n niveau
           ;  de la classe et des instances dans le selectionneur d'instances
           (l-in (intersection
                         (mapcar '#:ivan:ed-util:symbole
                                 (:list-instances nom-classe-ch))
                         (mapcar '#:ivan:ed-util:symbole 
                                 (send 'get-strings (component 'sm-instances b))) ))
           ; liste en sortie : elle sera renvoyee
           l-out
           ; variable de message
           msg
           )
          (:envoie-msg b
                  (list (font #:ivan:in-global:fstdb #M#:txt-const:pat-sel)))
          
          (setq l-out (sortl (:evaluer-requete
                                     #:ivan:re-global:list-attr
                                     #:ivan:re-global:list-comp
                                     #:ivan:re-global:list-val
                                     #:ivan:re-global:list-conn
                                     l-in) )
                )
          (:envoie-msg b nil)
          (ifn l-out
               (:envoie-msg b
                       (list (font #:ivan:in-global:fstdb #M#:txt-const:desole)))
               ; sinon
               (setq msg (if (= (length l-out) 1) " instance" " instances"))
               (:envoie-msg b
                       (list 
                             (row
                                  (font #:ivan:in-global:fstd #M#:txt-const:found)
                                  (font #:ivan:in-global:fstdb (string (length l-out)))
                                  (font #:ivan:in-global:fstd (catenate msg " !"))
                                  )) )
               
               )
          ; reinitialiser la selection
          (#:ivan:in-interf:reinit-sm
                  (component 'scroll-selection b)  
                  (component 'sm-selection     b)
                  l-out
                  )
          (:reinit-card 
                  (component 'card-selection b)
                  l-out
                  )
          (:autorise-sel b)
          )
    )
;;;-----------------------------------------------------------------------------------------
;;;
;;;                                     cluster SELECTION
;;;
;;;-----------------------------------------------------------------------------------------

;;;*****************************************************************************************
;;; ouvrir une fenetre pour saisir le nom du fichier a charger
;;; transferer la liste dans le selectionneur d'entree

(de :action-bout-chargersel (b)
    (lets
          (
           (edl-chemin (lineedit 0 0
                               (+ #5wc (send 'width #:shirka:*bases*))
                               #:shirka:*bases*) )
           (img-chemin
                   (#:ivan:ed-util:boxedimage
                           (row
                                (font #:ivan:in-global:fstd #M#:txt-const:chemin)
                                edl-chemin
                                )))
           (list-files (cdr (#:systeme:unix:catalogue (pathname #:shirka:*bases*))))
           
           (nom-base
                   (ask-string
                           (column
                                   (font #:ivan:in-global:fstdb #M#:txt-const:liste-inst)
                                   ""
                                   img-chemin
                                   ""
                                   (font #:ivan:in-global:fstd #M#:txt-const:charg-fic)
                                   )
                           list-files
                           nil
                           )
                   )
           (nom-fichier (catenate (send 'get-line edl-chemin) nom-base))
           list-in ; liste des instances
           )
          (when nom-base
                ; charger le fichier
                (setq
                      list-in (mapcar 'string (#:ivan:ed-util:lire nom-fichier))
                      )
                ; initialiser le selectionneur
                (#:ivan:in-interf:reinit-sm
                        (component 'scroll-instances b)
                        (component 'sm-instances     b)
                        list-in
                        )
                (:reinit-card
                        (component 'card-instances b)
                        list-in
                        )
                
                )
          )
    )
;;;*****************************************************************************************
;;; declenche la sauvegarde de la liste du selectionneur des instances trouvees

(de :action-bout-sauversel (b)
    (:sauver-liste
            b
            (send 'get-strings (component 'sm-selection b))
	    #M#:txt-const:sauv-l-inst)
    )
;;;*****************************************************************************************
;;; la liste resultat est reprise en entree

(de :action-bout-conserver (b)
    (let
         (
          (list-sel (send 'get-strings (component 'sm-selection b)))
          )
         ; reinitialiser la liste des instances de la classe
         (#:ivan:in-interf:reinit-sm
                 (component 'scroll-instances b)  
                 (component 'sm-instances     b)
                 list-sel
                 )
         (:reinit-card
                 (component 'card-instances b)
                 list-sel
                 )
         ; reinitialiser la selection
         (#:ivan:in-interf:reinit-sm
                 (component 'scroll-selection b)  
                 (component 'sm-selection     b)
                 nil
                 )    
         (:reinit-card 
                 (component 'card-selection b) 
                 nil
                 )
         )
    )
;;;*****************************************************************************************
;;;                             ACTION DES STRINGSELECTOR DE LA REQUETE 
;;;*****************************************************************************************

;;;*****************************************************************************************
;;; on a clique sur une valeur : modifier la valeur

(de :action-modif-val (ss)
    (lets
          (
           (valeur   (send 'get-selected-string ss))
           (n        ({application}:userdata ss))
           ; attr-int est la forme interne du critere :
           ; c'est soit un vecteur, soit une liste de vecteurs
           (attr-int (nth n #:ivan:re-global:list-attr ))
           (val-prec (nth n #:ivan:re-global:list-val  ))
           (comp     (nth n #:ivan:re-global:list-comp ))
           nom-classe
           nom-attr
           type-attr
           )
          (ifn valeur
               't
               ; sinon
               (send 'set-selected-string ss "")
               (when val-prec
                     ; il y a une valeur = appeler l'editeur d'attribut
                     (if (listp attr-int)
                         (setq
                               nom-attr     (vref (nth (- (length attr-int) 1) attr-int) 0)
                               nom-classe   (vref (nth (- (length attr-int) 1) attr-int) 1)
                               type-attr    (vref (nth (- (length attr-int) 1) attr-int) 2)
                               )
                         ; sinon
                         (setq
                               nom-attr   (vref attr-int 0)
                               nom-classe (vref attr-int 1)
                               type-attr  (vref attr-int 2)
                               )
                         )
                     ;
                     ; appeler l'editeur d'attributs
                     ;
                     (setq
                           val-attr (#:ivan:re-edattr:ea
                                            nom-classe
                                            nom-attr
                                            val-prec
                                            comp
                                            )
                           )
                     ;
                     ; remplacer la valeur
                     ;
                     (setq 
                           #:ivan:re-global:list-val 
                           (#:ivan:ed-util:remp-nlv (+ 1 n) #:ivan:re-global:list-val val-attr)
                           )
                     ;
                     ; redessiner l'image
                     ;
                     (:nouv-img-requete ss)
                     )
               )
          )
    )
;;;*****************************************************************************************
;;; on a clique sur le comparateur = ouvrir une fenetre pour le modifier

(de :action-modif-comp (ss)
    (lets
          (
           (list-comp
                   (list 
                         #M#:txt-const:text-egal
                         #M#:txt-const:text-diff
                         #M#:txt-const:text-sup
                         #M#:txt-const:text-inf
                         #M#:txt-const:text-supegal
                         #M#:txt-const:text-infegal
                         #M#:txt-const:text-cont
                         #M#:txt-const:text-ncont
                         #M#:txt-const:text-deb
                         #M#:txt-const:text-fin
                         #M#:txt-const:text-incl
                         #M#:txt-const:text-appart
                         ))
           (valeur        (send 'get-selected-string ss))
           (list-param    ({application}:userdata ss))
           (n             (nth 0 list-param))
           (bliste        (nth 1 list-param))
           (btype-simple  (nth 2 list-param)) 
           (type-attr     (nth 3 list-param))  
           (val-prec      (nth n #:ivan:re-global:list-val))
           nouv-comp
           )
          (ifn valeur
               't
               ; sinon
               (send 'set-selected-string ss "")
               (when val-prec
                     ; il y a une valeur = modifier le comparateur
                     ;
                     ; verifier les comparateurs possibles
                     ;
                     (if bliste
                         (progn
                                (setq
                                      list-comp (remove #M#:txt-const:text-deb  list-comp)
                                      list-comp (remove #M#:txt-const:text-fin  list-comp)
                                      list-comp (remove #M#:txt-const:text-incl list-comp))
                                )
                         (setq
                               list-comp (remove #M#:txt-const:text-cont  list-comp)
                               list-comp (remove #M#:txt-const:text-ncont list-comp))
                         )
                     (ifn btype-simple
                          (progn
                                 (setq
                                       list-comp (remove #M#:txt-const:text-deb     list-comp)
                                       list-comp (remove #M#:txt-const:text-fin     list-comp)
                                       list-comp (remove #M#:txt-const:text-incl    list-comp)
                                       list-comp (remove #M#:txt-const:text-sup     list-comp)
                                       list-comp (remove #M#:txt-const:text-inf     list-comp)
                                       list-comp (remove #M#:txt-const:text-supegal list-comp)
                                       list-comp (remove #M#:txt-const:text-infegal list-comp))
                                 )
                          (cond
                                ( (or (eq type-attr 'entier) (eq type-attr 'reel))
                                 (setq
                                       list-comp (remove #M#:txt-const:text-deb  list-comp)
                                       list-comp (remove #M#:txt-const:text-fin  list-comp)
                                       list-comp (remove #M#:txt-const:text-incl list-comp))
                                 )
                                ( t
                                   ; chaine, symbole, booleen
                                   (setq              
                                         list-comp (remove #M#:txt-const:text-sup     list-comp)
                                         list-comp (remove #M#:txt-const:text-inf     list-comp)
                                         list-comp (remove #M#:txt-const:text-supegal list-comp)
                                         list-comp (remove #M#:txt-const:text-infegal list-comp))
                                   (when (eq type-attr 'booleen)
                                         (setq
                                               list-comp (remove #M#:txt-const:text-deb   list-comp)
                                               list-comp (remove #M#:txt-const:text-fin   list-comp)
                                               list-comp (remove #M#:txt-const:text-incl  list-comp)
                                               list-comp (remove #M#:txt-const:text-cont  list-comp)
                                               list-comp (remove #M#:txt-const:text-ncont list-comp))
                                         ) ))
                          )
                     ;
                     ; saisir le nouveau comparateur
                     ;                
                     (setq nouv-comp
                           (ask-string
                                   (font #:ivan:in-global:fstd #M#:txt-const:sel-com-rm)
                                   list-comp
                                   nil
                                   ))
                     (when nouv-comp
                           ;
                           ; remplacer le comparateur
                           ;
                           (setq 
                                 #:ivan:re-global:list-comp
                                 (#:ivan:ed-util:remp-nlv 
                                         (+ 1 n)
                                         #:ivan:re-global:list-comp
                                         nouv-comp)
                                 )
                           ;
                           ; redessiner l'image
                           ;
                           (:nouv-img-requete ss)
                           )
                     )
               )
          )
    )
;;;*****************************************************************************************
;;; on a clique sur un connecteur = le remplacer

(de :action-modif-conn (ss)
    (lets
          (
           (valeur   (send 'get-selected-string ss))
           (n        ({application}:userdata ss))
           (val-prec (nth n #:ivan:re-global:list-val))
           nouv-conn
           )
          (ifn valeur
               't
               ; sinon
               (send 'set-selected-string ss "")
               (when val-prec
                     ; il y a une valeur = modifier le comparateur
                     (setq nouv-conn
                           (ask-string
                                   (font #:ivan:in-global:fstd #M#:txt-const:sel-con-rm)
                                   (list #M#:txt-const:et #M#:txt-const:ou #M#:txt-const:sauf)
                                   nil
                                   ))
                     (when nouv-conn
                           ;
                           ; remplacer le comparateur
                           ;
                           (setq 
                                 #:ivan:re-global:list-conn
                                 (#:ivan:ed-util:remp-nlv (+ 1 n) #:ivan:re-global:list-conn nouv-conn)
                                 )
                           ;
                           ; redessiner l'image
                           ;
                           (:nouv-img-requete ss)
                           )
                     )
               )
          )
    )

;;;*****************************************************************************************
;;;                                   
;;;                           FONCTIONS UTILITAIRES
;;;
;;;*****************************************************************************************

;;;*****************************************************************************************
;;; construction de l'image de visualisation du commentaire de l'instance
;;; suivant la taille du commentaire, l'image est affichee ou non dans un scroller

(de :nouv-img-commentaire (composant nom-classe-sym)
    (lets
          (
           ; vecteur du schema de classe
           (schema-clas   (if nom-classe-sym
                              (#:ivan:ed-util:schema nom-classe-sym)
                              nil))
           
           ; commentaire de la classe (liste de chaines) 
           (list-comm (if schema-clas
                          (#:ivan:ed-util:val-fac-he schema-clas 'lui-meme '$com)
                          nil))
           (list-com
                   (mapcar '(lambda (chaine)
                                    (let ( (chaine-font (font #:ivan:ed-global:fstd chaine)))
                                         chaine-font
                                         ))
                           list-comm ))
           ; colonne des lignes
           (col-com (apply 'column list-com))
           
           ; largeur max d'1 ligne en nombre de caracteres (attention #wc = 9)
           (larg (#:ivan:ed-util:larg-max list-comm))
           
           ; hauteur du commentaire (nombre de lignes)
           (haut (ifn (null list-com)
                      (length list-com)))
           
           ; hauteur de l'image commentaire si scroller
           (h-scroll (+ {scroller}:scrollbarheight
                        (#:ivan:ed-util:hc :%h-max)
                        2))
           ; largeur de l'image commentaire si scroller
           (w-scroll (+ {scroller}:scrollbarwidth
                        (#:ivan:ed-util:wc :%l-ei)
                        2))
           
           ; img du commentaire     
           (img-com 
                   (cond
                         ; pas de commentaire = 1 ligne vide
                         ((null list-com)
                          (rectangle 0 0 w-scroll h-scroll)
                          )
                         ; le commentaire tient en largeur et hauteur : boite rectangulaire
                         ((and (<= larg :%l-ei) (<= haut :%h-max))
                          (view
                                (rectangle 0 0 w-scroll (#:ivan:ed-util:hc haut))
                                col-com
                                )
                          )
                         ; le commentaire est trop large et trop haut : scroller   
                         ((and (> larg :%l-ei) (> haut :%h-max))
                          (scroller 0 0
                                  w-scroll h-scroll
                                  col-com
                                  t t
                                  )
                          )
                         ; le commentaire est trop large : scroller horizontal                   
                         ((> larg :%l-ei)
                          (horizontalscroller 0 0
                                  w-scroll h-scroll                               
                                  col-com
                                  )
                          )
                         ; le commentaire est trop haut : scroller vertical              
                         (t         
                            (verticalscroller 0 0
                                    w-scroll h-scroll
                                    col-com
                                    )
                            )
                         ))
           )
          ; image finale
          (if list-com
              (send 'set-image (component 'sa-com composant)
                    (#:ivan:ed-util:clusterimage img-com #M#:txt-const:comment)
                    )
              (send 'set-image (component 'sa-com composant)
                    img-com
                    )
              )
          )
    )
;;;*****************************************************************************************
;;; renvoie l'image du type de l'attribut

(de :nouv-img-type (composant 
                           nom-attr-sym
                           type-attr
                           btype-simple
                           liste-de 
                           spec-loc)
    
    (let
         (
          bout-classe
          img 
          trouve
          )
         (ifn nom-attr-sym
              ; alors
              (setq img (rectangle 0 0 #20wc #2hc))
              ; sinon
              (setq 
                    bout-classe (if btype-simple
                                    (font #:ivan:ed-global:fstd
                                          (catenate (string type-attr) " "))
                                    ; sinon on doit pouvoir cliquer sur la classe
                                    (standardbutton (font #:ivan:ed-global:fstd
                                                          (catenate (string type-attr) " ")))
                                    )
                    
                    img (centeredview 
                                (rectangle 0 0 #20wc #2hc)
                                (row
                                     (centeredview
                                             (rectangle 0 0 0 (send 'height bout-classe))
                                             liste-de
                                             )
                                     bout-classe
                                     " "
                                     (centeredview
                                             (rectangle 0 0 0 (send 'height bout-classe))
                                             spec-loc
                                             )
                                     )
                                )
                    )
              ; faut-il activer le bouton de classe ? 
              (unless btype-simple
                      (add-component (component 'top composant) 'bout-classe bout-classe)
                      (ifn (member type-attr
                                   (#:shirka:noyau:l-inst-t (#:shirka:noyau:schema 'objet)))
                           ; le bouton n'est pas active car la classe n'est pas chargee 
                           (inhibit-application bout-classe)
                           ; sinon
                           ({application}:userdata bout-classe (string type-attr))
                           (set-action bout-classe ':action-bout-classe)
                           ) 
                      )
              )
         ; image du champ
         (send 'set-image (component 'sa-type composant) img)
         
         )
    )
;;;*****************************************************************************************

(de :nouv-img-requete (appli)
    (lets
          (
           (scroll-req (component 'scroll-requete appli))
           (li         (:const-img-req))
           (col        (apply 'column li))
           
           nouv-scroll-req
           )
          ; scroller
          (setq nouv-scroll-req
                (scroller 0 0
                        (send 'width scroll-req) (send 'height scroll-req)
                        col
                        t t))
          
          (send 'set-image scroll-req nouv-scroll-req)
          
          ; repositionner le scroller en fin 
          (#:ivan:ed-util:positionne nouv-scroll-req t)
          
          (:envoie-msg appli nil)
          
          (:autorise-img appli)
          
          )
    )
;;;*****************************************************************************************

(de :const-img-req ()
    (lets
          (
           ; liste pour le parcours
           (list-while #:ivan:re-global:list-attr)
           ; la variable globale #:ivan:re-global:list-attr contient la
           ; liste des "champs". Un champ porte sur l'attribut.
           ; soit l'attribut est de type simple : c'est alors un vecteur
           ; attribut classe
           ; soit la requete est complexe : c'est alors une liste de vecteurs.
           ; Les classes ne doivent pas etre visualisees a l'ecran
           ; je cree la liste liste-attr qui contient le nom de l'attribut
           ; ou la concatenation des noms d'attributs
           (list-attr (mapcar 'string
                              (mapcan '(lambda (attr-int)                                   
                                               (if (listp attr-int)                              
                                                   (list (:img-attr attr-int))
                                                   ; elt 0 est l'attribut
                                                   (list (vref attr-int 0))  
                                                   )                                             
                                               )                                               
                                      #:ivan:re-global:list-attr)))
           ; liste des images constituees pour chaque attribut
           list-img
           ; largeur max d'un attribut
           (larg-img-attr (+ 0 (#:ivan:ed-util:larg-max list-attr)))
           ; largeur max d'un comparateur
           (larg-img-comp (+ 0 (plength #M#:txt-const:sup-or-eq)))
           ; largeur max des valeurs
           (larg-img-val (+ 2 (#:ivan:ed-util:larg-max
                                      (#:ivan:ed-util:list-alpha
                                              (#:ivan:ed-util:list-plate 
                                                      #:ivan:re-global:list-val)))))
           ; largeur max d'un connecteur = sauf
           (larg-img-conn 4)
           
           ; compteur 
           (n 0)
           )
          ; boucle de parcours de la liste des attributs
          (while list-while
                 (lets
                       (
                        ; attr est la forme visuelle du critere
                        (attr     (nth n list-attr))
                        ; attr-int est la forme interne du critere :
                        ; c'est soit un vecteur, soit une liste de vecteurs
                        (attr-int (nth n #:ivan:re-global:list-attr))
                        (comp     (nth n #:ivan:re-global:list-comp))
                        (val      (nth n #:ivan:re-global:list-val))
                        (conn     (nth n #:ivan:re-global:list-conn))
                        ; pour pouvoir afficher les chaines entre "",
                        ; il faut retrouver le type de l'attribut
                        ; type shirka de l'attribut pour mettre entre des chaines
                        nom-classe
                        nom-attr
                        schema-clas
                        type-attr
                        list-nature
                        bliste
                        btype-simple
                        ; les images
                        img-attr
                        img-comp
                        img-val
                        img-conn
                        )
                       
                       (if (listp attr-int)
                           (setq
                                 nom-attr     (vref (nth (- (length attr-int) 1) attr-int) 0)
                                 nom-classe   (vref (nth (- (length attr-int) 1) attr-int) 1)
                                 type-attr    (vref (nth (- (length attr-int) 1) attr-int) 2)
                                 )
                           ; sinon
                           (setq
                                 nom-attr   (vref attr-int 0)
                                 nom-classe (vref attr-int 1)
                                 type-attr  (vref attr-int 2)
                                 )
                           )
                       (setq
                             ; nature shirka de l'attribut
                             list-nature (#:ivan:ed-util:nature
                                                 (#:ivan:ed-util:schema nom-classe)
                                                 nom-attr)
                             ; l'attribut est-il une liste ?
                             bliste (eq (car list-nature) '$liste-de)
                             ; est-ce un type simple ?
                             btype-simple (not (null (#:ivan:ed-util:type-simple? type-attr)))
                             img-attr (stringselector (list attr))
                             img-comp (stringselector (mapcar 'string (list comp)))
                             ; transformer la valeurs en une liste de chaine
                             val      (if (listp val)
                                          (mapcar 'string val)
                                          (list   (string val))
                                          )
                             img-val  (if val
                                          (if (equal type-attr 'chaine)
                                              (stringselector (#:ivan:ed-edinst:list-ch val))
                                              (stringselector (mapcar 'string val)))
                                          (stringselector nil) )
                             img-conn (stringselector (mapcar 'string (list conn)))
                             )
                       ; recadrer les differentes images
                       (send 'w img-attr (#:ivan:ed-util:wc larg-img-attr))
                       (send 'w img-comp (#:ivan:ed-util:wc larg-img-comp))
                       (send 'w img-val  (#:ivan:ed-util:wc larg-img-val))
                       (send 'w img-conn (#:ivan:ed-util:wc larg-img-conn))
                       
                       ; action de l'attribut
                       ({application}:userdata img-attr n)
                       (set-action img-attr  ':action-modif-val)
                       
                       ; action du comparateur
                       ({application}:userdata img-comp (list n bliste btype-simple type-attr))
                       (set-action img-comp  ':action-modif-comp)
                       
                       ; action des valeurs
                       ({application}:userdata img-val n)
                       (set-action img-val   ':action-modif-val)
                       
                       ; action du connecteurs
                       ({application}:userdata img-conn n)
                       (set-action img-conn  ':action-modif-conn)
                       
                       ; inserer l'image dans la liste des images
                       (newr list-img (row
                                           " "
                                           img-attr                           
                                           " "
                                           img-comp 
                                           " "
                                           img-val  
                                           " "
                                           img-conn))
                       ; traiter l'attribut suivant
                       (nextl list-while)
                       ; incrementer le compteur
                       (setq n (+ 1 n))
                       ) ; lets
                 ) ; while
          ; renvoyer la liste
          list-img
          )
    )
;;;*****************************************************************************************
;;; renvoie la liste des attributs de la classe selectionnee

(de :list-attributs (nom-classe-sym)
    (let
         (
          list-attribut
          schema-clas
          l-attr
          )
         (when nom-classe-sym
               (setq
                     schema-clas     (#:shirka:noyau:schema             nom-classe-sym)
                     ; je ne veux pas faire de requete sur est-un, util-par, nom-inst 
                     ; donc j'utilise la classe d'attribut attribut-shirka
                     list-attribut   (#:shirka:noyau:l-nom-att-he-class schema-clas nil '(attribut-shirka))
                     ))
         ; renvoyer la liste finale
         (if #:ivan:ed-global:attralpha
             (sortl list-attribut)
             list-attribut)
         )
    )
;;;*****************************************************************************************
;;; renvoie la liste des instances de la classe

(de :list-instances (nom-classe-ch)
    (let
         (
          (nom-classe-sym (#:ivan:ed-util:symbole nom-classe-ch))
          list-instances
          list-in
          schema-clas
          )
         (when nom-classe-sym
               (setq
                     schema-clas     (#:shirka:noyau:schema    nom-classe-sym)
                     list-instances  (#:ivan:ed-util:l-inst-t-ch schema-clas)
                     list-in         (if (listp list-instances)
                                         list-instances
                                         list-in)
                     ))
         (sortl list-in)
         )
    )
;;;*****************************************************************************************
;;; fonction d'inhibition des comparateurs selon le type de l'attribut

(de :inhibit-comp (appli bliste btype-simple type-attr)
    (if bliste
        (progn
               (inhibit-application (component 'bout-deb  appli))
               (inhibit-application (component 'bout-fin  appli))
               (inhibit-application (component 'bout-incl appli))
               )
        (inhibit-application (component 'bout-cont appli))
        (inhibit-application (component 'bout-ncont appli))
        )
    (ifn btype-simple
         (progn
                (inhibit-application (component 'bout-deb     appli))
                (inhibit-application (component 'bout-fin     appli))
                (inhibit-application (component 'bout-incl    appli))
                (inhibit-application (component 'bout-sup     appli))
                (inhibit-application (component 'bout-inf     appli))
                (inhibit-application (component 'bout-supegal appli))
                (inhibit-application (component 'bout-infegal appli))
                )
         ; sinon    
         (cond
               ( (or (eq type-attr 'entier) (eq type-attr 'reel))
                (inhibit-application (component 'bout-deb  appli))
                (inhibit-application (component 'bout-fin  appli))
                (inhibit-application (component 'bout-incl appli))
                )
               ( t
                  ; chaine , symbole ou booleen
                  (inhibit-application (component 'bout-sup     appli))
                  (inhibit-application (component 'bout-inf     appli))
                  (inhibit-application (component 'bout-supegal appli))
                  (inhibit-application (component 'bout-infegal appli))
                  
                  (when (eq type-attr 'booleen)
                        (inhibit-application (component 'bout-deb  appli))
                        (inhibit-application (component 'bout-fin  appli))
                        (inhibit-application (component 'bout-incl appli)) 
                        (inhibit-application (component 'bout-cont appli))
                        (inhibit-application (component 'bout-ncont appli))
                        )
                  )
               )
         )
    )
;;;*****************************************************************************************
;;; fonction d'autorisation des blocs semantiques du module en fonction du top

(de :autorise-img (c)
    (let
         (
          (nom-classe (send 'get-line (component 'edl-classe c)))
          )
         (inhibit-application  (component 'img-attributs    c))
         
         (when (componentp 'bout-classe c)
               (inhibit-application  (component 'bout-classe     c)))
         
         (inhibit-application  (component 'img-comparateurs c))
         (inhibit-application  (component 'img-connecteurs  c))
         (inhibit-application  (component 'bout-executer   c))
         (inhibit-application  (component 'bout-sauver     c))
         (cond
               ( (or (eq #:ivan:re-global:top 0)
                     (eq #:ivan:re-global:top 2) )
                (authorize-application (component 'img-attributs    c))
                )
               ( (or (eq #:ivan:re-global:top 1)
                     (eq #:ivan:re-global:top 3) )
                (when (componentp 'bout-classe c)
                      (authorize-application (component 'bout-classe     c)))
                (authorize-application (component 'img-comparateurs    c))
                )
               ( (eq #:ivan:re-global:top 4) 
                ; rien
                )
               
               ( (eq #:ivan:re-global:top 5)
                (authorize-application (component 'img-connecteurs  c))
                (authorize-application (component 'bout-executer   c))
                (authorize-application (component 'bout-sauver     c))
                )
               ( t
		(when #:trace:re
                  (print "valeur du top incorrecte : " #:ivan:re-global:top)
		)
		(when #:trace:re
                  (print erreur)
		)
                  )
               )
         )
    )
;;;*****************************************************************************************
;;;  fonction autorisation/inhibition des boutons du cluster de selection

(de :autorise-sel (c)
    (let
         (
          (list-sel (send 'get-strings (component 'sm-selection c)))
          )
         (if list-sel 
             (progn
                    (authorize-application (component 'bout-conserver  c))
                    (authorize-application (component 'bout-sauversel  c))
                    )
             (inhibit-application (component 'bout-conserver  c))
             (inhibit-application (component 'bout-sauversel  c)) 
             )
         )
    )
;;;*****************************************************************************************
;;; envoi du message list-msg au composant sa-warn de l'application appli

(de :envoie-msg (appli list-msg)
    (lets
          (
           (sa-warn (component 'sa-warn appli))
           (col-msg (apply 'column list-msg))
           (message (centeredview
                            (rectangle 0 0 (send 'width sa-warn) (send 'height sa-warn))
                            col-msg))
           )
          (send 'set-image sa-warn message)
          )
    )
;;;*****************************************************************************************
;;; reinitialise l'editeur de ligne (contenant le nom de la classe)

(de :reinit-edl-classe (edl msg)
    ; deproteger
    ({medite}:readonly-flag edl nil)
    ; nom de la classe ou rien
    (send 'set-line edl msg)
    ; proteger     
    ({medite}:readonly-flag edl t) 
    ; declencher l'action de l'editeur de ligne
    (:action-edl-classe edl)
    )
;;;*****************************************************************************************
;;; reinitialise la cardinalite 

(de :reinit-card (edl-card liste)
    ; deproteger
    ({medite}:readonly-flag edl-card nil)
    ; cardinalite
    (send 'set-line edl-card (string (length liste)) )
    ; proteger     
    ({medite}:readonly-flag edl-card t) 
    )
;;;*****************************************************************************************

(de :reinit-var-global (nom-classe-ch)
    (lets
          (
           (nom-classe-sym (#:ivan:ed-util:symbole nom-classe-ch))
           )
          ;
          ; reinitialiser les variables globales
          ;
          (setq 
                #:ivan:re-global:nom-classe nom-classe-ch
                #:ivan:re-global:classe-ea  nom-classe-ch
                #:ivan:re-global:nom-attr   nil
                
                #:ivan:re-global:list-attr  nil
                #:ivan:re-global:list-comp  nil
                #:ivan:re-global:list-val   nil
                #:ivan:re-global:list-conn  nil
                #:ivan:re-global:top 0
                
                #:ivan:re-global:list-requete
                (list
                      #:ivan:re-global:nom-classe
                      #:ivan:re-global:list-attr
                      #:ivan:re-global:list-comp
                      #:ivan:re-global:list-val
                      #:ivan:re-global:list-conn
                      )
                )
          )
    )
;;;*****************************************************************************************
;;; renvoie l'image de l'attribut dans le scroller de requetes
;;; exemple : ([a b c] [d e f] [g h i]) donne a.d.g

(de :img-attr (l)  
    (cond                                                     
          ( (null l) "")                                        
          ( (null (cdr l)) (vref (car l) 0) )                           
          (t (catenate (vref (car  l) 0) "." (:img-attr (cdr l))))              
          )                                                         
    )
;;;*****************************************************************************************
;;; ouvre une fenetre pour sauvegarder une liste

(de :sauver-liste (b liste titre)
    (lets
          (
           (edl-chemin (lineedit 0 0
                               (+ #5wc (send 'width #:shirka:*bases*))
                               #:shirka:*bases*) )
           (img-chemin
                   (#:ivan:ed-util:boxedimage
                           (row
                                (font #:ivan:in-global:fstd #M#:txt-const:chemin)
                                edl-chemin
                                )))
           
           nom-base
           nom-fichier
           (bsauver t)
           bfini
           )
          ;------------------------------------------------------------------------
          (while (not bfini)
                 (setq
                       nom-base
                       (ask-string
                               (column                 
                                       (font #:ivan:in-global:fstdb
                                             titre)
                                       ""
                                       img-chemin
                                       ""
                                       (font #:ivan:in-global:fstd #M#:txt-const:sauve-fic)
                                       )
                               (cdr (#:systeme:unix:catalogue (pathname #:shirka:*bases*)))
                               t
                               )
                       nom-fichier (catenate (send 'get-line edl-chemin) nom-base)
                       )
                 (ifn nom-base
                      
                      ; l'utilisateur a clique sur annuler
                      (setq
                            bfini   t
                            bsauver nil )
                      
                      ; sinon
                      (when (probefile nom-fichier)
                            (setq bsauver
                                  (confirm
                                          (column 
                                                  (font #:ivan:in-global:fstdb 
							(catenate #M#:txt-const:attention " "))
                                                  (row 
                                                       (font #:ivan:in-global:fstd
								#M#:txt-const:le-fic)
                                                       (font #:ivan:in-global:fstdb
                                                             	(string nom-base))
                                                       (font #:ivan:in-global:fstd
								#M#:txt-const:ecrase) )
                                                  ""
                                                  ""
                                                  (row 
                                                       (font #:ivan:in-global:fstd
								#M#:txt-const:clic-on)
                                                       (font #:ivan:in-global:fstdb
								#M#:txt-const:d-accord)
                                                       (font #:ivan:in-global:fstd
								#M#:txt-const:pour-sauver) )
                                                  (row 
                                                       (font #:ivan:in-global:fstdb
								#M#:txt-const:annuler)
                                                       (font #:ivan:in-global:fstd
								#M#:txt-const:pour-aband) )
                                                  )))
                            ) ; endwhen
                      (when bsauver
                            (#:ivan:ed-util:ecrire nom-fichier liste)
                            (:envoie-msg b 
                                    (list
                                          (row
                                               (font #:ivan:in-global:fstd #M#:txt-const:sauv-fic)
                                               (font #:ivan:in-global:fstdb
                                                     nom-base)
                                               (font #:ivan:in-global:fstd #M#:txt-const:eff)
                                               )))
                            )
                      (setq bfini t) ; sortir de la boucle
                      )
                 )
          )
    )
