;;; Repertoire: ~/LeShirka/LeIvan/CodeIvan/Unix/
;
;************************************************************************
;
;                                    UNIX.(STD).LL
;
;                        Jerome Euzenat (compatible unix.ll)
;
;***************************************************************************
;
; Ce fichier contient les fonctions lisp d'appel a UNIX qui permettent:
; -- de designer des catalogues.
; -- de se deplacer dans l'arborescence des catalogues.
;
; Cette version (std) utilise le systeme de pathname virtuels de lelisp.
;***************************************************************************

(defvar #:sys-package:colon '#:systeme:unix)

(defvar :wildfile
        (make-pathname () () () '#:pathname:wild '#:pathname:wild ()))

(defvar :wildfilter
        (make-pathname () () () '#:pathname:wild () ()))

(de :compath (p s)
    (merge-pathnames
            (make-pathname () () (reverse (cons s (reverse (pathname-directory p)))) () ())
            p)
    )

(de :comfile (p s)
    (merge-pathnames (pathname s) p)
    )

(de :decompath (p)
    (merge-pathnames
            (make-pathname () () (reverse (cdr (reverse (pathname-directory p)))) () ())
            p)
    )

(de :str2path (s) (pathname s)
    )

(de :dir (p) (directory-namestring p)
    )

(de :file (p) (file-namestring p)
    )

(de :path2str (p) (namestring p)
    )

;;;***************************************************************************
;;; Cette fonction place le process dans le path donne en argument

(defmacro :aller-dans (p) `(current-directory ,p))

;;;***************************************************************************
;;; Cette fonction fait remonter le process d'un catalogue

(defmacro :remonter (p) `(current-directory (:decompath ,p)))

;;;***************************************************************************
;;; catalogue
;;; Prend en argument un path (catalogue) et un filtre (path de fichier)
;;; Par defaut, le filtre utilise est "*.*" sous unix
;;; Rend un cons dont le car est la liste des catalogues et le 
;;; cdr est la liste des fichiers correspondant a filtre
;;; On est oblige de jouer avec les fonctions standard unix qui
;;; ne rendent pas les catalogues mais uniquement les fichiers

(de :catalogue (p . f)
    (:trier-file
            (if f
                (wildcard (merge-pathnames (car f) p))
                (wildcard (merge-pathnames :wildfilter p))
                ))
    )

;;;***************************************************************************
;;; listdir - NON EXPORTE
;;; fonctionne en Aida version 1.5

(de :listdir (p)
    (mapcan
            (function
                    (lambda (f)
                            (when (directoryp f) (list f))))
            (wildcard (merge-pathnames :wildfilter p))
            )
    )

;;;***************************************************************************

(de :trier-file (fl)
    (when fl
          (let (((dirs . files) (:trier-file (cdr fl))))
               (if (directoryp (car fl))
                   (cons (cons (file-namestring (car fl)) dirs) files)
                   (cons dirs (cons (file-namestring (car fl)) files)))))
    )

;;;***************************************************************************
